c;;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; RGGdat_fit.f ---
c;; Author          : Friedrich Jegerlehner
c;; Created On      : Mon Feb 20 16:00:34 2012
c;; Last Modified By: Friedrich Jegerlehner
c;; Last Modified On: Thu Jan  8 03:33:18 2015
c;; RCS: $Id$
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; Copyright (C) 2013 Friedrich Jegerlehner
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;;
c smoothed version of function RSGG_fun.f interpolating RGG0321.f dataset
c using piecewise Chebyshev polynomial fits.
c  19/04/2010   add option for error estimates with future experimental error reduction
c               if errors up to energy efuture are reduced to futureprecision
c               ifuture=1, efuture=2.5 GeV, futureprecision=0.01
c               test parameters via common /future/ifuture,efuture,futureprecision
c fitranges: 2 m_pi   CHPTCUT   =>  RCHPT -> RCHPTnew via r3gfitchptail
c            CHPTCUT  M_omega   =>  r3gfitrholow
c            M_omega  EFITOMBG  =>  r3gfitrhohig
c            EFITOMBG EFITMIN   =>  epem3gfit079081
c            EFITMIN  1.4 GeV   =>  epem3gfit0814new
c            1.4 GeV  3.2 GeV   =>  epem3gfit1425
c  07/06/2013 changed flavor splittrig [us] component 9/10 --> 10/10
c fit up to 2.125 GeV
      function rsggsmoothed(s,IER)
      implicit none
      INTEGER IER,IER1,IER_SAV,ICH,IGS,iso,ifuture,NFext,NFext1,nf
      REAL *8 rsggsmoothed,RSGG,RS40smoothed,
     &     rggfitchptail,epemggfit0814new,
     &     epemggfit079081,epemggfit1425,rggfitrholow,rggfitrhohig
      REAL *8 S,MP2,M2,POLP,POLM,E,EFITOMBG,EFITMIN,EFITMAX,ESAV,RES,
     &     EP,EMA,EMI,eps,CHPTCUT,RP,RM,MOM,EC,ECUT,ECHARMTH,RSBG,reno,
     &     RESULT,CHPTCUTLOC,efuture,futureprecision,
     &     RSGG1,RGG,R33,FGG,ERXYDAT
      real *8 mq(6),mp(6),th(6)
      COMMON/MFPI/MP2,M2/ERR/IER1/POL/POLP,POLM
      COMMON/CHAN/ICH
      COMMON/RCUTS/CHPTCUT,EC,ECUT
      COMMON/GUSA/IGS,iso
      EXTERNAL RSGG,RS40smoothed,rggfitchptail,
     &     epemggfit0814new,epemggfit079081,epemggfit1425,
     &     rggfitrholow,rggfitrhohig
      common /future/efuture,futureprecision,ifuture
      common /cufit/CHPTCUTLOC,ERXYDAT,ECHARMTH
      common /quama/mq,mp,th
      RSGG1=0.d0
      IER_SAV=1
      if ((IER.GT.1).and.(ifuture.eq.1).and.(sqrt(s).le.efuture)) then
         IER_SAV=IER
         IER=1
      endif
      reno=1.d0
      IER1=IER
      E=DSQRT(S)
c common CUT set for R33dat_fit.f
c      CHPTCUTLOC=0.312D0
      CHPTCUTLOC=0.320D0
      MOM=0.77703500000000003d0
      IF (IGS.EQ.1) MOM=0.61d0
      EFITOMBG=0.78783499999999995d0
      EFITMIN=0.81d0
      EFITMAX=1.40d0
      EMA=sqrt(POLP)
      EMI=sqrt(POLM)
      EP=sqrt(M2)
      ERXYDAT=2.125d0
c extend to get smooth mactch to rewighted RS40smooth
      ERXYDAT=2.150d0
      ECHARMTH=th(4)  ! 4.2 GeV
      eps=0.02d0
      RSBG=0.d0
      IF (E.LE.CHPTCUTLOC) THEN
         ICH=0
c         RSBG=rggfitchptail(s)
         RSBG=RS40smoothed(s,IER)
      ELSE IF ((E.GT.CHPTCUTLOC).AND.(E.LE.MOM)) THEN
         ICH=1
         RSBG=rggfitrholow(s)
      ELSE IF ((E.GT.MOM).AND.(E.LE.EFITOMBG)) THEN
         ICH=2
         RSBG=rggfitrhohig(s)
      ELSE IF ((E.GT.EFITOMBG).AND.(E.LT.EFITMIN)) THEN
         ICH=3
         RSBG=epemggfit079081(s)
      ELSE IF ((E.GE.EFITMIN).AND.(E.LE.EFITMAX)) THEN
         ICH=4
         RSBG=epemggfit0814new(s)
      ELSE IF ((E.GT.EFITMAX).AND.(E.LE.ERXYDAT)) THEN
         ICH=5
         RSBG=epemggfit1425(s)
      ELSE IF (E.GT.ERXYDAT) THEN
         ICH=7
         RSBG=RS40smoothed(s,IER)
      ENDIF
c      CALL RENOoldnew(E,reno)
      RESULT=RSBG*reno
      if (IER_SAV.GT.1) then
         IER=IER_SAV
         if (IER_SAV.EQ.2) RESULT=0.d0
         if (IER_SAV.EQ.3) RESULT=RESULT*futureprecision
      endif
      rsggsmoothed=RESULT
      return
      end
C
      function epemggfit0814new(s)
c Chebyshev Polynomial fits are for R-value (IER=1), statistical (IER=2) and
c systematic (IER=3) errors;
c Note: syst error is the "true" one, not represented as a fraction as in the data sets
      implicit none
      integer np,nm,ier,IRESON,ini
      parameter(np=8,nm=12)
      real *8 epemggfit0814new,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     aresl,astal,asysl,aresm,astam,asysm,aresh,astah,asysh,
     &     aresfm,astafm,asysfm,aresfp,astafp,asysfp,
     &     aresfr,astafr,asysfr,polynom
      dimension aresl(nm),astal(nm),asysl(nm)
      dimension aresm(np),astam(np),asysm(np)
      dimension aresh(nm),astah(nm),asysh(nm)
      dimension aresfm(np),astafm(np),asysfm(np)
      dimension aresfp(np),astafp(np),asysfp(np)
      dimension aresfr(np),astafr(np),asysfr(np)
      real *8 MFI,GFI,PFI,fsta3,fsys3
      real *8 EFIMM,EFIMX,EFIPX,EFIPP,CHPTCUTLOC,ERXYDAT,ECHARMTH
      real *8 fac,rbw,BW
      REAL *8 UGM2,sMeV
      real *8 etest0,etest1,stest0,stest1
      external BW
      REAL*8 FSTA(5),FSYS(5)
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      COMMON/BWFI/MFI,GFI,PFI
C common Psi and Upsilon parameters set in this subroutine
      COMMON/RESRELERR/FSTA,FSYS
      COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP
      COMMON/RESFIT/fac,IRESON
      COMMON/ERR/IER
      common /cufit/CHPTCUTLOC,ERXYDAT,ECHARMTH
      data ini /0/

c      ++++ epemggfit0814new ++++

      include 'epemggfit0814new.h'

      fsta3=FSTA(3)
      fsys3=FSYS(3)
      e=sqrt(s)
      yfit=0.d0
      UGM2=1.D6
      sMeV=s*UGM2
      EFIMM=0.959d0
      EFIMX=1.000d0
      EFIPX=1.040d0
      EFIPP=ERXYDAT
c Fit ranges phi region
C  0.810 -- 1.000
C  0.843 -- 1.197
C  1.062 -- 1.438
      if ((e.ge.0.81d0).and.(e.le.EFIMM)) then
         e1=0.81d0
         en=0.999999d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresl,nm)
         else if (ier.eq.2) then
            yfit=polynom(ex,astal,nm)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysl,nm)
         endif
      else if ((e.gt.EFIMM).and.(e.le.EFIMX)) then
         e1=0.959d0
         en=0.999999d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresfm,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,astafm,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysfm,np)
         endif
      else if ((e.gt.EFIMX).and.(e.lt.EFIPX)) then
c update jan 2012
         e1=1.00001d0
         en=1.03999D0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresfr,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,astafr,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysfr,np)
         endif
         if (IRESON.eq.1) then
            rbw=fac*BW(sMeV,MFI,GFI,PFI)
            if (ier.eq.2) then
               rbw=rbw*fsta3
               yfit=sqrt(yfit**2+rbw**2)
            else
               if (ier.eq.3) rbw=rbw*fsys3
               yfit=yfit+rbw
            endif
         endif
      else if (e.lt.1.435d0) then
         e1=1.04000001d0
         en=1.435d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresh,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,astah,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysh,np)
         endif
      else
         write (*,*)  ' Warning: E out of fit range in epemggfit0814new'
      endif
      epemggfit0814new=yfit
      return
      end
c
      function epemggfit079081(s)
      implicit none
      integer np,ier,IRESON
      parameter(np=7)
      real *8 epemggfit079081,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     ares,asta,asys,polynom,MOM,GOM,POM,fsta2,fsys2
      REAL*8 FSTA(5),FSYS(5)
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      real *8 UGM2,sMeV
      dimension ares(np),asta(np),asys(np)
      real *8 fac,rbw,BW
      external BW
      COMMON/BWOM/MOM,GOM,POM
      COMMON/RESRELERR/FSTA,FSYS
      COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP
      COMMON/RESFIT/fac,IRESON
      COMMON/ERR/IER

c      ++++ epemggfit079081 ++++

      include 'epemggfit079081.h'

c omega  .41871054d0,.810d0
      UGM2=1.D6
      sMeV=s*UGM2
      fsta2=FSTA(2)
      fsys2=FSYS(2)

      e=sqrt(s)
      yfit=0.d0
      e1=0.787d0
      en=0.80999d0
      if ((e.gt.0.81d0).or.(e.lt.e1)) then
         epemggfit079081=0.d0
         return
      endif
      es=(en+e1)
      ed=(en-e1)
      amap=2.d0/ed
      bmap=-es/ed
      ex=amap*e+bmap
      if (ier.eq.1) then
        yfit=polynom(ex,ares,np)
      else if (ier.eq.2) then
        yfit=polynom(ex,asta,np)
      else if (ier.eq.3) then
        yfit=polynom(ex,asys,np)
      endif
      if ((IRESON.eq.1).and.(e.ge.EOMM).and.(e.lt.EOMP)) then
         rbw=fac*BW(sMeV,MOM,GOM,POM)
            if (ier.eq.2) then
               rbw=rbw*fsta2
               yfit=sqrt(yfit**2+rbw**2)
            else 
               if (ier.eq.3) rbw=rbw*fsys2
               yfit=yfit+rbw
            endif
      endif
      epemggfit079081=yfit
      return
      end
c
      function epemggfit1425(s)
      implicit none
      integer np,nm,nx,ier,IRESON
      parameter(np=12,nm=6,nx=8)
      real *8 epemggfit1425,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     areslow,astalow,asyslow,aresmed,astamed,asysmed,
     &     ares,asta,asys,ares1,asta1,asys1,
     &     polynom,CHPTCUTLOC,ERXYDAT,ECHARMTH
      real *8 fres2023,fsta2023,fsys2023
      dimension areslow(np),astalow(np),asyslow(np)
      dimension aresmed(nx),astamed(nx),asysmed(nx)
      dimension ares(np),asta(np),asys(np)
      dimension ares1(np),asta1(np),asys1(np)
      dimension fres2023(nm),fsta2023(nm),fsys2023(nm)
      COMMON/ERR/IER
      common /cufit/CHPTCUTLOC,ERXYDAT,ECHARMTH

c      ++++ epemggfit1425 ++++

      include 'epemggfit1425.h'

      e=sqrt(s)
      yfit=0.d0
      if ((e.lt.1.38d0).or.(e.gt.ERXYDAT)) then
         epemggfit1425=0.d0
         return
      else if (e.le.1.45d0) then
c   1.375  --  2.038
         e1=1.375d0
         en=2.038d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,areslow,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,astalow,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asyslow,np)
         endif
      else if (e.lt.1.975d0) then
c   1.395 -- 2.000
         e1=1.395d0
         en=2.000d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,ares,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,asta,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asys,np)
         endif
      else if (e.le.ERXYDAT) then
c  1.975 -- 2.125 update jan 2012 / jan 2015
         e1=1.975d0
         en=ERXYDAT
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,fres2023,nm)
         else if (ier.eq.2) then
            yfit=polynom(ex,fsta2023,nm)
         else if (ier.eq.3) then
            yfit=polynom(ex,fsys2023,nm)
         endif
      else
          write (*,*)  ' Warning: E out of fit range in epemggfit1425'
      endif
         epemggfit1425=yfit
      return
      end
c
      function rggfitchptail(s)
      implicit none
      integer ier
      real *8 rggfitchptail,s,rfitchptail
      COMMON/ERR/IER
      rggfitchptail=rfitchptail(s)
      return
      end
c
      function rggfitrholow(s)
      implicit none
      integer np,ier,IRESON
      parameter(np=14)
      real *8 rggfitrholow,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     ares,asta,asys,polynom,MOM,GOM,POM,fsta2,fsys2
      REAL*8 FSTA(5),FSYS(5)
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      dimension ares(np),asta(np),asys(np)
      real *8 fac,rbw,BW
      REAL*8 UGM2,sMeV
      external BW
      COMMON/BWOM/MOM,GOM,POM
      COMMON/RESRELERR/FSTA,FSYS
      COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP
      COMMON/RESFIT/fac,IRESON
      COMMON/ERR/IER

c      ++++ rggfitrholow ++++

      include 'rggfitrholow.h'

c omega  .41871054d0,.810d0
c omega  .41871054d0,.810d0
      UGM2=1.D6
      sMeV=s*UGM2
      fsta2=FSTA(2)
      fsys2=FSYS(2)

      e=sqrt(s)
      yfit=0.d0
      e1=0.318d0
      en=0.778d0
      if ((e.gt.en).or.(e.lt.e1)) then
         rggfitrholow=0.d0
         return
      endif
c 0.318 -- 0.778 updated nov 2010 (incl BaBar/KlOE) / jan 2012
      es=(en+e1)
      ed=(en-e1)
      amap=2.d0/ed
      bmap=-es/ed
      ex=amap*e+bmap
      if (ier.eq.1) then
        yfit=polynom(ex,ares,np)
      else if (ier.eq.2) then
        yfit=polynom(ex,asta,np)
      else if (ier.eq.3) then
        yfit=polynom(ex,asys,np)
      endif
      if ((IRESON.eq.1).and.(e.ge.EOMM).and.(e.lt.EOMP)) then
         rbw=fac*BW(sMeV,MOM,GOM,POM)
         if (ier.eq.2) then
            rbw=rbw*fsta2
            yfit=sqrt(yfit**2+rbw**2)
         else 
            if (ier.eq.3) rbw=rbw*fsys2
            yfit=yfit+rbw
         endif
      endif
      rggfitrholow=yfit
      return
      end
c
      function rggfitrhohig(s)
      implicit none
      integer np,ier,IRESON
      parameter(np=9)
      real *8 rggfitrhohig,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     ares,asta,asys,polynom,MOM,GOM,POM,fsta2,fsys2
      REAL*8 FSTA(5),FSYS(5)
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      dimension ares(np),asta(np),asys(np)
      real *8 fac,rbw,BW
      REAL*8 UGM2,sMeV
      external BW
      COMMON/BWOM/MOM,GOM,POM
      COMMON/RESRELERR/FSTA,FSYS
      COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP
      COMMON/RESFIT/fac,IRESON
      COMMON/ERR/IER

c      ++++ rggfitrhohig ++++

      include 'rggfitrhohig.h'

c omega  .41871054d0,.810d0
c omega  .41871054d0,.810d0
      UGM2=1.D6
      sMeV=s*UGM2
      fsta2=FSTA(2)
      fsys2=FSYS(2)

      e=sqrt(s)
      yfit=0.d0
      e1=0.769000000000000d0
      en=0.789000000000000d0
      if ((e.gt.en).or.(e.lt.e1)) then
         rggfitrhohig=0.d0
         return
      endif
c  0.769  -- 0.789 update jan 2012
      es=(en+e1)
      ed=(en-e1)
      amap=2.d0/ed
      bmap=-es/ed
      ex=amap*e+bmap
      if (ier.eq.1) then
        yfit=polynom(ex,ares,np)
      else if (ier.eq.2) then
        yfit=polynom(ex,asta,np)
      else if (ier.eq.3) then
        yfit=polynom(ex,asys,np)
      endif
      if ((IRESON.eq.1).and.(e.ge.EOMM).and.(e.lt.EOMP)) then
         rbw=fac*BW(sMeV,MOM,GOM,POM)
         if (ier.eq.2) then
            rbw=rbw*fsta2
            yfit=sqrt(yfit**2+rbw**2)
         else 
            if (ier.eq.3) rbw=rbw*fsys2
            yfit=yfit+rbw
         endif
      endif
      rggfitrhohig=yfit
      return
      end
