c;;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; Rdat_fit.f --- 
c;; Author          : Friedrich Jegerlehner
c;; Created On      : Sun Jan 17 05:02:10 2010
c;; Last Modified By: Friedrich Jegerlehner
c;; Last Modified On: Tue Jan  6 01:44:44 2015
c;; RCS: $Id$
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; Copyright (C) 2009 Friedrich Jegerlehner
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;;
c smoothed version of R(s) using piecewise Chebyshev polynomial fits. 
c  19/04/2010   add option for error estimates with future experimental error reduction
c               if errors up to energy efuture are reduced to futureprecision
c               ifuture=1, efuture=2.5 GeV, futureprecision=0.01         
c               test parameters via common /future/ifuture,efuture,futureprecision
c fitranges: 2 m_pi   CHPTCUT   =>  RCHPT -> RCHPTnew
c            CHPTCUT  M_omega   =>  rfitrholow
c            M_omega  EFITOMBG  =>  rfitrhohig        IGS=1 rsigpp
c            EFITOMBG EFITMIN   =>  epemfit079081     IGS=1 rsigpp
c            EFITMIN  1.4 GeV   =>  epemfit0814new
c            1.4 GeV  3.2 GeV   =>  epemfit1432
c            3.2 GeV  EC        =>  epemfir32cut
      function rs40smoothed(s,IER)
      implicit none
      INTEGER IER,IER1,IER_SAV,ICH,IGS,iso,ifuture,
     &     IRESON,iresonances,iresonances_sav,IRESON_SAV
      REAL *8 rs40smoothed,RS40,RS40x,RCHPT,RCHPTnew,rfitchptail,
     &     rsigpp,epemfit0814new,
     &     epemfit079081,epemfit1432,rfitrholow,rfitrhohig,epemfit32cut
      REAL *8 S,MP2,M2,POLP,POLM,E,EFITOMBG,EFITMIN,EFITMAX,ESAV,RES,
     &     EP,EMA,EMI,eps,CHPTCUT,RP,RM,MOM,EC,ECUT,ECUTFIT,RSBG,reno,
     &     RESULT,efuture,futureprecision,facbw
      COMMON/MFPI/MP2,M2/ERR/IER1/POL/POLP,POLM
      COMMON/CHAN/ICH
      COMMON/RCUTS/CHPTCUT,EC,ECUT
      COMMON/GUSA/IGS,iso
      common /RESFIT/facbw,IRESON
      common /RES/iresonances
      EXTERNAL RS40,RS40x,RCHPT,RCHPTnew,rfitchptail,
     &     rsigpp,epemfit0814new,
     &     epemfit079081,epemfit1432,rfitrholow,rfitrhohig,epemfit32cut
      common /future/efuture,futureprecision,ifuture
      iresonances_sav=iresonances
      IRESON_SAV=IRESON
      iresonances=IRESON
      IER_SAV=1
      if ((IER.GT.1).and.(ifuture.eq.1).and.(sqrt(s).le.efuture)) then
         IER_SAV=IER
         IER=1
      endif
      reno=1.d0
      IER1=IER
      E=DSQRT(S)
c     CHPTCUT=0.318D0
      MOM=0.77703500000000003d0
      IF (IGS.EQ.1) MOM=0.61d0
      EFITOMBG=0.78783499999999995d0
      EFITOMBG=0.78700000000000001d0
      EFITMIN=0.81d0
      EFITMAX=1.40d0
      EMA=sqrt(POLP)
      EMI=sqrt(POLM)
      EP=sqrt(M2)
      ECUTFIT=DMIN1(ECUT,40.d0)
      eps=0.02d0
      RSBG=0.d0
      IF (E.LE.CHPTCUT) THEN
         ICH=0
c take parameter uncertainty as syst error
         RSBG=rfitchptail(s)
      ELSE IF ((E.GT.CHPTCUT).AND.(E.LE.MOM)) THEN
         ICH=1
         RSBG=rfitrholow(s)
      ELSE IF ((E.GT.MOM).AND.(E.LE.EFITOMBG)) THEN
         ICH=2
         IF (IGS.EQ.0) THEN
c     ###########  Chebyshev fit
            RSBG=rfitrhohig(s)
         ELSE
c     ########### alternative GS fit
            RSBG=rsigpp(s,iso)
         ENDIF
      ELSE IF ((E.GT.EFITOMBG).AND.(E.LT.EFITMIN)) THEN
         ICH=3
         IF (IGS.EQ.0) THEN
c     ###########  Chebychev fit
            RSBG=epemfit079081(s)
         ELSE
c     ########### alternative GS fit
            RSBG=rsigpp(s,iso)
         ENDIF
      ELSE IF ((E.GE.EFITMIN).AND.(E.LE.1.4349d0)) THEN
         ICH=4
         RSBG=epemfit0814new(s)
      ELSE IF ((E.GE.1.4d0).AND.(E.LE.3.2d0)) THEN
         ICH=5
         RSBG=epemfit1432(s)
      ELSE IF ((E.GT.3.2d0).AND.(E.LE.ECUTFIT)) THEN
c     ECUTFIT don't go beyond 15 GeV
         ICH=6
         RSBG=epemfit32cut(s)
      ELSE
         ICH=7
         RSBG=RS40(s,IER)
      ENDIF
      RESULT=RSBG*reno
      if (IER_SAV.GT.1) then
         IER=IER_SAV
         if (IER_SAV.EQ.2) RESULT=0.d0
         if (IER_SAV.EQ.3) RESULT=RESULT*futureprecision
      endif
c      CALL RENOoldnew(E,reno)
      rs40smoothed=RESULT
      iresonances=iresonances_sav
      IRESON=IRESON_SAV
      return
      end
C
      function epemfit0814new(s)
c Chebyshev Polynomial fits are for R-value (IER=1), statistical (IER=2) and
c systematic (IER=3) errors;
c Note: syst error is the "true" one, not represented as a fraction as in the data sets
      implicit none
      integer np,nm,ier,IRESON,ini
      parameter(np=8,nm=12)
      real *8 epemfit0814new,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     aresl,astal,asysl,aresm,astam,asysm,aresh,astah,asysh,
     &     aresfm,astafm,asysfm,aresfp,astafp,asysfp,
     &     aresfr,astafr,asysfr,polynom
      dimension aresl(nm),astal(nm),asysl(nm)
      dimension aresm(np),astam(np),asysm(np)
      dimension aresh(nm),astah(nm),asysh(nm)
      dimension aresfm(np),astafm(np),asysfm(np)
      dimension aresfp(np),astafp(np),asysfp(np)
      dimension aresfr(np),astafr(np),asysfr(np)
      real *8 MFI,GFI,PFI,fsta3,fsys3
      real *8 EFIMM,EFIMX,EFIPX,EFIPP
      real *8 fac,rbw,BW
      REAL *8 UGM2,sMeV
      real *8 etest0,etest1,stest0,stest1
      external BW
      REAL*8 FSTA(5),FSYS(5)
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      COMMON/BWFI/MFI,GFI,PFI
C common Psi and Upsilon parameters set in this subroutine
      COMMON/RESRELERR/FSTA,FSYS
      COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP
      COMMON/RESFIT/fac,IRESON
      COMMON/ERR/IER
      data ini /0/

c      ++++ epemfit0814new ++++  

      include 'epemfit0814new.h'

      fsta3=FSTA(3)
      fsys3=FSYS(3)
      e=sqrt(s)
      yfit=0.d0
      UGM2=1.D6
      sMeV=s*UGM2
      EFIMM=0.95899999999999996d0
      EFIMX=1.00000000000000000d0
      EFIPX=1.0400000000000000d0
      EFIPP=1.1120000000000001d0
c Fit ranges phi region
C  0.810 -- 1.000
C  0.843 -- 1.197
C  1.062 -- 1.438
      if ((e.ge.0.81d0).and.(e.le.EFIMM)) then
         e1=0.81d0
         en=0.999999d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresl,nm)
         else if (ier.eq.2) then
            yfit=polynom(ex,astal,nm)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysl,nm)
         endif
      else if ((e.gt.EFIMM).and.(e.le.EFIMX)) then
         e1=0.958d0
         en=1.000d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresfm,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,astafm,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysfm,np)
         endif
      else if ((e.gt.EFIMX).and.(e.lt.EFIPX)) then
c update jan 2012
         e1=1.00001d0
         en=1.03999D0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresfr,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,astafr,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysfr,np)
         endif
c test background subtraction for phi
c         if (ini.eq.0) then            
c            ini=1
c            etest0=1.00d0
c            etest1=1.04d0
c            stest0=etest0**2
c            stest1=etest1**2
c            write (*,*) ' R phi at:',etest0,fac*BW(stest0,MFI,GFI,PFI)
c            write (*,*) ' R phi at:',etest1,fac*BW(stest1,MFI,GFI,PFI)
c         endif
         if (IRESON.eq.1) then
            rbw=fac*BW(sMeV,MFI,GFI,PFI)
            if (ier.eq.2) then
               rbw=rbw*fsta3
               yfit=sqrt(yfit**2+rbw**2)
            else 
               if (ier.eq.3) rbw=rbw*fsys3
               yfit=yfit+rbw
            endif
         endif
       else if (e.lt.1.435d0) then
         e1=1.04000001d0
         en=1.435d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresh,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,astah,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysh,np)
         endif
      else
         write (*,*)  ' Warning: s out of fit range in epemfit0814new'
      endif
      epemfit0814new=yfit
      return
      end
c
      function ffppg2fit0810(s)
c fitted multi hadron channels n>2 (G2 files from dataset0821 in ../for/vap/dat/FFPPG2.DAT)
      implicit none
      integer np,ier
      parameter(np=12)
      real *8 ffppg2fit0810,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     ares,asta,asys,polynom
      dimension ares(np),asta(np),asys(np)
      COMMON/ERR/IER

c      ++++ ffppg2fit0810 ++++  

      include 'ffppg2fit0810.h'

      e=sqrt(s)
      yfit=0.d0
      e1=0.81d0
      en=1.00d0
      if ((e.gt.en).or.(e.lt.e1)) then
         ffppg2fit0810=0.d0
         return
      endif
      es=(en+e1)
      ed=(en-e1)
      amap=2.d0/ed
      bmap=-es/ed
      ex=amap*e+bmap
      if (ier.eq.1) then
        yfit=polynom(ex,ares,np)
      else if (ier.eq.2) then
        yfit=polynom(ex,asta,np)
      else if (ier.eq.3) then
        yfit=polynom(ex,asys,np)
      endif
      ffppg2fit0810=yfit
      return
      end
c
      function ffppmfit0810(s)
c fitted multi hadron channels pp + n>2 (M files from dataset0821 in ../for/vap/dat/FFPPM.DAT)
      implicit none
      integer np,ier
      parameter(np=12)
      real *8 ffppmfit0810,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     ares,asta,asys,polynom
      dimension ares(np),asta(np),asys(np)
      COMMON/ERR/IER

c      ++++ ffppmfit0810 ++++  

      include 'ffppmfit0810.h'

      e=sqrt(s)
      yfit=0.d0
      e1=0.81d0
      en=1.00d0
      if ((e.gt.en).or.(e.lt.e1)) then
         ffppmfit0810=0.d0
         return
      endif
      es=(en+e1)
      ed=(en-e1)
      amap=2.d0/ed
      bmap=-es/ed
      ex=amap*e+bmap
      if (ier.eq.1) then
        yfit=polynom(ex,ares,np)
      else if (ier.eq.2) then
        yfit=polynom(ex,asta,np)
      else if (ier.eq.3) then
        yfit=polynom(ex,asys,np)
      endif
      ffppmfit0810=yfit
      return
      end
c
      function epemfit079081(s)
      implicit none
      integer np,ier,IRESON
      parameter(np=7)
      real *8 epemfit079081,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     ares,asta,asys,polynom,MOM,GOM,POM,fsta2,fsys2
      REAL*8 FSTA(5),FSYS(5)
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      real *8 UGM2,sMeV
      dimension ares(np),asta(np),asys(np)
      real *8 fac,rbw,BW
      external BW
      COMMON/BWOM/MOM,GOM,POM
      COMMON/RESRELERR/FSTA,FSYS
      COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP
      COMMON/RESFIT/fac,IRESON
      COMMON/ERR/IER

c      ++++ epemfit079081 ++++  

      include 'epemfit079081.h'

c omega  .41871054d0,.810d0
      UGM2=1.D6
      sMeV=s*UGM2
      fsta2=FSTA(2)
      fsys2=FSYS(2)
c
      e=sqrt(s)
      yfit=0.d0
      e1=0.787d0
      en=0.8099d0
      if ((e.gt.0.81d0).or.(e.lt.e1)) then
         epemfit079081=0.d0
         return
      endif
      es=(en+e1)
      ed=(en-e1)
      amap=2.d0/ed
      bmap=-es/ed
      ex=amap*e+bmap
      if (ier.eq.1) then
        yfit=polynom(ex,ares,np)
      else if (ier.eq.2) then
        yfit=polynom(ex,asta,np)
      else if (ier.eq.3) then
        yfit=polynom(ex,asys,np)
      endif
      if ((IRESON.eq.1).and.(e.ge.EOMM).and.(e.lt.EOMP)) then
         rbw=fac*BW(sMeV,MOM,GOM,POM)
            if (ier.eq.2) then
               rbw=rbw*fsta2
               yfit=sqrt(yfit**2+rbw**2)
            else 
               if (ier.eq.3) rbw=rbw*fsys2
               yfit=yfit+rbw
            endif
      endif
      epemfit079081=yfit
      return
      end
c
      function epemfit1432(s)
      implicit none
      integer np,nm,nx,ier,IRESON
      parameter(np=12,nm=6,nx=8)
      real *8 epemfit1432,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     areslow,astalow,asyslow,aresmed,astamed,asysmed,
     &     ares,asta,asys,ares1,asta1,asys1,
     &     arestoBES,astatoBES,asystoBES,polynom,eex,ebe
      real *8 fres2330,fsta2330,fsys2330,fres2023,fsta2023,fsys2023
      dimension areslow(np),astalow(np),asyslow(np)
      dimension aresmed(nx),astamed(nx),asysmed(nx)
      dimension ares(np),asta(np),asys(np)
      dimension ares1(np),asta1(np),asys1(np)
      dimension arestoBES(nm),astatoBES(nm),asystoBES(nm)
      dimension fres2330(nx),fsta2330(nx),fsys2330(nx)
      dimension fres2023(nm),fsta2023(nm),fsys2023(nm)
      REAL*8 MPS(6),GPS(6),PPS(6),MYP(6),GYP(6),PYP(6)
      REAL*8 STAPS(6),SYSPS(6),STAYP(6),SYSYP(6)
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      real *8 fac,rbw,BW,UGM2,sMeV
      external BW
      COMMON/RESFIT/fac,IRESON
      COMMON/ERR/IER
C common Psi and Upsilon parameters set in subroutine resonances
      COMMON/PSYPPAR/MPS,GPS,PPS,MYP,GYP,PYP,STAPS,SYSPS,STAYP,SYSYP
      COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP

c      ++++ epemfit1432 ++++  

      include 'epemfit1432.h'

      e=sqrt(s)
      yfit=0.d0
      UGM2=1.D6
      sMeV=s*UGM2
      eex=1.8590000000000000d0
      ebe=2.6000000000000001d0
c smoothing overlapps exten down to 1.38
c      if ((e.lt.1.4d0).or.(e.gt.3.2d0)) then
      if ((e.lt.1.38d0).or.(e.gt.3.2d0)) then
         epemfit1432=0.d0
         return
      else if (e.le.1.45d0) then
c   1.375  --  2.038
         e1=1.375d0
         en=2.038d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,areslow,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,astalow,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asyslow,np)
         endif 
      else if (e.lt.1.985d0) then
c   1.395 -- 2.000
         e1=1.395d0
         en=2.0d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,ares,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,asta,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asys,np)
         endif
      else if (e.lt.2.25d0) then
c  1.975 -- 2.243 update jan 2012
         e1=1.975d0
         en=2.243d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,fres2023,nm)
         else if (ier.eq.2) then
            yfit=polynom(ex,fsta2023,nm)
         else if (ier.eq.3) then
            yfit=polynom(ex,fsys2023,nm)
         endif
      else if (e.lt.3.00d0) then
         e1=2.25d0
         en=3.0d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,fres2330,nx)
         else if (ier.eq.2) then
            yfit=polynom(ex,fsta2330,nx)
         else if (ier.eq.3) then
            yfit=polynom(ex,fsys2330,nx)
         endif
      else if (e.le.3.2d0) then
         e1=2.9d0
c         en=3.69696d0
         en=3.7d0
c         e1=2.0d0
c         en=3.2d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,ares1,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,asta1,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asys1,np)
         endif
         if (IRESON.eq.1) then
c psi1  3.08587D0 3.10787D0
            if ((e.ge.EMIPS(1)).and.(e.le.EMAPS(1))) then ! psi1
               rbw=fac*BW(sMeV,MPS(1),GPS(1),PPS(1))
               if (ier.eq.2) then
                  rbw=rbw*STAPS(1)
                  yfit=sqrt(yfit**2+rbw**2)
               else 
                  if (ier.eq.3) rbw=rbw*SYSPS(1)
                  yfit=yfit+rbw
               endif
            endif
         endif
      endif
         epemfit1432=yfit
      return
      end
c
      function epemfit32cut(s)
      implicit none
      integer i,nx,np,nm,nh,ni,nb,ier,IRESON,iresonances,
     &     iresonances_sav,IRESON_SAV
      parameter(nx=12,nb=9,np=6,nm=6,ni=5,nh=3)
      real *8 epemfit32cut,s,e,ex,es,ed,e1,en,amap,bmap,yfit,rs40,rs40x,
     &    ares1,asta1,asys1,aresi,astai,asysi,aresin,astain,asysin,
     &    aresh,astah,asysh,aresy,astay,asysy,aresbes,astabes,asysbes,
     &    polynom,RSpQCD
      dimension ares1(nx),asta1(nx),asys1(nx)
      dimension aresi(ni),astai(ni),asysi(ni)
      dimension aresin(np),astain(np),asysin(np)
      dimension aresh(nh),astah(nh),asysh(nh)
      dimension aresy(ni),astay(ni),asysy(ni)
      dimension aresbes(nb),astabes(nb),asysbes(nb)
      REAL*8 MPS(6),GPS(6),PPS(6),MYP(6),GYP(6),PYP(6)
      REAL*8 STAPS(6),SYSPS(6),STAYP(6),SYSYP(6)
      REAL*8 fac,rbw,BW,CHPTCUT,EC,ECUT,RSx,ECUTFIT1,ECUTFIT2
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      REAL*8 UGM2,sMeV
      external BW,RS40,RS40x,RSpQCD
      COMMON/RESFIT/fac,IRESON
      common /RES/iresonances
      COMMON/ERR/IER
      COMMON/RCUTS/CHPTCUT,EC,ECUT
C cpmmon Psi and Upsilon parameters set in subroutine resonances
      COMMON/PSYPPAR/MPS,GPS,PPS,MYP,GYP,PYP,STAPS,SYSPS,STAYP,SYSYP
      COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP

c      ++++ epemfit32cut ++++  

      include 'epemfit32cut.h'

      iresonances_sav=iresonances
      IRESON_SAV=IRESON
      iresonances=IRESON
      e=sqrt(s)
      yfit=0.d0
      UGM2=1.D6
      sMeV=s*UGM2
      ECUTFIT1=DMIN1(ECUT,11.0d0)
      ECUTFIT2=DMIN1(ECUT,40.0d0)
c psi2  3.67496D0 3.69696D0
c psi3  3.7000D0  3.8400D0
c psi4  3.960D0   4.098D0
c psi5  4.102D0   4.275D0
c psi6  4.315D0   4.515D0
      if ((e.lt.3.2d0).or.(e.gt.40.d0)) then
         epemfit32cut=0.d0
         return
      else if (e.gt.ECUT) then
         epemfit32cut=RSpQCD(s,IER)
         return
      else if (e.lt.3.6d0) then
         e1=2.9d0
         en=3.7d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,ares1,nx)
         else if (ier.eq.2) then
            yfit=polynom(ex,asta1,nx)
         else if (ier.eq.3) then
            yfit=polynom(ex,asys1,nx)
         endif
         if (IRESON.eq.1) then
            do i=1,3 
               if ((E.GT.EMIPS(i)).and.(E.LT.EMAPS(i))) then
                  rbw=fac*BW(sMeV,MPS(i),GPS(i),PPS(i))
                  if (ier.eq.2) then
                     rbw=rbw*STAPS(i)
                     yfit=sqrt(yfit**2+rbw**2)
                  else 
                     if (ier.eq.3) rbw=rbw*SYSPS(i)
                     yfit=yfit+rbw
                  endif
               endif
            enddo
         endif
      else if (e.lt.3.71d0) then
c 3.598  --  3.710
         e1=3.598d0
         en=3.710d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresin,np)
         else if (ier.eq.2) then
            yfit=polynom(ex,astain,np)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysin,np)
         endif
         if (IRESON.eq.1) then
            do i=1,3 
               if ((E.GT.EMIPS(i)).and.(E.LT.EMAPS(i))) then
                  rbw=fac*BW(sMeV,MPS(i),GPS(i),PPS(i))
                  if (ier.eq.2) then
                     rbw=rbw*STAPS(i)
                     yfit=sqrt(yfit**2+rbw**2)
                  else 
                     if (ier.eq.3) rbw=rbw*SYSPS(i)
                     yfit=yfit+rbw
                  endif
               endif
            enddo
         endif
      else if (e.lt.3.78d0) then
c 3.598  --  3.787
         e1=3.598d0
         en=3.787d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresi,ni)
         else if (ier.eq.2) then
            yfit=polynom(ex,astai,ni)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysi,ni)
         endif
         if (IRESON.eq.1) then
            do i=1,3 
               if ((E.GT.EMIPS(i)).and.(E.LT.EMAPS(i))) then
                  rbw=fac*BW(sMeV,MPS(i),GPS(i),PPS(i))
                  if (ier.eq.2) then
                     rbw=rbw*STAPS(i)
                     yfit=sqrt(yfit**2+rbw**2)
                  else 
                     if (ier.eq.3) rbw=rbw*SYSPS(i)
                     yfit=yfit+rbw
                  endif
               endif
            enddo
         endif
      else if (e.lt.4.5d0) then
c it is important here to select 4.5 not 4.4 to get smooth transition
c psi3  3.7000D0  3.8400D0
c psi4  3.960D0   4.098D0
c psi5  4.102D0   4.275D0
c psi6  4.315D0   4.515D0
         yfit=RS40(s,IER)
      else if ((e.ge.EC).and.(e.le.9.5d0)) then
         yfit=RSpQCD(s,IER)
      else if (e.lt.9.5d0) then
         e1=4.57d0
         en=9.5d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresbes,nb)
         else if (ier.eq.2) then
            yfit=polynom(ex,astabes,nb)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysbes,nb)
         endif
         if (IRESON.eq.1) then
            i=1 
            if ((E.GT.EMIYP(i)).and.(E.LT.EMAYP(i))) then
               rbw=fac*BW(sMeV,MYP(i),GYP(i),PYP(i))
               if (ier.eq.2) then
                  rbw=rbw*STAYP(i)
                  yfit=sqrt(yfit**2+rbw**2)
               else 
                  if (ier.eq.3) rbw=rbw*SYSYP(i)
                  yfit=yfit+rbw
               endif
            endif
         endif
      else if ((e.ge.9.5d0).and.(e.le.14.d0)) then
         e1=09.50d0
         en=14.00d0
         es=(en+e1)
         ed=(en-e1)
         amap=2.d0/ed
         bmap=-es/ed
         ex=amap*e+bmap
         if (ier.eq.1) then
            yfit=polynom(ex,aresy,ni)
         else if (ier.eq.2) then
            yfit=polynom(ex,astay,ni)
         else if (ier.eq.3) then
            yfit=polynom(ex,asysy,ni)
         endif
         if (IRESON.eq.1) then
            do i=2,6 
               if ((E.GT.EMIYP(i)).and.(E.LT.EMAYP(i))) then
                  rbw=fac*BW(sMeV,MYP(i),GYP(i),PYP(i))
                  if (ier.eq.2) then
                     rbw=rbw*STAYP(i)
                     yfit=sqrt(yfit**2+rbw**2)
                  else 
                     if (ier.eq.3) rbw=rbw*SYSYP(i)
                     yfit=yfit+rbw
                  endif
               endif
            enddo
         endif
      else if ((e.ge.14.0d0).and.(e.le.40.d0)) then
            e1=14.04d0
            en=38.29d0
            es=(en+e1)
            ed=(en-e1)
            amap=2.d0/ed
            bmap=-es/ed
            ex=amap*e+bmap
            if (ier.eq.1) then
               yfit=polynom(ex,aresh,nh)
            else if (ier.eq.2) then
               yfit=polynom(ex,astah,nh)
            else if (ier.eq.3) then
               yfit=polynom(ex,asysh,nh)
            endif
      endif
      epemfit32cut=yfit
      iresonances=iresonances_sav
      IRESON=IRESON_SAV
      return
      end
c
      function rfitrholow(s)
      implicit none
      integer np,ier,IRESON
      parameter(np=14)
      real *8 rfitrholow,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     ares,asta,asys,polynom,MOM,GOM,POM,fsta2,fsys2
      REAL*8 FSTA(5),FSYS(5)
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      dimension ares(np),asta(np),asys(np)
      real *8 fac,rbw,BW
      REAL*8 UGM2,sMeV
      external BW
      COMMON/BWOM/MOM,GOM,POM
      COMMON/RESRELERR/FSTA,FSYS
      COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP
      COMMON/RESFIT/fac,IRESON
      COMMON/ERR/IER

c      ++++ rfitrholow ++++  

      include 'rfitrholow.h'

c omega  .41871054d0,.810d0
      fsta2=FSTA(2)
      fsys2=FSYS(2)
c
      e=sqrt(s)
      yfit=0.d0
      UGM2=1.D6
      sMeV=s*UGM2
      e1=0.318d0
      en=0.778d0
      if ((e.gt.en).or.(e.lt.e1)) then
         rfitrholow=0.d0
         return
      endif
      es=(en+e1)
      ed=(en-e1)
      amap=2.d0/ed
      bmap=-es/ed
      ex=amap*e+bmap
      if (ier.eq.1) then
        yfit=polynom(ex,ares,np)
      else if (ier.eq.2) then
        yfit=polynom(ex,asta,np)
      else if (ier.eq.3) then
        yfit=polynom(ex,asys,np)
      endif
      if ((IRESON.eq.1).and.(e.ge.EOMM).and.(e.lt.EOMP)) then
         rbw=fac*BW(sMeV,MOM,GOM,POM)
         if (ier.eq.2) then
            rbw=rbw*fsta2
            yfit=sqrt(yfit**2+rbw**2)
         else 
            if (ier.eq.3) rbw=rbw*fsys2
            yfit=yfit+rbw
         endif
      endif
      rfitrholow=yfit
      return
      end
c
      function rfitrhohig(s)
      implicit none
      integer np,ier,IRESON
      parameter(np=9)
      real *8 rfitrhohig,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     ares,asta,asys,polynom,MOM,GOM,POM,fsta2,fsys2
      REAL*8 FSTA(5),FSYS(5)
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      dimension ares(np),asta(np),asys(np)
      real *8 fac,rbw,BW
      REAL*8 UGM2,sMeV
      external BW
      COMMON/BWOM/MOM,GOM,POM
      COMMON/RESRELERR/FSTA,FSYS
      COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP
      COMMON/RESFIT/fac,IRESON
      COMMON/ERR/IER

c      ++++ rfitrhohig ++++  

      include 'rfitrhohig.h'

c omega  .41871054d0,.810d0
      fsta2=FSTA(2)
      fsys2=FSYS(2)
c
      e=sqrt(s)
      yfit=0.d0
      UGM2=1.D6
      sMeV=s*UGM2
      e1=0.769d0
      en=0.789d0
      if ((e.gt.en).or.(e.lt.e1)) then
         rfitrhohig=0.d0
         return
      endif
      es=(en+e1)
      ed=(en-e1)
      amap=2.d0/ed
      bmap=-es/ed
      ex=amap*e+bmap
      if (ier.eq.1) then
        yfit=polynom(ex,ares,np)
      else if (ier.eq.2) then
        yfit=polynom(ex,asta,np)
      else if (ier.eq.3) then
        yfit=polynom(ex,asys,np)
      endif
      if ((IRESON.eq.1).and.(e.ge.EOMM).and.(e.lt.EOMP)) then
         rbw=fac*BW(sMeV,MOM,GOM,POM)
         if (ier.eq.2) then
            rbw=rbw*fsta2
            yfit=sqrt(yfit**2+rbw**2)
         else 
            if (ier.eq.3) rbw=rbw*fsys2
            yfit=yfit+rbw
         endif
      endif
      rfitrhohig=yfit
      return
      end
c
      function rfitchptail(s)
      implicit none
      integer np,ier
      parameter(np=3)
      real *8 rfitchptail,s,e,ex,es,ed,e1,en,amap,bmap,yfit,
     &     ares,asta,asys,polynom
      dimension ares(np),asta(np),asys(np)
      COMMON/ERR/IER
c 0.285 - 0.325
c new feb 2012
      DATA ares/
     &      0.018753d0, 0.018778d0, 0.002144d0/
      DATA asta/
     &      0.000265d0, 0.000483d0, 0.000242d0/
      DATA asys/
     &      0.000295d0, 0.000353d0, 0.000089d0/
c
      e=sqrt(s)
      yfit=0.d0
      e1=0.285d0
      en=0.325d0
      if ((e.gt.en).or.(e.lt.e1)) then
         rfitchptail=0.d0
         return
      endif
      es=(en+e1)
      ed=(en-e1)
      amap=2.d0/ed
      bmap=-es/ed
      ex=amap*e+bmap
      if (ier.eq.1) then
        yfit=polynom(ex,ares,np)
      else if (ier.eq.2) then
        yfit=polynom(ex,asta,np)
      else if (ier.eq.3) then
        yfit=polynom(ex,asys,np)
      endif
      rfitchptail=yfit
      return
      end
c

      function polynom(x,a,na)
C map x to interval [-1,+1]
      implicit none
      INTEGER na,i
      REAL*8 x,a,polynom,t
      dimension a(na),t(na)
c      Tschebycheff Polynomials
      t(1)=1.d0
      t(2)=x
      do i=1,na-2
         t(i+2)=2.d0*x*t(i+1)-t(i)
      enddo
      polynom=0.d0
      do i=1,na
         polynom=polynom+a(i)*t(i)
      enddo
      return
      end
c
      function rsigpp(s,iso)
c modified jan 2012 ffppg2 --> ffppm incl all channels
      implicit none
      integer iso,iopt,ioptn,ioptc,IER,IRESON
      real *8 rsigpp,s,sMeV,e,y,beta,SM,UGM2,M2,e1,en,
     &             fpi2rnew,ffppmfit0810,epemfit0814new,eps
      real *8 MOM,GOM,POM,fsta2,fsys2
      REAL*8 FSTA(5),FSYS(5)
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      real *8 fac,rbw,BW
      double precision MPLUS,MZERO,MPI2
      external fpi2rnew,ffppmfit0810,epemfit0814new
      external BW
      common/pions/MPLUS,MZERO
      COMMON/BWOM/MOM,GOM,POM
      COMMON/RESRELERR/FSTA,FSYS
      COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP
      COMMON/RESFIT/fac,IRESON
      COMMON/ERR/IER
c omega  .41871054d0,.810d0
      fsta2=FSTA(2)
      fsys2=FSYS(2)
c
      e=sqrt(s)
      if ((e.gt.1.d0).or.(e.lt.0.61d0)) then
         write (*,*)  ' Warning: s out of fit range in rsigpp'
      endif
      ioptc=32 ! all 3 rho's with quadratic energy dependent width as used by Belle 2008
      ioptn=11 ! rho only  with linear energy dependent width as used by CMD-2 1999 ....
      iopt=ioptn
      eps=0.02
c      MPLUS=139.56995D0 now set in constants.f
      MPI2=MPLUS**2
      UGM2=1.D6
      M2=MPI2/UGM2
      sMeV=s*UGM2
      beta=SQRT(1.D0-4.D0*MPI2/sMeV)
c join GS parametrizaton and ffpp fit
      e1=0.81d0
      en=1.00d0
      if (e.le.e1) then
         y=beta**3*fpi2rnew(sMEV,iso,iopt)/4.d0
c     error of pipi data
         IF (IER.EQ.2) y=y*0.02d0
         IF (IER.EQ.3) y=y*0.006D0
         if ((IRESON.eq.1).and.(e.ge.EOMM).and.(e.lt.EOMP)) then
            rbw=fac*BW(sMeV,MOM,GOM,POM)
            if (ier.eq.2) then
               rbw=rbw*fsta2
               y=sqrt(y**2+rbw**2)
            else 
               if (ier.eq.3) rbw=rbw*fsys2
               y=y+rbw
            endif
         endif
      else if (e.gt.en) then
         y=epemfit0814new(s)
      else
         y=beta**3/4.d0*ffppmfit0810(s)
      endif
      rsigpp=y
      return
      end
c
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; fpi2rnew.f --- 
c;; Author          : Fred Jegerlehner
c;; Created On      : Thu May  1 23:40:25 2008
c;; Last Modified By: Fred Jegerlehner
c;; Last Modified On: Fri Aug  1 11:07:30 2008
c;; RCS: $Id$
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; Copyright (C) 2008 Fred Jegerlehner
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; 
      function fpi2rnew(s,iso,iopt)
c    PION FORM FACTOR  (absolute square)
c CMD-2 parametrization December 2001; update August 2003    
c version extended by rho--omega mixing flag iso : 
c iso=1 calculate I=1 part only (omega switched off), iso=0 I=0 part else sum
c 01/08/2008 new option for energy dependent widths
c iopt=12 rho only quadratic, 32 all 3 rho's quadratic 
c iopt=11 rho only linear, 31 all 3 rho's linear
c iopt anything else [0,1,...] old CMD-2 version = iopt=11 
      implicit none
      character*4 DSET
      integer np,iso,iopt,i,ini
      parameter(np=8)
      complex*16 cI,cone,cdels,cepdom,cbeta,cgama,cnorm,cfpis,cfpis1,
     &           cBWnu,cBWde,cBWGSr0,cBWGSr1,cBWom,cBWGSr(3)
      real *8 fpi2rnew,fpi2rnew1,fpi2rnewtot,fpi2rnewie1,
     &        s,e,xp,rats,beps,ppis,lgs,hs,pi,ratm,bepm,ppim,lgm,hm,
     &        dhdsm,fs,d,grs,rBWnu
      real *8 a(np),mrho(3),grho(3),mr,gr,mp2,mr2,mo2,mr4,
     &     beta,phibeta,delta,phidelta,deltaarg,betaarg,gamaarg
      real *8 mpi,dmro(3),dgro(3),ddelta,dphidel,dbeta,dphibet,
     &     gama,dgama,phigama,dphigam,momega,dmomega,gomega,dgomega,
     &     mp,mo,go
      COMMON /DASE/DSET
      data pi /3.141592653589793d0/
      data ini /0/
      save
      if (DSET.ne.'WAVE') DSET='CMD2'
c      DSET='WAVE'
c Constants, Parameters
c      alp=1.D0/137.035999084D0  ! pm (51) April 2008
C CMD-2 Fit parameters 2003 PDG 2006
c      mpi0  =134.9766           !+/- 0.0006  neutral pion
      mpi    =139.57018d0     !+/- 0.00035  mass of charged pion
      mrho(1)=775.5d0         ! PDG 2006
      dmro(1)=0.4d0           
      grho(1)=143.85d0
      dgro(1)=1.5d0           
      mrho(2)=1446.0d0    ! take values from Belle fit
      dmro(2)=29.0d0          
      grho(2)=434.0d0
      dgro(2)=62.0d0          
c      mrho(2)=1459.0d0      ! PDG 2008
c      dmro(2)=11.0d0          
c      grho(2)=171.0d0
c      dgro(2)=50.0d0          
      mrho(3)=1728.8d0      ! take values from Belle fit
      dmro(3)=91.0d0
      grho(3)=193.0d0        
      dgro(3)=62.0d0
c      mrho(3)=1688.8d0    ! PDG 2006
c      dmro(3)=2.1d0
c      grho(3)=161.0d0        
c      dgro(3)=10.0d0
      delta =1.57d-3       ! CMD2 2002 not changed  (ckeck 2008)
      ddelta=0.16d-3         ! +/- 0.15d-3 +/- 0.05d-3   modulus of delta
      phidelta=10.4d0        ! CMD2 hep-ex/0610021v3
      dphidel=3.8d0           ! +/- 1.6 +/- 3.5       phase angle in degrees
c      beta   =0.0859d0        ! CMD2 hep-ex/0610021v3
      beta   =0.08d0        ! CMD2 hep-ex/0610021v3
      dbeta  =0.0040d0        ! +/-  .0030 +/- .0027
      phibeta=180.0d0
      dphibet=10.0d0
      gama   =0.000d0         ! */-  0.009
      gama   =0.028d0         ! */-  0.009
      dgama  =0.028d0
      phigama=24.000d0
      dphigam=24.000d0
      momega =782.65d0       ! +/- 0.12 PDG 2004/2006
      dmomega=000.12d0      
      gomega =  8.49d0         ! +/- 0.08  PDG 2004/2006
      dgomega=  0.08d0        
      if (ini.eq.0) then
c CMD2 fit May 2008 chi^2=29/51 and S=0.7; incl. SND would give 159/51 and S=1.7
      if (DSET.eq.'CMD2') then
      a(1)=7.7437D+02  
      a(2)=1.4421D+02  
      a(3)=9.9859D-02
      a(4)=1.8000D+02  
      a(5)=1.4460D+03  
      a(6)=4.3400D+02
      a(7)=1.8300D-03  
      a(8)=1.0600D+01
      else if (DSET.eq.'WAVE') then
      a(1)=7.7449D+02  
      a(2)=1.5063D+02  
      a(3)=1.2615D-01
      a(4)=1.8000D+02  
      a(5)=1.4460D+03  
      a(6)=4.3400D+02
      a(7)=1.988D-03  
      a(8)=0.900D+01
      else
      endif
c      a(1)=7.7359D+02    ! mrho(1)   
c      a(2)=1.4305D+02    ! grho(1) 
c      a(3)=9.2313D-02    ! beta    
c      a(4)=1.8000D+02    ! phibeta 
c      a(5)=1.4590D+03    ! mrho(2) 
c      a(6)=1.7100D+02    ! grho(2) 
c      a(7)=1.5700D-03    ! delta   
c      a(8)=1.0400D+01    ! phidelta
      ini=1
      endif

      mrho(1) =a(1)
      grho(1) =a(2)
      beta    =a(3)
      phibeta =a(4)
      mrho(2) =a(5)
      grho(2) =a(6)
      delta   =a(7)
      phidelta=a(8)
      
      mp=mpi
      mo=momega
      go=gomega
 
      cI   =DCMPLX(0.D0,1.D0)
      cone =DCMPLX(1.d0,0.0d0)
      mp2  =mp*mp
      mo2  =mo*mo
c Auxiliary variables
      e    =dsqrt(s)
      xp   =4.d0*mp2
      rats =xp/s
      beps =dsqrt(1.d0-rats)
      ppis =0.5d0*e *beps
      lgs  =dlog((e +2.d0*ppis)/(2.d0*mp))
      hs   =2.d0/pi*ppis/e *lgs
      do i=1,3 
        mr   =mrho(i)
        gr   =grho(i)
        mr2  =mr *mr
        mr4  =mr2*mr2
        ratm =xp/mr2
        bepm =dsqrt(1.d0-ratm)
        ppim =0.5d0*mr*bepm
        lgm  =dlog((mr+2.d0*ppim)/(2.d0*mp))
        hm   =2.d0/pi*ppim/mr*lgm
        dhdsm=hm*(0.125d0/ppim**2-0.5d0/mr2)+0.5d0/pi/mr2
        fs   =gr*mr2/ppim**3*(ppis**2*(hs-hm)+(mr2-s)*ppim**2*dhdsm)
        d    =3.d0/pi*mp2/ppim**2*lgm+mr/2.d0/pi/ppim-mp2*mr/pi/ppim**3
        if (iopt.eq.11) then
           if (i.eq.1) then
c     GS parametrization R.R. Akhmetshin hep-ex/9904027 take linear mr/e factor
c     in Belle 2008 M.~Fujikawa et al arXiv:0805.3773 [hep-ex] fit a quadratic (mr/e)**2 dependence is assumed
c     only rho taken with energy dependent width
c     changed from linear to quadratic mr/e 24/06/2008
              grs=gr*(ppis/ppim)**3*(mr/e)
           else 
              grs=gr
           endif
c option used by Belle 2008 all 3 rho's have same type of energy dependent with
        else if (iopt.eq.31) then
           grs=gr*(ppis/ppim)**3*(mr/e)
        else if (iopt.eq.12) then
           if (i.eq.1) then
c     GS parametrization R.R. Akhmetshin hep-ex/9904027 take linear mr/e factor
c     in Belle 2008 M.~Fujikawa et al arXiv:0805.3773 [hep-ex] fit a quadratic (mr/e)**2 dependence is assumed
c     only rho taken with energy dependent width
c     changed from linear to quadratic mr/e 24/06/2008
              grs=gr*(ppis/ppim)**3*(mr/e)**2
           else 
              grs=gr
           endif
c option used by Belle 2008 all 3 rho's have same type of energy dependent with
        else if (iopt.eq.32) then
           grs=gr*(ppis/ppim)**3*(mr/e)**2
        else
c original CMD-2 fit version
           if (i.eq.1) then
              grs=gr*(ppis/ppim)**3*(mr/e)
           else 
              grs=gr
           endif           
        endif
        rBWnu=mr2*(1.d0+d*gr/mr)
        cBWnu=DCMPLX(rBWnu,0.D0)
        cBWde=DCMPLX(mr2-s+fs,-mr*grs)
        cBWGSr(i)=cBWnu/cBWde
      enddo  
c delta argument in radians       
      deltaarg=phidelta*2.d0*pi/360.d0
      gamaarg=phigama*2.d0*pi/360.d0
      cdels =DCMPLX(dcos(deltaarg),dsin(deltaarg))*
     &       DCMPLX(delta*s/mo2,0.d0)
      cBWom =DCMPLX(mo2,0.d0)/DCMPLX(mo2-s,-mo*go)
      cepdom=cone+cdels*cBWom
      betaarg=phibeta*2.d0*pi/360.d0
      cbeta =DCMPLX(dcos(betaarg),dsin(betaarg))*
     &       DCMPLX(beta,0.d0)
      cgama =DCMPLX(dcos(gamaarg),dsin(gamaarg))*
     &       DCMPLX(gama,0.d0)
      cnorm =cone+cbeta+cgama
      cfpis  =(cBWGSr(1)*cepdom+cbeta*cBWGSr(2)+cgama*cBWGSr(3))/cnorm
      cfpis1 =(cBWGSr(1)+cbeta*cBWGSr(2)+cgama*cBWGSr(3))/cnorm
      fpi2rnewtot=cfpis *DCONJG(cfpis )
      fpi2rnewie1=cfpis1*DCONJG(cfpis1)
      if (iso.eq.0) then        
        fpi2rnew =fpi2rnewtot-fpi2rnewie1
      else if (iso.eq.1) then
        fpi2rnew =fpi2rnewie1
      else 
        fpi2rnew =fpi2rnewtot
      endif
      return
      end
