*;;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;
*;; alphaQEDc_sub.f --- 
*;; Author          : Fred Jegerlehner
*;; Created On      : Wed Jan 11 03:26:27 2006
*;; Last Modified By: Friedrich Jegerlehner
*;; Last Modified On: Tue Jul 18 12:57:38 2017
*;; RCS: $Id$
*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
*;; Copyright (c) 2010 Fred Jegerlehner
*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
*;; 
      SUBROUTINE alphaQEDcx(s,cvpt_new,cvpt_hig,cvpt_low,alphac,alphah
     &     ,alphal,alphact,alphaht,alphalt,calept1,cahadr1,caltop1)
C Link files dggvapx,dhadr5n17,cggvapx,chadr5n17
C Calculating the running QED coupling alpha(E) as a complex function
C Leptons: 1--loop, 2--loop exact, 3--loop in high energy approximation
C Quarks form routine dhadr5n including effective s--channel shifts in low energy range
C r1....,r2....,r3.... : one-loop, two-loop, three-loop result
C  ..real: real-part, ..imag: imaginary part;
C ......l: light fermions only (high energy approximation for 3--loops) 
C dallepQED1n=r1real,dallepQED2n=r2real,dallepQED3l=r3reall
C dallepQED3l also returns one-loop and two-loop high energy (light fermion) 
C approximations 
c 21/09/2009 common pQCD parameter setting via constants_qcd.f      
c FJ 28/01/2012 recalculated Nall=2055 --> 2177
c 29/01/2012 common resonance parameters setting via resonances_dat.f
c 21/09/2012 new option for W contribution: iOSW=0 strict MSbar, iOSW=1 including 
c            high energy asymptotic constant of the OS W contribution at s>>MW2
c 10/07/2017 providing statistical and systematic hadronic errors separately now
c            allows to set up the covariance matrix
      implicit none
      double precision s,null
c      double precision dalept,dahadr,daltop,Dalphaweak1MSb
      double complex cggvapx,cerror,cerrorsta,cerrorsys,
     &     calept,cahadr,caltop,cDalphaweak1MSb
      double complex cone,calp,cvpt_new,cvpt_hig,cvpt_low,alphac,alphah
     &     ,alphal,alphact,alphaht,alphalt,calept1,cahadr1,caltop1
      external cggvapx
********************* part to be included and edited in main program **************
c check cuts CHPTCUT,EC,ECUT   ! specify where to use CHPT and pQCD in constants_qcd
c decommment next line in main
c      double precision pi1,ALINP,ERRAL,EINP,MTOP1 ! get QCD parameters as set in constants_qcd
c QCD parameters required to calculare imaginary part (R) and top contribution 
      include 'common.h'      
c decomment next 3 lines in main
c      common/var1/pi1,ALINP,ERRAL,EINP,MTOP1
c      common /parm/st2,als,mtop
c      COMMON/SMOO/Ismooth
c decomment next line if needed (provided also by cggvap)
c      common /resu/dalept,dahadr,daltop,Dalphaweak1MSb   ! real results
      common /cres/calept,cahadr,caltop,cDalphaweak1MSb  ! complex results
c decomment next 2 lines in main
c      call constants()
c      call constants_qcd()
c      sin2ell=0.23153 ! pm 0.00016 LEPEEWG Phys Rep 427 (2006) 257
c decommment next line and change default as appropriate in main      
c      st2=0.23153d0   ! Reference value for weak mixing parameter
c decomment next 2 lines in main
c      als=ALINP       ! alpha strong
c      mtop=MTOP1      ! top quark mass
c      Ismooth=0       ! flag for imaginary part: 0= R(s) data; 1= R(s) fits
c following two entried set as default in chad5n09.f
c      IRESON=1        ! include narrow resonances in fits
c      iresonances=1   ! include narrow resonances in data sets
c decomment next line in main
c      call Rdata()
c decomment next line in main
c      call resonances_renocomplex()
cc      call resonances_renoreal()
C LEPTONflag=all,had,lep,ele,muo,tau -> iLEP=-3,-2,-1,1,2,3
C Default: 
C      LEPTONflag='all'
C      iLEP  = -3  ! for sum of leptons + quarks              
C decomment in main the following 2 lines for changing the default
c      LEPTONflag='all'   ! choose had,lep,top,ele,muo,tau
c      iLEP=LFLAG(LEPTONflag)
************************************************************************
c
      null=0.d0
      cone=DCMPLX(1.d0,null)
      calp=DCMPLX(alp,null)
c
c     vacuum polarization
c
c      ' e,dalpha,dalpha_low,dalpha_hig (no top)' 
c      ' e,alpha ,alphal ,alphah (no top)    '
c      ' e,alphat,alphalt,alphaht (with top and W)   ' 
c      ' e,dalept,dahadr,daltop' (individual)
c
      cvpt_new= cggvapx(s,cerror,cerrorsta,cerrorsys)           ! central value
      cvpt_hig= cvpt_new+cerror            !    ...        + total error 
      cvpt_low= cvpt_new-cerror            !    ...        - total error
      alphac  =calp/(cone-cvpt_new)        ! alpha no top
      alphah  =calp/(cone-cvpt_hig)
      alphal  =calp/(cone-cvpt_low)
      alphact =calp/(cone-cvpt_new-caltop-cDalphaweak1MSb) ! alpha incl top + W
      alphaht =calp/(cone-cvpt_hig-caltop-cDalphaweak1MSb)
      alphalt =calp/(cone-cvpt_low-caltop-cDalphaweak1MSb)
      calept1 =calept                      ! lepton VP
      cahadr1 =cahadr                      ! hadronic VP 5 flavors
      caltop1 =caltop                      ! top contribution                 
c
c
c     delta alpha: leptons + 5 quarks no top
c
c     e,dvpt_new,dvpt_low,dvpt_hig
c
c     alpha: leptons + 5 quarks no top
c
c     e,alphac ,alphal ,alphah
c
c     alpha: leptons + 5 quarks + top + W (1-loop MSbar) [above 2 M_W threshold]
c
c     e,alphact,alphalt,alphaht
c
c     delta alpha: leptons, 5 quarks, top
c
c     e,dalept,dahadr,daltop
c
      return
      END
c
