c;;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; chadr5n17.f --- 
c;; Author          : Friedrich Jegerlehner
c;; Created On      : Thu Jul  6 12:37:04 2017
c;; Last Modified By: Friedrich Jegerlehner
c;; Last Modified On: Wed Aug  9 13:59:29 2017
c;; RCS: $Id$
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; Copyright (C) 2017 Friedrich Jegerlehner
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; 
c **********************************************************************
c *         F. Jegerlehner, Humboldt Univerity Berlin, Grermany        *
c **********************************************************************
       subroutine chadr5x(de,dst2,cder,cerrdersta,cerrdersys,
     &     cdeg,cerrdegsta,cerrdegsys)
c Update March 2012
c Provides imaginary part of 5 flavor hadronic contribution to photon vacuum polarization
c and converts dhadr5n12 to double complex variables in chadr5n12
c
c R(s) data/fits in Rdat_all.f [data] via interface Rdat_fun.f / Rdat_fit.f [fits]
c
c Pi' (q^2) photon vacuum polarization function ~ Pi(q^2)/q^2
c Convention Re alpha = - [Re Pi (q^2)-Pi (0)];
c            Im alpha =- Im Pi' (q^2) = -1/3 alpha * R(q^2)  [alpha = classical zero momentum feine structure constent]
c Options for R(q^2): iresonances=0,1 switch off,on narrow resonances
c omega, phi, psi(i) i=1,2,3, Upsilon(i) i=1,2,3,4,5,6
c Ismooth=0,1 data,smoothed data [mostly Chebyshev fits], IGS=0,1
c use [1] or not [0] Gounaris-Sakurai fit for rho resonance
c 08/10/13 save and restore environment variables changed temporarily by this routine
c Update December 2014: hadr5n12 --> hadr5n14 
c  -- included new data by KLOE 2012, BaBar 2012...2014, Novosibirsk 2014
c  -- up-to-date PDG parameters for omega and phi, 
c  -- option for using directly the omega and phi data
c  -- using local complex VP subtraction for BW resonances (numerical integration by NAG routine)  
       implicit none
       integer i,IRESON,iresonances,iresonances1,Ismooth,
     &      IGS,IGS1,iso,IER,IER_SAV,IOR,NF,ICHK,
     &      IOR_SAV,IRESON_SAV,iresonances_sav,iso_sav,IGS_SAV
       real *8 de,dst2,dder,ddeg,derrdersta,derrdersys,
     &      derrdegsta,derrdegsys
       real *8 s,RSDATn,sthres,null,fac,rder,rdeg,
     &      rerrdersta,rerrdegsta,rerrdersys,rerrdegsys
       real *8 rer,ider,ideg,ierrdersta,ierrdersys,
     &      ierrdegsta,ierrdegsys,ierrder1,rrer
       real *8 RR(3),RG(3),RES(3),REBW,fac2
       double precision pi,ALINP,ERRAL,EINP,MTOP
       double precision CHPTCUT,EC,ECUT
c       double precision R3(3),rdgg,errdgg,ierrdgg,idgg
       double complex cder,cdeg,cerrdersta,cerrdegsta,
     &      cerrdersys,cerrdegsys
c     &      ,cdgg,cerrdgg
       double precision ALS2,EST,ESY,MZINP,MTOP2
       double precision dalept,dahadr,daltop,Dalphaweak1MSb
       double precision alp,adp,adp2,adp3,facbw,ml(3)
       double precision Ruds,Rudsc,Rudscb,Rc,Rb
       double precision cuds,cc,cb,crho,comega,cphi,cpsi,cups
       double precision MPLUS,MZERO,RS3GFUN
       external dggvapx,RSDATn,RS3GFUN
       common /params/ALP,adp,adp2,adp3,ml
       common /resu/dalept,dahadr,daltop,Dalphaweak1MSb
c      FUNCTION RSpQCD(s,IER) in Rdat_fun.f requires QCD parameters incl errors
c provided via the following common block:
       COMMON/QCDPA/ALS2,EST,ESY,MZINP,MTOP2,NF,IOR,ICHK ! global INPUT here
c to set QCD parameters alpha_s with stat and sys error
c (one may be zero the other the tot error), scale sqrt(s) usually = M_Z, top mass,
c number of external flavors e.g 5 no top, 6 incl top, ior=number of loops in R(s)
c to be calculated, ICHK is dummy here
c (in some programs used for alternative routines calculating R(s))
       common/var1/pi,ALINP,ERRAL,EINP,MTOP
c commons needed to get R(s)
       COMMON/RCUTS/CHPTCUT,EC,ECUT
c resonances flag iresonances=1 include resonances as data here;
c                            =0 include them elsewhere
       COMMON/RESFIT/facbw,IRESON
       COMMON/RES/iresonances
       COMMON/RBW/REBW
       COMMON/ERR/IER
       common/srdeg/fac,sthres
c switch for using Gounaris-Sakurai parametrization: IGS=1, iso=2
c provides e^+e^- data fit
c iso=1 isovector part, iso=0 isoscalar part
       COMMON /GUSA/IGS,iso
c flag for imaginary part Ismooth=0 R(s) data,=1 R(s) fits
       COMMON/SMOO/Ismooth ! to be set in main program

       call constants_qcd()  ! fills commons var and RCUTS

c save environment and restore on return
       IOR_SAV=IOR
       IRESON_SAV=IRESON
       iresonances_sav=iresonances
       iso_sav=iso
       IGS_SAV=IGS

       IOR=4           ! don't change for consistency with real part calculation
       IRESON=1        ! don't change provides full R including narrow resonances in fits
       iresonances=1   ! don't change provides full R including narrow resonances in data
       iso=2           ! don't change
       IGS=0           ! Chebyshev fits to be preferred
c fill common QCDPA
       ALS2=ALINP
       EST=ERRAL
       ESY=0.D0
       MZINP=EINP
       MTOP2=MTOP
       IOR=4                    ! default
       NF =5                    ! default
       MPLUS=0.13957018D0       ! PDG 2004 now set in resonances_dat.f via common/pions/
       sthres=(2.d0*MPLUS)**2   ! threshold for imaginary part [very small between m_\pi^0 and 2 m_\pi]
       fac=-alp/3.d0
       fac2=fac/dst2
       do i=1,3
          RR(i) =null
          RG(i) =null
          RES(i)=null
       enddo 
       ider=null
       ideg=null
       ierrdersta=null
       ierrdersys=null
       ierrdegsta=null
       ierrdegsys=null
       s=de*abs(de)
c provide double precision version for real part
       call dhadr5x(de,dst2,dder,derrdersta,derrdersys,
     &      ddeg,derrdegsta,derrdegsys)
       if (s.gt.sthres) then
c get R(s)
          IER_SAV=IER
          do i=1,3
             IER=i
c R(s) incl resonances (if iresonances=1)
             RR(i)=RSDATn(s,IER,Ismooth)
             RG(i)=RS3GFUN(s,IER,Ismooth)
C separate narrow resonance contribution needed in flavor splitting for SU(2) coupling  
             RES(i)=rebw
          enddo
          IER=IER_SAV
          rerrdersta=sqrt(RR(2)**2)
          rerrdersys=sqrt(RR(3)**2)
          rerrdegsta=sqrt(RG(2)**2)
          rerrdegsys=sqrt(RG(3)**2)
          rder=RR(1)
          rdeg=RG(1)
       else 
          rerrdersta=null
          rerrdersys=null
          rerrdegsta=null
          rerrdegsys=null
          rder=null
          rdeg=null
       endif
       ider   =fac*rder
       ideg   =fac2*rdeg
       ierrdersta=abs(fac*rerrdersta)
       ierrdersys=abs(fac*rerrdersys)
       ierrdegsta=abs(fac2*rerrdegsta)
       ierrdegsys=abs(fac2*rerrdegsys)
       cder=DCMPLX(dder,ider)
       cdeg=DCMPLX(ddeg,ideg)
       cerrdersta=DCMPLX(derrdersta,ierrdersta)
       cerrdersys=DCMPLX(derrdersys,ierrdersys)
       cerrdegsta=DCMPLX(derrdegsta,ierrdegsta)
       cerrdegsys=DCMPLX(derrdegsys,ierrdegsys)

       IOR=IOR_SAV
       IRESON=IRESON_SAV
       iresonances=iresonances_sav
       iso=iso_sav
       IGS=IGS_SAV

       return
       end
C
      FUNCTION RSDATn(s,IER,Ismooth)
C      --------------------------------
c interface for switching between data and fits
c returns NULL in chosen pQCD ranges [RCUTS]
      IMPLICIT NONE
      INTEGER IER,IER1,Ismooth
      double precision E,S,RSDATn,RS40,RS40smoothed
      EXTERNAL RS40,RS40smoothed
      COMMON/ERR/IER1
      IER1=IER
      E=SQRT(S)
      if (Ismooth.eq.1) then
         RSDATn=RS40smoothed(s,IER)
      else
         RSDATn=RS40(s,IER)
      endif
      RETURN
      END
C
       FUNCTION RS3GFUN(s,IER,Ismooth)
C      -----------------
c interface for imaginary part of 3 gamma correlator (deg)
c Update Decembber 2014: new flavor separation and recombination 
c redesigned after cross check on the lattice by Harvey Meyer        
       IMPLICIT NONE
       INTEGER IER,Ismooth
       double precision E,S,RS3GFUN,RS3G,RS3Gsmoothed
       EXTERNAL RS3G,RS3Gsmoothed
       E=SQRT(S)
       if (Ismooth.eq.1) then
          RS3GFUN=RS3Gsmoothed(s,IER)
       else
          RS3GFUN=RS3G(s,IER)
       endif
       RETURN
       END
C
       FUNCTION RS33FUN(s,IER,Ismooth)
C      -----------------
c interface for imaginary part of 3 3 correlator (dgg)
c Update Decembber 2014: new flavor separation and recombination 
c redesigned after cross check on the lattice by Harvey Meyer        
       IMPLICIT NONE
       INTEGER IER,Ismooth
       double precision E,S,RS33FUN,RS33,RS33smoothed
       EXTERNAL RS33,RS33smoothed
       E=SQRT(S)
       if (Ismooth.eq.1) then
          RS33FUN=RS33smoothed(s,IER)
       else
          RS33FUN=RS33(s,IER)
       endif
       RETURN
       END

C tools for calculating R(s) in pQCD, needed for imaginary part of complex alpha_em
c
C Download the Harlander-Steinhauser program rhad from http://www.rhad.de/
c expanding the tar file creates directory ./rhad-1.01/ with all files
c put the following subroutine into that directory and compile it like the example
c..   by Robert V. Harlander and Matthias Steinhauser,
c..   Comp. Phys. Comm. 153 (2003) 244,
c..   CERN-TH/2002-379, DESY 02-223, hep-ph/0212294.
c..   Changes: [March 30, 2009] v1.01
c..            - exact results at alphas^4 included
c
c choose option lverbose = .false. in parameters.f
c you may also want to switch off warnings on non-perturbative regions
c just before format statement 2003 in rhad.f, I use iwarn flag to switch it off
c         if (iwarn.eq.1) then
c            write(6,2003) dsqrt(scms)
c         endif
c also change in parameters.f iunit = 6 to something not used by this package
c copy rqcdHSn.f from current directory to ./rhad-1.01/ compile and link it

