C Higher order kernels for hadronic contribution to g-2
c    2:      function k2a(s,m2)          Diag a) asymptotic expansion
c   23:      function k2a_ana(s,m2)              analytic
c  156:      function k2a_num(s,m2)              exact form numerical
c   61:      function k2b(s,m2,mf2)      Diag b) asymptotic
c   95:      function k2b_ana(s,m2,mf2)          analytic m_e=0
c  122:      function k2c(s,t,m2)        Diag c) analytic
c      
      function k2a(s,m2)
c BK96 Eq. (7) divided by 2 
c R. Barbieri, E. Remiddi, Nucl. Phys. B90  (1975) 233
      implicit none
      real *8 s,m2,k2a,zeta2,x1,x2,x3,lx1,lx2
      data zeta2/1.644934066848226d0/
      x1=m2/s
      x2=x1*x1
      x3=x2*x1
      lx1=-log(x1)
      lx2=lx1*lx1
      K2a=x1*((223.d0/54.d0-2.d0*zeta2-23.d0/36.d0*lx1)
     &   +x1*(8785.d0/1152.d0-37.d0/8.d0*zeta2-367.d0/216.d0*lx1
     &                                         +19.d0/144.d0*lx2) 
     &   +x2*(13072841.d0/432000.d0-883.d0/40.d0*zeta2
     &                    -10079.d0/3600.d0*lx1+141.d0/80.d0*lx2) 
     &   +x3*(2034703.d0/16000.d0-3903.d0/40.d0*zeta2
     &                     -6517.d0/1800.d0*lx1+961.d0/80.d0*lx2))
      return
      end
c      
      function k2aele(s,m2)
c BK96 Eq. (7) divided by 2 
c R. Barbieri, E. Remiddi, Nucl. Phys. B90  (1975) 233
c truncated form of k2a for m=m_electron
      implicit none
      real *8 s,m2,k2aele,zeta2,x1,lx1
      data zeta2/1.644934066848226d0/
      x1=m2/s
      lx1=-log(x1)
      K2aele=x1*((223.d0/54.d0-2.d0*zeta2-23.d0/36.d0*lx1))
      return
      end
c      
      function k2aana(s,m2)
c R. Barbieri, E. Remiddi, Nucl. Phys. B90  (1975) 233
      implicit none
      real *8 s,m2,k2aana,zeta2,zeta3,b,y,b2,b3,lnb,lny,lny2,bm4,
     &        rtbbm4,Dpb,Dmb,Tb,ddilog,dli3,k2aasy,k2anul
      data zeta2,zeta3/1.644934066848226d0,1.20205690315959d0/
      common /resu/k2aasy,k2anul
      b=s/m2
      y=(sqrt(b)-sqrt(bm4))/(sqrt(b)+sqrt(bm4))
      b2=b*b
      b3=b*b2
      bm4=b-4.d0
      rtbbm4=sqrt(b*bm4)
      lnb=log(b)
      lny=log(y)
      lny2=lny*lny
      Dpb=ddilog(y)+ lny*log(1.d0-y)-0.25d0*lny2-zeta2  
      Dmb=ddilog(-y)+0.25d0*lny2+0.5d0*zeta2   
      Tb = -6.d0*dli3(y)-3.d0*dli3(-y)+lny2*log(1.d0-y)
     &     +0.5d0*(lny2+6.d0*zeta2)*log(1.d0+y)+2.d0*lny
     &     *(ddilog(-y)+2.d0*ddilog(y))
      k2aana = -139.d0/144.d0+115.d0/72.d0*b+(19.d0/12.d0-7.d0/36.d0*b
     &     + 23.d0/144.d0*b2 + 1.d0/bm4)*lnb + (-4.d0/3.d0
     &     +127.d0/36.d0*b-115.d0/72.d0*b2+23.d0/144.d0*b3)*lny/rtbbm4
     &     +(9.d0/4.d0+5.d0/24.d0*b-0.5d0*b2-2.d0/b)*zeta2
     &     +5.d0/96.d0*b2*lnb**2+(-0.5d0*b+17.d0/24.d0*b2-7.d0/48.d0*b3)
     &     *lny/rtbbm4*lnb+(19.d0/24.d0+53.d0/48.d0*b-29.d0/96.d0*b2
     &     -1.d0/(3.d0*b)+2.d0/bm4)*lny2+(-2.d0*b+17.d0/6.d0*b2
     &     -7.d0/12.d0*b3)*Dpb/rtbbm4+(13.d0/3.d0-7.d0/6.d0*b
     &     +1.d0/4.d0*b2-1.d0/6.d0*b3-4.d0/bm4)*Dmb/rtbbm4+(0.5d0
     &     -7.d0/6.d0*b+0.5d0*b2)*Tb
      k2aasy=-1.d0/b*(23.d0/36.d0*lnb+2.d0*zeta2-223.d0/54.d0)
      k2anul=197.d0/144.d0+0.5d0*zeta2-3.d0*zeta2*log(2.d0)
     &     +0.75d0*zeta3
c      write (2,*) b,y,Dpb,Dmb,Tb 
      return
      end
c
      function k2b(s,m2,mf2)
c BK96 Eq. (8)
      implicit none
      real *8 s,m2,mf2,k2b,pi,pi2,x1,x2,x3,x4,x5,lx1,lx2,
     &          y1,ly1,ly2,z1,lz1,lz2
      x1=m2/s
      x2=x1*x1
      x3=x2*x1
      x4=x3*x1
      x5=x4*x1
      lx1=-log(x1)
      lx2=lx1*lx1
      y1=m2/mf2
      ly1=log(y1)
      ly2=ly1*ly1
      z1=s/mf2
      lz1=log(z1)
      lz2=lz1*lz1
      K2b=     x1*(  -1.d0/18.d0 + 1.d0/9.d0*ly1   
     &       + x1*( - 55.d0/48.d0 +pi2/18.d0 + 5.d0/9.d0*lz1
     &                 + 5.d0/36.d0*ly1 - 1.d0/6.d0*lz2 + 1.d0/6.d0*ly2)  
     &       + x2*( - 11299.d0/1800.d0 +pi2/3.d0 + 10.d0/3.d0*lz1
     &                 -1.d0/10.d0*ly1 -  lz2 +  ly2 ) 
     &       - x3*(   6419.d0/225.d0 - 14.d0/9.d0*pi2 + 76.d0/45.d0*ly1
     &             - 14.d0/3.d0*ly2 - 140.d0/9.d0*lz1 + 14.d0/3.d0*lz2)  
     &       - x4*( 53350.d0/441.d0 - 20.d0/3.d0*pi2 + 592.d0/63.d0*ly1
     &             - 20.d0*ly2 - 200.d0/3.d0*lz1 + 20.d0*lz2 )) 
     &       +1.d0/y1*(x1 - 2.d0/3.d0*x2
     &       - x3*( - 2.d0*lx1 + 25.d0/6.d0) 
     &       - x4*( - 12.d0*lx1 + 97.d0/5.d0)   
     &       - x5*( - 56.d0*lx1 + 416.d0/5.d0 ))
      return
      end
c
      function k2bana(s,m2,mf2)
c BK96 Eq. (8)
      implicit none
      real *8 k2bana,s,m2,mf2,y1,ly1,b,x1,x2,r1,r2,ln1,ln2,ddilog
      y1=m2/mf2
      ly1=log(y1)
      b  =s/m2
      x1 =(b+sqrt(b**2-4.d0*b))/2.d0
      x2 =(b-sqrt(b**2-4.d0*b))/2.d0
      r1 =-x1/(1.d0-x1)
      r2 =-x2/(1.d0-x2)
      ln1=log(r1)
      ln2=log(r2)
      K2bana = - (5.d0/9.d0-1.d0/3.d0*ly1)
     &                 *(1.d0/2.d0-(x1+x2)+1.d0/(x1-x2)*(
     &                     x1**2*(x1-1.d0)*ln1 
     &                   -x2**2*(x2-1.d0)*ln2 ))
     &         -5.d0/12.d0+1.d0/3.d0*(x1+x2)+1.d0/3.d0/(x1-x2)*(
     &            x1**2*(1.d0-x1)*(ddilog(1.d0/x1) -1.d0/2.d0*ln1**2 )
     &          -x2**2*(1.d0-x2)*(ddilog(1.d0/x2) -1.d0/2.d0*ln2**2 )) 
      return
      end
c
c      adp=alpha/pi
c      a2c =  adp**3/9.d0
c      double-integral ds/s ds'/s' R(s) R(s') K2c(s,s')
c
      function k2c(s,t,m2)
      implicit none
      real *8 b,s,m2,c,t,b2,c2,c3,lnb,lnc,
     &         bmc,rtb,rtc,argb,argc,logargb,logargc,cmf,K2c
      b=s/m2
      c=t/m2
      b2=b*b
      c2=c*c
      c3=c*c2
      lnb=log(b)
      lnc=log(c)
      bmc=2.d0*(b-c)
      rtb  =sqrt(- (4.d0 - b)*b)
      rtc  =sqrt(- (4.d0 - c)*c)
      argb=(b + rtb)/(b - rtb)
      argc=(c + rtc)/(c - rtc)
      logargb=log(argb)
      logargc=log(argc)
      cmf=2.d0*(c-4.d0)
      if (b.eq.c) then 
         K2c =  1.d0/2.d0 - 2.d0*c
     &         + c*( -2.d0 +c-4.d0*lnc+3.d0*c*lnc)/2.d0
     &         + c*(-2.d0+4.d0*c-c2)/cmf
     &         + c*(12.d0-42.d0*c+22.d0*c2-3.d0*c3)*logargc/(cmf*rtc)
         else
         K2c =  1.d0/2.d0 - b - c
     &         - (( 2.d0 - b)*b2*lnb)/(bmc) 
     &         - (b2*( 2.d0 - 4.d0*b + b2)*logargb)/(bmc*rtb)
     &         + (( 2.d0 - c)*c2*lnc)/(bmc) 
     &         + (c2*( 2.d0 - 4.d0*c + c2)*logargc)/(bmc*rtc)
      endif
      return
      end

      function k2anum(s,m2)
      implicit none
      integer N,i,ij
      parameter(n=446)
      real *8 k2anum,k2a,s,m2,e,eps
      real *8 xx(n),yy(n)
C 446
        DATA XX/
     &  5.0000D+00,6.0000D+00,7.0000D+00,8.0000D+00,9.0000D+00,
     &  1.0000D+01,1.1000D+01,1.2000D+01,1.3000D+01,1.4000D+01,
     &  1.5000D+01,1.6000D+01,1.7000D+01,1.8000D+01,1.9000D+01,
     &  2.0000D+01,2.1000D+01,2.2000D+01,2.3000D+01,2.4000D+01,
     &  2.5000D+01,2.6000D+01,2.7000D+01,2.8000D+01,2.9000D+01,
     &  3.0000D+01,3.1000D+01,3.2000D+01,3.3000D+01,3.4000D+01,
     &  3.5000D+01,3.6000D+01,3.7000D+01,3.8000D+01,3.9000D+01,
     &  4.0000D+01,4.1000D+01,4.2000D+01,4.3000D+01,4.4000D+01,
     &  4.5000D+01,4.6000D+01,4.7000D+01,4.8000D+01,4.9000D+01,
     &  5.0000D+01,5.1000D+01,5.2000D+01,5.3000D+01,5.4000D+01,
     &  5.5000D+01,5.6000D+01,5.7000D+01,5.8000D+01,5.9000D+01,
     &  6.0000D+01,6.1000D+01,6.2000D+01,6.3000D+01,6.4000D+01,
     &  6.5000D+01,6.6000D+01,6.7000D+01,6.8000D+01,6.9000D+01,
     &  7.0000D+01,7.1000D+01,7.2000D+01,7.3000D+01,7.4000D+01,
     &  7.5000D+01,7.6000D+01,7.7000D+01,7.8000D+01,7.9000D+01,
     &  8.0000D+01,8.1000D+01,8.2000D+01,8.3000D+01,8.4000D+01,
     &  8.5000D+01,8.6000D+01,8.7000D+01,8.8000D+01,8.9000D+01,
     &  9.0000D+01,9.1000D+01,9.2000D+01,9.3000D+01,9.4000D+01,
     &  9.5000D+01,9.6000D+01,9.7000D+01,9.8000D+01,9.9000D+01,
     &   100.00D0, 101.00D0, 102.00D0, 103.00D0, 104.00D0,
     &   105.00D0, 106.00D0, 107.00D0, 108.00D0, 109.00D0,
     &   110.00D0, 111.00D0, 112.00D0, 113.00D0, 114.00D0,
     &   115.00D0, 116.00D0, 117.00D0, 118.00D0, 119.00D0,
     &   120.00D0, 121.00D0, 122.00D0, 123.00D0, 124.00D0,
     &   125.00D0, 126.00D0, 127.00D0, 128.00D0, 129.00D0,
     &   130.00D0, 131.00D0, 132.00D0, 133.00D0, 134.00D0,
     &   135.00D0, 136.00D0, 137.00D0, 138.00D0, 139.00D0,
     &   140.00D0, 141.00D0, 142.00D0, 143.00D0, 144.00D0,
     &   145.00D0, 146.00D0, 147.00D0, 148.00D0, 149.00D0,
     &   150.00D0, 151.00D0, 152.00D0, 153.00D0, 154.00D0,
     &   155.00D0, 156.00D0, 157.00D0, 158.00D0, 159.00D0,
     &   160.00D0, 161.00D0, 162.00D0, 163.00D0, 164.00D0,
     &   165.00D0, 166.00D0, 167.00D0, 168.00D0, 169.00D0,
     &   170.00D0, 171.00D0, 172.00D0, 173.00D0, 174.00D0,
     &   175.00D0, 176.00D0, 177.00D0, 178.00D0, 179.00D0,
     &   180.00D0, 181.00D0, 182.00D0, 183.00D0, 184.00D0,
     &   185.00D0, 186.00D0, 187.00D0, 188.00D0, 189.00D0,
     &   190.00D0, 191.00D0, 192.00D0, 193.00D0, 194.00D0,
     &   195.00D0, 196.00D0, 197.00D0, 198.00D0, 199.00D0,
     &   200.00D0, 201.00D0, 202.00D0, 203.00D0, 204.00D0,
     &   205.00D0, 206.00D0, 207.00D0, 208.00D0, 209.00D0,
     &   210.00D0, 211.00D0, 212.00D0, 213.00D0, 214.00D0,
     &   215.00D0, 216.00D0, 217.00D0, 218.00D0, 219.00D0,
     &   220.00D0, 221.00D0, 222.00D0, 223.00D0, 224.00D0,
     &   225.00D0, 226.00D0, 227.00D0, 228.00D0, 229.00D0,
     &   230.00D0, 231.00D0, 232.00D0, 233.00D0, 234.00D0,
     &   235.00D0, 236.00D0, 237.00D0, 238.00D0, 239.00D0,
     &   240.00D0, 241.00D0, 242.00D0, 243.00D0, 244.00D0,
     &   245.00D0, 246.00D0, 247.00D0, 248.00D0, 249.00D0,
     &   250.00D0, 251.00D0, 252.00D0, 253.00D0, 254.00D0,
     &   255.00D0, 256.00D0, 257.00D0, 258.00D0, 259.00D0,
     &   260.00D0, 261.00D0, 262.00D0, 263.00D0, 264.00D0,
     &   265.00D0, 266.00D0, 267.00D0, 268.00D0, 269.00D0,
     &   270.00D0, 271.00D0, 272.00D0, 273.00D0, 274.00D0,
     &   275.00D0, 276.00D0, 277.00D0, 278.00D0, 279.00D0,
     &   280.00D0, 281.00D0, 282.00D0, 283.00D0, 284.00D0,
     &   285.00D0, 286.00D0, 287.00D0, 288.00D0, 289.00D0,
     &   290.00D0, 291.00D0, 292.00D0, 293.00D0, 294.00D0,
     &   295.00D0, 296.00D0, 297.00D0, 298.00D0, 299.00D0,
     &   300.00D0, 301.00D0, 302.00D0, 303.00D0, 304.00D0,
     &   305.00D0, 306.00D0, 307.00D0, 308.00D0, 309.00D0,
     &   310.00D0, 311.00D0, 312.00D0, 313.00D0, 314.00D0,
     &   315.00D0, 316.00D0, 317.00D0, 318.00D0, 319.00D0,
     &   320.00D0, 321.00D0, 322.00D0, 323.00D0, 324.00D0,
     &   325.00D0, 326.00D0, 327.00D0, 328.00D0, 329.00D0,
     &   330.00D0, 331.00D0, 332.00D0, 333.00D0, 334.00D0,
     &   335.00D0, 336.00D0, 337.00D0, 338.00D0, 339.00D0,
     &   340.00D0, 341.00D0, 342.00D0, 343.00D0, 344.00D0,
     &   345.00D0, 346.00D0, 347.00D0, 348.00D0, 349.00D0,
     &   350.00D0, 351.00D0, 352.00D0, 353.00D0, 354.00D0,
     &   355.00D0, 356.00D0, 357.00D0, 358.00D0, 359.00D0,
     &   360.00D0, 361.00D0, 362.00D0, 363.00D0, 364.00D0,
     &   365.00D0, 366.00D0, 367.00D0, 368.00D0, 369.00D0,
     &   370.00D0, 371.00D0, 372.00D0, 373.00D0, 374.00D0,
     &   375.00D0, 376.00D0, 377.00D0, 378.00D0, 379.00D0,
     &   380.00D0, 381.00D0, 382.00D0, 383.00D0, 384.00D0,
     &   385.00D0, 386.00D0, 387.00D0, 388.00D0, 389.00D0,
     &   390.00D0, 391.00D0, 392.00D0, 393.00D0, 394.00D0,
     &   395.00D0, 396.00D0, 397.00D0, 398.00D0, 399.00D0,
     &   400.00D0, 401.00D0, 402.00D0, 403.00D0, 404.00D0,
     &   405.00D0, 406.00D0, 407.00D0, 408.00D0, 409.00D0,
     &   410.00D0, 411.00D0, 412.00D0, 413.00D0, 414.00D0,
     &   415.00D0, 416.00D0, 417.00D0, 418.00D0, 419.00D0,
     &   420.00D0, 421.00D0, 422.00D0, 423.00D0, 424.00D0,
     &   425.00D0, 426.00D0, 427.00D0, 428.00D0, 429.00D0,
     &   430.00D0, 431.00D0, 432.00D0, 433.00D0, 434.00D0,
     &   435.00D0, 436.00D0, 437.00D0, 438.00D0, 439.00D0,
     &   440.00D0, 441.00D0, 442.00D0, 443.00D0, 444.00D0,
     &   445.00D0, 446.00D0, 447.00D0, 448.00D0, 449.00D0,
     &   450.00D0/
        DATA YY/
     &  1.3012D-01,1.1964D-01,1.1110D-01,1.0398D-01,9.7914D-02,
     &  9.2671D-02,8.8080D-02,8.4017D-02,8.0389D-02,7.7124D-02,
     &  7.4167D-02,7.1473D-02,6.9005D-02,6.6735D-02,6.4638D-02,
     &  6.2693D-02,6.0883D-02,5.9194D-02,5.7613D-02,5.6129D-02,
     &  5.4734D-02,5.3418D-02,5.2175D-02,5.0999D-02,4.9884D-02,
     &  4.8825D-02,4.7817D-02,4.6857D-02,4.5942D-02,4.5067D-02,
     &  4.4230D-02,4.3429D-02,4.2662D-02,4.1925D-02,4.1218D-02,
     &  4.0537D-02,3.9883D-02,3.9252D-02,3.8645D-02,3.8058D-02,
     &  3.7492D-02,3.6946D-02,3.6417D-02,3.5906D-02,3.5411D-02,
     &  3.4931D-02,3.4466D-02,3.4016D-02,3.3579D-02,3.3154D-02,
     &  3.2742D-02,3.2341D-02,3.1952D-02,3.1573D-02,3.1204D-02,
     &  3.0845D-02,3.0496D-02,3.0155D-02,2.9823D-02,2.9500D-02,
     &  2.9184D-02,2.8876D-02,2.8575D-02,2.8282D-02,2.7995D-02,
     &  2.7715D-02,2.7441D-02,2.7173D-02,2.6911D-02,2.6655D-02,
     &  2.6405D-02,2.6159D-02,2.5919D-02,2.5684D-02,2.5454D-02,
     &  2.5228D-02,2.5007D-02,2.4790D-02,2.4577D-02,2.4369D-02,
     &  2.4164D-02,2.3964D-02,2.3767D-02,2.3574D-02,2.3384D-02,
     &  2.3198D-02,2.3015D-02,2.2835D-02,2.2659D-02,2.2485D-02,
     &  2.2315D-02,2.2147D-02,2.1982D-02,2.1820D-02,2.1661D-02,
     &  2.1505D-02,2.1350D-02,2.1199D-02,2.1050D-02,2.0903D-02,
     &  2.0758D-02,2.0616D-02,2.0476D-02,2.0338D-02,2.0202D-02,
     &  2.0068D-02,1.9936D-02,1.9806D-02,1.9678D-02,1.9552D-02,
     &  1.9427D-02,1.9305D-02,1.9184D-02,1.9065D-02,1.8947D-02,
     &  1.8831D-02,1.8717D-02,1.8604D-02,1.8493D-02,1.8383D-02,
     &  1.8275D-02,1.8168D-02,1.8063D-02,1.7959D-02,1.7856D-02,
     &  1.7754D-02,1.7654D-02,1.7555D-02,1.7458D-02,1.7361D-02,
     &  1.7266D-02,1.7172D-02,1.7079D-02,1.6987D-02,1.6896D-02,
     &  1.6807D-02,1.6718D-02,1.6631D-02,1.6544D-02,1.6459D-02,
     &  1.6374D-02,1.6291D-02,1.6208D-02,1.6126D-02,1.6046D-02,
     &  1.5966D-02,1.5887D-02,1.5809D-02,1.5732D-02,1.5655D-02,
     &  1.5580D-02,1.5505D-02,1.5431D-02,1.5358D-02,1.5285D-02,
     &  1.5214D-02,1.5143D-02,1.5073D-02,1.5003D-02,1.4935D-02,
     &  1.4867D-02,1.4799D-02,1.4733D-02,1.4667D-02,1.4601D-02,
     &  1.4537D-02,1.4473D-02,1.4409D-02,1.4346D-02,1.4284D-02,
     &  1.4223D-02,1.4162D-02,1.4101D-02,1.4041D-02,1.3982D-02,
     &  1.3924D-02,1.3865D-02,1.3808D-02,1.3751D-02,1.3694D-02,
     &  1.3638D-02,1.3582D-02,1.3527D-02,1.3473D-02,1.3419D-02,
     &  1.3365D-02,1.3312D-02,1.3260D-02,1.3207D-02,1.3156D-02,
     &  1.3104D-02,1.3054D-02,1.3003D-02,1.2953D-02,1.2904D-02,
     &  1.2855D-02,1.2806D-02,1.2758D-02,1.2710D-02,1.2662D-02,
     &  1.2615D-02,1.2569D-02,1.2522D-02,1.2476D-02,1.2431D-02,
     &  1.2386D-02,1.2341D-02,1.2296D-02,1.2252D-02,1.2208D-02,
     &  1.2165D-02,1.2122D-02,1.2079D-02,1.2037D-02,1.1995D-02,
     &  1.1953D-02,1.1911D-02,1.1870D-02,1.1830D-02,1.1789D-02,
     &  1.1749D-02,1.1709D-02,1.1670D-02,1.1630D-02,1.1591D-02,
     &  1.1553D-02,1.1514D-02,1.1476D-02,1.1438D-02,1.1401D-02,
     &  1.1363D-02,1.1326D-02,1.1290D-02,1.1253D-02,1.1217D-02,
     &  1.1181D-02,1.1145D-02,1.1110D-02,1.1074D-02,1.1039D-02,
     &  1.1005D-02,1.0970D-02,1.0936D-02,1.0902D-02,1.0868D-02,
     &  1.0835D-02,1.0801D-02,1.0768D-02,1.0735D-02,1.0703D-02,
     &  1.0670D-02,1.0638D-02,1.0606D-02,1.0574D-02,1.0543D-02,
     &  1.0511D-02,1.0480D-02,1.0449D-02,1.0419D-02,1.0388D-02,
     &  1.0358D-02,1.0328D-02,1.0298D-02,1.0268D-02,1.0238D-02,
     &  1.0209D-02,1.0180D-02,1.0151D-02,1.0122D-02,1.0093D-02,
     &  1.0065D-02,1.0036D-02,1.0008D-02,9.9802D-03,9.9525D-03,
     &  9.9249D-03,9.8974D-03,9.8701D-03,9.8430D-03,9.8161D-03,
     &  9.7893D-03,9.7627D-03,9.7362D-03,9.7099D-03,9.6838D-03,
     &  9.6578D-03,9.6320D-03,9.6063D-03,9.5808D-03,9.5554D-03,
     &  9.5302D-03,9.5051D-03,9.4801D-03,9.4553D-03,9.4307D-03,
     &  9.4062D-03,9.3818D-03,9.3576D-03,9.3335D-03,9.3095D-03,
     &  9.2857D-03,9.2620D-03,9.2385D-03,9.2150D-03,9.1917D-03,
     &  9.1686D-03,9.1456D-03,9.1227D-03,9.0999D-03,9.0772D-03,
     &  9.0547D-03,9.0323D-03,9.0100D-03,8.9878D-03,8.9658D-03,
     &  8.9439D-03,8.9221D-03,8.9004D-03,8.8788D-03,8.8574D-03,
     &  8.8360D-03,8.8148D-03,8.7937D-03,8.7727D-03,8.7518D-03,
     &  8.7310D-03,8.7103D-03,8.6897D-03,8.6693D-03,8.6489D-03,
     &  8.6287D-03,8.6085D-03,8.5885D-03,8.5685D-03,8.5487D-03,
     &  8.5290D-03,8.5093D-03,8.4898D-03,8.4703D-03,8.4510D-03,
     &  8.4318D-03,8.4126D-03,8.3936D-03,8.3746D-03,8.3557D-03,
     &  8.3370D-03,8.3183D-03,8.2997D-03,8.2812D-03,8.2628D-03,
     &  8.2445D-03,8.2263D-03,8.2081D-03,8.1901D-03,8.1721D-03,
     &  8.1543D-03,8.1365D-03,8.1188D-03,8.1011D-03,8.0836D-03,
     &  8.0662D-03,8.0488D-03,8.0315D-03,8.0143D-03,7.9972D-03,
     &  7.9801D-03,7.9632D-03,7.9463D-03,7.9295D-03,7.9128D-03,
     &  7.8961D-03,7.8795D-03,7.8631D-03,7.8466D-03,7.8303D-03,
     &  7.8140D-03,7.7978D-03,7.7817D-03,7.7657D-03,7.7497D-03,
     &  7.7338D-03,7.7179D-03,7.7022D-03,7.6865D-03,7.6709D-03,
     &  7.6553D-03,7.6398D-03,7.6244D-03,7.6091D-03,7.5938D-03,
     &  7.5786D-03,7.5635D-03,7.5484D-03,7.5334D-03,7.5184D-03,
     &  7.5036D-03,7.4887D-03,7.4740D-03,7.4593D-03,7.4447D-03,
     &  7.4301D-03,7.4156D-03,7.4012D-03,7.3868D-03,7.3725D-03,
     &  7.3583D-03,7.3441D-03,7.3299D-03,7.3159D-03,7.3019D-03,
     &  7.2879D-03,7.2740D-03,7.2602D-03,7.2464D-03,7.2327D-03,
     &  7.2190D-03,7.2054D-03,7.1918D-03,7.1783D-03,7.1649D-03,
     &  7.1515D-03,7.1382D-03,7.1249D-03,7.1117D-03,7.0985D-03,
     &  7.0854D-03,7.0724D-03,7.0593D-03,7.0464D-03,7.0335D-03,
     &  7.0206D-03,7.0078D-03,6.9951D-03,6.9824D-03,6.9697D-03,
     &  6.9571D-03,6.9446D-03,6.9321D-03,6.9196D-03,6.9072D-03,
     &  6.8949D-03,6.8826D-03,6.8703D-03,6.8581D-03,6.8460D-03,
     &  6.8339D-03/
        save XX,YY
        e=s/m2
c PHI resonance range 
       ij=N
       eps=0.5d0
       IF ((E.GT.xx(1)).AND.(E.LT.xx(N))) THEN
         do while (xx(ij).ge.e)
           ij=ij-1
         enddo
           k2anum=YY(IJ)
     &           +(YY(IJ+1)-YY(IJ))/(XX(IJ+1)-XX(IJ))*(E-XX(IJ))
       ELSE IF (E.GT.xx(N)) then
         k2anum=-k2a(s,m2)
       ELSE
         k2anum=0.D0
         IF ((E.GT.(xx(1)-eps)).AND.(E.LT.(xx(1)+eps)))
     &        k2anum=YY(1)
         IF ((E.GT.(xx(N)-eps)).AND.(E.LT.(xx(N)+eps)))
     &        k2anum=YY(N)
       ENDIF
       k2anum=-k2anum
       RETURN
       END
