C--------------------------------------------
C     spline3.for
C--------------------------------------------
C==============================================================================
      SUBROUTINE SPLIN3(X,Y,DERIV,N,NC,Z,FVALUE,FDERIV,M,MC,IOP)
C==============================================================================
C
C     CERN LIBRARY PROGRAM NO E-209.
C
C     REVISED VERSION JULY 1973.
C
C     MODIFIED BY M. BATINIC TO AVOID WARNING MESSAGES
C          LABEL USED ACROSS BLOCK
C     WHICH CAN BE SEVERE ERROR ON SOME MACHINE, DEPENDING
C     ON WHAT HAPPEND WHEN GO OUTSIDE DO LOOP AND RETURN IN IT.
C
C     PURPOSE = TO COMPUTE A NATURAL SPLINE APPROXIMATION OF THIRD ORDER
C               FOR A FUNCTION Y(X) GIVEN IN THE N POINTS (X(I),Y(I)) ,
C               I=1(1)N.
C
C     PARAMETERS (IN LIST).
C
C     X       = AN ARRAY STORING THE INPUT ARGUMENTS.DIMENSION X(N).
C     Y       = AN ARRAY STORING THE INPUT FUNCTION VALUES.THE ELEMENT
C               Y(I) REPRESENT THE FUNCTION VALUE Y(X) FOR X=X(I).
C     DERIV   = AN ARRAY USED FOR STORING THE COMPUTED DERIVATIVES OF
C               THE FUNCTION Y(X).IN DERIV(I,1) AND DERIV(I,2) ARE STOR-
C               ED THE FIRST-AND SECOND ORDER DERIVATIVES OF Y(X) FOR
C               X=X(I) RESPECTIVELY.
C     N       = NUMBER OF INPUT FUNCTION VALUES.
C     NC      = ARRAY DERIV IS DIMENSIONED DERIV(NC,2) IN CALLING
C               PROGRAM.
C     Z       = AN ARRAY STORING THE ARGUMENTS FOR THE INTERPOLATED
C               VALUES TO BE COMPUTED.
C     FVALUE  = AN ARRAY STORING THE COMPUTED INTERPOLATED VALUES.
C               FVALUE(J) REPRESENT THE FUNCTION VALUE FVALUE(Z) FOR
C               Z=Z(J).
C     FDERIV    = AN ARRAY USED FOR STORING THE DERIVATIVES OF THE COM-
C               PUTED INTERPOLATED VALUES.EXPLANATION AS FOR DERIV.
C     M       = NUMBER OF INTERPOLATED VALUES TO BE COMPUTED.
C     MC      = ARRAY FDERIV IS DIMENSIONED FDERIV(MC,2) IN CALLING
C               PROGRAM.
C     IOP     = OPTION PARAMETER.FOR IOP.LE.0 THE DERIVATIVES FOR EACH
C               SUB-INTERVAL IN THE SPLINE APPROXIMATION ARE COMPUTED.
C                                  IOP=-1, THE SECOND ORDER END-POINT
C                                          DERIVATIVES ARE COMPUTED BY
C                                          LINEAR EXTRAPOLATION.
C                                  IOP=0 , THE SECOND ORDER END-POINT
C                                          DERIVATIVES ASSUMED TO BE GI-
C                                          VEN (SEE COMMON /SPAPPR/).
C                                  IOP=1 , COMPUTE SPLINE APPROXIMATIONS
C                                          FOR THE ARGUMENTS GIVEN IN
C                                          THE ARRAY Z,THE DERIVATIVES
C                                          BEEING ASSUMED TO HAVE BEEN
C                                          CALCULATED IN A PREVIOUS CALL
C                                          ON THE ROUTINE.
C
C     PARAMETERS (IN COMMON BLOCK / SPAPPR /).
C
C     SECD1   = VALUE OF THE SECOND DERIVATIVE D2Y(X)/DX2 FOR THE INPUT
C               ARGUMENT X=X(1).
C     SECDN   = VALUE OF THE SECOND DERIVATIVE D2Y(X)/DX2 FOR THE INPUT
C               ARGUMENT X=X(N).
C               NB. VALUES HAVE TO BE ASSIGNED TO SECD1 AND SECDN IN THE
C               CALLING PROGRAM.IF A NATURAL SPLINE FIT IS WANTED PUT
C               SECD1=SECDN=0.
C     VOFINT  = COMPUTED APPROXIMATION FOR THE INTEGRAL OF Y(X) TAKEN
C               FROM X(1) TO X(N).
C     IERR    = ERROR PARAMETER.IERR=0,NO ERRORS OCCURED.
C                               IERR=1,THE NUMBER OF POINTS TOO SMALL
C                                      I.E.N LESS THAN 4.
C                               IERR=2,THE ARGUMENTS X(I) NOT IN INCREA-
C                                      SING ORDER.
C                               IERR=3,ARGUMENT TO BE USED IN INTERPOLA-
C                                      TION ABOVE RANGE.
C                               IERR=4,ARGUMENT TO BE USED IN INTERPOLA-
C                                      TION BELOW RANGE.
C     NXY     = N (SEE ABOVE),HAS TO BE STORED FOR ENTRIES CORRESPONDING
C               TO IOP=1.
C
C==============================================================================
C
      IMPLICIT REAL*8(A-H,O-Z)
C
      DIMENSION X(NC),Y(NC),DERIV(NC,2),Z(MC),FVALUE(MC),FDERIV(MC,2)
C
      COMMON /SPAPPR / SECD1, SECDN, VOFINT, IERR, NXY
C
      DATA ZERO  / 0.0D0 /
      DATA HALF  / 0.5D0 /
      DATA ONE   / 1.0D0 /
      DATA THREE / 3.0D0 /
      DATA THIRD / 0.333333333333333D0 /
      DATA SIXTH / 0.166666666666667D0 /
C
      IF (IOP.GT.0) GOTO 1110
C
      IERR=0
C
C     CHECK IF ENOUGH DATA-POINTS ARE AVAILABLEI.E. IF N LESS THAN 4 NO
C     THIRD ORDER SPLINE APPROXIMATION IS POSSIBLE.
C
      IF (N.LT.4) THEN
         IERR=1
         GOTO 2000
      END IF
C
C     START CALCULATION OF COEFFICIENTS TO BE USED IN THE SYSTEM OF EQU-
C     ATIONS FOR THE SECOND ORDER DERIVATIVES OF Y(X).
C
      IF (IOP.EQ.-1) THEN
         SECD1=ZERO
         SECDN = ZERO
         BET1=ONE/(ONE+HALF*(X(2)-X(1))/(X(3)-X(2)))
         ALF1=BET1*(ONE- ((X(2)-X(1))/(X(3)-X(2)))**2)
         BETN=ONE/(ONE+HALF*(X(N)-X(N-1))/(X(N-1)-X(N-2)))
         ALFN=BETN*(ONE- ((X(N)-X(N-1))/(X(N-1)-X(N-2)))**2)
      END IF
C
      DERIV(1,2)=SECD1
      DERIV(N,2)=SECDN
      DERIV(1,1)=ZERO
      DXPLUS=X(2)-X(1)
C
C     CHECK IF ARGUMENTS ARE IN INCREASING ORDER.
C     IF NOT PRINT ERROR MESSAGE AND STOP.
C
      IF (DXPLUS.LE.ZERO) THEN
         IN=1
         IERR=2
         GOTO 2000
      END IF
C
      DYPLUS=(Y(2)-Y(1))/DXPLUS
      IU=N-1
C
      DO 1040 I=2,IU
C
         DXMIN =DXPLUS
         DYMIN =DYPLUS
         DXPLUS=X(I+1)-X(I)
C
C        CHECK IF ARGUMENTS ARE IN INCREASING ORDER.IF NOT PRINT ERROR
C        MESSAGE AND STOP.
C
         IF (DXPLUS.LE.ZERO) THEN
            IN=I
            IERR=2
            GOTO 2000
         END IF
C
         DXINV =ONE/(DXPLUS+DXMIN)
         DYPLUS=(Y(I+1)-Y(I))/DXPLUS
         DIVDIF=DXINV*(DYPLUS-DYMIN)
         ALF   =HALF*DXINV*DXMIN
         BET   =HALF-ALF
C
         IF (I.EQ.2)  DIVDIF=DIVDIF-THIRD*ALF*DERIV(1,2)
         IF (I.EQ.IU) DIVDIF=DIVDIF-THIRD*BET*DERIV(N,2)
         IF (I.EQ.2) ALF=ZERO
C
         IF (IOP.EQ.-1) THEN
            IF (I.EQ.2) THEN
               BET=BET*ALF1
               DIVDIF=DIVDIF*BET1
            ELSE
               IF (I.EQ.IU) THEN
                  ALF=ALF*ALFN
                  DIVDIF=DIVDIF*BETN
               END IF
            END IF
         END IF
C
         DXINV =ONE/(ONE+ALF*DERIV(I-1,1))
         DERIV(I,1)=-DXINV*BET
         DERIV(I,2)= DXINV*(THREE*DIVDIF-ALF*DERIV(I-1,2))
C
 1040 CONTINUE
C
C     COMPUTE THE SECOND DERIVATIVES BY BACKWARDS RECURRENCE RELATION.
C     THE SECOND ORDER DERIVATIVES FOR X=X(N-1) ALREADY COMPUTED.
C
      DO 1060 I=2,IU
         J=N-I
         DERIV(J,2)=DERIV(J,1)*DERIV(J+1,2)+DERIV(J,2)
 1060 CONTINUE
C
      IF (IOP.EQ.-1) THEN
         DERIV(1,2)= ((X(3)-X(1))/(X(3)-X(2)))*DERIV(2,2)
     &              -((X(2)-X(1))/(X(3)-X(2)))*DERIV(3,2)
         DERIV(N,2)=-((X(N)-X(N-1))/(X(N-1)-X(N-2)))*DERIV(N-2,2)
     &              +((X(N)-X(N-2))/(X(N-1)-X(N-2)))*DERIV(N-1,2)
      END IF
C
C     CALCULATION OF THE SECOND ORDER DERIVATIVES FINISHED.START CAL-
C     CULATION OF THE FIRST ORDER DERIVATIVES AND OF THE INTEGRAL.
C
      VOFINT=ZERO
      DO 1080 I=1,IU
         DXPLUS=X(I+1)-X(I)
         DYPLUS=Y(I+1)-Y(I)
         DIVDIF=DYPLUS/DXPLUS
         DERIV(I,1)=DIVDIF-DXPLUS*(THIRD*DERIV(I,2)+SIXTH*DERIV(I+1,2))
         DXPLUS=HALF*DXPLUS
         VOFINT=VOFINT+DXPLUS*(Y(I+1)+Y(I)
     &          -THIRD*(DERIV(I+1,2)+DERIV(I,2))*DXPLUS**2)
 1080 CONTINUE
C
C     COMPUTE THE LAST FIRST ORDER DERIVATIVE.
C
      DXPLUS=X(N)-X(N-1)
      DYPLUS=Y(N)-Y(N-1)
      DIVDIF=DYPLUS/DXPLUS
      DERIV(N,1)=DIVDIF+DXPLUS*(SIXTH*DERIV(N-1,2)+THIRD*DERIV(N,2))
C
C     CALCULATION OF FIRST ORDER DERIVATIVES AND INTEGRAL FINISHED.
C
C     SET VALUE OF N IN COMMON BLOCK / SPAPPR /.
C
      NXY=N
C
C     COMPUTE INTERPOLATED VALUES IF ANY.
C
 1110 IF (M.LT.1) RETURN
C
      XL=X(1)
      XU=X(2)
      IP=3
      IL=0
C
      DO 1160 J=1,M
C
      IERR34=0
C
      ARG=Z(J)
      IF (ARG.GT.XU) THEN
         IF(IP.GT.NXY) THEN
            IERR34=3
            IP=NXY+1
            GOTO 1155
         END IF
         IPP=IP
         DO 1180 I=IPP,NXY
            IF (ARG.LE.X(I)) THEN
               XL=X(I-1)
               XU=X(I)
               IP=I+1
               IL=0
               GOTO 1150
            END IF
 1180    CONTINUE
         IERR34=3
         IP=NXY+1
         GOTO 1155
      END IF
      IF (ARG.LT.XL) THEN
         IPP=IP
         DO 1200 I=1,IPP
            II=IP-I-2
            IF (II.EQ.0) THEN
               IERR34=4
               IP=3
               GOTO 1155
            END IF
            IF (ARG.GE.X(II)) THEN
               XL=X(II)
               XU=X(II+1)
               IP=II+2
               IL=0
               GOTO 1150
            END IF
 1200    CONTINUE
         IERR34=4
         IP=3
         GOTO 1155
      END IF
C
1150  IF (IL.EQ.0) THEN
C
C        COMPUTE POLYNOMIAL COEFFICIENTS IF NEEDED.
C
         II=IP-2
         A0=Y(II)
         A1=DERIV(II,1)
         A4=DERIV(II,2)
         A6=(DERIV(II+1,2)-A4)/(XU-XL)
         A2=HALF*A4
         A3=SIXTH*A6
         A5=HALF*A6
         IL=1
      END IF
C
C     COMPUTE VALUES.
C
      ARG=ARG-XL
      FVALUE(J)  =((A3*ARG+A2)*ARG+A1)*ARG+A0
      FDERIV(J,1)= (A5*ARG+A4)*ARG+A1
      FDERIV(J,2)=  A6*ARG+A4
C
 1155 CONTINUE
C
      IF (IERR34.NE.0) THEN
         IERR=IERR34
         WRITE(6,3000) IERR, ARG
         FVALUE(J)=ZERO
         FDERIV(J,1)=ZERO
         FDERIV(J,2)=ZERO
         II=IP-2
         XL=X(II)
         XU=X(II+1)
         IL=0
      END IF
C
 1160 CONTINUE
C
C     CALCULATION OF INTERPOLATED VALUES FINISHED.
C
      RETURN
C
 2000 CONTINUE
C
C     SPLINE ERROR # 1 OR # 2 OCCURS.
C
      IF (IERR.EQ.1) WRITE(6,3000)  IERR
      IF (IERR.EQ.2) WRITE(6,3000)  IERR , X(IN) , X(IN+1)
C
 3000 FORMAT(/,5X,'*** SUBROUTINE SPLIN3 ERROR NO ',I2,' ***',
     &       2(4X,E21.14))
C
      RETURN
C
      END
