c;;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; hadr5n09.f -- 
c;; Author          : Friedrich Jegerlehner
c;; Created On      : Wed Sep  9 17:52:59 2009
c;; Last Modified By: Friedrich Jegerlehner
c;; Last Modified On: Sun Apr  1 14:46:51 2012
c;; RCS: $Id$
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; Copyright (C) 2009 Friedrich Jegerlehner
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; 
       subroutine hadr5n09(e,st2,der,errder,deg,errdeg)
c Provides real part of 5 flavor hadronic contribution to photon vacuum polarization
c single precision HADR5 ; double precision DHADR5
c ******************************************************************
c *                                                                *
c *      subroutine for the evaluation of the light hadron         *
c *           contributions to Delta_r  and  Delta_g               *
c *                    using fits to the                           *
c *          QED vacuum polarization from e^+ e^- data             *
c *                                                                *
c *    F. Jegerlehner, Institut fr Physik                         *
c *                    Humboldt-Universitt zu Berlin              *
c *                    Newtonstrae 15, D-12489 Berlin, Germany    *
c *    E-mail: fjeger@physik.hu-berlin.de                          *
c *    Phone :   +49-3375 500 277                                  *
c *                                                                *
c *    Reference: F. Jegerlehner, Z. Phys. C32 (1986) 195          *
c *               H. Burkhardt et al., Z. Phys. C42 (1989) 497     *
c *               S. Eidelman, F. Jegerlehner, Z. Phys. C (1995)   *
c *               F. Jegerlehner,                                  *
c *                    Nucl.Phys.Proc.Suppl.131:213-222,2004       *
c *                    Nucl.Phys.Proc.Suppl.162:22-32,2006         *
c *                    Nucl.Phys.Proc.Suppl.181-182:135-140,2008   *
c *                                                                *
c ******************************************************************
c       VERSION: 09/09/2009
c
C  Notation: E energy ( momentum transfer ): E>0 timelike , E<0 spacelike
c      sin2ell=0.23153 ! pm 0.00016 LEPEEWG Phys Rep 427 (2006) 257
C            st2 is sin^2(Theta); st2=0.23153 is the reference value
C  the routine returns the hadronic contribution of 5 flavors (u,d,s,c,b)
C                 to   DER=Delta_r with hadronic error ERRDER
C                and   DEG=Delta_g with hadronic error ERRDEG
C  The effective value of the fine structure constant alphaQED at energy
C  E is alphaQED(E)=alphaQED(0)/(1-Delta_r) ,similarly for the SU(2)
C  coupling alphaSU2(E)=alphaSU2(0)/(1-Delta_g), where Delta_r(g) is the
C  sum of leptonic, hadronic contributions (top to be added).
C
      IMPLICIT NONE
      INTEGER NA,NB,NC,IJ,I,ini
      PARAMETER(NA=979,NB=1840,NC=200)
      real e,st2,der,errder,deg,errdeg,fac
      real dal(2),dg2(2)
      REAL ETA(NA),DAT(NA,2),ESA(NB),DAS(NB,2),EMA(NC),DAM(NC,2),
     &     DGT(NA,2),DGS(NB,2),DGM(NC,2)
      COMMON /DATT09/ETA,DAT,DGT
      COMMON /DATS09/ESA,DAS,DGS
      COMMON /DATM09/EMA,DAM,DGM
      data ini/0/
C initialize data

      call dalhad_spaceliken

      call dalhad_timeliken

      call dalhad_timelike1n

c boundaries of data sets
c space-like and unphysical region  -1.0000E+03 - 0.27599999D0 (ETA,DAT)
c time-like R data range 0.28207018D0 - 38.29000000D0          (ESA,DAS)
c time-like1 pQCD range 1.3005E+01 - 2.0000E+04                (EMA,DAM)
c 2 pion threshold at 0.27914....
      fac=0.23153d0/st2
      if (e.eq.0.0) then
        der=0.0
        errder=0.0
        deg=0.0
        errdeg=0.0
        return
c first bridge cracks between regions
      else if ((e.ge.ETA(NA)).and.(e.lt.ESA(1))) then
        DO I=1,2
        dal(i)=DAT(NA,I)
     &    +(DAS(1,I)-DAT(NA,I))/(ESA(1)-ETA(NA))*(E-ETA(NA))
        dg2(i)=DGT(NA,I)
     &    +(DGS(1,I)-DGT(NA,I))/(ESA(1)-ETA(NA))*(E-ETA(NA))
        ENDDO
        der=dal(1)
        errder=dal(2)
        deg=dg2(1)*fac
        errdeg=dg2(2)*fac
        return
      else if ((e.lt.ETA(NA)).and.(e.ge.ETA(1))) then
        ij=NA
        do while (ETA(ij).gt.e)
          ij=ij-1
        enddo
        DO I=1,2
        dal(i)=DAT(IJ,I)
     &    +(DAT(IJ+1,I)-DAT(IJ,I))/(ETA(IJ+1)-ETA(IJ))*(E-ETA(IJ))
        dg2(i)=DGT(IJ,I)
     &    +(DGT(IJ+1,I)-DGT(IJ,I))/(ETA(IJ+1)-ETA(IJ))*(E-ETA(IJ))
        ENDDO
        der=dal(1)
        errder=dal(2)
        deg=dg2(1)*fac
        errdeg=dg2(2)*fac
        return
      else if ((e.lt.EMA(NC)).and.(e.ge.EMA(1))) then
        ij=NC
        do while (EMA(ij).gt.e)
          ij=ij-1
        enddo
        DO I=1,2
        dal(i)=DAM(IJ,I)
     &    +(DAM(IJ+1,I)-DAM(IJ,I))/(EMA(IJ+1)-EMA(IJ))*(E-EMA(IJ))
        dg2(i)=DGM(IJ,I)
     &    +(DGM(IJ+1,I)-DGM(IJ,I))/(EMA(IJ+1)-EMA(IJ))*(E-EMA(IJ))
        ENDDO
        der=dal(1)
        errder=dal(2)
        deg=dg2(1)*fac
        errdeg=dg2(2)*fac
        return
      else if ((e.lt.ESA(NB)).and.(e.ge.ESA(1))) then
        if ((e.gt.3.96).and.(e.lt.4.52).and.(ini.eq.0)) then
c           write (*,*) ' ******************************************'
c           write (*,*) ' * Note: fluctuations in the Psi(4,5,6)   *'
c           write (*,*) ' * region are because we are using the    *'
c           write (*,*) ' * CB and BES data without any smoothing  *'
c           write (*,*) ' *********** FJ@HU Berlin *****************'
           ini=1
        endif
        ij=NB
        do while (ESA(ij).gt.e)
          ij=ij-1
        enddo
        DO I=1,2
        dal(i)=DAS(IJ,I)
     &    +(DAS(IJ+1,I)-DAS(IJ,I))/(ESA(IJ+1)-ESA(IJ))*(E-ESA(IJ))
        dg2(i)=DGS(IJ,I)
     &    +(DGS(IJ+1,I)-DGS(IJ,I))/(ESA(IJ+1)-ESA(IJ))*(E-ESA(IJ))
        ENDDO
        der=dal(1)
        errder=dal(2)
        deg=dg2(1)*fac
        errdeg=dg2(2)*fac
        return
      else
         write(*,*) ' out of range! '
      endif
      return
      end

       subroutine dhadr5n09(de,dst2,dder,derrder,ddeg,derrdeg)
c **********************************************************************
c *    F. Jegerlehner, University of Silesia, PL-40007 Katowice, Poland *
c **********************************************************************
c Converts hadr5n09 to double precision variables in dhadr5n09
c
       implicit none
       real *8 de,dst2,dder,ddeg,derrder,derrdeg
       real    se,sst2,sder,sdeg,serrder,serrdeg
       se  =sngl(de)
       sst2=sngl(dst2)
       call hadr5n09(se,sst2,sder,serrder,sdeg,serrdeg)
       dder   =dble(sder)
       ddeg   =dble(sdeg)
       derrder=dble(serrder)
       derrdeg=dble(serrdeg)
       return
       end
c
c Following datasets obtained with run parameters specified in the header
c 
c ******************************************************************************
c *                                                                            *
c *    --- intRdat1 ---                              Tue Mar 16 19:12:00 2010  *
c *                                                                            *
c ***************************# FJ@HU Berlin #***********************************
c 
c    Delta alpha [5 quarks]     Update version (02/02/2010) 
c
c  ISCAN: 1, Ismooth: 0, IGS: 0, DSET:     , iplot: 0
c  IKE,Emin,Emax,NP,N,EPSREL: 1, 1.1200E+01, 1.1200E+01,   1, 10, 1.00E-03,
c  ALS(sta)(sys):  0.1184( 0.0007)( 0.0000), MZINP:   91.1876, MTOP:    171.30
c  NF,IOR,ICHK: 5,4,2,ISU2: 0, sin2W:   0.23153
c  CHPTCUT,EC,ECUT: 0.3180,     5.20,    11.50
 
       subroutine dalhad_timeliken
       IMPLICIT NONE
       INTEGER NB,I,J
       PARAMETER(NB=1840)
       REAL ESA(NB),DAS(NB,2),DGS(NB,2)
       COMMON /DATS09/ESA,DAS,DGS
  
       DATA  (ESA(I), I=   1, 100) /
     &   0.28207018D0, 0.28800000D0, 0.29400000D0, 0.30000000D0,
     &   0.30600000D0, 0.31200000D0, 0.32050000D0, 0.32600000D0,
     &   0.33100000D0, 0.33600000D0, 0.34100000D0, 0.34600000D0,
     &   0.35200000D0, 0.35600000D0, 0.36100000D0, 0.36600000D0,
     &   0.37100000D0, 0.37600000D0, 0.38000000D0, 0.38500000D0,
     &   0.38900000D0, 0.39400000D0, 0.39800000D0, 0.40300000D0,
     &   0.40700000D0, 0.41100000D0, 0.41600000D0, 0.41871100D0,
     &   0.42000000D0, 0.42400000D0, 0.42800000D0, 0.43200000D0,
     &   0.43600000D0, 0.44000000D0, 0.44400000D0, 0.44800000D0,
     &   0.45200000D0, 0.45600000D0, 0.46000000D0, 0.46400000D0,
     &   0.46700000D0, 0.47100000D0, 0.47500000D0, 0.47800000D0,
     &   0.48200000D0, 0.48600000D0, 0.48900000D0, 0.49300000D0,
     &   0.49600000D0, 0.50000000D0, 0.52000000D0, 0.54000000D0,
     &   0.56600000D0, 0.59200000D0, 0.60000000D0, 0.60200000D0,
     &   0.60800000D0, 0.61600000D0, 0.62400000D0, 0.63200000D0,
     &   0.64000000D0, 0.64300000D0, 0.64800000D0, 0.65600000D0,
     &   0.66300000D0, 0.67100000D0, 0.67800000D0, 0.68600000D0,
     &   0.68800000D0, 0.69300000D0, 0.69800000D0, 0.70000000D0,
     &   0.70700000D0, 0.71400000D0, 0.72100000D0, 0.72800000D0,
     &   0.73500000D0, 0.74200000D0, 0.74800000D0, 0.75000000D0,
     &   0.75500000D0, 0.75743600D0, 0.76007300D0, 0.76207300D0,
     &   0.76507300D0, 0.76851700D0, 0.77060500D0, 0.77205400D0,
     &   0.77314300D0, 0.77400300D0, 0.77470900D0, 0.77530500D0,
     &   0.77582000D0, 0.77627100D0, 0.77667300D0, 0.77726600D0,
     &   0.77746600D0, 0.77766900D0, 0.77795100D0, 0.77821500D0/
       DATA  (ESA(I), I= 101, 200) /
     &   0.77846300D0, 0.77869800D0, 0.77892200D0, 0.77913700D0,
     &   0.77934400D0, 0.77954500D0, 0.77974200D0, 0.77993500D0,
     &   0.78012700D0, 0.78031800D0, 0.78051000D0, 0.78070600D0,
     &   0.78090800D0, 0.78112100D0, 0.78135100D0, 0.78161200D0,
     &   0.78193600D0, 0.78268000D0, 0.78341500D0, 0.78373400D0,
     &   0.78399100D0, 0.78421700D0, 0.78442600D0, 0.78462500D0,
     &   0.78481700D0, 0.78500500D0, 0.78519100D0, 0.78537800D0,
     &   0.78556600D0, 0.78575600D0, 0.78595100D0, 0.78615100D0,
     &   0.78635800D0, 0.78657300D0, 0.78679800D0, 0.78703400D0,
     &   0.78728400D0, 0.78755000D0, 0.78773500D0, 0.78793500D0,
     &   0.78814300D0, 0.78847900D0, 0.78884800D0, 0.78925800D0,
     &   0.78971900D0, 0.79024600D0, 0.79085900D0, 0.79158800D0,
     &   0.79248000D0, 0.79361500D0, 0.79513900D0, 0.79736300D0,
     &   0.79800000D0, 0.80000000D0, 0.80200000D0, 0.80400000D0,
     &   0.80600000D0, 0.80800000D0, 0.81200000D0, 0.81400000D0,
     &   0.81600000D0, 0.82000000D0, 0.82200000D0, 0.82400000D0,
     &   0.82600000D0, 0.82800000D0, 0.83000000D0, 0.83100000D0,
     &   0.83700000D0, 0.84300000D0, 0.84900000D0, 0.85400000D0,
     &   0.86000000D0, 0.86600000D0, 0.87200000D0, 0.87700000D0,
     &   0.88300000D0, 0.88900000D0, 0.89400000D0, 0.89700000D0,
     &   0.90250000D0, 0.90500000D0, 0.91100000D0, 0.91600000D0,
     &   0.92200000D0, 0.92700000D0, 0.93300000D0, 0.93800000D0,
     &   0.94300000D0, 0.94900000D0, 0.95200000D0, 0.95400000D0,
     &   0.95800000D0, 0.96000000D0, 0.96200000D0, 0.96400000D0,
     &   0.96600000D0, 0.96800000D0, 0.97000000D0, 0.97400000D0/
       DATA  (ESA(I), I= 201, 300) /
     &   0.97800000D0, 0.98000000D0, 0.98070000D0, 0.98140000D0,
     &   0.98210000D0, 0.98280000D0, 0.98350000D0, 0.98420000D0,
     &   0.98490000D0, 0.98560000D0, 0.98630000D0, 0.98700000D0,
     &   0.98770000D0, 0.98840000D0, 0.98910000D0, 0.98980000D0,
     &   0.99050000D0, 0.99120000D0, 0.99190000D0, 0.99260000D0,
     &   0.99330000D0, 0.99400000D0, 0.99470000D0, 0.99540000D0,
     &   0.99610000D0, 0.99680000D0, 0.99750000D0, 0.99820000D0,
     &   0.99890000D0, 0.99960000D0, 0.99990000D0, 1.00001000D0,
     &   1.00030000D0, 1.00100000D0, 1.00170000D0, 1.00240000D0,
     &   1.00310000D0, 1.00380000D0, 1.00450000D0, 1.00520000D0,
     &   1.00590000D0, 1.00660000D0, 1.00730000D0, 1.00800000D0,
     &   1.00870000D0, 1.00940000D0, 1.01010000D0, 1.01080000D0,
     &   1.01150000D0, 1.01220000D0, 1.01290000D0, 1.01360000D0,
     &   1.01430000D0, 1.01500000D0, 1.01520000D0, 1.01540000D0,
     &   1.01560000D0, 1.01580000D0, 1.01600000D0, 1.01620000D0,
     &   1.01640000D0, 1.01660000D0, 1.01680000D0, 1.01700000D0,
     &   1.01720000D0, 1.01740000D0, 1.01760000D0, 1.01780000D0,
     &   1.01800000D0, 1.01820000D0, 1.01840000D0, 1.01860000D0,
     &   1.01880000D0, 1.01900000D0, 1.01920000D0, 1.01940000D0,
     &   1.01960000D0, 1.01980000D0, 1.02000000D0, 1.02020000D0,
     &   1.02040000D0, 1.02060000D0, 1.02080000D0, 1.02100000D0,
     &   1.02120000D0, 1.02140000D0, 1.02160000D0, 1.02180000D0,
     &   1.02200000D0, 1.02220000D0, 1.02240000D0, 1.02260000D0,
     &   1.02280000D0, 1.02300000D0, 1.02320000D0, 1.02340000D0,
     &   1.02360000D0, 1.02380000D0, 1.02400000D0, 1.02420000D0/
       DATA  (ESA(I), I= 301, 400) /
     &   1.02440000D0, 1.02460000D0, 1.02480000D0, 1.02500000D0,
     &   1.02570000D0, 1.02640000D0, 1.02710000D0, 1.02780000D0,
     &   1.02850000D0, 1.02920000D0, 1.02990000D0, 1.03060000D0,
     &   1.03130000D0, 1.03200000D0, 1.03270000D0, 1.03340000D0,
     &   1.03410000D0, 1.03480000D0, 1.03550000D0, 1.03620000D0,
     &   1.03690000D0, 1.03760000D0, 1.03830000D0, 1.03900000D0,
     &   1.03970000D0, 1.03999000D0, 1.04001000D0, 1.04040000D0,
     &   1.04110000D0, 1.04180000D0, 1.04250000D0, 1.04320000D0,
     &   1.04390000D0, 1.04460000D0, 1.04530000D0, 1.04600000D0,
     &   1.04670000D0, 1.04740000D0, 1.04810000D0, 1.04880000D0,
     &   1.04950000D0, 1.05020000D0, 1.05090000D0, 1.05160000D0,
     &   1.05230000D0, 1.05300000D0, 1.05370000D0, 1.05440000D0,
     &   1.05510000D0, 1.05580000D0, 1.05650000D0, 1.05720000D0,
     &   1.05790000D0, 1.05860000D0, 1.05930000D0, 1.06000000D0,
     &   1.06200000D0, 1.06700000D0, 1.07700000D0, 1.08700000D0,
     &   1.09700000D0, 1.10700000D0, 1.11700000D0, 1.12700000D0,
     &   1.13700000D0, 1.13800000D0, 1.15200000D0, 1.16300000D0,
     &   1.16700000D0, 1.17700000D0, 1.18700000D0, 1.18800000D0,
     &   1.19700000D0, 1.20700000D0, 1.21200000D0, 1.21700000D0,
     &   1.22700000D0, 1.23700000D0, 1.24700000D0, 1.25700000D0,
     &   1.26300000D0, 1.26700000D0, 1.27700000D0, 1.28700000D0,
     &   1.28800000D0, 1.29700000D0, 1.30700000D0, 1.31200000D0,
     &   1.31700000D0, 1.32700000D0, 1.33700000D0, 1.34700000D0,
     &   1.35000000D0, 1.35700000D0, 1.36200000D0, 1.36700000D0,
     &   1.37700000D0, 1.38700000D0, 1.38800000D0, 1.39500000D0/
       DATA  (ESA(I), I= 401, 500) /
     &   1.41000000D0, 1.41900000D0, 1.42700000D0, 1.43500000D0,
     &   1.43800000D0, 1.45200000D0, 1.45700000D0, 1.46000000D0,
     &   1.46400000D0, 1.47200000D0, 1.48100000D0, 1.49000000D0,
     &   1.49100000D0, 1.50000000D0, 1.50100000D0, 1.50600000D0,
     &   1.51000000D0, 1.52300000D0, 1.52500000D0, 1.53800000D0,
     &   1.53900000D0, 1.54000000D0, 1.55000000D0, 1.56000000D0,
     &   1.56200000D0, 1.57400000D0, 1.57500000D0, 1.58000000D0,
     &   1.58100000D0, 1.58700000D0, 1.59000000D0, 1.59900000D0,
     &   1.60000000D0, 1.61000000D0, 1.61900000D0, 1.62400000D0,
     &   1.62500000D0, 1.63800000D0, 1.64000000D0, 1.65000000D0,
     &   1.65700000D0, 1.66000000D0, 1.66200000D0, 1.66300000D0,
     &   1.67300000D0, 1.67500000D0, 1.68100000D0, 1.68700000D0,
     &   1.69000000D0, 1.70000000D0, 1.71000000D0, 1.71200000D0,
     &   1.71300000D0, 1.71900000D0, 1.72200000D0, 1.72500000D0,
     &   1.73800000D0, 1.74000000D0, 1.74600000D0, 1.75000000D0,
     &   1.75600000D0, 1.75700000D0, 1.76000000D0, 1.76200000D0,
     &   1.76400000D0, 1.76900000D0, 1.77500000D0, 1.78100000D0,
     &   1.79000000D0, 1.79900000D0, 1.80000000D0, 1.81200000D0,
     &   1.81900000D0, 1.82500000D0, 1.82600000D0, 1.83400000D0,
     &   1.83500000D0, 1.83800000D0, 1.84500000D0, 1.85700000D0,
     &   1.85900000D0, 1.87400000D0, 1.88100000D0, 1.88600000D0,
     &   1.90000000D0, 1.91900000D0, 1.92500000D0, 1.93000000D0,
     &   1.93800000D0, 1.94000000D0, 1.94500000D0, 1.94600000D0,
     &   1.95000000D0, 1.95500000D0, 1.96000000D0, 1.96200000D0,
     &   1.98100000D0, 1.99050000D0, 2.00000000D0, 2.05000000D0/
       DATA  (ESA(I), I= 501, 600) /
     &   2.10000000D0, 2.15000000D0, 2.22000000D0, 2.26000000D0,
     &   2.30000000D0, 2.40000000D0, 2.45000000D0, 2.50000000D0,
     &   2.55000000D0, 2.60000000D0, 2.65000000D0, 2.70000000D0,
     &   2.75000000D0, 2.80000000D0, 2.85000000D0, 2.90000000D0,
     &   2.95000000D0, 2.98000000D0, 3.04293500D0, 3.08587000D0,
     &   3.08688300D0, 3.08789600D0, 3.08890900D0, 3.08992200D0,
     &   3.09093500D0, 3.09194800D0, 3.09296100D0, 3.09397400D0,
     &   3.09498700D0, 3.09600000D0, 3.09605000D0, 3.09610000D0,
     &   3.09615000D0, 3.09620000D0, 3.09625000D0, 3.09630000D0,
     &   3.09635000D0, 3.09640000D0, 3.09645000D0, 3.09650000D0,
     &   3.09655000D0, 3.09660000D0, 3.09665000D0, 3.09665500D0,
     &   3.09673500D0, 3.09677100D0, 3.09679300D0, 3.09680900D0,
     &   3.09682000D0, 3.09682900D0, 3.09683600D0, 3.09684300D0,
     &   3.09684800D0, 3.09685300D0, 3.09685700D0, 3.09686100D0,
     &   3.09686400D0, 3.09686800D0, 3.09687000D0, 3.09687300D0,
     &   3.09687600D0, 3.09687800D0, 3.09688100D0, 3.09688300D0,
     &   3.09688500D0, 3.09688700D0, 3.09688900D0, 3.09689100D0,
     &   3.09689300D0, 3.09689500D0, 3.09689700D0, 3.09689900D0,
     &   3.09690100D0, 3.09690400D0, 3.09690600D0, 3.09690900D0,
     &   3.09691200D0, 3.09692000D0, 3.09692800D0, 3.09693100D0,
     &   3.09693400D0, 3.09693600D0, 3.09693900D0, 3.09694100D0,
     &   3.09694300D0, 3.09694500D0, 3.09694700D0, 3.09694900D0,
     &   3.09695100D0, 3.09695300D0, 3.09695500D0, 3.09695700D0,
     &   3.09695900D0, 3.09696200D0, 3.09696400D0, 3.09696700D0,
     &   3.09697000D0, 3.09697200D0, 3.09697600D0, 3.09697900D0/
       DATA  (ESA(I), I= 601, 700) /
     &   3.09698300D0, 3.09698700D0, 3.09699200D0, 3.09699700D0,
     &   3.09700400D0, 3.09701100D0, 3.09702000D0, 3.09703100D0,
     &   3.09704700D0, 3.09706900D0, 3.09710500D0, 3.09715000D0,
     &   3.09720000D0, 3.09725000D0, 3.09730000D0, 3.09735000D0,
     &   3.09740000D0, 3.09745000D0, 3.09750000D0, 3.09755000D0,
     &   3.09760000D0, 3.09765000D0, 3.09770000D0, 3.09775000D0,
     &   3.09780000D0, 3.09785000D0, 3.09790000D0, 3.09795000D0,
     &   3.09800000D0, 3.09898700D0, 3.09997400D0, 3.10096100D0,
     &   3.10194800D0, 3.10293500D0, 3.10392200D0, 3.10490900D0,
     &   3.10589600D0, 3.10688300D0, 3.10787000D0, 3.15000000D0,
     &   3.25000000D0, 3.30000000D0, 3.40000000D0, 3.45000000D0,
     &   3.50000000D0, 3.55000000D0, 3.59000000D0, 3.60200000D0,
     &   3.63000000D0, 3.65000000D0, 3.66000000D0, 3.67000000D0,
     &   3.67496000D0, 3.67551000D0, 3.67606000D0, 3.67661000D0,
     &   3.67716000D0, 3.67771000D0, 3.67826000D0, 3.67881000D0,
     &   3.67936000D0, 3.67991000D0, 3.68046000D0, 3.68101000D0,
     &   3.68156000D0, 3.68211000D0, 3.68266000D0, 3.68321000D0,
     &   3.68376000D0, 3.68431000D0, 3.68486000D0, 3.68527300D0,
     &   3.68541000D0, 3.68552000D0, 3.68569900D0, 3.68574600D0,
     &   3.68578100D0, 3.68580900D0, 3.68583200D0, 3.68585100D0,
     &   3.68586800D0, 3.68588300D0, 3.68589500D0, 3.68590700D0,
     &   3.68591800D0, 3.68592800D0, 3.68593700D0, 3.68594600D0,
     &   3.68595300D0, 3.68596100D0, 3.68596800D0, 3.68597500D0,
     &   3.68598200D0, 3.68598800D0, 3.68599500D0, 3.68600100D0,
     &   3.68600700D0, 3.68601300D0, 3.68602000D0, 3.68602600D0/
       DATA  (ESA(I), I= 701, 800) /
     &   3.68603300D0, 3.68603900D0, 3.68604700D0, 3.68605500D0,
     &   3.68606600D0, 3.68607500D0, 3.68608500D0, 3.68609000D0,
     &   3.68609500D0, 3.68610000D0, 3.68611400D0, 3.68612500D0,
     &   3.68613300D0, 3.68614100D0, 3.68614700D0, 3.68615400D0,
     &   3.68616000D0, 3.68616700D0, 3.68617300D0, 3.68617900D0,
     &   3.68618500D0, 3.68619200D0, 3.68619800D0, 3.68620500D0,
     &   3.68621200D0, 3.68621900D0, 3.68622700D0, 3.68623400D0,
     &   3.68624300D0, 3.68625200D0, 3.68626200D0, 3.68627300D0,
     &   3.68628500D0, 3.68629700D0, 3.68631200D0, 3.68632900D0,
     &   3.68634800D0, 3.68637100D0, 3.68639900D0, 3.68643400D0,
     &   3.68648100D0, 3.68654800D0, 3.68651000D0, 3.68666000D0,
     &   3.68690700D0, 3.68706000D0, 3.68761000D0, 3.68816000D0,
     &   3.68871000D0, 3.68926000D0, 3.68981000D0, 3.69036000D0,
     &   3.69091000D0, 3.69146000D0, 3.69201000D0, 3.69256000D0,
     &   3.69311000D0, 3.69366000D0, 3.69421000D0, 3.69476000D0,
     &   3.69531000D0, 3.69586000D0, 3.69641000D0, 3.69696000D0,
     &   3.72128100D0, 3.73082800D0, 3.73660200D0, 3.74060200D0,
     &   3.74360300D0, 3.74597300D0, 3.74791700D0, 3.74955600D0,
     &   3.75096800D0, 3.75220600D0, 3.75330900D0, 3.75430100D0,
     &   3.75520500D0, 3.75603600D0, 3.75680500D0, 3.75752300D0,
     &   3.75819800D0, 3.75883700D0, 3.75944400D0, 3.76002600D0,
     &   3.76058600D0, 3.76112800D0, 3.76165500D0, 3.76217200D0,
     &   3.76268100D0, 3.76318700D0, 3.76369200D0, 3.76420200D0,
     &   3.76472200D0, 3.76526100D0, 3.76582800D0, 3.76644200D0,
     &   3.76713800D0, 3.76800500D0, 3.77000000D0, 3.77199500D0/
       DATA  (ESA(I), I= 801, 900) /
     &   3.77286200D0, 3.77355800D0, 3.77417200D0, 3.77474000D0,
     &   3.77527800D0, 3.77579800D0, 3.77630800D0, 3.77681400D0,
     &   3.77731900D0, 3.77782800D0, 3.77834500D0, 3.77887200D0,
     &   3.77941400D0, 3.77997400D0, 3.78055600D0, 3.78116300D0,
     &   3.78180200D0, 3.78247700D0, 3.78319500D0, 3.78396400D0,
     &   3.78479500D0, 3.78569900D0, 3.78669100D0, 3.78779400D0,
     &   3.78903200D0, 3.79044400D0, 3.79208300D0, 3.79402700D0,
     &   3.79639700D0, 3.79939800D0, 3.80339800D0, 3.80917200D0,
     &   3.81871900D0, 3.82200000D0, 3.83200000D0, 3.85000000D0,
     &   3.85200000D0, 3.86700000D0, 3.87300000D0, 3.87800000D0,
     &   3.88400000D0, 3.89000000D0, 3.89600000D0, 3.90000000D0,
     &   3.90200000D0, 3.90800000D0, 3.91400000D0, 3.92000000D0,
     &   3.92600000D0, 3.93200000D0, 3.93800000D0, 3.94400000D0,
     &   3.95000000D0, 3.95600000D0, 3.96200000D0, 3.96800000D0,
     &   3.97400000D0, 3.98000000D0, 3.98600000D0, 3.99200000D0,
     &   3.99800000D0, 4.00000000D0, 4.00400000D0, 4.01000000D0,
     &   4.01600000D0, 4.02000000D0, 4.02200000D0, 4.02800000D0,
     &   4.03000000D0, 4.03400000D0, 4.04000000D0, 4.04600000D0,
     &   4.05000000D0, 4.05200000D0, 4.05800000D0, 4.06400000D0,
     &   4.07000000D0, 4.07600000D0, 4.08200000D0, 4.08800000D0,
     &   4.09000000D0, 4.09400000D0, 4.10000000D0, 4.10500000D0,
     &   4.10900000D0, 4.11000000D0, 4.11300000D0, 4.11800000D0,
     &   4.12100000D0, 4.12500000D0, 4.12900000D0, 4.13000000D0,
     &   4.13300000D0, 4.13600000D0, 4.14100000D0, 4.14700000D0,
     &   4.15000000D0, 4.15300000D0, 4.15900000D0, 4.16500000D0/
       DATA  (ESA(I), I= 901,1000) /
     &   4.17100000D0, 4.17700000D0, 4.18000000D0, 4.18200000D0,
     &   4.18500000D0, 4.19000000D0, 4.19500000D0, 4.20000000D0,
     &   4.20100000D0, 4.20800000D0, 4.21200000D0, 4.21400000D0,
     &   4.22000000D0, 4.22600000D0, 4.23200000D0, 4.23800000D0,
     &   4.24000000D0, 4.24400000D0, 4.25000000D0, 4.25600000D0,
     &   4.26200000D0, 4.26800000D0, 4.27400000D0, 4.28000000D0,
     &   4.29200000D0, 4.30000000D0, 4.30400000D0, 4.31000000D0,
     &   4.31600000D0, 4.32800000D0, 4.33000000D0, 4.34000000D0,
     &   4.35000000D0, 4.35200000D0, 4.36400000D0, 4.37600000D0,
     &   4.38000000D0, 4.38200000D0, 4.38800000D0, 4.39000000D0,
     &   4.39400000D0, 4.40000000D0, 4.40600000D0, 4.41000000D0,
     &   4.41200000D0, 4.41800000D0, 4.42400000D0, 4.43000000D0,
     &   4.43600000D0, 4.44000000D0, 4.44200000D0, 4.44800000D0,
     &   4.45000000D0, 4.46000000D0, 4.47200000D0, 4.48400000D0,
     &   4.49600000D0, 4.50450000D0, 4.54000000D0, 4.55000000D0,
     &   4.57000000D0, 4.60000000D0, 4.62000000D0, 4.65000000D0,
     &   4.66000000D0, 4.70000000D0, 4.71000000D0, 4.80000000D0,
     &   4.90000000D0, 4.98000000D0, 5.06500000D0, 5.15000000D0,
     &   5.22500000D0, 5.25000000D0, 5.30000000D0, 5.40000000D0,
     &   5.45000000D0, 5.50000000D0, 5.60000000D0, 5.70000000D0,
     &   5.75000000D0, 5.80000000D0, 5.85000000D0, 5.90000000D0,
     &   5.95000000D0, 6.00000000D0, 6.05000000D0, 6.10000000D0,
     &   6.15000000D0, 6.20000000D0, 6.25000000D0, 6.30000000D0,
     &   6.35000000D0, 6.40000000D0, 6.45000000D0, 6.50000000D0,
     &   6.55000000D0, 6.60000000D0, 6.65000000D0, 6.70000000D0/
       DATA  (ESA(I), I=1001,1100) /
     &   6.75000000D0, 6.80000000D0, 6.85000000D0, 6.90000000D0,
     &   6.95000000D0, 7.00000000D0, 7.05000000D0, 7.10000000D0,
     &   7.15000000D0, 7.20000000D0, 7.25000000D0, 7.30000000D0,
     &   7.35000000D0, 7.40000000D0, 7.44000000D0, 7.50000000D0,
     &   7.60000000D0, 7.70000000D0, 7.80000000D0, 7.90000000D0,
     &   8.00000000D0, 8.10000000D0, 8.20000000D0, 8.30000000D0,
     &   8.40000000D0, 8.50000000D0, 8.60000000D0, 8.70000000D0,
     &   8.80000000D0, 8.90000000D0, 8.91000000D0, 9.00000000D0,
     &   9.10000000D0, 9.20000000D0, 9.28000000D0, 9.30000000D0,
     &   9.35000000D0, 9.39000000D0, 9.40000000D0, 9.41000000D0,
     &   9.41500000D0, 9.42000000D0, 9.43000000D0, 9.44000000D0,
     &   9.44937000D0, 9.44959000D0, 9.44981000D0, 9.45003000D0,
     &   9.45025000D0, 9.45047000D0, 9.45069000D0, 9.45091000D0,
     &   9.45113000D0, 9.45135000D0, 9.45157000D0, 9.45179000D0,
     &   9.45201000D0, 9.45223000D0, 9.45245000D0, 9.45267000D0,
     &   9.45289000D0, 9.45311000D0, 9.45333000D0, 9.45355000D0,
     &   9.45377000D0, 9.45399000D0, 9.45421000D0, 9.45443000D0,
     &   9.45465000D0, 9.45487000D0, 9.45509000D0, 9.45531000D0,
     &   9.45553000D0, 9.45575000D0, 9.45597000D0, 9.45619000D0,
     &   9.45641000D0, 9.45663000D0, 9.45685000D0, 9.45707000D0,
     &   9.45729000D0, 9.45751000D0, 9.45773000D0, 9.45795000D0,
     &   9.45817000D0, 9.45839000D0, 9.45861000D0, 9.45883000D0,
     &   9.45905000D0, 9.45927000D0, 9.45949000D0, 9.45971000D0,
     &   9.45993000D0, 9.46000000D0, 9.46014595D0, 9.46019268D0,
     &   9.46021366D0, 9.46022606D0, 9.46023464D0, 9.46024132D0/
       DATA  (ESA(I), I=1101,1200) /
     &   9.46024704D0, 9.46025085D0, 9.46025467D0, 9.46025848D0,
     &   9.46026039D0, 9.46026325D0, 9.46026516D0, 9.46026802D0,
     &   9.46026897D0, 9.46027088D0, 9.46027279D0, 9.46027470D0,
     &   9.46027565D0, 9.46027660D0, 9.46027851D0, 9.46027946D0,
     &   9.46028042D0, 9.46028233D0, 9.46028328D0, 9.46028423D0,
     &   9.46028519D0, 9.46028709D0, 9.46028805D0, 9.46028900D0,
     &   9.46029091D0, 9.46029186D0, 9.46029377D0, 9.46029568D0,
     &   9.46030045D0, 9.46030426D0, 9.46030617D0, 9.46030807D0,
     &   9.46030998D0, 9.46031094D0, 9.46031189D0, 9.46031284D0,
     &   9.46031475D0, 9.46031570D0, 9.46031666D0, 9.46031761D0,
     &   9.46031952D0, 9.46032047D0, 9.46032143D0, 9.46032333D0,
     &   9.46032429D0, 9.46032619D0, 9.46032715D0, 9.46032906D0,
     &   9.46033096D0, 9.46033287D0, 9.46033478D0, 9.46033669D0,
     &   9.46033955D0, 9.46034145D0, 9.46034527D0, 9.46034908D0,
     &   9.46035290D0, 9.46035862D0, 9.46036530D0, 9.46037388D0,
     &   9.46038628D0, 9.46040726D0, 9.46045494D0, 9.46059000D0,
     &   9.46081000D0, 9.46103000D0, 9.46125000D0, 9.46147000D0,
     &   9.46169000D0, 9.46191000D0, 9.46213000D0, 9.46235000D0,
     &   9.46257000D0, 9.46279000D0, 9.46301000D0, 9.46323000D0,
     &   9.46345000D0, 9.46367000D0, 9.46389000D0, 9.46411000D0,
     &   9.46433000D0, 9.46455000D0, 9.46477000D0, 9.46499000D0,
     &   9.46521000D0, 9.46543000D0, 9.46565000D0, 9.46587000D0,
     &   9.46609000D0, 9.46631000D0, 9.46653000D0, 9.46675000D0,
     &   9.46697000D0, 9.46719000D0, 9.46741000D0, 9.46763000D0,
     &   9.46785000D0, 9.46807000D0, 9.46829000D0, 9.46851000D0/
       DATA  (ESA(I), I=1201,1300) /
     &   9.46873000D0, 9.46895000D0, 9.46917000D0, 9.46939000D0,
     &   9.46961000D0, 9.46983000D0, 9.47005000D0, 9.47027000D0,
     &   9.47049000D0, 9.47071000D0, 9.47093000D0, 9.47115000D0,
     &   9.47137000D0, 9.48000000D0, 9.48400000D0, 9.49000000D0,
     &   9.51000000D0, 9.60000000D0, 9.70000000D0, 9.80000000D0,
     &   9.90000000D0,10.00000000D0,10.01226000D0,10.01281000D0,
     &  10.01336000D0,10.01391000D0,10.01446000D0,10.01501000D0,
     &  10.01556000D0,10.01611000D0,10.01666000D0,10.01721000D0,
     &  10.01776000D0,10.01831000D0,10.01886000D0,10.01941000D0,
     &  10.01996000D0,10.02051000D0,10.02106000D0,10.02161000D0,
     &  10.02216000D0,10.02250000D0,10.02253750D0,10.02257500D0,
     &  10.02261250D0,10.02265000D0,10.02268750D0,10.02272500D0,
     &  10.02276250D0,10.02280000D0,10.02283750D0,10.02287500D0,
     &  10.02291250D0,10.02295000D0,10.02298750D0,10.02302500D0,
     &  10.02306250D0,10.02310000D0,10.02313750D0,10.02317500D0,
     &  10.02321250D0,10.02322000D0,10.02322500D0,10.02323000D0,
     &  10.02323400D0,10.02323700D0,10.02324000D0,10.02324200D0,
     &  10.02324400D0,10.02324600D0,10.02324800D0,10.02324930D0,
     &  10.02324950D0,10.02324960D0,10.02324980D0,10.02324988D0,
     &  10.02324994D0,10.02325000D0,10.02325060D0,10.02325150D0,
     &  10.02325250D0,10.02325340D0,10.02325440D0,10.02325540D0,
     &  10.02325630D0,10.02325680D0,10.02325730D0,10.02325780D0,
     &  10.02325850D0,10.02325900D0,10.02325950D0,10.02325980D0,
     &  10.02325990D0,10.02326000D0,10.02326010D0,10.02326050D0,
     &  10.02326100D0,10.02326150D0,10.02326390D0,10.02326490D0/
       DATA  (ESA(I), I=1301,1400) /
     &  10.02326680D0,10.02326770D0,10.02326870D0,10.02326970D0,
     &  10.02327060D0,10.02327160D0,10.02327250D0,10.02327350D0,
     &  10.02327440D0,10.02327540D0,10.02327630D0,10.02327730D0,
     &  10.02327820D0,10.02328010D0,10.02328110D0,10.02328210D0,
     &  10.02328300D0,10.02328490D0,10.02328590D0,10.02328750D0,
     &  10.02332500D0,10.02336250D0,10.02340000D0,10.02343750D0,
     &  10.02347500D0,10.02351250D0,10.02355000D0,10.02358750D0,
     &  10.02362500D0,10.02366250D0,10.02370000D0,10.02373750D0,
     &  10.02377500D0,10.02381250D0,10.02385000D0,10.02388750D0,
     &  10.02392500D0,10.02396250D0,10.02400000D0,10.02436000D0,
     &  10.02491000D0,10.02546000D0,10.02601000D0,10.02656000D0,
     &  10.02711000D0,10.02766000D0,10.02821000D0,10.02876000D0,
     &  10.02931000D0,10.02986000D0,10.03041000D0,10.03096000D0,
     &  10.03151000D0,10.03206000D0,10.03261000D0,10.03316000D0,
     &  10.03371000D0,10.03426000D0,10.04000000D0,10.10000000D0,
     &  10.20000000D0,10.30000000D0,10.34420000D0,10.34493333D0,
     &  10.34566667D0,10.34640000D0,10.34713333D0,10.34786667D0,
     &  10.34860000D0,10.34933333D0,10.35006667D0,10.35080000D0,
     &  10.35153333D0,10.35226667D0,10.35300000D0,10.35309167D0,
     &  10.35318333D0,10.35327500D0,10.35336667D0,10.35345833D0,
     &  10.35355000D0,10.35364167D0,10.35373333D0,10.35382500D0,
     &  10.35391667D0,10.35400833D0,10.35410000D0,10.35419167D0,
     &  10.35428333D0,10.35437500D0,10.34420000D0,10.35446667D0,
     &  10.35455833D0,10.35465000D0,10.35474167D0,10.35483334D0,
     &  10.35492500D0,10.35501667D0,10.35510834D0,10.35512330D0/
       DATA  (ESA(I), I=1401,1500) /
     &  10.35514660D0,10.35515710D0,10.35516340D0,10.35516780D0,
     &  10.35517110D0,10.35517370D0,10.35517580D0,10.35517760D0,
     &  10.35517920D0,10.35518060D0,10.35518180D0,10.35518290D0,
     &  10.35518390D0,10.35518480D0,10.35518570D0,10.35518650D0,
     &  10.35518720D0,10.35518790D0,10.35518860D0,10.35518930D0,
     &  10.35518990D0,10.35519050D0,10.35519110D0,10.35519170D0,
     &  10.35519230D0,10.35519280D0,10.35519340D0,10.35519400D0,
     &  10.35519460D0,10.35519530D0,10.35519600D0,10.35519680D0,
     &  10.35519770D0,10.35520000D0,10.35520230D0,10.35520320D0,
     &  10.35520400D0,10.35520470D0,10.35520540D0,10.35520600D0,
     &  10.35520660D0,10.35520720D0,10.35520770D0,10.35520830D0,
     &  10.35520890D0,10.35520950D0,10.35521010D0,10.35521070D0,
     &  10.35521140D0,10.35521210D0,10.35521280D0,10.35521350D0,
     &  10.35521430D0,10.35521520D0,10.35521610D0,10.35521710D0,
     &  10.35521820D0,10.35521940D0,10.35522080D0,10.35522240D0,
     &  10.35522420D0,10.35522630D0,10.35522890D0,10.35523220D0,
     &  10.35523660D0,10.35524290D0,10.35525340D0,10.35527670D0,
     &  10.35529167D0,10.35538334D0,10.35547500D0,10.35556667D0,
     &  10.35565834D0,10.35575000D0,10.35584167D0,10.35593334D0,
     &  10.35602500D0,10.35611667D0,10.35620834D0,10.35630000D0,
     &  10.35639167D0,10.35648334D0,10.35657500D0,10.35666667D0,
     &  10.35740000D0,10.35813333D0,10.35886667D0,10.35960000D0,
     &  10.36033333D0,10.36106667D0,10.36180000D0,10.36253333D0,
     &  10.36326667D0,10.36400000D0,10.36473333D0,10.36546667D0,
     &  10.36620000D0,10.40000000D0,10.47300000D0,10.52897000D0/
       DATA  (ESA(I), I=1501,1600) /
     &  10.54217000D0,10.54889000D0,10.55317000D0,10.55622000D0,
     &  10.55855000D0,10.56042000D0,10.56197000D0,10.56328000D0,
     &  10.56442000D0,10.56543000D0,10.56633000D0,10.56714000D0,
     &  10.56788000D0,10.56857000D0,10.56921000D0,10.56980000D0,
     &  10.57036000D0,10.57090000D0,10.57141000D0,10.57189000D0,
     &  10.57237000D0,10.57282000D0,10.57327000D0,10.57371000D0,
     &  10.57415000D0,10.57459000D0,10.57503000D0,10.57547000D0,
     &  10.57594000D0,10.57643000D0,10.57695000D0,10.57755000D0,
     &  10.57829000D0,10.58000000D0,10.58171000D0,10.58245000D0,
     &  10.58305000D0,10.58357000D0,10.58406000D0,10.58453000D0,
     &  10.58497000D0,10.58541000D0,10.58585000D0,10.58629000D0,
     &  10.58673000D0,10.58718000D0,10.58763000D0,10.58811000D0,
     &  10.58859000D0,10.58910000D0,10.58964000D0,10.59020000D0,
     &  10.59079000D0,10.59143000D0,10.59212000D0,10.59286000D0,
     &  10.59367000D0,10.59457000D0,10.59558000D0,10.59671000D0,
     &  10.59803000D0,10.59958000D0,10.60145000D0,10.60378000D0,
     &  10.60683000D0,10.61111000D0,10.61783000D0,10.63103000D0,
     &  10.68700000D0,10.69000000D0,10.71309000D0,10.72975000D0,
     &  10.74256000D0,10.75286000D0,10.76141000D0,10.76869000D0,
     &  10.77500000D0,10.78057000D0,10.78554000D0,10.79004000D0,
     &  10.79414000D0,10.79791000D0,10.80141000D0,10.80468000D0,
     &  10.80775000D0,10.81065000D0,10.81341000D0,10.81604000D0,
     &  10.81858000D0,10.82102000D0,10.82339000D0,10.82570000D0,
     &  10.82796000D0,10.83019000D0,10.83240000D0,10.83460000D0,
     &  10.83681000D0,10.83906000D0,10.84135000D0,10.84374000D0/
       DATA  (ESA(I), I=1601,1700) /
     &  10.84626000D0,10.84900000D0,10.85212000D0,10.85601000D0,
     &  10.86500000D0,10.87288000D0,10.87627000D0,10.87895000D0,
     &  10.88128000D0,10.88340000D0,10.88539000D0,10.88728000D0,
     &  10.88910000D0,10.89088000D0,10.89262000D0,10.89434000D0,
     &  10.89605000D0,10.89775000D0,10.89946000D0,10.90119000D0,
     &  10.90292000D0,10.90468000D0,10.90648000D0,10.90830000D0,
     &  10.91017000D0,10.91208000D0,10.91406000D0,10.91609000D0,
     &  10.91820000D0,10.92038000D0,10.92266000D0,10.92504000D0,
     &  10.92754000D0,10.93017000D0,10.93295000D0,10.93590000D0,
     &  10.93905000D0,10.94242000D0,10.94606000D0,10.95000000D0,
     &  10.97500000D0,10.97638000D0,10.97771000D0,10.97900000D0,
     &  10.98025000D0,10.98146000D0,10.98263000D0,10.98377000D0,
     &  10.98489000D0,10.98598000D0,10.98705000D0,10.98810000D0,
     &  10.98913000D0,10.99014000D0,10.99114000D0,10.99213000D0,
     &  10.99310000D0,10.99407000D0,10.99504000D0,10.99600000D0,
     &  10.99697000D0,10.99793000D0,10.99890000D0,10.99988000D0,
     &  11.00087000D0,11.00188000D0,11.00290000D0,11.00396000D0,
     &  11.00506000D0,11.00621000D0,11.00742000D0,11.00873000D0,
     &  11.01018000D0,11.01186000D0,11.01399000D0,11.01900000D0,
     &  11.02401000D0,11.02614000D0,11.02782000D0,11.02927000D0,
     &  11.03058000D0,11.03179000D0,11.03294000D0,11.03404000D0,
     &  11.03510000D0,11.03612000D0,11.03713000D0,11.03812000D0,
     &  11.03910000D0,11.04007000D0,11.04103000D0,11.04200000D0,
     &  11.04296000D0,11.04392000D0,11.04490000D0,11.04587000D0,
     &  11.04686000D0,11.04786000D0,11.04887000D0,11.04990000D0/
       DATA  (ESA(I), I=1701,1800) /
     &  11.05095000D0,11.05202000D0,11.05311000D0,11.05423000D0,
     &  11.05537000D0,11.05654000D0,11.05775000D0,11.05900000D0,
     &  11.06029000D0,11.06162000D0,11.06300000D0,11.07000000D0,
     &  11.09200000D0,11.11500000D0,11.13700000D0,11.16000000D0,
     &  11.18200000D0,11.20500000D0,11.22700000D0,11.25000000D0,
     &  11.27300000D0,11.29500000D0,11.31800000D0,11.34000000D0,
     &  11.50000000D0,11.65500000D0,11.67700000D0,11.70000000D0,
     &  11.72200000D0,11.74500000D0,11.76700000D0,11.79000000D0,
     &  11.81200000D0,11.83500000D0,11.85800000D0,11.88000000D0,
     &  11.90300000D0,11.92500000D0,11.94800000D0,11.97000000D0,
     &  11.99300000D0,12.01500000D0,12.03800000D0,12.06000000D0,
     &  12.08300000D0,12.10500000D0,12.12700000D0,12.15000000D0,
     &  12.17200000D0,12.19500000D0,12.21700000D0,12.24000000D0,
     &  12.26200000D0,12.28500000D0,12.30700000D0,12.33000000D0,
     &  12.35200000D0,12.37500000D0,12.39800000D0,12.42000000D0,
     &  12.44300000D0,12.46500000D0,12.48800000D0,12.51000000D0,
     &  12.53300000D0,12.55500000D0,12.57800000D0,12.60000000D0,
     &  12.62300000D0,12.64500000D0,12.66700000D0,12.69000000D0,
     &  12.71200000D0,12.73500000D0,12.75700000D0,12.78000000D0,
     &  12.80200000D0,12.82500000D0,12.84700000D0,12.87000000D0,
     &  12.89200000D0,12.91500000D0,12.93800000D0,12.96000000D0,
     &  12.98300000D0,13.00500000D0,13.02800000D0,13.05000000D0,
     &  13.07300000D0,13.09500000D0,13.11800000D0,13.14000000D0,
     &  13.16300000D0,13.18500000D0,13.20800000D0,13.23000000D0,
     &  13.25200000D0,13.27500000D0,13.29700000D0,13.32000000D0/
       DATA  (ESA(I), I=1801,1840) /
     &  13.34200000D0,13.36500000D0,13.38700000D0,13.41000000D0,
     &  13.43200000D0,13.45500000D0,13.47700000D0,13.50000000D0,
     &  13.75000000D0,14.00000000D0,14.04000000D0,16.00000000D0,
     &  17.00000000D0,20.00000000D0,21.95000000D0,22.05000000D0,
     &  25.00000000D0,25.01000000D0,27.50000000D0,27.60000000D0,
     &  27.66000000D0,29.00000000D0,29.93000000D0,30.10000000D0,
     &  30.38000000D0,30.80000000D0,31.10000000D0,31.29000000D0,
     &  33.20000000D0,33.30000000D0,33.89000000D0,34.00000000D0,
     &  34.50000000D0,34.70000000D0,34.86000000D0,35.01000000D0,
     &  35.45000000D0,36.10000000D0,36.38000000D0,38.29000000D0/
  
         DATA ((DAS(I,J), I=   1, 100), J=1,1) /
     &  -9.22013843E-04,-9.79581497E-04,-1.03638394E-03,-1.09236509E-03,
     &  -1.14737291E-03,-1.19286012E-03,-1.32504637E-03,-1.33921767E-03,
     &  -1.38070195E-03,-1.43257445E-03,-1.48585439E-03,-1.54084825E-03,
     &  -1.60726052E-03,-1.65044414E-03,-1.70283368E-03,-1.75552372E-03,
     &  -1.80817575E-03,-1.86028521E-03,-1.90170150E-03,-1.95654381E-03,
     &  -2.00214947E-03,-2.06094559E-03,-2.10953470E-03,-2.17166521E-03,
     &  -2.22108411E-03,-2.27060158E-03,-2.33542768E-03,-2.37075247E-03,
     &  -2.38732920E-03,-2.43880462E-03,-2.49049875E-03,-2.54335136E-03,
     &  -2.59764058E-03,-2.65119719E-03,-2.70540816E-03,-2.76153420E-03,
     &  -2.82034975E-03,-2.88232458E-03,-2.94801200E-03,-3.01769371E-03,
     &  -3.07031707E-03,-3.14175407E-03,-3.21557755E-03,-3.27215893E-03,
     &  -3.34857773E-03,-3.42378171E-03,-3.47962014E-03,-3.55436445E-03,
     &  -3.61108330E-03,-3.68706051E-03,-4.08767946E-03,-4.56272187E-03,
     &  -5.26402216E-03,-6.09468703E-03,-6.40424941E-03,-6.48422490E-03,
     &  -6.73010684E-03,-7.06590772E-03,-7.40342156E-03,-7.74688160E-03,
     &  -8.09839041E-03,-8.23373674E-03,-8.46479065E-03,-8.83877676E-03,
     &  -9.15017455E-03,-9.47724748E-03,-9.72692365E-03,-9.94936558E-03,
     &  -9.98955616E-03,-1.00615130E-02,-1.00895747E-02,-1.00865494E-02,
     &  -9.98689653E-03,-9.72618001E-03,-9.31346846E-03,-8.74325167E-03,
     &  -8.00313226E-03,-7.09253473E-03,-6.25068108E-03,-5.99071710E-03,
     &  -5.21501035E-03,-4.93587376E-03,-4.70465435E-03,-4.58983000E-03,
     &  -4.55267302E-03,-4.78844167E-03,-5.09856755E-03,-5.39047666E-03,
     &  -5.65320300E-03,-5.88215960E-03,-6.07387092E-03,-6.21704323E-03,
     &  -6.31255240E-03,-6.35437699E-03,-6.33796297E-03,-6.22033859E-03,
     &  -6.16252254E-03,-6.02267689E-03,-5.75374855E-03,-5.47477009E-03/
  
         DATA ((DAS(I,J), I= 101, 200), J=1,1) /
     &  -5.20979136E-03,-4.95464456E-03,-4.68904252E-03,-4.41163014E-03,
     &  -4.09656300E-03,-3.73921403E-03,-3.33302219E-03,-2.87351231E-03,
     &  -2.34914483E-03,-1.75477178E-03,-1.08458684E-03,-3.21988388E-04,
     &   5.49187713E-04, 1.56142958E-03, 2.76106025E-03, 4.24700796E-03,
     &   6.25416046E-03, 1.12795675E-02, 1.61474292E-02, 1.80186154E-02,
     &   1.93691433E-02, 2.04277721E-02, 2.12944718E-02, 2.20195696E-02,
     &   2.26263898E-02, 2.31385159E-02, 2.35696752E-02, 2.39286286E-02,
     &   2.42174820E-02, 2.44401572E-02, 2.45972069E-02, 2.46842822E-02,
     &   2.46955182E-02, 2.46241930E-02, 2.44640784E-02, 2.42211330E-02,
     &   2.39133483E-02, 2.35950337E-02, 2.34128400E-02, 2.32120957E-02,
     &   2.29914445E-02, 2.26283516E-02, 2.22247924E-02, 2.17744116E-02,
     &   2.12798646E-02, 2.07316187E-02, 2.01261415E-02, 1.94577639E-02,
     &   1.87182050E-02, 1.78911498E-02, 1.69592597E-02, 1.58923148E-02,
     &   1.56358629E-02, 1.49449436E-02, 1.44004798E-02, 1.39791585E-02,
     &   1.36561250E-02, 1.33829891E-02, 1.34309767E-02, 1.35300695E-02,
     &   1.35324046E-02, 1.34252500E-02, 1.33428450E-02, 1.32434493E-02,
     &   1.31313951E-02, 1.30093841E-02, 1.28833933E-02, 1.28200368E-02,
     &   1.24463608E-02, 1.21049225E-02, 1.17872278E-02, 1.15334472E-02,
     &   1.12343175E-02, 1.09203682E-02, 1.05722588E-02, 1.02536681E-02,
     &   9.85032878E-03, 9.43534250E-03, 9.09731992E-03, 8.90276401E-03,
     &   8.56107474E-03, 8.41039353E-03, 8.05537138E-03, 7.75898225E-03,
     &   7.39079404E-03, 7.06311277E-03, 6.63860172E-03, 6.25944436E-03,
     &   5.85667091E-03, 5.33950888E-03, 5.06966757E-03, 4.88565709E-03,
     &   4.50900828E-03, 4.30149819E-03, 4.09993344E-03, 3.88179166E-03,
     &   3.64411237E-03, 3.39020166E-03, 3.12247181E-03, 2.54834227E-03/
  
         DATA ((DAS(I,J), I= 201, 300), J=1,1) /
     &   1.90289132E-03, 1.53540191E-03, 1.39821831E-03, 1.25636440E-03,
     &   1.10954664E-03, 9.57473439E-04, 7.99889156E-04, 6.36564675E-04,
     &   4.67227490E-04, 2.91662496E-04, 1.09826209E-04,-7.83588180E-05,
     &  -2.72825601E-04,-4.73736211E-04,-6.81987783E-04,-8.97468508E-04,
     &  -1.11998445E-03,-1.34980387E-03,-1.58682673E-03,-1.83163321E-03,
     &  -2.08515054E-03,-2.34792723E-03,-2.62097921E-03,-2.90557167E-03,
     &  -3.20332263E-03,-3.51723444E-03,-3.85104722E-03,-4.20913033E-03,
     &  -4.59683433E-03,-5.02081936E-03,-5.20307268E-03,-5.21338809E-03,
     &  -5.56601205E-03,-6.15240673E-03,-6.74066624E-03,-7.35751757E-03,
     &  -8.01369323E-03,-8.71725712E-03,-9.47607932E-03,-1.02993499E-02,
     &  -1.11967497E-02,-1.21794714E-02,-1.32608079E-02,-1.44561203E-02,
     &  -1.57860347E-02,-1.72753271E-02,-1.89520232E-02,-2.08511507E-02,
     &  -2.30152441E-02,-2.54965461E-02,-2.83605392E-02,-3.16791633E-02,
     &  -3.55193354E-02,-3.99159653E-02,-4.12684117E-02,-4.26562185E-02,
     &  -4.40717460E-02,-4.55041686E-02,-4.69412321E-02,-4.83532574E-02,
     &  -4.97021469E-02,-5.09502936E-02,-5.20432570E-02,-5.29090771E-02,
     &  -5.34548823E-02,-5.35635302E-02,-5.30909400E-02,-5.18659182E-02,
     &  -4.96948810E-02,-4.63741850E-02,-4.17135512E-02,-3.55715001E-02,
     &  -2.79010481E-02,-1.87960905E-02,-8.52112647E-03, 2.49240961E-03,
     &   1.36958661E-02, 2.45096786E-02, 3.44219434E-02, 4.30538416E-02,
     &   5.01992486E-02, 5.58228852E-02, 6.00110729E-02, 6.29273834E-02,
     &   6.47694355E-02, 6.57385570E-02, 6.60205694E-02, 6.57766356E-02,
     &   6.51408028E-02, 6.42209513E-02, 6.31021443E-02, 6.18501515E-02,
     &   6.05151823E-02, 5.91349590E-02, 5.77375849E-02, 5.63436342E-02,
     &   5.49680708E-02, 5.36216724E-02, 5.23131795E-02, 5.10447190E-02/
  
         DATA ((DAS(I,J), I= 301, 400), J=1,1) /
     &   4.98165269E-02, 4.86316799E-02, 4.74909966E-02, 4.63944276E-02,
     &   4.28902857E-02, 3.98600311E-02, 3.72349194E-02, 3.49489292E-02,
     &   3.29472550E-02, 3.11855369E-02, 2.96250900E-02, 2.82339664E-02,
     &   2.69862966E-02, 2.58604367E-02, 2.48398990E-02, 2.39111302E-02,
     &   2.30614668E-02, 2.22805620E-02, 2.15595661E-02, 2.08909024E-02,
     &   2.02691566E-02, 1.96873404E-02, 1.91404986E-02, 1.86232822E-02,
     &   1.81313042E-02, 1.79566473E-02, 1.79309796E-02, 1.76085110E-02,
     &   1.71538102E-02, 1.67520375E-02, 1.63841033E-02, 1.60472931E-02,
     &   1.57324497E-02, 1.54382625E-02, 1.51627454E-02, 1.49036608E-02,
     &   1.46594341E-02, 1.44285858E-02, 1.42098398E-02, 1.40024028E-02,
     &   1.38055443E-02, 1.36186790E-02, 1.34402556E-02, 1.32702119E-02,
     &   1.31078599E-02, 1.29527279E-02, 1.28043643E-02, 1.26624532E-02,
     &   1.25262118E-02, 1.23952260E-02, 1.22695128E-02, 1.21487354E-02,
     &   1.20324065E-02, 1.19203095E-02, 1.18119579E-02, 1.16708589E-02,
     &   1.13901536E-02, 1.07801657E-02, 9.81813890E-03, 9.06739764E-03,
     &   8.51058610E-03, 8.01063215E-03, 7.50818160E-03, 7.10291560E-03,
     &   6.71908986E-03, 6.67950984E-03, 6.31451473E-03, 6.04230394E-03,
     &   5.95139969E-03, 5.73307810E-03, 5.52067880E-03, 5.49993062E-03,
     &   5.31633090E-03, 5.12069500E-03, 5.02752745E-03, 4.94402136E-03,
     &   4.79931127E-03, 4.67492693E-03, 4.55978757E-03, 4.43924813E-03,
     &   4.36481674E-03, 4.31405789E-03, 4.18514412E-03, 4.05827637E-03,
     &   4.04665983E-03, 3.94529093E-03, 3.85015935E-03, 3.80772533E-03,
     &   3.76229336E-03, 3.67176675E-03, 3.58657585E-03, 3.49863595E-03,
     &   3.47389311E-03, 3.41497304E-03, 3.36944258E-03, 3.32596920E-03,
     &   3.26590411E-03, 3.23645520E-03, 3.23468749E-03, 3.24929354E-03/
  
         DATA ((DAS(I,J), I= 401, 500), J=1,1) /
     &   3.29885836E-03, 3.22264320E-03, 3.12296396E-03, 3.01941921E-03,
     &   2.98682395E-03, 2.88294201E-03, 2.89067393E-03, 2.89791191E-03,
     &   2.91078879E-03, 2.95336669E-03, 3.01357039E-03, 3.07925371E-03,
     &   3.08652093E-03, 3.15401872E-03, 3.16217253E-03, 3.20779645E-03,
     &   3.25018751E-03, 3.42601963E-03, 3.45681674E-03, 3.66853393E-03,
     &   3.68364104E-03, 3.69841370E-03, 3.83574619E-03, 3.93837153E-03,
     &   3.95284220E-03, 4.01004110E-03, 4.01164711E-03, 4.00902487E-03,
     &   4.00677645E-03, 3.99259389E-03, 3.98753968E-03, 3.97929862E-03,
     &   3.98192421E-03, 3.97469533E-03, 3.95477242E-03, 3.95276437E-03,
     &   3.95238254E-03, 3.95356676E-03, 3.95880741E-03, 4.02310527E-03,
     &   4.09859297E-03, 4.13206871E-03, 4.15440945E-03, 4.16559680E-03,
     &   4.27875969E-03, 4.29934984E-03, 4.35143153E-03, 4.39481306E-03,
     &   4.41763749E-03, 4.50510205E-03, 4.61276934E-03, 4.63703567E-03,
     &   4.64964228E-03, 4.73108238E-03, 4.77345965E-03, 4.81705221E-03,
     &   4.99795371E-03, 5.01865071E-03, 5.06437162E-03, 5.07974141E-03,
     &   5.07170325E-03, 5.06840363E-03, 5.05739334E-03, 5.04941613E-03,
     &   5.04121779E-03, 5.02105201E-03, 5.00321239E-03, 5.01256325E-03,
     &   5.06037262E-03, 5.11312924E-03, 5.11837200E-03, 5.14204261E-03,
     &   5.13000069E-03, 5.13934741E-03, 5.14335159E-03, 5.16805054E-03,
     &   5.16950856E-03, 5.17524929E-03, 5.18988733E-03, 5.12715507E-03,
     &   5.10224256E-03, 4.84587077E-03, 4.71589597E-03, 4.62420605E-03,
     &   4.41310797E-03, 4.28932935E-03, 4.27705996E-03, 4.26674778E-03,
     &   4.24232284E-03, 4.23621569E-03, 4.20824045E-03, 4.19918141E-03,
     &   4.16028886E-03, 4.10433492E-03, 4.04577345E-03, 4.02071336E-03,
     &   3.68104247E-03, 3.45281345E-03, 3.44262393E-03, 3.77060245E-03/
  
         DATA ((DAS(I,J), I= 501, 600), J=1,1) /
     &   4.32091269E-03, 4.60460173E-03, 4.93444233E-03, 5.03984784E-03,
     &   5.07351044E-03, 4.99762567E-03, 4.88600099E-03, 4.71237466E-03,
     &   4.50622689E-03, 4.30063683E-03, 4.05891714E-03, 3.75363888E-03,
     &   3.35718930E-03, 2.82405459E-03, 2.07294651E-03, 9.43280986E-04,
     &  -9.37750726E-04,-2.81980226E-03,-1.34493663E-02,-8.94837629E-02,
     &  -9.91203975E-02,-1.10919677E-01,-1.25701907E-01,-1.44762288E-01,
     &  -1.70272684E-01,-2.06174410E-01,-2.60440853E-01,-3.52003694E-01,
     &  -5.39416898E-01,-1.13773820E+00,-1.20311776E+00,-1.27640098E+00,
     &  -1.35910779E+00,-1.45317285E+00,-1.56109654E+00,-1.68616674E+00,
     &  -1.83279305E+00,-2.00702518E+00,-2.21738478E+00,-2.47625228E+00,
     &  -2.80228489E+00,-3.22485534E+00,-3.79269116E+00,-3.86029691E+00,
     &  -5.37025707E+00,-6.46918617E+00,-7.35452384E+00,-8.12948579E+00,
     &  -8.73262978E+00,-9.26675643E+00,-9.70240402E+00,-1.01472062E+01,
     &  -1.04629341E+01,-1.07681742E+01,-1.09972696E+01,-1.12045372E+01,
     &  -1.13396415E+01,-1.14825590E+01,-1.15337972E+01,-1.15790950E+01,
     &  -1.15787191E+01,-1.15482260E+01,-1.14489414E+01,-1.13418437E+01,
     &  -1.11975766E+01,-1.10121535E+01,-1.07815028E+01,-1.05015604E+01,
     &  -1.01683872E+01,-9.77831399E+00,-9.32811236E+00,-8.81518909E+00,
     &  -8.23779668E+00,-7.24967310E+00,-6.51094564E+00,-5.28997446E+00,
     &  -3.94882117E+00, 4.73510750E-03, 3.95838269E+00, 5.29961186E+00,
     &   6.52067586E+00, 7.25947238E+00, 8.24770701E+00, 8.82517658E+00,
     &   9.33817820E+00, 9.78845836E+00, 1.01786095E+01, 1.05118593E+01,
     &   1.07918763E+01, 1.10225993E+01, 1.12080922E+01, 1.13524261E+01,
     &   1.14595877E+01, 1.15589622E+01, 1.15895115E+01, 1.15899660E+01,
     &   1.15447403E+01, 1.14935468E+01, 1.13507110E+01, 1.12156616E+01/
  
         DATA ((DAS(I,J), I= 601, 700), J=1,1) /
     &   1.10084601E+01, 1.07794231E+01, 1.04742470E+01, 1.01585742E+01,
     &   9.71383695E+00, 9.27824288E+00, 8.74417193E+00, 8.14108099E+00,
     &   7.36617558E+00, 6.48089023E+00, 5.38201199E+00, 4.41904050E+00,
     &   3.67594507E+00, 3.14248452E+00, 2.74233152E+00, 2.43165577E+00,
     &   2.18373836E+00, 1.98144761E+00, 1.81332585E+00, 1.67143437E+00,
     &   1.55010557E+00, 1.44518850E+00, 1.35357518E+00, 1.27289344E+00,
     &   1.20130193E+00, 1.13734987E+00, 1.07987897E+00, 1.02795337E+00,
     &   9.80808788E-01, 5.15951091E-01, 3.51204831E-01, 2.66890522E-01,
     &   2.15666179E-01, 1.81247900E-01, 1.56530370E-01, 1.37918728E-01,
     &   1.23399028E-01, 1.11755281E-01, 1.02209837E-01, 2.56289261E-02,
     &   1.23982562E-02, 1.05747791E-02, 8.62495844E-03, 7.91123129E-03,
     &   7.03778737E-03, 5.79491695E-03, 4.23137812E-03, 3.49992850E-03,
     &   6.91345904E-04,-3.63324035E-03,-8.13508799E-03,-1.80673682E-02,
     &  -2.95184014E-02,-3.14448142E-02,-3.35869092E-02,-3.59776792E-02,
     &  -3.86611886E-02,-4.16949790E-02,-4.51527357E-02,-4.91303576E-02,
     &  -5.37550473E-02,-5.91991553E-02,-6.57021929E-02,-7.36070987E-02,
     &  -8.34225877E-02,-9.59371667E-02,-1.12441735E-01,-1.35204242E-01,
     &  -1.68606062E-01,-2.22330123E-01,-3.22701011E-01,-4.81446756E-01,
     &  -5.72494120E-01,-6.72592009E-01,-9.23664279E-01,-1.01654096E+00,
     &  -1.09471522E+00,-1.16253575E+00,-1.22102867E+00,-1.27033737E+00,
     &  -1.31418120E+00,-1.35156094E+00,-1.37967634E+00,-1.40523594E+00,
     &  -1.42543387E+00,-1.44014911E+00,-1.44955494E+00,-1.45441285E+00,
     &  -1.45439727E+00,-1.44955859E+00,-1.44043381E+00,-1.42606163E+00,
     &  -1.40574495E+00,-1.38304632E+00,-1.34973871E+00,-1.31480181E+00,
     &  -1.27348946E+00,-1.22538038E+00,-1.16017209E+00,-1.09616272E+00/
  
         DATA ((DAS(I,J), I= 701, 800), J=1,1) /
     &  -1.01175630E+00,-9.31010675E-01,-8.11503285E-01,-6.79156309E-01,
     &  -4.78719664E-01,-3.02134387E-01,-9.74883493E-02, 6.39746325E-03,
     &   1.10284574E-01, 2.13387513E-01, 4.91542491E-01, 6.92008307E-01,
     &   8.24381319E-01, 9.43917734E-01, 1.02468653E+00, 1.10912088E+00,
     &   1.17315466E+00, 1.23839141E+00, 1.28652468E+00, 1.32786072E+00,
     &   1.36282079E+00, 1.39615436E+00, 1.41887435E+00, 1.43921472E+00,
     &   1.45360931E+00, 1.46275511E+00, 1.46761614E+00, 1.46764979E+00,
     &   1.46281328E+00, 1.45342686E+00, 1.43873077E+00, 1.41855151E+00,
     &   1.39300952E+00, 1.36490901E+00, 1.32754466E+00, 1.28371420E+00,
     &   1.23441613E+00, 1.17593049E+00, 1.10811123E+00, 1.02992660E+00,
     &   9.37007545E-01, 8.26363410E-01, 8.86195941E-01, 6.86057633E-01,
     &   4.95102392E-01, 4.21570906E-01, 2.74824347E-01, 2.04438913E-01,
     &   1.63266697E-01, 1.36276295E-01, 1.17225383E-01, 1.03066914E-01,
     &   9.21324650E-02, 8.34331773E-02, 7.63475307E-02, 7.04649167E-02,
     &   6.55031959E-02, 6.12619279E-02, 5.75951851E-02, 5.43939142E-02,
     &   5.15750478E-02, 4.90716869E-02, 4.68325164E-02, 4.48218888E-02,
     &   1.81292380E-02, 1.53377486E-02, 1.40596302E-02, 1.32896796E-02,
     &   1.27544710E-02, 1.23521310E-02, 1.20340853E-02, 1.17743984E-02,
     &   1.15576033E-02, 1.13738056E-02, 1.12161322E-02, 1.10804021E-02,
     &   1.09629067E-02, 1.08612758E-02, 1.07738006E-02, 1.06989238E-02,
     &   1.06355721E-02, 1.05828999E-02, 1.05404126E-02, 1.05075125E-02,
     &   1.04839899E-02, 1.04696701E-02, 1.04645209E-02, 1.04686414E-02,
     &   1.04822906E-02, 1.05059605E-02, 1.05402322E-02, 1.05861707E-02,
     &   1.06451610E-02, 1.07195475E-02, 1.08124162E-02, 1.09295744E-02,
     &   1.10821400E-02, 1.12982442E-02, 1.18683934E-02, 1.24476278E-02/
  
         DATA ((DAS(I,J), I= 801, 900), J=1,1) /
     &   1.26731148E-02, 1.28354036E-02, 1.29626549E-02, 1.30662003E-02,
     &   1.31514029E-02, 1.32218696E-02, 1.32798649E-02, 1.33269108E-02,
     &   1.33639427E-02, 1.33918343E-02, 1.34111461E-02, 1.34222250E-02,
     &   1.34253738E-02, 1.34207297E-02, 1.34083492E-02, 1.33882503E-02,
     &   1.33602525E-02, 1.33242223E-02, 1.32798578E-02, 1.32267652E-02,
     &   1.31643302E-02, 1.30919640E-02, 1.30088553E-02, 1.29136607E-02,
     &   1.28052049E-02, 1.26814483E-02, 1.25398679E-02, 1.23771334E-02,
     &   1.21889319E-02, 1.19716322E-02, 1.17119598E-02, 1.13423207E-02,
     &   1.07994738E-02, 1.06399382E-02, 1.02032311E-02, 9.52478673E-03,
     &   9.45185467E-03, 8.96234368E-03, 8.76829851E-03, 8.57603122E-03,
     &   8.42316033E-03, 8.24445687E-03, 7.96930676E-03, 7.85606698E-03,
     &   7.86857778E-03, 7.67468311E-03, 7.68357444E-03, 7.92635871E-03,
     &   7.81344947E-03, 7.95575562E-03, 7.81017554E-03, 7.58420316E-03,
     &   7.81607235E-03, 7.52394902E-03, 6.94534732E-03, 6.78466568E-03,
     &   6.97997359E-03, 6.99439388E-03, 6.95721507E-03, 6.82476671E-03,
     &   6.42934457E-03, 6.24112844E-03, 5.80911405E-03, 5.45199018E-03,
     &   5.29821840E-03, 5.56289886E-03, 6.00564343E-03, 6.94841368E-03,
     &   7.28420581E-03, 7.70062314E-03, 7.96542733E-03, 8.25670777E-03,
     &   8.33770809E-03, 8.32531049E-03, 8.73044678E-03, 9.10235352E-03,
     &   9.01650828E-03, 8.97570417E-03, 9.03526413E-03, 9.08349431E-03,
     &   9.12043414E-03, 9.10519555E-03, 9.12906191E-03, 9.12116720E-03,
     &   9.00721638E-03, 8.95602092E-03, 8.97977670E-03, 9.05628792E-03,
     &   9.15350110E-03, 9.40262172E-03, 9.51662684E-03, 9.52437877E-03,
     &   9.52024762E-03, 9.42919734E-03, 9.18633870E-03, 9.17333915E-03,
     &   9.40010690E-03, 9.66398128E-03, 9.89995313E-03, 9.91536787E-03/
  
         DATA ((DAS(I,J), I= 901,1000), J=1,1) /
     &   1.00125264E-02, 1.01603422E-02, 1.03110779E-02, 1.05778890E-02,
     &   1.08579258E-02, 1.09735285E-02, 1.10924942E-02, 1.13536757E-02,
     &   1.14068431E-02, 1.13006730E-02, 1.10807391E-02, 1.10058906E-02,
     &   1.11360288E-02, 1.10888834E-02, 1.07057610E-02, 1.05786421E-02,
     &   1.06579382E-02, 1.06577367E-02, 1.01997155E-02, 9.73936045E-03,
     &   9.59177608E-03, 9.52243399E-03, 9.46590676E-03, 9.41461717E-03,
     &   9.14532442E-03, 9.01431500E-03, 9.05934634E-03, 9.04372108E-03,
     &   8.84991027E-03, 8.47008853E-03, 8.41952164E-03, 8.31644036E-03,
     &   8.38706477E-03, 8.41617201E-03, 8.47585388E-03, 8.53197591E-03,
     &   8.55105848E-03, 8.56604187E-03, 8.49745071E-03, 8.39284888E-03,
     &   8.31267869E-03, 8.67626744E-03, 8.97091712E-03, 8.95828672E-03,
     &   8.98950969E-03, 8.99913176E-03, 8.97415910E-03, 9.27853509E-03,
     &   9.70674366E-03, 9.82325348E-03, 9.81341741E-03, 9.86071945E-03,
     &   9.89538748E-03, 9.95517438E-03, 1.00175194E-02, 9.98464549E-03,
     &   9.93941746E-03, 9.91563423E-03, 9.66495337E-03, 9.61124497E-03,
     &   9.70571113E-03, 9.73630264E-03, 9.62178555E-03, 9.55130179E-03,
     &   9.54316208E-03, 9.55010845E-03, 9.56222900E-03, 9.79710782E-03,
     &   1.00505847E-02, 1.01005964E-02, 1.01005361E-02, 1.01372156E-02,
     &   9.97844261E-03, 1.01640331E-02, 1.02852098E-02, 1.04683500E-02,
     &   1.05452218E-02, 1.06164667E-02, 1.07474078E-02, 1.08676399E-02,
     &   1.09247611E-02, 1.09802251E-02, 1.10342143E-02, 1.10868733E-02,
     &   1.11383189E-02, 1.11883873E-02, 1.12376650E-02, 1.12859725E-02,
     &   1.13333669E-02, 1.13798971E-02, 1.14256053E-02, 1.14705285E-02,
     &   1.15146987E-02, 1.15581444E-02, 1.16008902E-02, 1.16429583E-02,
     &   1.16843680E-02, 1.17251360E-02, 1.17652791E-02, 1.18048090E-02/
  
         DATA ((DAS(I,J), I=1001,1100), J=1,1) /
     &   1.18437375E-02, 1.18820750E-02, 1.19198301E-02, 1.19570102E-02,
     &   1.19936212E-02, 1.20296681E-02, 1.20651545E-02, 1.21000828E-02,
     &   1.21344543E-02, 1.21682692E-02, 1.22015262E-02, 1.22342232E-02,
     &   1.22663564E-02, 1.22979210E-02, 1.23227589E-02, 1.23593167E-02,
     &   1.24183400E-02, 1.24748913E-02, 1.25288360E-02, 1.25799950E-02,
     &   1.26281332E-02, 1.26729409E-02, 1.27140076E-02, 1.27507828E-02,
     &   1.27825141E-02, 1.28081486E-02, 1.28260710E-02, 1.28342562E-02,
     &   1.28289738E-02, 1.28041716E-02, 1.28002573E-02, 1.27486978E-02,
     &   1.26392726E-02, 1.24171595E-02, 1.20354323E-02, 1.18764463E-02,
     &   1.12153389E-02, 9.98449385E-03, 9.41523747E-03, 8.61499587E-03,
     &   8.07997006E-03, 7.40997917E-03, 5.39562244E-03, 1.36662220E-03,
     &  -9.17567367E-03,-9.64606621E-03,-1.01362568E-02,-1.06475185E-02,
     &  -1.11812359E-02,-1.17389177E-02,-1.23222104E-02,-1.29329153E-02,
     &  -1.35730061E-02,-1.42446509E-02,-1.49502362E-02,-1.56923954E-02,
     &  -1.64740412E-02,-1.72984043E-02,-1.81690772E-02,-1.90900670E-02,
     &  -2.00658565E-02,-2.11014774E-02,-2.22025963E-02,-2.33756188E-02,
     &  -2.46278136E-02,-2.59674631E-02,-2.74040463E-02,-2.89484634E-02,
     &  -3.06133119E-02,-3.24132310E-02,-3.43653319E-02,-3.64897434E-02,
     &  -3.88103072E-02,-4.13554754E-02,-4.41594818E-02,-4.72638885E-02,
     &  -5.07196580E-02,-5.45899700E-02,-5.89541153E-02,-6.39129808E-02,
     &  -6.95969387E-02,-7.61774660E-02,-8.38847289E-02,-9.30350367E-02,
     &  -1.04075296E-01,-1.17658152E-01,-1.34775755E-01,-1.57013426E-01,
     &  -1.87070635E-01,-2.29947583E-01,-2.96056639E-01,-4.11241024E-01,
     &  -6.61947118E-01,-8.17442105E-01,-1.57084603E+00,-2.19362271E+00,
     &  -2.64670968E+00,-2.99862541E+00,-3.28891027E+00,-3.54402768E+00/
  
         DATA ((DAS(I,J), I=1101,1200), J=1,1) /
     &  -3.78191370E+00,-3.94838496E+00,-4.11880594E+00,-4.28772995E+00,
     &  -4.36983684E+00,-4.48600961E+00,-4.55657627E+00,-4.64617730E+00,
     &  -4.67029585E+00,-4.70774683E+00,-4.72709316E+00,-4.72366605E+00,
     &  -4.71182890E+00,-4.69229033E+00,-4.62649855E+00,-4.57890097E+00,
     &  -4.51956971E+00,-4.36426824E+00,-4.26674013E+00,-4.15457057E+00,
     &  -4.02551953E+00,-3.72073345E+00,-3.54065517E+00,-3.34473270E+00,
     &  -2.89724984E+00,-2.64853401E+00,-2.09929289E+00,-1.49240111E+00,
     &   1.74876711E-01, 1.50025191E+00, 2.10876052E+00, 2.65726528E+00,
     &   3.14235582E+00, 3.35950018E+00, 3.55654704E+00, 3.73595224E+00,
     &   4.04466920E+00, 4.17346572E+00, 4.28786473E+00, 4.38629721E+00,
     &   4.54324657E+00, 4.60275292E+00, 4.65159218E+00, 4.71832272E+00,
     &   4.73863232E+00, 4.75623253E+00, 4.75517042E+00, 4.73647075E+00,
     &   4.69976416E+00, 4.64862037E+00, 4.58650739E+00, 4.51616473E+00,
     &   4.40021250E+00, 4.31862908E+00, 4.14934159E+00, 3.97934362E+00,
     &   3.81235340E+00, 3.57429961E+00, 3.31895688E+00, 3.02838813E+00,
     &   2.67611872E+00, 2.22259726E+00, 1.59014741E+00, 8.73531923E-01,
     &   5.05488978E-01, 3.57831768E-01, 2.78342911E-01, 2.28689718E-01,
     &   1.94734279E-01, 1.70050628E-01, 1.51298376E-01, 1.36569467E-01,
     &   1.24694889E-01, 1.14918481E-01, 1.06729485E-01, 9.97704693E-02,
     &   9.37838135E-02, 8.85791460E-02, 8.40127451E-02, 7.99740761E-02,
     &   7.63767348E-02, 7.31522041E-02, 7.02454534E-02, 6.76117757E-02,
     &   6.52144757E-02, 6.30231520E-02, 6.10124032E-02, 5.91608408E-02,
     &   5.74503267E-02, 5.58653786E-02, 5.43927009E-02, 5.30208120E-02,
     &   5.17397451E-02, 5.05408065E-02, 4.94163785E-02, 4.83597579E-02,
     &   4.73650230E-02, 4.64269228E-02, 4.55407848E-02, 4.47024374E-02/
  
         DATA ((DAS(I,J), I=1201,1300), J=1,1) /
     &   4.39081446E-02, 4.31545504E-02, 4.24386322E-02, 4.17576599E-02,
     &   4.11091617E-02, 4.04908940E-02, 3.99008160E-02, 3.93370672E-02,
     &   3.87979477E-02, 3.82819016E-02, 3.77875019E-02, 3.73134375E-02,
     &   3.68585016E-02, 2.72040501E-02, 2.52713575E-02, 2.37007328E-02,
     &   1.83913295E-02, 1.47958581E-02, 1.40893255E-02, 1.36581375E-02,
     &   1.30636346E-02, 9.06830130E-03, 3.77989013E-03, 3.25227510E-03,
     &   2.66617238E-03, 2.01124502E-03, 1.27457623E-03, 4.39808236E-04,
     &  -5.14087474E-04,-1.61460728E-03,-2.89841078E-03,-4.41548612E-03,
     &  -6.23581414E-03,-8.46047476E-03,-1.12410819E-02,-1.48158746E-02,
     &  -1.95818908E-02,-2.62537256E-02,-3.62603273E-02,-5.29348471E-02,
     &  -8.62693534E-02,-1.30965713E-01,-1.38465238E-01,-1.46783569E-01,
     &  -1.56062391E-01,-1.66478033E-01,-1.78252736E-01,-1.91670601E-01,
     &  -2.07100688E-01,-2.25031256E-01,-2.46121975E-01,-2.71286109E-01,
     &  -3.01824803E-01,-3.39656273E-01,-3.87728049E-01,-4.50807019E-01,
     &  -5.37117174E-01,-6.62086991E-01,-8.58066792E-01,-1.20352507E+00,
     &  -1.90982280E+00,-2.12157403E+00,-2.26992471E+00,-2.41068551E+00,
     &  -2.50092798E+00,-2.54066662E+00,-2.53979988E+00,-2.50659355E+00,
     &  -2.43865750E+00,-2.32770551E+00,-2.16535815E+00,-2.02870936E+00,
     &  -2.00537684E+00,-1.99347491E+00,-1.96919736E+00,-1.95930886E+00,
     &  -1.95182574E+00,-1.94428532E+00,-1.86571392E+00,-1.73699089E+00,
     &  -1.57869238E+00,-1.42275907E+00,-1.23523384E+00,-1.03385165E+00,
     &  -8.42114683E-01,-7.31864266E-01,-6.19317078E-01,-5.04791032E-01,
     &  -3.41777068E-01,-2.23912813E-01,-1.05281904E-01,-3.38937351E-02,
     &  -1.00811986E-02, 1.37344314E-02, 3.75500631E-02, 1.32750820E-01,
     &   2.51381889E-01, 3.69246409E-01, 9.12976825E-01, 1.12310562E+00/
  
         DATA ((DAS(I,J), I=1301,1400), J=1,1) /
     &   1.48596625E+00, 1.63910717E+00, 1.79420772E+00, 1.93320595E+00,
     &   2.04460884E+00, 2.15353082E+00, 2.23868831E+00, 2.31974725E+00,
     &   2.38124311E+00, 2.43777600E+00, 2.47888439E+00, 2.51468360E+00,
     &   2.53885170E+00, 2.56819799E+00, 2.57354031E+00, 2.57302309E+00,
     &   2.56818198E+00, 2.54664804E+00, 2.53025012E+00, 2.49835561E+00,
     &   1.54043461E+00, 1.04255753E+00, 7.82011846E-01, 6.25030158E-01,
     &   5.20749673E-01, 4.46634497E-01, 3.91318324E-01, 3.48482863E-01,
     &   3.14346185E-01, 2.86509906E-01, 2.63381074E-01, 2.43860944E-01,
     &   2.27167449E-01, 2.12728951E-01, 2.00118018E-01, 1.89008701E-01,
     &   1.79148136E-01, 1.70337163E-01, 1.62416802E-01, 1.13815058E-01,
     &   8.04805365E-02, 6.38059828E-02, 5.37993335E-02, 4.71274384E-02,
     &   4.23613490E-02, 3.87864692E-02, 3.60057596E-02, 3.37809795E-02,
     &   3.19605119E-02, 3.04432729E-02, 2.91593964E-02, 2.80588425E-02,
     &   2.71049173E-02, 2.62701279E-02, 2.55334499E-02, 2.48785311E-02,
     &   2.42924617E-02, 2.37649138E-02, 2.03245309E-02, 1.50619151E-02,
     &   1.39045791E-02, 1.24509509E-02, 5.68508314E-03, 5.09461363E-03,
     &   4.41145791E-03, 3.61481854E-03, 2.67373346E-03, 1.54485127E-03,
     &   1.65608115E-04,-1.55790025E-03,-3.77324837E-03,-6.72627553E-03,
     &  -1.08596134E-02,-1.70585434E-02,-2.73881972E-02,-2.91845765E-02,
     &  -3.11440020E-02,-3.32902200E-02,-3.56510098E-02,-3.82599459E-02,
     &  -4.11589930E-02,-4.43990173E-02,-4.80435137E-02,-5.21742240E-02,
     &  -5.68948470E-02,-6.23408224E-02,-6.86947324E-02,-7.62033537E-02,
     &  -8.52350987E-02,-9.62199840E-02, 5.68508314E-03,-1.09955272E-01,
     &  -1.27615796E-01,-1.51165203E-01,-1.84309802E-01,-2.33708529E-01,
     &  -3.15870635E-01,-4.79325808E-01,-9.57668713E-01,-1.13723231E+00/
  
         DATA ((DAS(I,J), I=1401,1500), J=1,1) /
     &  -1.59080432E+00,-1.92273675E+00,-2.18546918E+00,-2.40525621E+00,
     &  -2.59158732E+00,-2.75077085E+00,-2.88566976E+00,-3.00367763E+00,
     &  -3.10796786E+00,-3.19618370E+00,-3.26713932E+00,-3.32607706E+00,
     &  -3.37235654E+00,-3.40600134E+00,-3.42984447E+00,-3.44076307E+00,
     &  -3.44069854E+00,-3.43002480E+00,-3.40701387E+00,-3.36978279E+00,
     &  -3.32502632E+00,-3.26701255E+00,-3.19438342E+00,-3.10578589E+00,
     &  -2.99991507E+00,-2.89762175E+00,-2.75707375E+00,-2.59632007E+00,
     &  -2.41481124E+00,-2.17661281E+00,-1.91045872E+00,-1.57399551E+00,
     &  -1.15909345E+00, 1.39284125E-02, 1.18695261E+00, 1.60185669E+00,
     &   1.93832211E+00, 2.20447835E+00, 2.44267909E+00, 2.62418998E+00,
     &   2.78494576E+00, 2.92549586E+00, 3.02779092E+00, 3.13366379E+00,
     &   3.22226333E+00, 3.29489440E+00, 3.35291004E+00, 3.39766829E+00,
     &   3.43490136E+00, 3.45791415E+00, 3.46858964E+00, 3.46865580E+00,
     &   3.45773894E+00, 3.43389759E+00, 3.40025443E+00, 3.35397659E+00,
     &   3.29494771E+00, 3.22399367E+00, 3.13577943E+00, 3.03149078E+00,
     &   2.91348441E+00, 2.77858695E+00, 2.61940487E+00, 2.43307520E+00,
     &   2.21328959E+00, 1.95055856E+00, 1.61862752E+00, 1.16505695E+00,
     &   9.85392084E-01, 5.07125097E-01, 3.43696378E-01, 2.61526830E-01,
     &   2.12129327E-01, 1.78988895E-01, 1.55440768E-01, 1.37779375E-01,
     &   1.24046131E-01, 1.13061143E-01, 1.04076631E-01, 9.65013338E-02,
     &   9.01726280E-02, 8.47510754E-02, 8.00557506E-02, 7.59495682E-02,
     &   5.52174244E-02, 4.49027947E-02, 3.86252981E-02, 3.46003470E-02,
     &   3.15696502E-02, 2.93695958E-02, 2.77527793E-02, 2.62971838E-02,
     &   2.51834078E-02, 2.41642511E-02, 2.34240834E-02, 2.27409977E-02,
     &   2.21487208E-02, 1.58176077E-02, 1.42920230E-02, 1.35296684E-02/
  
         DATA ((DAS(I,J), I=1501,1600), J=1,1) /
     &   1.32031356E-02, 1.29697421E-02, 1.27850684E-02, 1.26324266E-02,
     &   1.25032926E-02, 1.23924259E-02, 1.22968839E-02, 1.22150434E-02,
     &   1.21445978E-02, 1.20844090E-02, 1.20341318E-02, 1.19931206E-02,
     &   1.19606315E-02, 1.19360531E-02, 1.19195654E-02, 1.19110291E-02,
     &   1.19100634E-02, 1.19169173E-02, 1.19315523E-02, 1.19536164E-02,
     &   1.19847591E-02, 1.20231463E-02, 1.20713420E-02, 1.21288047E-02,
     &   1.21972971E-02, 1.22775601E-02, 1.23702376E-02, 1.24758364E-02,
     &   1.26032716E-02, 1.27523355E-02, 1.29282949E-02, 1.31528294E-02,
     &   1.34577632E-02, 1.42387654E-02, 1.50209481E-02, 1.53270428E-02,
     &   1.55527374E-02, 1.57298238E-02, 1.58800252E-02, 1.60085987E-02,
     &   1.61152885E-02, 1.62090679E-02, 1.62904320E-02, 1.63600156E-02,
     &   1.64185515E-02, 1.64678202E-02, 1.65072506E-02, 1.65394698E-02,
     &   1.65625705E-02, 1.65782618E-02, 1.65861835E-02, 1.65862713E-02,
     &   1.65787881E-02, 1.65633806E-02, 1.65399001E-02, 1.65085189E-02,
     &   1.64686473E-02, 1.64195592E-02, 1.63606242E-02, 1.62921236E-02,
     &   1.62111197E-02, 1.61172125E-02, 1.60082490E-02, 1.58814457E-02,
     &   1.57318831E-02, 1.55517470E-02, 1.53263168E-02, 1.50195245E-02,
     &   1.44390580E-02, 1.44204240E-02, 1.42916790E-02, 1.42098987E-02,
     &   1.41503435E-02, 1.41037494E-02, 1.40657732E-02, 1.40340117E-02,
     &   1.40070852E-02, 1.39840064E-02, 1.39642042E-02, 1.39471698E-02,
     &   1.39326413E-02, 1.39203628E-02, 1.39101297E-02, 1.39018157E-02,
     &   1.38953300E-02, 1.38905924E-02, 1.38875439E-02, 1.38861623E-02,
     &   1.38864309E-02, 1.38883552E-02, 1.38919631E-02, 1.38972957E-02,
     &   1.39044067E-02, 1.39134097E-02, 1.39244195E-02, 1.39375734E-02,
     &   1.39531073E-02, 1.39714122E-02, 1.39927001E-02, 1.40178231E-02/
  
         DATA ((DAS(I,J), I=1601,1700), J=1,1) /
     &   1.40475234E-02, 1.40834667E-02, 1.41287537E-02, 1.41909440E-02,
     &   1.43505593E-02, 1.44924241E-02, 1.45492963E-02, 1.45913235E-02,
     &   1.46253816E-02, 1.46541729E-02, 1.46791877E-02, 1.47010872E-02,
     &   1.47204411E-02, 1.47377221E-02, 1.47530549E-02, 1.47667225E-02,
     &   1.47788800E-02, 1.47895969E-02, 1.47990506E-02, 1.48073193E-02,
     &   1.48143476E-02, 1.48202925E-02, 1.48251882E-02, 1.48289975E-02,
     &   1.48317980E-02, 1.48335787E-02, 1.48343636E-02, 1.48341415E-02,
     &   1.48329082E-02, 1.48306647E-02, 1.48273737E-02, 1.48230233E-02,
     &   1.48175682E-02, 1.48109784E-02, 1.48031999E-02, 1.47941762E-02,
     &   1.47838183E-02, 1.47721024E-02, 1.47588989E-02, 1.47441805E-02,
     &   1.46584758E-02, 1.46552894E-02, 1.46525264E-02, 1.46501657E-02,
     &   1.46482075E-02, 1.46466491E-02, 1.46454850E-02, 1.46447013E-02,
     &   1.46442942E-02, 1.46442576E-02, 1.46446026E-02, 1.46453319E-02,
     &   1.46464456E-02, 1.46479423E-02, 1.46498401E-02, 1.46521470E-02,
     &   1.46548400E-02, 1.46579801E-02, 1.46615854E-02, 1.46656288E-02,
     &   1.46702110E-02, 1.46752525E-02, 1.46808721E-02, 1.46870987E-02,
     &   1.46932634E-02, 1.47008616E-02, 1.47091551E-02, 1.47184358E-02,
     &   1.47287776E-02, 1.47403540E-02, 1.47533589E-02, 1.47683570E-02,
     &   1.47860094E-02, 1.48077256E-02, 1.48369390E-02, 1.49105310E-02,
     &   1.49852805E-02, 1.50156793E-02, 1.50386159E-02, 1.50575208E-02,
     &   1.50738072E-02, 1.50881330E-02, 1.51010807E-02, 1.51128397E-02,
     &   1.51235839E-02, 1.51333769E-02, 1.51425494E-02, 1.51510404E-02,
     &   1.51589685E-02, 1.51663611E-02, 1.51732163E-02, 1.51796668E-02,
     &   1.51855936E-02, 1.51910807E-02, 1.51962454E-02, 1.52009404E-02,
     &   1.52053230E-02, 1.52093500E-02, 1.52130299E-02, 1.52164033E-02/
  
         DATA ((DAS(I,J), I=1701,1800), J=1,1) /
     &   1.52194705E-02, 1.52222337E-02, 1.52246966E-02, 1.52268830E-02,
     &   1.52287760E-02, 1.52303967E-02, 1.52317577E-02, 1.52328558E-02,
     &   1.52336903E-02, 1.52342066E-02, 1.52343762E-02, 1.52308528E-02,
     &   1.51990234E-02, 1.51627004E-02, 1.51339091E-02, 1.51097699E-02,
     &   1.50908543E-02, 1.50739715E-02, 1.50594919E-02, 1.50451836E-02,
     &   1.50309435E-02, 1.50167294E-02, 1.50005700E-02, 1.49831582E-02,
     &   1.49707181E-02, 1.49582779E-02, 1.49814785E-02, 1.50037979E-02,
     &   1.50237494E-02, 1.50434779E-02, 1.50614901E-02, 1.50795962E-02,
     &   1.50963438E-02, 1.51133544E-02, 1.51299340E-02, 1.51454471E-02,
     &   1.51613522E-02, 1.51763029E-02, 1.51916900E-02, 1.52062002E-02,
     &   1.52211742E-02, 1.52353273E-02, 1.52499614E-02, 1.52638166E-02,
     &   1.52781636E-02, 1.52917641E-02, 1.53052526E-02, 1.53192417E-02,
     &   1.53325215E-02, 1.53463049E-02, 1.53593985E-02, 1.53729973E-02,
     &   1.53859231E-02, 1.53993546E-02, 1.54121275E-02, 1.54254060E-02,
     &   1.54380386E-02, 1.54511765E-02, 1.54642465E-02, 1.54766871E-02,
     &   1.54896315E-02, 1.55019563E-02, 1.55147836E-02, 1.55270000E-02,
     &   1.55397178E-02, 1.55518325E-02, 1.55644473E-02, 1.55764664E-02,
     &   1.55889840E-02, 1.56009128E-02, 1.56127992E-02, 1.56251817E-02,
     &   1.56369846E-02, 1.56492821E-02, 1.56610056E-02, 1.56732219E-02,
     &   1.56848696E-02, 1.56970083E-02, 1.57085834E-02, 1.57206479E-02,
     &   1.57321534E-02, 1.57441464E-02, 1.57561040E-02, 1.57675091E-02,
     &   1.57793990E-02, 1.57896457E-02, 1.58014588E-02, 1.58127281E-02,
     &   1.58244785E-02, 1.58356886E-02, 1.58473780E-02, 1.58585304E-02,
     &   1.58701603E-02, 1.58812565E-02, 1.58928281E-02, 1.59038691E-02,
     &   1.59148837E-02, 1.59263708E-02, 1.59373319E-02, 1.59487636E-02/
  
         DATA ((DAS(I,J), I=1801,1840), J=1,1) /
     &   1.59596722E-02, 1.59710494E-02, 1.59819063E-02, 1.59932300E-02,
     &   1.60040360E-02, 1.60153069E-02, 1.60260628E-02, 1.60372817E-02,
     &   1.61575624E-02, 1.62749239E-02, 1.62934417E-02, 1.71256051E-02,
     &   1.75039600E-02, 1.85048951E-02, 1.90716391E-02, 1.90992374E-02,
     &   1.98589322E-02, 1.98613446E-02, 2.04307800E-02, 2.04525514E-02,
     &   2.04655749E-02, 2.07490926E-02, 2.09381602E-02, 2.09720669E-02,
     &   2.10274860E-02, 2.11096390E-02, 2.11676184E-02, 2.12040428E-02,
     &   2.15581377E-02, 2.15760596E-02, 2.16809079E-02, 2.17002498E-02,
     &   2.17873708E-02, 2.18218595E-02, 2.18493052E-02, 2.18749194E-02,
     &   2.19494153E-02, 2.20577653E-02, 2.21038308E-02, 2.24082797E-02/
  
         DATA ((DAS(I,J), I=   1, 100), J=2,2) /
     &   7.43217590E-06, 7.90437200E-06, 8.43031546E-06, 9.02300316E-06,
     &   9.72691956E-06, 1.05076282E-05, 1.29323311E-05, 1.20972889E-05,
     &   1.24257406E-05, 1.29427889E-05, 1.28414256E-05, 1.30817247E-05,
     &   1.30926955E-05, 1.31234207E-05, 1.31389605E-05, 1.32775000E-05,
     &   1.36604764E-05, 1.41832871E-05, 1.46384752E-05, 1.52885606E-05,
     &   1.57200695E-05, 1.61482435E-05, 1.64260746E-05, 1.65838615E-05,
     &   1.67444259E-05, 1.70249676E-05, 1.76341659E-05, 1.79584848E-05,
     &   1.81248714E-05, 1.85814914E-05, 1.89703865E-05, 1.92804827E-05,
     &   1.97990019E-05, 2.02869836E-05, 2.07611228E-05, 2.12331255E-05,
     &   2.17255740E-05, 2.23504214E-05, 2.30290142E-05, 2.37835230E-05,
     &   2.44628297E-05, 2.53206833E-05, 2.63000473E-05, 2.71790115E-05,
     &   2.85264926E-05, 3.03809868E-05, 3.18761990E-05, 3.27707850E-05,
     &   3.30235346E-05, 3.32977784E-05, 3.96937234E-05, 4.72152471E-05,
     &   5.40299307E-05, 5.57401473E-05, 6.04914229E-05, 6.16963550E-05,
     &   5.77342339E-05, 5.52874854E-05, 5.51457467E-05, 5.65080064E-05,
     &   5.91202701E-05, 6.00349997E-05, 6.23569857E-05, 6.60750029E-05,
     &   7.07672569E-05, 7.68170619E-05, 8.33276051E-05, 9.12672427E-05,
     &   9.40049946E-05, 9.90366786E-05, 1.04801320E-04, 1.06312459E-04,
     &   1.13211028E-04, 1.18764043E-04, 1.24647254E-04, 1.26670957E-04,
     &   1.25609727E-04, 1.24972434E-04, 1.25882931E-04, 1.63562641E-04,
     &   1.64940042E-04, 1.76255769E-04, 1.85914738E-04, 1.96044405E-04,
     &   2.14334389E-04, 2.52040657E-04, 2.83794595E-04, 3.07197470E-04,
     &   3.28352229E-04, 3.46736064E-04, 3.62464697E-04, 3.93444645E-04,
     &   4.49316125E-04, 5.37659462E-04, 7.01896805E-04, 8.08359772E-04,
     &   6.64570010E-04, 5.84871309E-04, 5.19480063E-04, 4.82045353E-04/
  
         DATA ((DAS(I,J), I= 101, 200), J=2,2) /
     &   4.58383652E-04, 4.42061572E-04, 4.29824837E-04, 4.19760939E-04,
     &   4.14320737E-04, 4.07852549E-04, 3.99804249E-04, 3.90095375E-04,
     &   3.78553590E-04, 3.65170564E-04, 3.49736488E-04, 3.31752314E-04,
     &   3.10968074E-04, 2.86789814E-04, 2.58433103E-04, 2.24290898E-04,
     &   1.81917450E-04, 1.31315563E-04, 2.01805794E-04, 2.45685383E-04,
     &   2.79954826E-04, 3.08011096E-04, 3.31744104E-04, 3.52153968E-04,
     &   3.69689292E-04, 3.84669748E-04, 3.97592268E-04, 4.08724817E-04,
     &   4.18116436E-04, 4.25783767E-04, 4.32000086E-04, 4.36763969E-04,
     &   4.40160933E-04, 4.42185846E-04, 4.42910587E-04, 4.42344510E-04,
     &   4.40536061E-04, 4.37562592E-04, 4.34998684E-04, 4.31879019E-04,
     &   4.28252005E-04, 4.21675995E-04, 4.13822252E-04, 4.04546116E-04,
     &   3.93831825E-04, 3.81547310E-04, 3.67456161E-04, 3.51665822E-04,
     &   3.33490528E-04, 3.12979454E-04, 2.89608131E-04, 2.64266152E-04,
     &   2.58771857E-04, 2.45131638E-04, 2.33172205E-04, 2.21506146E-04,
     &   2.06906904E-04, 1.88303926E-04, 1.92543821E-04, 1.81800368E-04,
     &   1.73918852E-04, 1.65950104E-04, 1.65087666E-04, 1.57033034E-04,
     &   1.50580971E-04, 1.45199753E-04, 1.40735038E-04, 1.38482326E-04,
     &   1.26976557E-04, 1.17953761E-04, 1.11531270E-04, 1.07334969E-04,
     &   1.03701436E-04, 1.01110719E-04, 9.93842264E-05, 9.84868719E-05,
     &   9.79604132E-05, 9.78131103E-05, 9.82713063E-05, 9.86595848E-05,
     &   9.97057607E-05, 1.00267582E-04, 1.01880603E-04, 1.03659421E-04,
     &   1.06124861E-04, 1.08350130E-04, 1.11352722E-04, 1.14034696E-04,
     &   1.17462901E-04, 1.22442149E-04, 1.26056637E-04, 1.28756659E-04,
     &   1.35708756E-04, 1.39178948E-04, 1.38333110E-04, 1.40953128E-04,
     &   1.45783093E-04, 1.52295620E-04, 1.59308536E-04, 1.70878888E-04/
  
         DATA ((DAS(I,J), I= 201, 300), J=2,2) /
     &   1.78681816E-04, 1.83728549E-04, 1.85884485E-04, 1.88255854E-04,
     &   1.90851380E-04, 1.93675560E-04, 1.96730661E-04, 2.00004740E-04,
     &   2.03486186E-04, 2.07216774E-04, 2.11201675E-04, 2.15447187E-04,
     &   2.19957578E-04, 2.24694971E-04, 2.29680431E-04, 2.34935106E-04,
     &   2.40271685E-04, 2.45795774E-04, 2.51580591E-04, 2.57649375E-04,
     &   2.64048505E-04, 2.70847645E-04, 2.78158751E-04, 2.86143869E-04,
     &   2.95033383E-04, 3.05154102E-04, 3.16982684E-04, 3.31332742E-04,
     &   3.50083069E-04, 3.81507095E-04, 4.19327401E-04, 4.87308155E-04,
     &   4.02421220E-04, 3.93687519E-04, 4.00285179E-04, 4.11233399E-04,
     &   4.24904209E-04, 4.40815556E-04, 4.58632931E-04, 4.78457581E-04,
     &   5.00475918E-04, 5.24941967E-04, 5.52180170E-04, 5.82599575E-04,
     &   6.16705931E-04, 6.55122279E-04, 6.98614278E-04, 7.48126015E-04,
     &   8.04817843E-04, 8.70103289E-04, 9.45665420E-04, 1.03339317E-03,
     &   1.13514713E-03, 1.25184830E-03, 1.28776968E-03, 1.32463053E-03,
     &   1.36220228E-03, 1.40019438E-03, 1.43816250E-03, 1.47547340E-03,
     &   1.51129823E-03, 1.54452413E-03, 1.57366633E-03, 1.59678397E-03,
     &   1.61137987E-03, 1.61430381E-03, 1.60168243E-03, 1.56891591E-03,
     &   1.51080387E-03, 1.42188204E-03, 1.29705641E-03, 1.13258603E-03,
     &   9.27376044E-04, 6.84452225E-04, 4.12986581E-04, 1.44747883E-04,
     &   2.21783088E-04, 4.99241890E-04, 7.62766810E-04, 9.93964400E-04,
     &   1.18609010E-03, 1.33768090E-03, 1.45086336E-03, 1.52992732E-03,
     &   1.58013553E-03, 1.60686853E-03, 1.61510593E-03, 1.60917357E-03,
     &   1.59267036E-03, 1.56849888E-03, 1.53894668E-03, 1.50578449E-03,
     &   1.47036290E-03, 1.43369868E-03, 1.39654774E-03, 1.35946437E-03,
     &   1.32284851E-03, 1.28698298E-03, 1.25205445E-03, 1.21818197E-03/
  
         DATA ((DAS(I,J), I= 301, 400), J=2,2) /
     &   1.18546468E-03, 1.15394661E-03, 1.12361586E-03, 1.09448096E-03,
     &   1.00158203E-03, 9.21522734E-04, 8.52470908E-04, 7.92652011E-04,
     &   7.40565583E-04, 6.94959189E-04, 6.54810102E-04, 6.19286786E-04,
     &   5.87712855E-04, 5.59538531E-04, 5.34313348E-04, 5.11670676E-04,
     &   4.91319022E-04, 4.73029698E-04, 4.56635181E-04, 4.42035276E-04,
     &   4.31341409E-04, 4.21392173E-04, 4.14324001E-04, 4.12259445E-04,
     &   4.27061269E-04, 5.12962283E-04, 5.12888148E-04, 4.07334864E-04,
     &   3.73873697E-04, 3.54081936E-04, 3.38907736E-04, 3.26146131E-04,
     &   3.15050171E-04, 3.05138077E-04, 2.96142675E-04, 2.87895536E-04,
     &   2.80275995E-04, 2.73197726E-04, 2.66597787E-04, 2.60425081E-04,
     &   2.54640881E-04, 2.49237233E-04, 2.44208336E-04, 2.39472677E-04,
     &   2.34987376E-04, 2.30734343E-04, 2.26745114E-04, 2.23009108E-04,
     &   2.19516596E-04, 2.16258804E-04, 2.13228473E-04, 2.10362978E-04,
     &   2.07545917E-04, 2.04897846E-04, 2.02413650E-04, 1.99983122E-04,
     &   1.94156409E-04, 1.84130721E-04, 1.65864051E-04, 1.55280633E-04,
     &   1.51784260E-04, 1.54888415E-04, 1.52325829E-04, 1.48917443E-04,
     &   1.48944667E-04, 1.48890106E-04, 1.49213053E-04, 1.52468517E-04,
     &   1.53585007E-04, 1.56565177E-04, 1.60707343E-04, 1.60956847E-04,
     &   1.65361584E-04, 1.70845841E-04, 1.73301883E-04, 1.76822577E-04,
     &   1.82865091E-04, 1.89955469E-04, 1.97720905E-04, 2.06610347E-04,
     &   2.11825684E-04, 2.16031382E-04, 2.25996821E-04, 2.37661264E-04,
     &   2.38767332E-04, 2.50207422E-04, 2.64475413E-04, 2.71966300E-04,
     &   2.80420563E-04, 2.98489539E-04, 3.19717631E-04, 3.45616232E-04,
     &   3.54182370E-04, 3.76546124E-04, 3.93922368E-04, 4.13497847E-04,
     &   4.60074564E-04, 5.25993330E-04, 5.34232976E-04, 6.08988071E-04/
  
         DATA ((DAS(I,J), I= 401, 500), J=2,2) /
     &   7.61299250E-04, 7.70972840E-04, 6.51752874E-04, 5.59508904E-04,
     &   5.31195559E-04, 4.28032146E-04, 4.17002697E-04, 4.08637134E-04,
     &   3.92855703E-04, 3.60446690E-04, 3.23375597E-04, 2.84219664E-04,
     &   2.80524475E-04, 2.45765878E-04, 2.40777748E-04, 2.19677347E-04,
     &   2.07243425E-04, 1.73017207E-04, 1.69655115E-04, 1.57353051E-04,
     &   1.57075954E-04, 1.57305510E-04, 1.51081023E-04, 1.42165365E-04,
     &   1.39283190E-04, 1.35784068E-04, 1.35497808E-04, 1.39621348E-04,
     &   1.39678364E-04, 1.37376251E-04, 1.39348491E-04, 1.33660246E-04,
     &   1.34293099E-04, 1.32974861E-04, 1.33997793E-04, 1.29875969E-04,
     &   1.29914505E-04, 1.26822803E-04, 1.28526599E-04, 1.30981709E-04,
     &   1.29332649E-04, 1.31605330E-04, 1.31384256E-04, 1.32822733E-04,
     &   1.49349823E-04, 1.53153161E-04, 1.70475623E-04, 1.75516799E-04,
     &   1.81083250E-04, 1.82793485E-04, 1.94035309E-04, 1.94999389E-04,
     &   1.95803673E-04, 2.01411400E-04, 2.00944553E-04, 2.00045869E-04,
     &   2.06565363E-04, 2.08877784E-04, 2.14811960E-04, 2.17712705E-04,
     &   2.22038411E-04, 2.22185096E-04, 2.24583874E-04, 2.25106731E-04,
     &   2.27213195E-04, 2.31113145E-04, 2.35020950E-04, 2.41362189E-04,
     &   2.53602567E-04, 2.64081345E-04, 2.66287594E-04, 2.90041278E-04,
     &   3.07944385E-04, 3.23898934E-04, 3.26765773E-04, 3.50533179E-04,
     &   3.54662716E-04, 3.60021008E-04, 3.80936158E-04, 4.11978734E-04,
     &   4.17260492E-04, 4.69275507E-04, 4.95937057E-04, 5.12475166E-04,
     &   5.43376901E-04, 5.66391900E-04, 5.71423023E-04, 5.74253774E-04,
     &   5.76979232E-04, 5.79370840E-04, 5.84431162E-04, 5.85339194E-04,
     &   5.90619552E-04, 5.98374597E-04, 6.08704286E-04, 6.14426036E-04,
     &   7.51597849E-04, 8.77143720E-04, 7.53592291E-04, 5.48645529E-04/
  
         DATA ((DAS(I,J), I= 501, 600), J=2,2) /
     &   4.49160325E-04, 3.95520989E-04, 3.67023632E-04, 3.50223546E-04,
     &   3.45947825E-04, 3.17042562E-04, 2.95852740E-04, 2.80820539E-04,
     &   2.72196110E-04, 2.70715808E-04, 2.74075038E-04, 2.83195437E-04,
     &   2.97761037E-04, 3.19014519E-04, 3.53989371E-04, 4.14523093E-04,
     &   5.27088835E-04, 6.47387429E-04, 1.36928848E-03, 6.72689815E-03,
     &   7.40851287E-03, 8.24324310E-03, 9.28916808E-03, 1.06379909E-02,
     &   1.24434860E-02, 1.49847099E-02, 1.88261983E-02, 2.53083623E-02,
     &   3.85769709E-02, 8.09389118E-02, 8.55679153E-02, 9.07565168E-02,
     &   9.66123330E-02, 1.03272342E-01, 1.10913578E-01, 1.19768830E-01,
     &   1.30150309E-01, 1.42486351E-01, 1.57380309E-01, 1.75708750E-01,
     &   1.98792652E-01, 2.28711678E-01, 2.68915862E-01, 2.73702517E-01,
     &   3.80611479E-01, 4.58418427E-01, 5.21102571E-01, 5.75971836E-01,
     &   6.18675957E-01, 6.56493476E-01, 6.87338429E-01, 7.18831553E-01,
     &   7.41185888E-01, 7.62797658E-01, 7.79018195E-01, 7.93693262E-01,
     &   8.03258994E-01, 8.13377912E-01, 8.17005702E-01, 8.20212906E-01,
     &   8.20186286E-01, 8.18027296E-01, 8.10997698E-01, 8.03414910E-01,
     &   7.93200432E-01, 7.80072001E-01, 7.63741347E-01, 7.43920715E-01,
     &   7.20331207E-01, 6.92713032E-01, 6.60837614E-01, 6.24521345E-01,
     &   5.83640498E-01, 5.13678843E-01, 4.61375068E-01, 3.74927222E-01,
     &   2.79970190E-01, 2.68331972E-04, 2.79880200E-01, 3.74842604E-01,
     &   4.61297023E-01, 5.13605688E-01, 5.83575205E-01, 6.24461515E-01,
     &   6.60783331E-01, 6.92664310E-01, 7.20288001E-01, 7.43882930E-01,
     &   7.63708849E-01, 7.80044625E-01, 7.93177991E-01, 8.03397201E-01,
     &   8.10984509E-01, 8.18020479E-01, 8.20183445E-01, 8.20215631E-01,
     &   8.17013533E-01, 8.13388904E-01, 8.03275766E-01, 7.93713933E-01/
  
         DATA ((DAS(I,J), I= 601, 700), J=2,2) /
     &   7.79043539E-01, 7.62827142E-01, 7.41219894E-01, 7.18869462E-01,
     &   6.87380939E-01, 6.56539776E-01, 6.18726201E-01, 5.76025836E-01,
     &   5.21160574E-01, 4.58480135E-01, 3.80676787E-01, 3.12495989E-01,
     &   2.59882979E-01, 2.22112653E-01, 1.93780839E-01, 1.71784242E-01,
     &   1.54231095E-01, 1.39908432E-01, 1.28005021E-01, 1.17958780E-01,
     &   1.09368429E-01, 1.01940070E-01, 9.54536512E-02, 8.97412154E-02,
     &   8.46723909E-02, 8.01444596E-02, 7.60754103E-02, 7.23989820E-02,
     &   6.90610626E-02, 3.61486794E-02, 2.44849531E-02, 1.85160168E-02,
     &   1.48899374E-02, 1.24537740E-02, 1.07044503E-02, 9.38744340E-03,
     &   8.36016026E-03, 7.53650512E-03, 6.86142068E-03, 1.47783276E-03,
     &   7.51407588E-04, 5.97755727E-04, 5.26514330E-04, 5.27030993E-04,
     &   5.80162714E-04, 7.07240441E-04, 9.33256507E-04, 1.02347489E-03,
     &   1.40963979E-03, 2.13164236E-03, 2.92724797E-03, 4.72906208E-03,
     &   6.83257129E-03, 7.18762737E-03, 7.58180304E-03, 8.02179058E-03,
     &   8.51600600E-03, 9.07511640E-03, 9.71278188E-03, 1.04467838E-02,
     &   1.13007062E-02, 1.23065090E-02, 1.35086090E-02, 1.49705987E-02,
     &   1.67868315E-02, 1.91035561E-02, 2.21602118E-02, 2.63775022E-02,
     &   3.25681778E-02, 4.25285400E-02, 6.11421404E-02, 9.05855956E-02,
     &   1.07472246E-01, 1.26034120E-01, 1.72620538E-01, 1.89870602E-01,
     &   2.04385856E-01, 2.16977290E-01, 2.27836381E-01, 2.36990166E-01,
     &   2.45129308E-01, 2.52068397E-01, 2.57287696E-01, 2.62032570E-01,
     &   2.65782200E-01, 2.68514137E-01, 2.70260533E-01, 2.71162806E-01,
     &   2.71160412E-01, 2.70262894E-01, 2.68569787E-01, 2.65902750E-01,
     &   2.62132399E-01, 2.57919886E-01, 2.51738389E-01, 2.45254419E-01,
     &   2.37587163E-01, 2.28658420E-01, 2.16556111E-01, 2.04676254E-01/
  
         DATA ((DAS(I,J), I= 701, 800), J=2,2) /
     &   1.89010754E-01, 1.74024655E-01, 1.51844495E-01, 1.27281370E-01,
     &   9.00812829E-02, 5.73086282E-02, 1.93342026E-02, 6.73097784E-04,
     &   1.92517341E-02, 3.83817333E-02, 9.00037730E-02, 1.27209347E-01,
     &   1.51777372E-01, 1.73963001E-01, 1.88953468E-01, 2.04624248E-01,
     &   2.16508719E-01, 2.28616430E-01, 2.37549770E-01, 2.45221547E-01,
     &   2.51709931E-01, 2.57896412E-01, 2.62113033E-01, 2.65887968E-01,
     &   2.68559355E-01, 2.70256570E-01, 2.71158492E-01, 2.71164486E-01,
     &   2.70266512E-01, 2.68524059E-01, 2.65796116E-01, 2.62050447E-01,
     &   2.57309430E-01, 2.52093556E-01, 2.45158220E-01, 2.37022724E-01,
     &   2.27872379E-01, 2.17016726E-01, 2.04428636E-01, 1.89916584E-01,
     &   1.72669577E-01, 1.52132438E-01, 1.63238231E-01, 1.26089389E-01,
     &   9.06441511E-02, 7.69948095E-02, 4.97539387E-02, 3.66877417E-02,
     &   2.90446966E-02, 2.40346474E-02, 2.04987635E-02, 1.78705110E-02,
     &   1.58404857E-02, 1.42254945E-02, 1.29101651E-02, 1.18182383E-02,
     &   1.08972941E-02, 1.01101179E-02, 9.42955320E-03, 8.83532537E-03,
     &   8.31199123E-03, 7.84763644E-03, 7.43286003E-03, 7.06003175E-03,
     &   2.26733057E-03, 1.80742598E-03, 1.61528399E-03, 1.50994914E-03,
     &   1.44221129E-03, 1.39509003E-03, 1.36001600E-03, 1.33284469E-03,
     &   1.31102892E-03, 1.29290963E-03, 1.27752902E-03, 1.26411630E-03,
     &   1.25217166E-03, 1.24133276E-03, 1.23126920E-03, 1.22180559E-03,
     &   1.21273353E-03, 1.20388701E-03, 1.19522797E-03, 1.18659271E-03,
     &   1.17790122E-03, 1.16910763E-03, 1.16014358E-03, 1.15093464E-03,
     &   1.14141410E-03, 1.13151941E-03, 1.12120777E-03, 1.11039104E-03,
     &   1.09898628E-03, 1.08686858E-03, 1.07392594E-03, 1.05991737E-03,
     &   1.04442196E-03, 1.02642061E-03, 9.95307669E-04, 9.83145218E-04/
  
         DATA ((DAS(I,J), I= 801, 900), J=2,2) /
     &   9.82982726E-04, 9.84267817E-04, 9.86001834E-04, 9.87801544E-04,
     &   9.89456495E-04, 9.90862608E-04, 9.91930743E-04, 9.92616869E-04,
     &   9.92865869E-04, 9.92659194E-04, 9.91940539E-04, 9.90714759E-04,
     &   9.88938677E-04, 9.86596267E-04, 9.83618926E-04, 9.80006889E-04,
     &   9.75747025E-04, 9.70749138E-04, 9.64988726E-04, 9.58447793E-04,
     &   9.50995318E-04, 9.42626555E-04, 9.33251887E-04, 9.22683639E-04,
     &   9.10905739E-04, 8.97610473E-04, 8.82633818E-04, 8.65691790E-04,
     &   8.46178242E-04, 8.24109731E-04, 7.97634931E-04, 7.63903822E-04,
     &   7.17693062E-04, 7.04540725E-04, 6.66024285E-04, 6.13510307E-04,
     &   6.08701407E-04, 5.75772209E-04, 5.64942186E-04, 5.57375464E-04,
     &   5.46868799E-04, 5.36785709E-04, 5.29572160E-04, 5.25829542E-04,
     &   5.21983164E-04, 5.16937063E-04, 5.07894059E-04, 5.05107480E-04,
     &   4.87922512E-04, 4.68929301E-04, 4.68991889E-04, 4.55254509E-04,
     &   4.42507677E-04, 4.42253967E-04, 4.40042400E-04, 4.44864082E-04,
     &   4.60083480E-04, 5.05133553E-04, 4.29596000E-04, 3.92278890E-04,
     &   4.01577173E-04, 4.11102895E-04, 4.27395002E-04, 4.25312356E-04,
     &   4.14008336E-04, 4.13871221E-04, 4.10014278E-04, 3.94757512E-04,
     &   3.86812901E-04, 3.66811714E-04, 3.63505242E-04, 3.64095290E-04,
     &   3.56114036E-04, 3.50587012E-04, 3.46932551E-04, 3.38961447E-04,
     &   3.32676980E-04, 3.30073494E-04, 3.29375203E-04, 3.28578136E-04,
     &   3.28722672E-04, 3.27063531E-04, 3.25075436E-04, 3.22565041E-04,
     &   3.22659094E-04, 3.24118636E-04, 3.22524014E-04, 3.14162473E-04,
     &   3.09108905E-04, 3.07101281E-04, 3.05721071E-04, 3.04868410E-04,
     &   3.03834828E-04, 3.03642666E-04, 3.02422360E-04, 2.99358570E-04,
     &   2.97953247E-04, 2.96599888E-04, 2.95287895E-04, 2.93163977E-04/
  
         DATA ((DAS(I,J), I= 901,1000), J=2,2) /
     &   2.92046080E-04, 2.91443027E-04, 2.91296432E-04, 2.91305313E-04,
     &   2.93300411E-04, 2.89042582E-04, 2.85933306E-04, 2.85476015E-04,
     &   2.85136129E-04, 2.83702268E-04, 2.83735168E-04, 2.84877524E-04,
     &   2.81278906E-04, 2.84991480E-04, 2.92883782E-04, 2.80766639E-04,
     &   2.79871992E-04, 2.76284141E-04, 2.76170358E-04, 2.80152038E-04,
     &   2.73803127E-04, 2.72280016E-04, 2.70885324E-04, 2.69806959E-04,
     &   2.67433483E-04, 2.68752592E-04, 2.68738106E-04, 2.68424876E-04,
     &   2.66815126E-04, 2.67781972E-04, 2.68453930E-04, 2.65083248E-04,
     &   2.64855372E-04, 2.64816090E-04, 2.61138809E-04, 2.61445838E-04,
     &   2.62353345E-04, 2.61430610E-04, 2.62267708E-04, 2.63576248E-04,
     &   2.58550813E-04, 2.58165441E-04, 2.58096480E-04, 2.63566793E-04,
     &   2.61184786E-04, 2.60794968E-04, 2.59196680E-04, 2.63856403E-04,
     &   2.72474353E-04, 2.77479587E-04, 2.72554603E-04, 2.72267283E-04,
     &   2.76099645E-04, 2.73664054E-04, 2.76459648E-04, 2.86637955E-04,
     &   3.20730591E-04, 3.01068677E-04, 3.02766446E-04, 3.14333273E-04,
     &   2.93288990E-04, 2.89763502E-04, 2.88531527E-04, 3.04324085E-04,
     &   3.06455062E-04, 3.36946735E-04, 3.39906994E-04, 3.89862421E-04,
     &   4.55236627E-04, 4.86886830E-04, 6.02560767E-04, 6.93978154E-04,
     &   6.88677988E-04, 5.84269264E-04, 4.83399608E-04, 3.95299841E-04,
     &   3.70037481E-04, 3.50606483E-04, 3.22238579E-04, 3.02214235E-04,
     &   2.94195136E-04, 2.87154245E-04, 2.80913255E-04, 2.75336379E-04,
     &   2.70318100E-04, 2.65772455E-04, 2.61637403E-04, 2.57856107E-04,
     &   2.54383772E-04, 2.51183168E-04, 2.48223070E-04, 2.45477071E-04,
     &   2.42922678E-04, 2.40540599E-04, 2.38314191E-04, 2.36229012E-04,
     &   2.34272474E-04, 2.32433552E-04, 2.30702552E-04, 2.29070916E-04/
  
         DATA ((DAS(I,J), I=1001,1100), J=2,2) /
     &   2.27531069E-04, 2.26076284E-04, 2.24700575E-04, 2.23398601E-04,
     &   2.22165591E-04, 2.20997279E-04, 2.19889846E-04, 2.18839878E-04,
     &   2.17844317E-04, 2.16900436E-04, 2.16005807E-04, 2.15158275E-04,
     &   2.14355939E-04, 2.13597137E-04, 2.13020465E-04, 2.12204591E-04,
     &   2.10971646E-04, 2.09892547E-04, 2.08964743E-04, 2.08189040E-04,
     &   2.07569967E-04, 2.07116427E-04, 2.06842730E-04, 2.06770215E-04,
     &   2.06929790E-04, 2.07365998E-04, 2.08144364E-04, 2.09360611E-04,
     &   2.11167256E-04, 2.13816517E-04, 2.14142674E-04, 2.17768218E-04,
     &   2.23986572E-04, 2.35002277E-04, 2.53081460E-04, 2.60727827E-04,
     &   2.94307141E-04, 3.64672884E-04, 3.99944649E-04, 4.51599023E-04,
     &   4.87179083E-04, 5.32642104E-04, 6.73498774E-04, 9.64912323E-04,
     &   1.74608338E-03, 1.78117130E-03, 1.81774697E-03, 1.85590606E-03,
     &   1.89575261E-03, 1.93740000E-03, 1.98097194E-03, 2.02660379E-03,
     &   2.07444385E-03, 2.12465504E-03, 2.17741668E-03, 2.23292663E-03,
     &   2.29140376E-03, 2.35309082E-03, 2.41825774E-03, 2.48720560E-03,
     &   2.56027124E-03, 2.63783270E-03, 2.72031577E-03, 2.80820174E-03,
     &   2.90203674E-03, 3.00244309E-03, 3.11013298E-03, 3.22592540E-03,
     &   3.35076687E-03, 3.48575729E-03, 3.63218232E-03, 3.79155430E-03,
     &   3.96566457E-03, 4.15665085E-03, 4.36708525E-03, 4.60009049E-03,
     &   4.85949558E-03, 5.15004742E-03, 5.47770342E-03, 5.85004357E-03,
     &   6.27686316E-03, 6.77104586E-03, 7.34988480E-03, 8.03714535E-03,
     &   8.86640504E-03, 9.88669936E-03, 1.11725716E-02, 1.28431322E-02,
     &   1.51012010E-02, 1.83224593E-02, 2.32892218E-02, 3.19431731E-02,
     &   5.07793082E-02, 6.24620772E-02, 1.19067595E-01, 1.65858767E-01,
     &   1.99900630E-01, 2.26341181E-01, 2.48151209E-01, 2.67318997E-01/
  
         DATA ((DAS(I,J), I=1101,1200), J=2,2) /
     &   2.85192135E-01, 2.97699654E-01, 3.10503929E-01, 3.23195733E-01,
     &   3.29364687E-01, 3.38093120E-01, 3.43395021E-01, 3.50127035E-01,
     &   3.51939139E-01, 3.54752953E-01, 3.56206507E-01, 3.55949019E-01,
     &   3.55059657E-01, 3.53591663E-01, 3.48648519E-01, 3.45072362E-01,
     &   3.40614616E-01, 3.28946323E-01, 3.21618726E-01, 3.13191071E-01,
     &   3.03495056E-01, 2.80595510E-01, 2.67065658E-01, 2.52345383E-01,
     &   2.18724583E-01, 2.00037780E-01, 1.58771581E-01, 1.13173951E-01,
     &   1.20975267E-02, 1.11674372E-01, 1.57393477E-01, 1.98604350E-01,
     &   2.35050729E-01, 2.51365472E-01, 2.66170226E-01, 2.79649508E-01,
     &   3.02844393E-01, 3.12521285E-01, 3.21116445E-01, 3.28511990E-01,
     &   3.40304095E-01, 3.44774995E-01, 3.48444444E-01, 3.53458120E-01,
     &   3.54984045E-01, 3.56306402E-01, 3.56226602E-01, 3.54821634E-01,
     &   3.52063748E-01, 3.48221150E-01, 3.43554402E-01, 3.38269331E-01,
     &   3.29557467E-01, 3.23427843E-01, 3.10708731E-01, 2.97936238E-01,
     &   2.85389728E-01, 2.67503984E-01, 2.48319267E-01, 2.26487910E-01,
     &   2.00020785E-01, 1.65946278E-01, 1.18428330E-01, 6.45868334E-02,
     &   3.69347975E-02, 2.58410501E-02, 1.98690092E-02, 1.61386164E-02,
     &   1.35876447E-02, 1.17332909E-02, 1.03245805E-02, 9.21815524E-03,
     &   8.32618326E-03, 7.59185190E-03, 6.97678613E-03, 6.45413087E-03,
     &   6.00453031E-03, 5.61368113E-03, 5.27078503E-03, 4.96753712E-03,
     &   4.69744554E-03, 4.45536233E-03, 4.23715280E-03, 4.03945794E-03,
     &   3.85952065E-03, 3.69505672E-03, 3.54415762E-03, 3.40521630E-03,
     &   3.27686993E-03, 3.15795521E-03, 3.04747315E-03, 2.94456105E-03,
     &   2.84846999E-03, 2.75854674E-03, 2.67421887E-03, 2.59498271E-03,
     &   2.52039325E-03, 2.45005592E-03, 2.38361957E-03, 2.32077071E-03/
  
         DATA ((DAS(I,J), I=1201,1300), J=2,2) /
     &   2.26122857E-03, 2.20474093E-03, 2.15108060E-03, 2.10004239E-03,
     &   2.05144050E-03, 2.00510628E-03, 1.96088632E-03, 1.91864073E-03,
     &   1.87824171E-03, 1.83957226E-03, 1.80252508E-03, 1.76700156E-03,
     &   1.73291092E-03, 1.00617065E-03, 8.54703065E-04, 7.10607440E-04,
     &   5.10739629E-04, 3.08078099E-04, 2.78100129E-04, 2.77199440E-04,
     &   3.19227423E-04, 1.09129281E-03, 2.26692705E-03, 2.38506666E-03,
     &   2.51641308E-03, 2.66329510E-03, 2.82862372E-03, 3.01608722E-03,
     &   3.23042763E-03, 3.47784605E-03, 3.76661072E-03, 4.10799653E-03,
     &   4.51778819E-03, 5.01878549E-03, 5.64518718E-03, 6.45073187E-03,
     &   7.52497827E-03, 9.02911590E-03, 1.12854696E-02, 1.50458919E-02,
     &   2.25642751E-02, 3.26458424E-02, 3.43374404E-02, 3.62137350E-02,
     &   3.83066857E-02, 4.06560669E-02, 4.33120088E-02, 4.63385959E-02,
     &   4.98190764E-02, 5.38635858E-02, 5.86209244E-02, 6.42970962E-02,
     &   7.11855974E-02, 7.97191169E-02, 9.05625236E-02, 1.04791076E-01,
     &   1.24259844E-01, 1.52449001E-01, 1.96655751E-01, 2.74580090E-01,
     &   4.33898324E-01, 4.81662654E-01, 5.15125841E-01, 5.46876991E-01,
     &   5.67232817E-01, 5.76196589E-01, 5.76001078E-01, 5.68510787E-01,
     &   5.53186577E-01, 5.28159342E-01, 4.91538948E-01, 4.60715335E-01,
     &   4.55452260E-01, 4.52767566E-01, 4.47291323E-01, 4.45060793E-01,
     &   4.43372839E-01, 4.41671959E-01, 4.23948756E-01, 3.94912942E-01,
     &   3.59205843E-01, 3.24032262E-01, 2.81732551E-01, 2.36307158E-01,
     &   1.93057429E-01, 1.68188467E-01, 1.42801434E-01, 1.16968049E-01,
     &   8.01974029E-02, 5.36111607E-02, 2.68523527E-02, 1.07511019E-02,
     &   5.38249483E-03, 2.43129240E-04, 5.37259525E-03, 2.68424549E-02,
     &   5.36012988E-02, 8.01876007E-02, 2.02835654E-01, 2.50233996E-01/
  
         DATA ((DAS(I,J), I=1301,1400), J=2,2) /
     &   3.32083775E-01, 3.66627484E-01, 4.01613224E-01, 4.32966797E-01,
     &   4.58095739E-01, 4.82665069E-01, 5.01873886E-01, 5.20158201E-01,
     &   5.34029708E-01, 5.46781728E-01, 5.56054473E-01, 5.64129634E-01,
     &   5.69581188E-01, 5.76200777E-01, 5.77405834E-01, 5.77289166E-01,
     &   5.76197167E-01, 5.71339797E-01, 5.67640948E-01, 5.60446563E-01,
     &   3.44370118E-01, 2.32064948E-01, 1.73294191E-01, 1.37884174E-01,
     &   1.14361871E-01, 9.76439024E-02, 8.51663905E-02, 7.55041300E-02,
     &   6.78040381E-02, 6.15251157E-02, 5.63080436E-02, 5.19049798E-02,
     &   4.81395150E-02, 4.48827057E-02, 4.20381369E-02, 3.95322850E-02,
     &   3.73081116E-02, 3.53206929E-02, 3.35341680E-02, 2.25717031E-02,
     &   1.50533241E-02, 1.12929054E-02, 9.03655617E-03, 7.53242442E-03,
     &   6.45818567E-03, 5.65265075E-03, 5.02626136E-03, 4.52527935E-03,
     &   4.11550652E-03, 3.77414378E-03, 3.48538068E-03, 3.23795230E-03,
     &   3.02359799E-03, 2.83611620E-03, 2.67076441E-03, 2.52385382E-03,
     &   2.39247280E-03, 2.27429187E-03, 1.50674678E-03, 4.13408252E-04,
     &   3.06866867E-04, 3.83138924E-04, 1.36599778E-03, 1.46002875E-03,
     &   1.56879106E-03, 1.69596182E-03, 1.84656469E-03, 2.02763653E-03,
     &   2.24933117E-03, 2.52688842E-03, 2.88425821E-03, 3.36133790E-03,
     &   4.02996211E-03, 5.03380297E-03, 6.70801037E-03, 6.99927012E-03,
     &   7.31699052E-03, 7.66502528E-03, 8.04788269E-03, 8.47101162E-03,
     &   8.94122247E-03, 9.46677036E-03, 1.00579599E-02, 1.07280568E-02,
     &   1.14938905E-02, 1.23774414E-02, 1.34083407E-02, 1.46266377E-02,
     &   1.60883410E-02, 1.78748755E-02, 1.36599778E-03, 2.01077364E-02,
     &   2.29775876E-02, 2.68031103E-02, 3.21558111E-02, 4.01766168E-02,
     &   5.35145489E-02, 8.00454852E-02, 1.57679474E-01, 1.86821441E-01/
  
         DATA ((DAS(I,J), I=1401,1500), J=2,2) /
     &   2.60432536E-01, 3.14302237E-01, 3.56941306E-01, 3.92610695E-01,
     &   4.22850481E-01, 4.48684463E-01, 4.70577275E-01, 4.89728832E-01,
     &   5.06654145E-01, 5.20970741E-01, 5.32486168E-01, 5.42051213E-01,
     &   5.49561946E-01, 5.55022194E-01, 5.58891731E-01, 5.60663743E-01,
     &   5.60653301E-01, 5.58921092E-01, 5.55186682E-01, 5.49144485E-01,
     &   5.41880992E-01, 5.32465972E-01, 5.20679028E-01, 5.06300575E-01,
     &   4.89118842E-01, 4.72517698E-01, 4.49708212E-01, 4.23619549E-01,
     &   3.94162538E-01, 3.55505388E-01, 3.12311323E-01, 2.57706813E-01,
     &   1.90372506E-01, 2.73845949E-04, 1.90367186E-01, 2.57701823E-01,
     &   3.12306690E-01, 3.55501106E-01, 3.94158631E-01, 4.23615976E-01,
     &   4.49704978E-01, 4.72514805E-01, 4.89116231E-01, 5.06298298E-01,
     &   5.20677076E-01, 5.32464336E-01, 5.41879659E-01, 5.49143442E-01,
     &   5.55185961E-01, 5.58920673E-01, 5.60653166E-01, 5.60663874E-01,
     &   5.58892143E-01, 5.55022896E-01, 5.49562913E-01, 5.42052446E-01,
     &   5.32487664E-01, 5.20972490E-01, 5.06656152E-01, 4.89731093E-01,
     &   4.70579777E-01, 4.48687198E-01, 4.22853449E-01, 3.92613892E-01,
     &   3.56944728E-01, 3.14305879E-01, 2.60436394E-01, 1.86825510E-01,
     &   1.57667096E-01, 8.00454003E-02, 5.35188029E-02, 4.01797807E-02,
     &   3.21593646E-02, 2.68073942E-02, 2.29818692E-02, 2.01117395E-02,
     &   1.78791463E-02, 1.60926036E-02, 1.46307539E-02, 1.34126143E-02,
     &   1.23817099E-02, 1.14980636E-02, 1.07323129E-02, 1.00622085E-02,
     &   6.71227950E-03, 5.03806709E-03, 4.03428482E-03, 3.36559193E-03,
     &   2.88859790E-03, 2.53123850E-03, 2.25357870E-03, 2.03201311E-03,
     &   1.85095767E-03, 1.70053214E-03, 1.57328683E-03, 1.46450635E-03,
     &   1.37045498E-03, 4.31501271E-04, 3.17784993E-04, 3.47086563E-04/
  
         DATA ((DAS(I,J), I=1501,1600), J=2,2) /
     &   3.85000428E-04, 4.17887112E-04, 4.46995119E-04, 4.72407362E-04,
     &   4.94905106E-04, 5.14657146E-04, 5.32047998E-04, 5.47171139E-04,
     &   5.60269579E-04, 5.71547805E-04, 5.80997404E-04, 5.88673198E-04,
     &   5.94720252E-04, 5.99223848E-04, 6.02114612E-04, 6.03445939E-04,
     &   6.03265235E-04, 6.01504102E-04, 5.98197946E-04, 5.93422059E-04,
     &   5.86822088E-04, 5.78810538E-04, 5.68875605E-04, 5.57153283E-04,
     &   5.43339149E-04, 5.27347185E-04, 5.09139004E-04, 4.88746333E-04,
     &   4.64671642E-04, 4.37359496E-04, 4.06499893E-04, 3.69823377E-04,
     &   3.26502670E-04, 2.71728535E-04, 3.24112097E-04, 3.67047042E-04,
     &   4.03625251E-04, 4.34505220E-04, 4.61898158E-04, 4.86089004E-04,
     &   5.06613304E-04, 5.24967355E-04, 5.41113230E-04, 5.55084598E-04,
     &   5.66963644E-04, 5.77055519E-04, 5.85218537E-04, 5.91972160E-04,
     &   5.96891944E-04, 6.00339655E-04, 6.02237289E-04, 6.02544298E-04,
     &   6.01329816E-04, 5.98547590E-04, 5.94139752E-04, 5.88173151E-04,
     &   5.80561126E-04, 5.71153781E-04, 5.59892028E-04, 5.46893747E-04,
     &   5.31595053E-04, 5.14087499E-04, 4.94140757E-04, 4.71325270E-04,
     &   4.45418595E-04, 4.15481382E-04, 3.81139156E-04, 3.39814760E-04,
     &   2.88669346E-04, 2.87750959E-04, 2.79655699E-04, 2.77056129E-04,
     &   2.76411163E-04, 2.76364588E-04, 2.76309894E-04, 2.76588189E-04,
     &   2.77114537E-04, 2.77680615E-04, 2.78217845E-04, 2.78639624E-04,
     &   2.78927126E-04, 2.79132335E-04, 2.79191302E-04, 2.79138899E-04,
     &   2.78932407E-04, 2.78515054E-04, 2.77945158E-04, 2.77184629E-04,
     &   2.76247564E-04, 2.75133722E-04, 2.73825219E-04, 2.72281510E-04,
     &   2.70536145E-04, 2.68576446E-04, 2.66394870E-04, 2.63990385E-04,
     &   2.61354564E-04, 2.58474535E-04, 2.55293181E-04, 2.51828526E-04/
  
         DATA ((DAS(I,J), I=1601,1700), J=2,2) /
     &   2.48176544E-04, 2.43637363E-04, 2.38769445E-04, 2.33674699E-04,
     &   2.26567100E-04, 2.29288531E-04, 2.32466577E-04, 2.35430973E-04,
     &   2.38486094E-04, 2.41338939E-04, 2.43824285E-04, 2.46215031E-04,
     &   2.48508841E-04, 2.50642362E-04, 2.52642284E-04, 2.54522274E-04,
     &   2.56276941E-04, 2.57897689E-04, 2.59395498E-04, 2.60774086E-04,
     &   2.62014164E-04, 2.63137822E-04, 2.64147646E-04, 2.65034295E-04,
     &   2.65792201E-04, 2.66425865E-04, 2.66958905E-04, 2.67389517E-04,
     &   2.67730449E-04, 2.67970096E-04, 2.68090954E-04, 2.68120872E-04,
     &   2.68105229E-04, 2.68074134E-04, 2.68042919E-04, 2.68066914E-04,
     &   2.68249150E-04, 2.67612214E-04, 2.66301584E-04, 2.64840106E-04,
     &   2.60177215E-04, 2.60062924E-04, 2.59924947E-04, 2.59759054E-04,
     &   2.59556873E-04, 2.59316017E-04, 2.59042299E-04, 2.58732058E-04,
     &   2.58381056E-04, 2.57380996E-04, 2.56355049E-04, 2.55354964E-04,
     &   2.54367474E-04, 2.53375033E-04, 2.52360175E-04, 2.51325948E-04,
     &   2.50279214E-04, 2.49196176E-04, 2.48073893E-04, 2.46914547E-04,
     &   2.45696991E-04, 2.44452476E-04, 2.43162888E-04, 2.41811837E-04,
     &   2.40370547E-04, 2.38934245E-04, 2.37402937E-04, 2.35862688E-04,
     &   2.34047192E-04, 2.32173330E-04, 2.30282559E-04, 2.28385912E-04,
     &   2.26187566E-04, 2.23828687E-04, 2.21405060E-04, 2.18087602E-04,
     &   2.18809264E-04, 2.20193122E-04, 2.21774625E-04, 2.23333999E-04,
     &   2.24673594E-04, 2.26073404E-04, 2.27500725E-04, 2.28905586E-04,
     &   2.30054864E-04, 2.31224173E-04, 2.32309181E-04, 2.33380824E-04,
     &   2.34408997E-04, 2.35384996E-04, 2.36324624E-04, 2.37237731E-04,
     &   2.38097714E-04, 2.38909522E-04, 2.39699699E-04, 2.40443865E-04,
     &   2.41166870E-04, 2.41863319E-04, 2.42526732E-04, 2.43168325E-04/
  
         DATA ((DAS(I,J), I=1701,1800), J=2,2) /
     &   2.43802872E-04, 2.44438085E-04, 2.45052867E-04, 2.45366973E-04,
     &   2.45700646E-04, 2.46078069E-04, 2.46536427E-04, 2.47118357E-04,
     &   2.47660207E-04, 2.47436060E-04, 2.47173452E-04, 2.45562654E-04,
     &   2.44833267E-04, 2.46221589E-04, 2.49779568E-04, 2.55814832E-04,
     &   2.63378901E-04, 2.72872254E-04, 2.83332769E-04, 2.95541844E-04,
     &   3.09143798E-04, 3.23525566E-04, 3.40332998E-04, 3.58393827E-04,
     &   3.65583146E-04, 3.72772465E-04, 3.58620203E-04, 3.45998317E-04,
     &   3.35543722E-04, 3.25960595E-04, 3.17844068E-04, 3.10265890E-04,
     &   3.03746692E-04, 2.97578342E-04, 2.91977335E-04, 2.87079163E-04,
     &   2.82377163E-04, 2.78232117E-04, 2.74224362E-04, 2.70668207E-04,
     &   2.67209391E-04, 2.64123588E-04, 2.61107234E-04, 2.58403726E-04,
     &   2.55749748E-04, 2.53361546E-04, 2.51107816E-04, 2.48883502E-04,
     &   2.46871859E-04, 2.44880370E-04, 2.43074064E-04, 2.41280958E-04,
     &   2.39650367E-04, 2.38027710E-04, 2.36548666E-04, 2.35073546E-04,
     &   2.33726128E-04, 2.32379564E-04, 2.31092411E-04, 2.29913319E-04,
     &   2.28731746E-04, 2.27647529E-04, 2.26559255E-04, 2.25559080E-04,
     &   2.24553647E-04, 2.23628261E-04, 2.22696705E-04, 2.21838154E-04,
     &   2.20972750E-04, 2.20174158E-04, 2.19402657E-04, 2.18623606E-04,
     &   2.17903455E-04, 2.17175452E-04, 2.16501766E-04, 2.15820025E-04,
     &   2.15188512E-04, 2.14548821E-04, 2.13955694E-04, 2.13354330E-04,
     &   2.12796237E-04, 2.12229898E-04, 2.11680305E-04, 2.11169610E-04,
     &   2.10650727E-04, 2.10155270E-04, 2.09664562E-04, 2.09207888E-04,
     &   2.08743200E-04, 2.08310432E-04, 2.07869764E-04, 2.07459088E-04,
     &   2.07040637E-04, 2.06650412E-04, 2.06252548E-04, 2.05881292E-04,
     &   2.05518812E-04, 2.05148907E-04, 2.04803439E-04, 2.04450695E-04/
  
         DATA ((DAS(I,J), I=1801,1840), J=2,2) /
     &   2.04121071E-04, 2.03784322E-04, 2.03469478E-04, 2.03147660E-04,
     &   2.02846623E-04, 2.02538765E-04, 2.02250644E-04, 2.01955854E-04,
     &   1.99122180E-04, 1.96830301E-04, 1.96503544E-04, 1.87356703E-04,
     &   1.85090828E-04, 1.81330494E-04, 1.79987838E-04, 1.79931764E-04,
     &   1.78641990E-04, 1.78638561E-04, 1.77918518E-04, 1.77894326E-04,
     &   1.77879959E-04, 1.77585579E-04, 1.77407423E-04, 1.77376880E-04,
     &   1.77327839E-04, 1.77257102E-04, 1.77208547E-04, 1.77178607E-04,
     &   1.76908799E-04, 1.76896079E-04, 1.76823552E-04, 1.76810489E-04,
     &   1.76752843E-04, 1.76730549E-04, 1.76713018E-04, 1.76696821E-04,
     &   1.76650606E-04, 1.76585687E-04, 1.76558887E-04, 1.76392685E-04/
c deg
         DATA ((DGS(I,J), I=   1, 100), J=1,1) /
     &  -1.92974439E-03,-2.05093273E-03,-2.17031053E-03,-2.28778799E-03,
     &  -2.40303139E-03,-2.49757669E-03,-2.77756821E-03,-2.80446845E-03,
     &  -2.89057190E-03,-2.99899280E-03,-3.11033160E-03,-3.22524457E-03,
     &  -3.36386996E-03,-3.45381800E-03,-3.56268621E-03,-3.67204739E-03,
     &  -3.78116068E-03,-3.88892490E-03,-3.97617334E-03,-4.08969783E-03,
     &  -4.18413072E-03,-4.30586934E-03,-4.40647327E-03,-4.53505571E-03,
     &  -4.63715091E-03,-4.73931429E-03,-4.87312863E-03,-4.94552370E-03,
     &  -4.97980617E-03,-5.08616612E-03,-5.19284523E-03,-5.30186975E-03,
     &  -5.41383649E-03,-5.52405644E-03,-5.63551990E-03,-5.75094393E-03,
     &  -5.87199508E-03,-5.99968141E-03,-6.13519100E-03,-6.27715197E-03,
     &  -6.38582093E-03,-6.53326136E-03,-6.68562135E-03,-6.80235437E-03,
     &  -6.95989059E-03,-7.11454626E-03,-7.22914417E-03,-7.38233284E-03,
     &  -7.49846367E-03,-7.65380564E-03,-8.47037254E-03,-9.43791063E-03,
     &  -1.08349108E-02,-1.24988316E-02,-1.31188615E-02,-1.32786953E-02,
     &  -1.37690698E-02,-1.44349858E-02,-1.50979262E-02,-1.57659277E-02,
     &  -1.64421743E-02,-1.67007046E-02,-1.71397605E-02,-1.78407976E-02,
     &  -1.84072971E-02,-1.89749334E-02,-1.93740903E-02,-1.96679267E-02,
     &  -1.97028654E-02,-1.97179436E-02,-1.96207940E-02,-1.95455668E-02,
     &  -1.90600909E-02,-1.81695609E-02,-1.68741732E-02,-1.51329293E-02,
     &  -1.28733715E-02,-1.00183987E-02,-7.15815443E-03,-6.16679320E-03,
     &  -3.15775955E-03,-1.72572733E-03,-1.44304235E-04, 1.08962083E-03,
     &   3.00941842E-03, 5.37462585E-03, 7.01534164E-03, 8.31571698E-03,
     &   9.39649995E-03, 1.03205834E-02, 1.11352015E-02, 1.18903388E-02,
     &   1.26036859E-02, 1.32978673E-02, 1.39954229E-02, 1.51081987E-02,
     &   1.54751001E-02, 1.59896673E-02, 1.67983691E-02, 1.75273515E-02/
  
         DATA ((DGS(I,J), I= 101, 200), J=1,1) /
     &   1.81248660E-02, 1.85998465E-02, 1.89947552E-02, 1.93098444E-02,
     &   1.95983986E-02, 1.98648274E-02, 2.01144691E-02, 2.03527383E-02,
     &   2.05867337E-02, 2.08194041E-02, 2.10430526E-02, 2.12601722E-02,
     &   2.14716953E-02, 2.16792386E-02, 2.18869599E-02, 2.20988076E-02,
     &   2.23323448E-02, 2.27651771E-02, 2.30507320E-02, 2.31111137E-02,
     &   2.31262794E-02, 2.31140079E-02, 2.30813450E-02, 2.30339845E-02,
     &   2.29706817E-02, 2.29002992E-02, 2.28248371E-02, 2.27370149E-02,
     &   2.26338638E-02, 2.25125068E-02, 2.23601938E-02, 2.21647199E-02,
     &   2.19070619E-02, 2.15697127E-02, 2.11363282E-02, 2.06182664E-02,
     &   2.00535405E-02, 1.95595004E-02, 1.93444593E-02, 1.91376060E-02,
     &   1.89281124E-02, 1.86288115E-02, 1.83456269E-02, 1.80710735E-02,
     &   1.78181335E-02, 1.75780222E-02, 1.73565481E-02, 1.71601104E-02,
     &   1.69968415E-02, 1.68709105E-02, 1.68032889E-02, 1.68289457E-02,
     &   1.68486560E-02, 1.69365048E-02, 1.70652644E-02, 1.72458521E-02,
     &   1.74704066E-02, 1.76687618E-02, 1.90447065E-02, 1.97780749E-02,
     &   2.02418405E-02, 2.07823606E-02, 2.09315225E-02, 2.10119368E-02,
     &   2.10371665E-02, 2.10165460E-02, 2.09659174E-02, 2.09325823E-02,
     &   2.06611504E-02, 2.03429645E-02, 1.99895074E-02, 1.96671541E-02,
     &   1.92425135E-02, 1.87414725E-02, 1.81296451E-02, 1.75338453E-02,
     &   1.67475671E-02, 1.59104902E-02, 1.52115403E-02, 1.48023605E-02,
     &   1.40699901E-02, 1.37408777E-02, 1.29492304E-02, 1.22705150E-02,
     &   1.14062406E-02, 1.06203648E-02, 9.58206612E-03, 8.63573761E-03,
     &   7.61068926E-03, 6.26400396E-03, 5.54524502E-03, 5.04816084E-03,
     &   4.01071331E-03, 3.43664084E-03, 2.86487169E-03, 2.24564316E-03,
     &   1.57125859E-03, 8.47338346E-04, 7.73099952E-05,-1.60386876E-03/
  
         DATA ((DGS(I,J), I= 201, 300), J=1,1) /
     &  -3.53522899E-03,-4.64333438E-03,-5.05820814E-03,-5.48802936E-03,
     &  -5.93373137E-03,-6.39626987E-03,-6.87655276E-03,-7.37546425E-03,
     &  -7.89402016E-03,-8.43315547E-03,-8.99348374E-03,-9.57574837E-03,
     &  -1.01804462E-02,-1.08086447E-02,-1.14630883E-02,-1.21444485E-02,
     &  -1.28533467E-02,-1.35915575E-02,-1.43602517E-02,-1.51623105E-02,
     &  -1.60016769E-02,-1.68818914E-02,-1.78080772E-02,-1.87867255E-02,
     &  -1.98263661E-02,-2.09406482E-02,-2.21487909E-02,-2.34801266E-02,
     &  -2.49913912E-02,-2.68744706E-02,-2.81480427E-02,-2.93301318E-02,
     &  -2.87842947E-02,-3.00911328E-02,-3.17308820E-02,-3.35522149E-02,
     &  -3.55426740E-02,-3.77108435E-02,-4.00738284E-02,-4.26560636E-02,
     &  -4.54861389E-02,-4.85986437E-02,-5.20355390E-02,-5.58464702E-02,
     &  -6.00962843E-02,-6.48634673E-02,-7.02392936E-02,-7.63372746E-02,
     &  -8.32957864E-02,-9.12842917E-02,-1.00511589E-01,-1.11208825E-01,
     &  -1.23593851E-01,-1.37779114E-01,-1.42142956E-01,-1.46620736E-01,
     &  -1.51187270E-01,-1.55806580E-01,-1.60434418E-01,-1.64982090E-01,
     &  -1.69333830E-01,-1.73363445E-01,-1.76893750E-01,-1.79691408E-01,
     &  -1.81455570E-01,-1.81806730E-01,-1.80278110E-01,-1.76315107E-01,
     &  -1.69290440E-01,-1.58544100E-01,-1.43458959E-01,-1.23575777E-01,
     &  -9.87411247E-02,-6.92575199E-02,-3.59805071E-02,-3.06352878E-04,
     &   3.59882422E-02, 7.10254459E-02, 1.03143516E-01, 1.31119118E-01,
     &   1.54288309E-01, 1.72531782E-01, 1.86127442E-01, 1.95603540E-01,
     &   2.01599488E-01, 2.04766775E-01, 2.05706970E-01, 2.04941977E-01,
     &   2.02905865E-01, 1.99948195E-01, 1.96344535E-01, 1.92308031E-01,
     &   1.88001348E-01, 1.83546782E-01, 1.79035375E-01, 1.74533812E-01,
     &   1.70090465E-01, 1.65739953E-01, 1.61508938E-01, 1.57407129E-01/
  
         DATA ((DGS(I,J), I= 301, 400), J=1,1) /
     &   1.53438156E-01, 1.49610310E-01, 1.45925922E-01, 1.42384660E-01,
     &   1.31073564E-01, 1.21299429E-01, 1.12839893E-01, 1.05482443E-01,
     &   9.90483829E-02, 9.33917473E-02, 8.83879119E-02, 8.39344120E-02,
     &   7.99481188E-02, 7.63600499E-02, 7.31164220E-02, 7.01731085E-02,
     &   6.74906699E-02, 6.50370511E-02, 6.27856157E-02, 6.07147074E-02,
     &   5.88100547E-02, 5.70582065E-02, 5.54589335E-02, 5.40382173E-02,
     &   5.30000039E-02, 5.39723064E-02, 5.38981751E-02, 5.11937996E-02,
     &   4.92846534E-02, 4.77794331E-02, 4.64592606E-02, 4.52735983E-02,
     &   4.41832458E-02, 4.31746281E-02, 4.22366210E-02, 4.13596331E-02,
     &   4.05367379E-02, 3.97620088E-02, 3.90305030E-02, 3.83387126E-02,
     &   3.76835570E-02, 3.70624955E-02, 3.64710781E-02, 3.59082145E-02,
     &   3.53715798E-02, 3.48594144E-02, 3.43700829E-02, 3.39023139E-02,
     &   3.34538870E-02, 3.30234296E-02, 3.26105448E-02, 3.22141118E-02,
     &   3.18327190E-02, 3.14655697E-02, 3.11113123E-02, 3.07693512E-02,
     &   2.98547882E-02, 2.78881550E-02, 2.48699361E-02, 2.26010505E-02,
     &   2.09269363E-02, 1.94896389E-02, 1.81250354E-02, 1.70260763E-02,
     &   1.60145964E-02, 1.59127149E-02, 1.49233059E-02, 1.42077503E-02,
     &   1.39702831E-02, 1.34062222E-02, 1.28674849E-02, 1.28152732E-02,
     &   1.23564712E-02, 1.18728878E-02, 1.16439322E-02, 1.14375062E-02,
     &   1.10773078E-02, 1.07662712E-02, 1.04797380E-02, 1.01854790E-02,
     &   1.00060485E-02, 9.88460988E-03, 9.57871147E-03, 9.27988670E-03,
     &   9.25244940E-03, 9.01339330E-03, 8.78754853E-03, 8.68642208E-03,
     &   8.57926329E-03, 8.36690857E-03, 8.16758530E-03, 7.96368806E-03,
     &   7.90630485E-03, 7.77026194E-03, 7.66598808E-03, 7.56642418E-03,
     &   7.42610425E-03, 7.35283693E-03, 7.34810077E-03, 7.37344609E-03/
  
         DATA ((DGS(I,J), I= 401, 500), J=1,1) /
     &   7.46849805E-03, 7.29749505E-03, 7.07698319E-03, 6.84852125E-03,
     &   6.77640842E-03, 6.54469871E-03, 6.55901468E-03, 6.57327683E-03,
     &   6.59932867E-03, 6.68799165E-03, 6.81465266E-03, 6.95349255E-03,
     &   6.96887289E-03, 7.11199266E-03, 7.12932659E-03, 7.22653647E-03,
     &   7.31709421E-03, 7.69398956E-03, 7.76011395E-03, 8.21514820E-03,
     &   8.24762695E-03, 8.27938648E-03, 8.57469903E-03, 8.79534710E-03,
     &   8.82643519E-03, 8.94921054E-03, 8.95263334E-03, 8.94678114E-03,
     &   8.94189521E-03, 8.91113674E-03, 8.90018936E-03, 8.88242206E-03,
     &   8.88810702E-03, 8.87276860E-03, 8.83017705E-03, 8.82615549E-03,
     &   8.82540013E-03, 8.82904293E-03, 8.84055731E-03, 8.98051279E-03,
     &   9.14441596E-03, 9.21711510E-03, 9.26564131E-03, 9.28994380E-03,
     &   9.53585232E-03, 9.58064463E-03, 9.69414012E-03, 9.78890926E-03,
     &   9.83876498E-03, 1.00296418E-02, 1.02643113E-02, 1.03171663E-02,
     &   1.03446186E-02, 1.05218921E-02, 1.06141290E-02, 1.07090043E-02,
     &   1.11030153E-02, 1.11482490E-02, 1.12486340E-02, 1.12829546E-02,
     &   1.12673334E-02, 1.12605024E-02, 1.12376181E-02, 1.12209935E-02,
     &   1.12038971E-02, 1.11618943E-02, 1.11252715E-02, 1.11474191E-02,
     &   1.12536918E-02, 1.13707615E-02, 1.13824392E-02, 1.14379335E-02,
     &   1.14145724E-02, 1.14370764E-02, 1.14461149E-02, 1.15026326E-02,
     &   1.15061847E-02, 1.15198005E-02, 1.15543021E-02, 1.14239360E-02,
     &   1.13710062E-02, 1.08240535E-02, 1.05465934E-02, 1.03509293E-02,
     &   9.90179535E-03, 9.64405871E-03, 9.62068483E-03, 9.60105428E-03,
     &   9.55260030E-03, 9.54049838E-03, 9.48282525E-03, 9.46381405E-03,
     &   9.38204633E-03, 9.26402017E-03, 9.14039727E-03, 9.08742582E-03,
     &   8.36506398E-03, 7.87796921E-03, 7.86186933E-03, 8.60340741E-03/
  
         DATA ((DGS(I,J), I= 501, 600), J=1,1) /
     &   9.82893808E-03, 1.04828988E-02, 1.12610407E-02, 1.15310575E-02,
     &   1.16501197E-02, 1.16232410E-02, 1.14648031E-02, 1.11846608E-02,
     &   1.08496889E-02, 1.05351720E-02, 1.01681193E-02, 9.69836275E-03,
     &   9.08048678E-03, 8.23987735E-03, 7.04366155E-03, 5.23077432E-03,
     &   2.19802911E-03,-8.42370506E-04,-1.80511640E-02,-1.41213964E-01,
     &  -1.56822623E-01,-1.75934050E-01,-1.99876863E-01,-2.30748847E-01,
     &  -2.72067681E-01,-3.30216939E-01,-4.18110803E-01,-5.66412173E-01,
     &  -8.69958593E-01,-1.83903664E+00,-1.94492939E+00,-2.06362335E+00,
     &  -2.19758034E+00,-2.34993384E+00,-2.52473361E+00,-2.72730493E+00,
     &  -2.96478980E+00,-3.24698674E+00,-3.58769792E+00,-4.00697544E+00,
     &  -4.53503762E+00,-5.21945834E+00,-6.13915966E+00,-6.24865802E+00,
     &  -8.69428098E+00,-1.04741731E+01,-1.19081196E+01,-1.31632949E+01,
     &  -1.41401835E+01,-1.50052873E+01,-1.57108886E+01,-1.64313172E+01,
     &  -1.69426892E+01,-1.74370745E+01,-1.78081313E+01,-1.81438344E+01,
     &  -1.83626576E+01,-1.85941354E+01,-1.86771238E+01,-1.87504910E+01,
     &  -1.87498820E+01,-1.87004936E+01,-1.85396863E+01,-1.83662244E+01,
     &  -1.81325607E+01,-1.78322381E+01,-1.74586623E+01,-1.70052507E+01,
     &  -1.64656232E+01,-1.58338370E+01,-1.51046632E+01,-1.42739016E+01,
     &  -1.33387219E+01,-1.17382971E+01,-1.05418094E+01,-8.56425052E+00,
     &  -6.39203745E+00, 1.13816831E-02, 6.41494869E+00, 8.58728466E+00,
     &   1.05649938E+01, 1.17615934E+01, 1.33621981E+01, 1.42975027E+01,
     &   1.51283912E+01, 1.58576922E+01, 1.64896047E+01, 1.70293561E+01,
     &   1.74828887E+01, 1.78565817E+01, 1.81570172E+01, 1.83907891E+01,
     &   1.85643544E+01, 1.87253074E+01, 1.87747869E+01, 1.87755231E+01,
     &   1.87022727E+01, 1.86193567E+01, 1.83880111E+01, 1.81692771E+01/
  
         DATA ((DGS(I,J), I= 601, 700), J=1,1) /
     &   1.78336809E+01, 1.74627188E+01, 1.69684369E+01, 1.64571542E+01,
     &   1.57368308E+01, 1.50313163E+01, 1.41663027E+01, 1.31895001E+01,
     &   1.19344163E+01, 1.05005545E+01, 8.72074479E+00, 7.16105783E+00,
     &   5.95749544E+00, 5.09347040E+00, 4.44535834E+00, 3.94216904E+00,
     &   3.54062696E+00, 3.21298458E+00, 2.94068438E+00, 2.71086830E+00,
     &   2.51435680E+00, 2.34442673E+00, 2.19604422E+00, 2.06536715E+00,
     &   1.94941317E+00, 1.84583252E+00, 1.75274914E+00, 1.66864725E+00,
     &   1.59228900E+00, 8.39376706E-01, 5.72543066E-01, 4.35981894E-01,
     &   3.53015244E-01, 2.97268720E-01, 2.57234136E-01, 2.27089023E-01,
     &   2.03571477E-01, 1.84712011E-01, 1.69251084E-01, 4.52070406E-02,
     &   2.38216101E-02, 2.09143192E-02, 1.79461910E-02, 1.69058653E-02,
     &   1.55536113E-02, 1.35522979E-02, 1.09314303E-02, 8.45095623E-03,
     &   5.38965033E-03,-1.64940325E-03,-8.94943372E-03,-2.50421058E-02,
     &  -4.35910073E-02,-4.67113525E-02,-5.01810251E-02,-5.40534626E-02,
     &  -5.84000342E-02,-6.33139380E-02,-6.89145192E-02,-7.53571015E-02,
     &  -8.28477103E-02,-9.16654947E-02,-1.02198382E-01,-1.15001816E-01,
     &  -1.30899743E-01,-1.51169276E-01,-1.77901234E-01,-2.14768923E-01,
     &  -2.68868687E-01,-3.55883578E-01,-5.18450493E-01,-7.75564820E-01,
     &  -9.23030680E-01,-1.08515531E+00,-1.49180721E+00,-1.64223592E+00,
     &  -1.76885169E+00,-1.87869793E+00,-1.97343661E+00,-2.05329997E+00,
     &  -2.12431210E+00,-2.18485459E+00,-2.23039199E+00,-2.27178988E+00,
     &  -2.30450368E+00,-2.32833737E+00,-2.34357162E+00,-2.35143978E+00,
     &  -2.35141455E+00,-2.34357753E+00,-2.32879849E+00,-2.30552045E+00,
     &  -2.27261433E+00,-2.23585025E+00,-2.18190323E+00,-2.12531731E+00,
     &  -2.05840532E+00,-1.98048495E+00,-1.87486966E+00,-1.77119621E+00/
  
         DATA ((DGS(I,J), I= 701, 800), J=1,1) /
     &  -1.63448646E+00,-1.50370596E+00,-1.31014456E+00,-1.09578738E+00,
     &  -7.71148059E-01,-4.85139855E-01,-1.53682743E-01, 1.45770074E-02,
     &   1.82838860E-01, 3.49830622E-01, 8.00347263E-01, 1.12503383E+00,
     &   1.33943318E+00, 1.53304159E+00, 1.66385962E+00, 1.80061461E+00,
     &   1.90432758E+00, 2.00998898E+00, 2.08794854E+00, 2.15489889E+00,
     &   2.21152233E+00, 2.26551141E+00, 2.30231007E+00, 2.33525457E+00,
     &   2.35856891E+00, 2.37338200E+00, 2.38125522E+00, 2.38130971E+00,
     &   2.37347622E+00, 2.35827339E+00, 2.33447072E+00, 2.30178716E+00,
     &   2.26041781E+00, 2.21490451E+00, 2.15438695E+00, 2.08339648E+00,
     &   2.00355033E+00, 1.90882345E+00, 1.79897925E+00, 1.67234669E+00,
     &   1.52184934E+00, 1.34264339E+00, 1.43955179E+00, 1.11539563E+00,
     &   8.06112899E-01, 6.87016815E-01, 4.49337071E-01, 3.35336434E-01,
     &   2.68651293E-01, 2.24935882E-01, 1.94079752E-01, 1.71147722E-01,
     &   1.53437512E-01, 1.39347508E-01, 1.27871057E-01, 1.18343114E-01,
     &   1.10306707E-01, 1.03437192E-01, 9.74982150E-02, 9.23131474E-02,
     &   8.77474477E-02, 8.36927624E-02, 8.00659783E-02, 7.68093588E-02,
     &   3.35746933E-02, 2.90536339E-02, 2.69837847E-02, 2.57369690E-02,
     &   2.48703258E-02, 2.42188590E-02, 2.37038996E-02, 2.32834450E-02,
     &   2.29324465E-02, 2.26348813E-02, 2.23796191E-02, 2.21598895E-02,
     &   2.19696870E-02, 2.18051740E-02, 2.16635834E-02, 2.15423936E-02,
     &   2.14398670E-02, 2.13546344E-02, 2.12858952E-02, 2.12326819E-02,
     &   2.11946552E-02, 2.11715324E-02, 2.11632617E-02, 2.11700044E-02,
     &   2.11921799E-02, 2.12305860E-02, 2.12861637E-02, 2.13606392E-02,
     &   2.14562563E-02, 2.15768134E-02, 2.17273101E-02, 2.19171552E-02,
     &   2.21643611E-02, 2.25145045E-02, 2.34382531E-02, 2.43767245E-02/
  
         DATA ((DGS(I,J), I= 801, 900), J=1,1) /
     &   2.47420731E-02, 2.50050360E-02, 2.52112377E-02, 2.53790370E-02,
     &   2.55171228E-02, 2.56313391E-02, 2.57253544E-02, 2.58016353E-02,
     &   2.58616969E-02, 2.59069554E-02, 2.59383190E-02, 2.59563502E-02,
     &   2.59615400E-02, 2.59541111E-02, 2.59341560E-02, 2.59017042E-02,
     &   2.58564644E-02, 2.57982214E-02, 2.57264874E-02, 2.56406257E-02,
     &   2.55396437E-02, 2.54225895E-02, 2.52880854E-02, 2.51341592E-02,
     &   2.49587123E-02, 2.47585150E-02, 2.45294914E-02, 2.42662625E-02,
     &   2.39618679E-02, 2.36104693E-02, 2.31906491E-02, 2.25931134E-02,
     &   2.17159435E-02, 2.14582927E-02, 2.07532904E-02, 1.96585962E-02,
     &   1.95409378E-02, 1.87518030E-02, 1.84391114E-02, 1.81290558E-02,
     &   1.78830678E-02, 1.75951962E-02, 1.71510686E-02, 1.69686654E-02,
     &   1.69894359E-02, 1.66769562E-02, 1.66930139E-02, 1.70879832E-02,
     &   1.69068664E-02, 1.71390366E-02, 1.69048072E-02, 1.65403036E-02,
     &   1.69174226E-02, 1.64460348E-02, 1.55107606E-02, 1.52522201E-02,
     &   1.55699090E-02, 1.55945245E-02, 1.55360453E-02, 1.53233867E-02,
     &   1.46829204E-02, 1.43776660E-02, 1.36787587E-02, 1.31045483E-02,
     &   1.28580374E-02, 1.32882744E-02, 1.40061178E-02, 1.55354674E-02,
     &   1.60800340E-02, 1.67558719E-02, 1.71868171E-02, 1.76606251E-02,
     &   1.77930743E-02, 1.77736691E-02, 1.84318683E-02, 1.90362367E-02,
     &   1.88991944E-02, 1.88350970E-02, 1.89335502E-02, 1.90136488E-02,
     &   1.90741387E-02, 1.90507760E-02, 1.90914068E-02, 1.90802641E-02,
     &   1.88970165E-02, 1.88144256E-02, 1.88538867E-02, 1.89794490E-02,
     &   1.91378848E-02, 1.95426862E-02, 1.97286456E-02, 1.97415284E-02,
     &   1.97358191E-02, 1.95893300E-02, 1.91976162E-02, 1.91785217E-02,
     &   1.95467881E-02, 1.99751542E-02, 2.03593059E-02, 2.03862286E-02/
  
         DATA ((DGS(I,J), I= 901,1000), J=1,1) /
     &   2.05455473E-02, 2.07869120E-02, 2.10320289E-02, 2.14648231E-02,
     &   2.19193633E-02, 2.21082258E-02, 2.23025341E-02, 2.27271828E-02,
     &   2.28136206E-02, 2.26439316E-02, 2.22890099E-02, 2.21684289E-02,
     &   2.23811517E-02, 2.23067342E-02, 2.16881465E-02, 2.14841962E-02,
     &   2.16132750E-02, 2.16142402E-02, 2.08743372E-02, 2.01306522E-02,
     &   1.98935473E-02, 1.97831663E-02, 1.96935388E-02, 1.96123926E-02,
     &   1.91800752E-02, 1.89704436E-02, 1.90446572E-02, 1.90212647E-02,
     &   1.87092696E-02, 1.80979052E-02, 1.80166394E-02, 1.78528565E-02,
     &   1.79704127E-02, 1.80181897E-02, 1.81186480E-02, 1.82133322E-02,
     &   1.82454993E-02, 1.82703968E-02, 1.81611893E-02, 1.79923984E-02,
     &   1.78638064E-02, 1.84545791E-02, 1.89336915E-02, 1.89144865E-02,
     &   1.89656827E-02, 1.89831425E-02, 1.89445681E-02, 1.94394243E-02,
     &   2.01348446E-02, 2.03247949E-02, 2.03094854E-02, 2.03879620E-02,
     &   2.04447331E-02, 2.05446666E-02, 2.06493532E-02, 2.05998072E-02,
     &   2.05302437E-02, 2.04943321E-02, 2.00991448E-02, 2.00151871E-02,
     &   2.01742293E-02, 2.02327717E-02, 2.00532482E-02, 1.99479590E-02,
     &   1.99377153E-02, 1.99606412E-02, 1.99831695E-02, 2.03892922E-02,
     &   2.08275834E-02, 2.09301717E-02, 2.09524380E-02, 2.10337210E-02,
     &   2.07950553E-02, 2.11016341E-02, 2.13097266E-02, 2.16294146E-02,
     &   2.17651357E-02, 2.18915228E-02, 2.21249180E-02, 2.23400563E-02,
     &   2.24424285E-02, 2.25418834E-02, 2.26387148E-02, 2.27331552E-02,
     &   2.28253916E-02, 2.29151568E-02, 2.30033984E-02, 2.30898231E-02,
     &   2.31745208E-02, 2.32575677E-02, 2.33390292E-02, 2.34189617E-02,
     &   2.34974135E-02, 2.35744263E-02, 2.36500363E-02, 2.37242742E-02,
     &   2.37971665E-02, 2.38687348E-02, 2.39390006E-02, 2.40079761E-02/
  
         DATA ((DGS(I,J), I=1001,1100), J=1,1) /
     &   2.40756739E-02, 2.41421034E-02, 2.42072704E-02, 2.42711785E-02,
     &   2.43338280E-02, 2.43952168E-02, 2.44553399E-02, 2.45141896E-02,
     &   2.45717552E-02, 2.46280230E-02, 2.46829765E-02, 2.47365954E-02,
     &   2.47888562E-02, 2.48397316E-02, 2.48794134E-02, 2.49371958E-02,
     &   2.50286801E-02, 2.51137894E-02, 2.51920208E-02, 2.52627338E-02,
     &   2.53251099E-02, 2.53780938E-02, 2.54203079E-02, 2.54499235E-02,
     &   2.54644622E-02, 2.54604749E-02, 2.54326932E-02, 2.53745253E-02,
     &   2.52738583E-02, 2.51110620E-02, 2.50901968E-02, 2.48499932E-02,
     &   2.44152307E-02, 2.36165167E-02, 2.23174144E-02, 2.17868871E-02,
     &   1.96067826E-02, 1.55891123E-02, 1.37375153E-02, 1.11377069E-02,
     &   9.40081214E-03, 7.22672955E-03, 6.94072285E-04,-1.23646227E-02,
     &  -4.65214572E-02,-4.80453723E-02,-4.96334195E-02,-5.12897228E-02,
     &  -5.30187675E-02,-5.48254403E-02,-5.67150753E-02,-5.86935069E-02,
     &  -6.07671290E-02,-6.29429645E-02,-6.52287442E-02,-6.76329982E-02,
     &  -7.01651623E-02,-7.28357012E-02,-7.56562525E-02,-7.86397963E-02,
     &  -8.18008540E-02,-8.51557245E-02,-8.87227642E-02,-9.25227227E-02,
     &  -9.65791457E-02,-1.00918862E-01,-1.05572579E-01,-1.10575604E-01,
     &  -1.15968744E-01,-1.21799422E-01,-1.28123065E-01,-1.35004878E-01,
     &  -1.42522091E-01,-1.50766870E-01,-1.59850108E-01,-1.69906440E-01,
     &  -1.81100948E-01,-1.93638293E-01,-2.07775322E-01,-2.23838841E-01,
     &  -2.42251167E-01,-2.63567770E-01,-2.88534230E-01,-3.18175176E-01,
     &  -3.53938281E-01,-3.97937676E-01,-4.53387248E-01,-5.25422367E-01,
     &  -6.22787479E-01,-7.61679843E-01,-9.75828461E-01,-1.34894783E+00,
     &  -2.16106553E+00,-2.66476377E+00,-5.10528110E+00,-7.12265479E+00,
     &  -8.59034898E+00,-9.73031704E+00,-1.06706429E+01,-1.14970501E+01/
  
         DATA ((DGS(I,J), I=1101,1200), J=1,1) /
     &  -1.22676393E+01,-1.28068932E+01,-1.33589415E+01,-1.39061406E+01,
     &  -1.41721112E+01,-1.45484321E+01,-1.47770202E+01,-1.50672667E+01,
     &  -1.51453944E+01,-1.52667101E+01,-1.53293791E+01,-1.53182776E+01,
     &  -1.52799333E+01,-1.52166416E+01,-1.50035209E+01,-1.48493370E+01,
     &  -1.46571440E+01,-1.41540727E+01,-1.38381478E+01,-1.34747946E+01,
     &  -1.30567569E+01,-1.20694569E+01,-1.14861255E+01,-1.08514697E+01,
     &  -9.40192907E+00,-8.59625863E+00,-6.81709029E+00,-4.85117294E+00,
     &   5.49675280E-01, 4.84299113E+00, 6.81414589E+00, 8.59092893E+00,
     &   1.01622929E+01, 1.08656933E+01, 1.15039913E+01, 1.20851424E+01,
     &   1.30851757E+01, 1.35023890E+01, 1.38729641E+01, 1.41918185E+01,
     &   1.47002279E+01, 1.48929881E+01, 1.50511941E+01, 1.52673558E+01,
     &   1.53331451E+01, 1.53901578E+01, 1.53867173E+01, 1.53261431E+01,
     &   1.52072386E+01, 1.50415675E+01, 1.48403636E+01, 1.46125011E+01,
     &   1.42368946E+01, 1.39726197E+01, 1.34242432E+01, 1.28735652E+01,
     &   1.23326302E+01, 1.15614975E+01, 1.07343604E+01, 9.79311498E+00,
     &   8.65200123E+00, 7.18289958E+00, 5.13419139E+00, 2.81284364E+00,
     &   1.62063420E+00, 1.14232493E+00, 8.84834833E-01, 7.23992037E-01,
     &   6.13999305E-01, 5.34040876E-01, 4.73296149E-01, 4.25584324E-01,
     &   3.87118591E-01, 3.55449507E-01, 3.28922561E-01, 3.06379914E-01,
     &   2.86987053E-01, 2.70127304E-01, 2.55335106E-01, 2.42252400E-01,
     &   2.30599297E-01, 2.20153847E-01, 2.10737788E-01, 2.02206301E-01,
     &   1.94440518E-01, 1.87341957E-01, 1.80828337E-01, 1.74830372E-01,
     &   1.69289309E-01, 1.64154994E-01, 1.59384359E-01, 1.54940211E-01,
     &   1.50790265E-01, 1.46906360E-01, 1.43263817E-01, 1.39840925E-01,
     &   1.36618501E-01, 1.33579535E-01, 1.30708891E-01, 1.27993056E-01/
  
         DATA ((DGS(I,J), I=1201,1300), J=1,1) /
     &   1.25419928E-01, 1.22978636E-01, 1.20659389E-01, 1.18453343E-01,
     &   1.16352491E-01, 1.14349566E-01, 1.12437956E-01, 1.10611635E-01,
     &   1.08865096E-01, 1.07193300E-01, 1.05591623E-01, 1.04055818E-01,
     &   1.02581978E-01, 7.13020295E-02, 6.50386865E-02, 5.99470360E-02,
     &   4.27330265E-02, 3.10142820E-02, 2.86495252E-02, 2.71779026E-02,
     &   2.51783334E-02, 1.21632361E-02,-4.97653177E-03,-6.68604537E-03,
     &  -8.58501913E-03,-1.07069381E-02,-1.30936441E-02,-1.57981248E-02,
     &  -1.88884982E-02,-2.24538336E-02,-2.66128836E-02,-3.15275762E-02,
     &  -3.74246016E-02,-4.46313901E-02,-5.36390681E-02,-6.52193678E-02,
     &  -8.06584229E-02,-1.02271037E-01,-1.34686031E-01,-1.88700553E-01,
     &  -2.96682121E-01,-4.41468229E-01,-4.65761625E-01,-4.92707398E-01,
     &  -5.22764508E-01,-5.56504145E-01,-5.94646213E-01,-6.38111014E-01,
     &  -6.88094049E-01,-7.46176944E-01,-8.14496580E-01,-8.96011320E-01,
     &  -9.94935982E-01,-1.11748429E+00,-1.27320423E+00,-1.47753730E+00,
     &  -1.75712363E+00,-2.16194102E+00,-2.79678255E+00,-3.91583288E+00,
     &  -6.20375802E+00,-6.88968826E+00,-7.37024377E+00,-7.82621319E+00,
     &  -8.11853752E+00,-8.24726376E+00,-8.24445610E+00,-8.13689014E+00,
     &  -7.91682346E+00,-7.55741434E+00,-7.03151911E+00,-6.58886982E+00,
     &  -6.51328827E+00,-6.47473412E+00,-6.39609133E+00,-6.36405931E+00,
     &  -6.33981908E+00,-6.31539321E+00,-6.06087524E+00,-5.64390001E+00,
     &  -5.13112029E+00,-4.62600226E+00,-4.01854781E+00,-3.36620623E+00,
     &  -2.74510864E+00,-2.38797214E+00,-2.02339566E+00,-1.65240902E+00,
     &  -1.12435445E+00,-7.42554278E-01,-3.58270674E-01,-1.27021472E-01,
     &  -4.98850200E-02, 2.72614526E-02, 1.04407931E-01, 4.12793751E-01,
     &   7.97077874E-01, 1.17887890E+00, 2.94019631E+00, 3.62087095E+00/
  
         DATA ((DGS(I,J), I=1301,1400), J=1,1) /
     &   4.79629304E+00, 5.29236564E+00, 5.79478612E+00, 6.24504603E+00,
     &   6.60591576E+00, 6.95874903E+00, 7.23460149E+00, 7.49717742E+00,
     &   7.69638225E+00, 7.87951044E+00, 8.01267372E+00, 8.12863885E+00,
     &   8.20692709E+00, 8.30198914E+00, 8.31929463E+00, 8.31761918E+00,
     &   8.30193728E+00, 8.23218194E+00, 8.17906380E+00, 8.07574725E+00,
     &   4.97273349E+00, 3.35994973E+00, 2.51595859E+00, 2.00744447E+00,
     &   1.66964647E+00, 1.42956359E+00, 1.25037671E+00, 1.11161887E+00,
     &   1.00103918E+00, 9.10868511E-01, 8.35946770E-01, 7.72714774E-01,
     &   7.18639157E-01, 6.71868199E-01, 6.31017310E-01, 5.95030638E-01,
     &   5.63089074E-01, 5.34547476E-01, 5.08890858E-01, 3.51453942E-01,
     &   2.43472324E-01, 1.89457692E-01, 1.57042544E-01, 1.35429736E-01,
     &   1.19990444E-01, 1.08409862E-01, 9.94018525E-02, 9.21946771E-02,
     &   8.62972001E-02, 8.13819780E-02, 7.72226919E-02, 7.36572464E-02,
     &   7.05667787E-02, 6.78622299E-02, 6.54754958E-02, 6.33536060E-02,
     &   6.14547415E-02, 5.97454469E-02, 4.85967939E-02, 3.15063080E-02,
     &   2.76860283E-02, 2.29067857E-02, 9.59062228E-04,-9.54169616E-04,
     &  -3.16764181E-03,-5.74872393E-03,-8.79771203E-03,-1.24550353E-02,
     &  -1.69233580E-02,-2.25068661E-02,-2.96836012E-02,-3.92499158E-02,
     &  -5.26396353E-02,-7.27204711E-02,-1.06182047E-01,-1.12001161E-01,
     &  -1.18348433E-01,-1.25300786E-01,-1.32948206E-01,-1.41399452E-01,
     &  -1.50790460E-01,-1.61286003E-01,-1.73091761E-01,-1.86472522E-01,
     &  -2.01764199E-01,-2.19405526E-01,-2.39987945E-01,-2.64310844E-01,
     &  -2.93567630E-01,-3.29151265E-01, 9.59062228E-04,-3.73644335E-01,
     &  -4.30852509E-01,-5.07136663E-01,-6.14502727E-01,-7.74521130E-01,
     &  -1.04067064E+00,-1.57015449E+00,-3.11966082E+00,-3.70132499E+00/
  
         DATA ((DGS(I,J), I=1401,1500), J=1,1) /
     &  -5.17059036E+00,-6.24582611E+00,-7.09690080E+00,-7.80886158E+00,
     &  -8.41244787E+00,-8.92809445E+00,-9.36507533E+00,-9.74734073E+00,
     &  -1.00851702E+01,-1.03709297E+01,-1.06007777E+01,-1.07916960E+01,
     &  -1.09416101E+01,-1.10505964E+01,-1.11278320E+01,-1.11632008E+01,
     &  -1.11629918E+01,-1.11284161E+01,-1.10538763E+01,-1.09332729E+01,
     &  -1.07882923E+01,-1.06003671E+01,-1.03650980E+01,-1.00781022E+01,
     &  -9.73515258E+00,-9.40379169E+00,-8.94851157E+00,-8.42777878E+00,
     &  -7.83981340E+00,-7.06821220E+00,-6.20605366E+00,-5.11614126E+00,
     &  -3.77214029E+00, 2.76541854E-02, 3.82745620E+00, 5.17146376E+00,
     &   6.26138329E+00, 7.12354882E+00, 7.89515750E+00, 8.48312955E+00,
     &   9.00386913E+00, 9.45915605E+00, 9.79052257E+00, 1.01334788E+01,
     &   1.04204812E+01, 1.06557565E+01, 1.08436878E+01, 1.09886742E+01,
     &   1.11092840E+01, 1.11838298E+01, 1.12184112E+01, 1.12186255E+01,
     &   1.11832623E+01, 1.11060325E+01, 1.09970515E+01, 1.08471427E+01,
     &   1.06559292E+01, 1.04260863E+01, 1.01403320E+01, 9.80250761E+00,
     &   9.42024707E+00, 8.98327090E+00, 8.46762903E+00, 7.86404740E+00,
     &   7.15209121E+00, 6.30102104E+00, 5.22578983E+00, 3.75652910E+00,
     &   3.17453689E+00, 1.62527648E+00, 1.09587832E+00, 8.29704706E-01,
     &   6.69690269E-01, 5.62337705E-01, 4.86057697E-01, 4.28846711E-01,
     &   3.84360258E-01, 3.48776291E-01, 3.19672510E-01, 2.95133625E-01,
     &   2.74632851E-01, 2.57070637E-01, 2.41860909E-01, 2.28559602E-01,
     &   1.61401020E-01, 1.27988112E-01, 1.07652774E-01, 9.46141537E-02,
     &   8.47962431E-02, 7.76690499E-02, 7.24311518E-02, 6.77155004E-02,
     &   6.41071119E-02, 6.08052258E-02, 5.84070749E-02, 5.61938307E-02,
     &   5.42747454E-02, 3.37427639E-02, 2.87505069E-02, 2.62426308E-02/
  
         DATA ((DGS(I,J), I=1501,1600), J=1,1) /
     &   2.51758732E-02, 2.44152537E-02, 2.38141197E-02, 2.33175853E-02,
     &   2.28976914E-02, 2.25372850E-02, 2.22267388E-02, 2.19607397E-02,
     &   2.17317680E-02, 2.15361102E-02, 2.13726337E-02, 2.12392340E-02,
     &   2.11334879E-02, 2.10534013E-02, 2.09995571E-02, 2.09715039E-02,
     &   2.09679949E-02, 2.09898297E-02, 2.10368902E-02, 2.11080364E-02,
     &   2.12085914E-02, 2.13326336E-02, 2.14884489E-02, 2.16742899E-02,
     &   2.18958597E-02, 2.21555580E-02, 2.24554710E-02, 2.27972401E-02,
     &   2.32097241E-02, 2.36922566E-02, 2.42618923E-02, 2.49888237E-02,
     &   2.59760995E-02, 2.85048550E-02, 3.10374349E-02, 3.20284710E-02,
     &   3.27591611E-02, 3.33324472E-02, 3.38186651E-02, 3.42348370E-02,
     &   3.45801407E-02, 3.48836234E-02, 3.51468893E-02, 3.53719939E-02,
     &   3.55613119E-02, 3.57206034E-02, 3.58480257E-02, 3.59520683E-02,
     &   3.60265730E-02, 3.60770558E-02, 3.61023502E-02, 3.61022544E-02,
     &   3.60776132E-02, 3.60272690E-02, 3.59507403E-02, 3.58485846E-02,
     &   3.57188786E-02, 3.55592564E-02, 3.53676620E-02, 3.51450002E-02,
     &   3.48817076E-02, 3.45764613E-02, 3.42222261E-02, 3.38098898E-02,
     &   3.33233425E-02, 3.27369267E-02, 3.20021380E-02, 3.09994168E-02,
     &   2.90814210E-02, 2.90190469E-02, 2.85865361E-02, 2.83104932E-02,
     &   2.81090328E-02, 2.79512416E-02, 2.78225390E-02, 2.77148175E-02,
     &   2.76234063E-02, 2.75449531E-02, 2.74775148E-02, 2.74193546E-02,
     &   2.73695794E-02, 2.73273128E-02, 2.72918508E-02, 2.72627582E-02,
     &   2.72397221E-02, 2.72224620E-02, 2.72107660E-02, 2.72045560E-02,
     &   2.72037515E-02, 2.72083773E-02, 2.72185038E-02, 2.72342570E-02,
     &   2.72558047E-02, 2.72835009E-02, 2.73177115E-02, 2.73588752E-02,
     &   2.74077427E-02, 2.74655604E-02, 2.75330150E-02, 2.76128279E-02/
  
         DATA ((DGS(I,J), I=1601,1700), J=1,1) /
     &   2.77073831E-02, 2.78220182E-02, 2.79666731E-02, 2.81655802E-02,
     &   2.86767458E-02, 2.91311474E-02, 2.93131638E-02, 2.94475570E-02,
     &   2.95563651E-02, 2.96482501E-02, 2.97279869E-02, 2.97976974E-02,
     &   2.98592078E-02, 2.99140304E-02, 2.99625684E-02, 3.00057265E-02,
     &   3.00440001E-02, 3.00776132E-02, 3.01071289E-02, 3.01327937E-02,
     &   3.01544408E-02, 3.01725593E-02, 3.01872542E-02, 3.01984179E-02,
     &   3.02062828E-02, 3.02108190E-02, 3.02120847E-02, 3.02100568E-02,
     &   3.02047034E-02, 3.01960338E-02, 3.01839086E-02, 3.01682887E-02,
     &   3.01490151E-02, 3.01259844E-02, 3.00990090E-02, 3.00678948E-02,
     &   3.00323354E-02, 2.99922410E-02, 2.99471613E-02, 2.98969920E-02,
     &   2.96041343E-02, 2.95929945E-02, 2.95832828E-02, 2.95748421E-02,
     &   2.95677424E-02, 2.95621422E-02, 2.95577025E-02, 2.95543672E-02,
     &   2.95523478E-02, 2.95517456E-02, 2.95524913E-02, 2.95544125E-02,
     &   2.95574091E-02, 2.95615229E-02, 2.95669565E-02, 2.95739061E-02,
     &   2.95824011E-02, 2.95926255E-02, 2.96045282E-02, 2.96178367E-02,
     &   2.96326869E-02, 2.96486755E-02, 2.96661203E-02, 2.96851418E-02,
     &   2.97036906E-02, 2.97267649E-02, 2.97521652E-02, 2.97809380E-02,
     &   2.98133990E-02, 2.98500830E-02, 2.98915009E-02, 2.99392909E-02,
     &   2.99954042E-02, 3.00642731E-02, 3.01570185E-02, 3.03920519E-02,
     &   3.06303420E-02, 3.07273389E-02, 3.08005804E-02, 3.08608643E-02,
     &   3.09126774E-02, 3.09581636E-02, 3.09992363E-02, 3.10365401E-02,
     &   3.10706375E-02, 3.11017115E-02, 3.11307796E-02, 3.11576221E-02,
     &   3.11826067E-02, 3.12058367E-02, 3.12273369E-02, 3.12475515E-02,
     &   3.12661181E-02, 3.12832821E-02, 3.12993728E-02, 3.13139057E-02,
     &   3.13273823E-02, 3.13397177E-02, 3.13509745E-02, 3.13612380E-02/
  
         DATA ((DGS(I,J), I=1701,1800), J=1,1) /
     &   3.13704373E-02, 3.13786443E-02, 3.13858943E-02, 3.13922171E-02,
     &   3.13975758E-02, 3.14020332E-02, 3.14057171E-02, 3.14084269E-02,
     &   3.14102564E-02, 3.14110621E-02, 3.14106780E-02, 3.13945413E-02,
     &   3.12767013E-02, 3.11437631E-02, 3.10359732E-02, 3.09426638E-02,
     &   3.08669889E-02, 3.07972951E-02, 3.07360818E-02, 3.06748130E-02,
     &   3.06138025E-02, 3.05535552E-02, 3.04863904E-02, 3.04158379E-02,
     &   3.02763958E-02, 3.01369536E-02, 3.01989196E-02, 3.02574304E-02,
     &   3.03088647E-02, 3.03589694E-02, 3.04041078E-02, 3.04489441E-02,
     &   3.04899766E-02, 3.05312583E-02, 3.05711450E-02, 3.06081818E-02,
     &   3.06458944E-02, 3.06811247E-02, 3.07171809E-02, 3.07510097E-02,
     &   3.07857592E-02, 3.08184655E-02, 3.08521538E-02, 3.08839363E-02,
     &   3.09167408E-02, 3.09477457E-02, 3.09784122E-02, 3.10101358E-02,
     &   3.10401790E-02, 3.10712930E-02, 3.11007887E-02, 3.11313636E-02,
     &   3.11603724E-02, 3.11904652E-02, 3.12190364E-02, 3.12486938E-02,
     &   3.12768682E-02, 3.13061297E-02, 3.13352020E-02, 3.13628404E-02,
     &   3.13915646E-02, 3.14188832E-02, 3.14472859E-02, 3.14743081E-02,
     &   3.15024119E-02, 3.15291583E-02, 3.15569832E-02, 3.15834715E-02,
     &   3.16110353E-02, 3.16372813E-02, 3.16634142E-02, 3.16906172E-02,
     &   3.17165281E-02, 3.17435054E-02, 3.17692061E-02, 3.17959694E-02,
     &   3.18214704E-02, 3.18480297E-02, 3.18733405E-02, 3.18997056E-02,
     &   3.19248345E-02, 3.19510136E-02, 3.19771006E-02, 3.20019688E-02,
     &   3.20278808E-02, 3.20490384E-02, 3.20747445E-02, 3.20992558E-02,
     &   3.21248016E-02, 3.21491618E-02, 3.21745521E-02, 3.21987655E-02,
     &   3.22240045E-02, 3.22480752E-02, 3.22731668E-02, 3.22970983E-02,
     &   3.23209630E-02, 3.23458417E-02, 3.23695719E-02, 3.23943116E-02/
  
         DATA ((DGS(I,J), I=1801,1840), J=1,1) /
     &   3.24179103E-02, 3.24425138E-02, 3.24659836E-02, 3.24904537E-02,
     &   3.25137970E-02, 3.25381361E-02, 3.25613551E-02, 3.25855655E-02,
     &   3.28446381E-02, 3.30966270E-02, 3.31363210E-02, 3.49058017E-02,
     &   3.57033300E-02, 3.77998026E-02, 3.89808859E-02, 3.90383187E-02,
     &   4.06168896E-02, 4.06218961E-02, 4.18026624E-02, 4.18477749E-02,
     &   4.18747600E-02, 4.24620344E-02, 4.28534843E-02, 4.29236712E-02,
     &   4.30383801E-02, 4.32084044E-02, 4.33283848E-02, 4.34037546E-02,
     &   4.41362376E-02, 4.41733013E-02, 4.43901174E-02, 4.44301114E-02,
     &   4.46102429E-02, 4.46815465E-02, 4.47382871E-02, 4.47912397E-02,
     &   4.49452371E-02, 4.51691946E-02, 4.52644033E-02, 4.58935166E-02/
  
         DATA ((DGS(I,J), I=   1, 100), J=2,2) /
     &   1.59728843E-05, 1.69888526E-05, 1.81209316E-05, 1.93973807E-05,
     &   2.09146355E-05, 2.25979103E-05, 2.78395030E-05, 2.60205034E-05,
     &   2.67233519E-05, 2.78347649E-05, 2.76044207E-05, 2.81144780E-05,
     &   2.81235246E-05, 2.81795570E-05, 2.81989221E-05, 2.84844406E-05,
     &   2.93003999E-05, 3.04195870E-05, 3.14044586E-05, 3.27999323E-05,
     &   3.37232663E-05, 3.46349071E-05, 3.52228196E-05, 3.55444155E-05,
     &   3.58753314E-05, 3.64664898E-05, 3.77671760E-05, 3.84562787E-05,
     &   3.88124641E-05, 3.97872466E-05, 4.06137457E-05, 4.12677364E-05,
     &   4.23749954E-05, 4.34148848E-05, 4.44237359E-05, 4.54270132E-05,
     &   4.64739022E-05, 4.78084713E-05, 4.92592380E-05, 5.08625988E-05,
     &   5.23211866E-05, 5.41604970E-05, 5.62635083E-05, 5.81547755E-05,
     &   6.10577175E-05, 6.50633700E-05, 6.82932445E-05, 7.02058460E-05,
     &   7.07289284E-05, 7.12879888E-05, 8.50037436E-05, 1.01125511E-04,
     &   1.15414435E-04, 1.18424438E-04, 1.28552886E-04, 1.31117941E-04,
     &   1.22138507E-04, 1.16290053E-04, 1.15432458E-04, 1.17822589E-04,
     &   1.22907028E-04, 1.24639446E-04, 1.29280218E-04, 1.36602154E-04,
     &   1.46125832E-04, 1.58388483E-04, 1.71685895E-04, 1.87749448E-04,
     &   1.93419620E-04, 2.03376860E-04, 2.14816041E-04, 2.17521595E-04,
     &   2.30303588E-04, 2.39251544E-04, 2.47824377E-04, 2.45364498E-04,
     &   2.31522931E-04, 2.10782506E-04, 1.83885823E-04, 2.81596934E-04,
     &   2.50277259E-04, 2.59607818E-04, 2.54172811E-04, 2.51889686E-04,
     &   2.40983023E-04, 2.55185066E-04, 2.71526918E-04, 2.68225800E-04,
     &   2.70006754E-04, 2.71724088E-04, 2.71445928E-04, 3.68187451E-04,
     &   5.59157691E-04, 8.25750240E-04, 1.26446059E-03, 1.51838194E-03,
     &   1.14251292E-03, 9.11007215E-04, 6.93722897E-04, 5.47483571E-04/
  
         DATA ((DGS(I,J), I= 101, 200), J=2,2) /
     &   4.41121660E-04, 3.60527430E-04, 2.99157175E-04, 2.53517595E-04,
     &   2.51220904E-04, 2.52432272E-04, 2.53689498E-04, 2.54724276E-04,
     &   2.55855954E-04, 2.57560077E-04, 2.59855414E-04, 2.61872061E-04,
     &   2.64106830E-04, 2.66953764E-04, 2.70375808E-04, 2.74185328E-04,
     &   2.78430830E-04, 2.85519276E-04, 2.88597575E-04, 2.89121680E-04,
     &   2.89237185E-04, 2.89260310E-04, 2.89437697E-04, 2.89874726E-04,
     &   2.90289394E-04, 2.89858319E-04, 2.89727656E-04, 2.89690983E-04,
     &   2.89578639E-04, 2.88595290E-04, 2.87415506E-04, 2.85821624E-04,
     &   2.83865135E-04, 2.81249401E-04, 2.78238550E-04, 2.74772725E-04,
     &   2.71150346E-04, 2.67987414E-04, 2.66403079E-04, 2.65419476E-04,
     &   2.64605842E-04, 2.63160339E-04, 2.62282417E-04, 2.61451314E-04,
     &   2.61162092E-04, 2.61433750E-04, 2.61673433E-04, 2.63868109E-04,
     &   2.65346768E-04, 2.68756151E-04, 2.73874487E-04, 2.87867887E-04,
     &   2.93368133E-04, 3.11087255E-04, 3.19593912E-04, 3.19253055E-04,
     &   3.03126076E-04, 2.69796200E-04, 3.20315686E-04, 3.03567697E-04,
     &   2.93325864E-04, 2.90417345E-04, 2.95557179E-04, 2.81118830E-04,
     &   2.70323489E-04, 2.61844700E-04, 2.55323985E-04, 2.51873776E-04,
     &   2.34755931E-04, 2.21896298E-04, 2.14085247E-04, 2.09606955E-04,
     &   2.06887622E-04, 2.06045265E-04, 2.06782502E-04, 2.08400939E-04,
     &   2.11396757E-04, 2.15137352E-04, 2.19492184E-04, 2.22364232E-04,
     &   2.28406074E-04, 2.31381172E-04, 2.39245590E-04, 2.46928739E-04,
     &   2.57219097E-04, 2.66583689E-04, 2.79172186E-04, 2.90767311E-04,
     &   3.04609038E-04, 3.24237949E-04, 3.36766157E-04, 3.45937618E-04,
     &   3.67825918E-04, 3.79250683E-04, 3.83760422E-04, 3.94637419E-04,
     &   4.09698589E-04, 4.28092524E-04, 4.47922947E-04, 4.85714035E-04/
  
         DATA ((DGS(I,J), I= 201, 300), J=2,2) /
     &   5.21278183E-04, 5.42874011E-04, 5.51416873E-04, 5.60506060E-04,
     &   5.70165991E-04, 5.80415753E-04, 5.91272434E-04, 6.02732792E-04,
     &   6.14796149E-04, 6.27546256E-04, 6.41012332E-04, 6.55227901E-04,
     &   6.70226314E-04, 6.85979961E-04, 7.02555577E-04, 7.20024558E-04,
     &   7.38147262E-04, 7.57137434E-04, 7.77167198E-04, 7.98343322E-04,
     &   8.20821208E-04, 8.44809245E-04, 8.70603673E-04, 8.98610639E-04,
     &   9.29392121E-04, 9.63744400E-04, 1.00285813E-03, 1.04880563E-03,
     &   1.10634608E-03, 1.19554387E-03, 1.29055783E-03, 1.44448976E-03,
     &   1.25885606E-03, 1.24787241E-03, 1.27406835E-03, 1.31195653E-03,
     &   1.35756210E-03, 1.40978472E-03, 1.46813969E-03, 1.53297803E-03,
     &   1.60491258E-03, 1.68477635E-03, 1.77363105E-03, 1.87280470E-03,
     &   1.98393650E-03, 2.10904447E-03, 2.25061085E-03, 2.41169412E-03,
     &   2.59605242E-03, 2.80826259E-03, 3.05377160E-03, 3.33869985E-03,
     &   3.66901695E-03, 4.04771131E-03, 4.16424400E-03, 4.28381514E-03,
     &   4.40570587E-03, 4.52893243E-03, 4.65205697E-03, 4.77306373E-03,
     &   4.88925626E-03, 4.99701382E-03, 5.09152324E-03, 5.16649246E-03,
     &   5.21382558E-03, 5.22330851E-03, 5.18238022E-03, 5.07611988E-03,
     &   4.88764994E-03, 4.59921836E-03, 4.19423408E-03, 3.66040733E-03,
     &   2.99381734E-03, 2.20326717E-03, 1.31465547E-03, 3.98073899E-04,
     &   6.74305524E-04, 1.59806108E-03, 2.45833463E-03, 3.21015192E-03,
     &   3.83403822E-03, 4.32597427E-03, 4.69313719E-03, 4.94956183E-03,
     &   5.11237046E-03, 5.19903625E-03, 5.22571715E-03, 5.20644420E-03,
     &   5.15288780E-03, 5.07445914E-03, 4.97857469E-03, 4.87097515E-03,
     &   4.75603913E-03, 4.63706294E-03, 4.51649701E-03, 4.39613813E-03,
     &   4.27728268E-03, 4.16084702E-03, 4.04744932E-03, 3.93749050E-03/
  
         DATA ((DGS(I,J), I= 301, 400), J=2,2) /
     &   3.83125352E-03, 3.72887566E-03, 3.63036495E-03, 3.53572369E-03,
     &   3.23380376E-03, 2.97341764E-03, 2.74861147E-03, 2.55369214E-03,
     &   2.38379185E-03, 2.23485632E-03, 2.10356976E-03, 1.98723319E-03,
     &   1.88364969E-03, 1.79102998E-03, 1.70791014E-03, 1.63309602E-03,
     &   1.56562593E-03, 1.50473572E-03, 1.44984772E-03, 1.40058200E-03,
     &   1.35994557E-03, 1.32335757E-03, 1.29444448E-03, 1.27804346E-03,
     &   1.30142528E-03, 1.49203215E-03, 1.49115519E-03, 1.24231271E-03,
     &   1.15160586E-03, 1.09410953E-03, 1.04841580E-03, 1.00926419E-03,
     &   9.74689489E-04, 9.43501977E-04, 9.15004244E-04, 8.88737267E-04,
     &   8.64367150E-04, 8.41646021E-04, 8.20384759E-04, 8.00429004E-04,
     &   7.81655700E-04, 7.63998273E-04, 7.47401264E-04, 7.31700042E-04,
     &   7.16793494E-04, 7.02622408E-04, 6.89207202E-04, 6.76507075E-04,
     &   6.64485160E-04, 6.53108353E-04, 6.42347903E-04, 6.32090892E-04,
     &   6.22141385E-04, 6.12672657E-04, 6.03664618E-04, 5.95096334E-04,
     &   5.72881918E-04, 5.29797703E-04, 4.61644387E-04, 4.18360822E-04,
     &   3.94657090E-04, 3.87899053E-04, 3.73957990E-04, 3.60426500E-04,
     &   3.54987908E-04, 3.54400346E-04, 3.49323559E-04, 3.52430518E-04,
     &   3.53627635E-04, 3.57384555E-04, 3.63983258E-04, 3.64326392E-04,
     &   3.72028417E-04, 3.82122316E-04, 3.86676925E-04, 3.93520373E-04,
     &   4.05285358E-04, 4.19450076E-04, 4.35211445E-04, 4.53504929E-04,
     &   4.64293011E-04, 4.73060074E-04, 4.93888040E-04, 5.18443634E-04,
     &   5.20775316E-04, 5.44976725E-04, 5.75279917E-04, 5.91225303E-04,
     &   6.09255771E-04, 6.47861216E-04, 6.93321758E-04, 7.48893210E-04,
     &   7.67291603E-04, 8.15359591E-04, 8.52732817E-04, 8.94859872E-04,
     &   9.95166453E-04, 1.13724795E-03, 1.15501443E-03, 1.31624995E-03/
  
         DATA ((DGS(I,J), I= 401, 500), J=2,2) /
     &   1.64488562E-03, 1.66572966E-03, 1.40837418E-03, 1.20927368E-03,
     &   1.14816660E-03, 9.25539758E-04, 9.01721765E-04, 8.83661702E-04,
     &   8.49603483E-04, 7.79669894E-04, 6.99693072E-04, 6.15247544E-04,
     &   6.07278879E-04, 5.32343584E-04, 5.21595589E-04, 4.76135208E-04,
     &   4.49346429E-04, 3.75632265E-04, 3.68387252E-04, 3.41813475E-04,
     &   3.41206223E-04, 3.41686755E-04, 3.28198992E-04, 3.08933674E-04,
     &   3.02722335E-04, 2.95053259E-04, 2.94425491E-04, 3.03212511E-04,
     &   3.03322055E-04, 2.98300207E-04, 3.02498008E-04, 2.90161075E-04,
     &   2.91507521E-04, 2.88543567E-04, 2.90625193E-04, 2.81696688E-04,
     &   2.81766223E-04, 2.74942018E-04, 2.78580627E-04, 2.83732160E-04,
     &   2.80088985E-04, 2.84944173E-04, 2.84441816E-04, 2.87527643E-04,
     &   3.23027088E-04, 3.31205677E-04, 3.68506448E-04, 3.79323695E-04,
     &   3.91307154E-04, 3.94902910E-04, 4.19081074E-04, 4.21144740E-04,
     &   4.22872490E-04, 4.34929305E-04, 4.33894582E-04, 4.31926913E-04,
     &   4.45895835E-04, 4.50874443E-04, 4.63644613E-04, 4.69878732E-04,
     &   4.79176174E-04, 4.79484861E-04, 4.84644006E-04, 4.85757482E-04,
     &   4.90293558E-04, 4.98681730E-04, 5.07079685E-04, 5.20738727E-04,
     &   5.47128074E-04, 5.69712436E-04, 5.74474395E-04, 6.25737423E-04,
     &   6.64390194E-04, 6.98837988E-04, 7.05028357E-04, 7.56351098E-04,
     &   7.65270605E-04, 7.76834112E-04, 8.21997979E-04, 8.89021993E-04,
     &   9.00426105E-04, 1.01275757E-03, 1.07033790E-03, 1.10605144E-03,
     &   1.17276179E-03, 1.22240890E-03, 1.23325367E-03, 1.23934811E-03,
     &   1.24520120E-03, 1.25036018E-03, 1.26127236E-03, 1.26322998E-03,
     &   1.27462225E-03, 1.29135766E-03, 1.31365684E-03, 1.32601217E-03,
     &   1.62232692E-03, 1.89352890E-03, 1.62655922E-03, 1.18333859E-03/
  
         DATA ((DGS(I,J), I= 501, 600), J=2,2) /
     &   9.67728540E-04, 8.51001801E-04, 7.88131782E-04, 7.50843894E-04,
     &   7.40607732E-04, 6.74160130E-04, 6.25074034E-04, 5.88395508E-04,
     &   5.64505500E-04, 5.54884616E-04, 5.53946898E-04, 5.63071273E-04,
     &   5.80173231E-04, 6.05541815E-04, 6.51285116E-04, 7.36869242E-04,
     &   9.05198559E-04, 1.09089167E-03, 2.23866864E-03, 1.08999006E-02,
     &   1.20034786E-02, 1.33550494E-02, 1.50486813E-02, 1.72329019E-02,
     &   2.01567736E-02, 2.42722727E-02, 3.04937552E-02, 4.09922316E-02,
     &   6.24824401E-02, 1.31093966E-01, 1.38591359E-01, 1.46995110E-01,
     &   1.56479523E-01, 1.67266456E-01, 1.79642641E-01, 1.93985123E-01,
     &   2.10799573E-01, 2.30779751E-01, 2.54902886E-01, 2.84588718E-01,
     &   3.21976780E-01, 3.70435425E-01, 4.35552535E-01, 4.43305289E-01,
     &   6.16461483E-01, 7.42482313E-01, 8.44009338E-01, 9.32878917E-01,
     &   1.00204509E+00, 1.06329663E+00, 1.11325499E+00, 1.16426315E+00,
     &   1.20046959E+00, 1.23547332E+00, 1.26174508E+00, 1.28551371E+00,
     &   1.30100694E+00, 1.31739616E+00, 1.32327194E+00, 1.32846653E+00,
     &   1.32842341E+00, 1.32492658E+00, 1.31354102E+00, 1.30125947E+00,
     &   1.28471549E+00, 1.26345189E+00, 1.23700177E+00, 1.20489908E+00,
     &   1.16669208E+00, 1.12196002E+00, 1.07033266E+00, 1.01151264E+00,
     &   9.45299545E-01, 8.31985421E-01, 7.47271068E-01, 6.07254917E-01,
     &   4.53456789E-01, 5.43595734E-04, 4.53311035E-01, 6.07117866E-01,
     &   7.47144660E-01, 8.31866934E-01, 9.45193792E-01, 1.01141573E+00,
     &   1.07024474E+00, 1.12188110E+00, 1.16662210E+00, 1.20483788E+00,
     &   1.23694914E+00, 1.26340755E+00, 1.28467914E+00, 1.30123079E+00,
     &   1.31351966E+00, 1.32491554E+00, 1.32841881E+00, 1.32847094E+00,
     &   1.32328463E+00, 1.31741396E+00, 1.30103411E+00, 1.28554719E+00/
  
         DATA ((DGS(I,J), I= 601, 700), J=2,2) /
     &   1.26178613E+00, 1.23552107E+00, 1.20052467E+00, 1.16432455E+00,
     &   1.11332384E+00, 1.06337162E+00, 1.00212647E+00, 9.32966379E-01,
     &   8.44103284E-01, 7.42582259E-01, 6.16567261E-01, 5.06137531E-01,
     &   4.20922336E-01, 3.59747249E-01, 3.13859350E-01, 2.78232352E-01,
     &   2.49802235E-01, 2.26604405E-01, 2.07324941E-01, 1.91053462E-01,
     &   1.77140032E-01, 1.65108635E-01, 1.54602860E-01, 1.45350675E-01,
     &   1.37140922E-01, 1.29807233E-01, 1.23216776E-01, 1.17262233E-01,
     &   1.11855959E-01, 5.85495116E-02, 3.96586800E-02, 2.99914792E-02,
     &   2.41189154E-02, 2.01736160E-02, 1.73407684E-02, 1.52081316E-02,
     &   1.35447537E-02, 1.22111888E-02, 1.11182630E-02, 2.42731752E-03,
     &   1.41562758E-03, 1.11585883E-03, 9.84677366E-04, 9.68034507E-04,
     &   1.04886503E-03, 1.25396258E-03, 1.62653291E-03, 1.75724833E-03,
     &   2.32749387E-03, 3.47600016E-03, 4.75690770E-03, 7.66858568E-03,
     &   1.10725635E-02, 1.16473128E-02, 1.22854246E-02, 1.29977382E-02,
     &   1.37978846E-02, 1.47031408E-02, 1.57356317E-02, 1.69241568E-02,
     &   1.83069144E-02, 1.99356687E-02, 2.18823598E-02, 2.42499859E-02,
     &   2.71913674E-02, 3.09433777E-02, 3.58938272E-02, 4.27241127E-02,
     &   5.27506147E-02, 6.88827315E-02, 9.90301599E-02, 1.46718351E-01,
     &   1.74068925E-01, 2.04132805E-01, 2.79586904E-01, 3.07526131E-01,
     &   3.31035901E-01, 3.51429739E-01, 3.69017771E-01, 3.83843788E-01,
     &   3.97026429E-01, 4.08265394E-01, 4.16718884E-01, 4.24403966E-01,
     &   4.30477092E-01, 4.34901902E-01, 4.37730469E-01, 4.39191843E-01,
     &   4.39187967E-01, 4.37734292E-01, 4.34992035E-01, 4.30672342E-01,
     &   4.24565655E-01, 4.17742817E-01, 4.07730894E-01, 3.97229068E-01,
     &   3.84810721E-01, 3.70349194E-01, 3.50747572E-01, 3.31506245E-01/
  
         DATA ((DGS(I,J), I= 701, 800), J=2,2) /
     &   3.06133469E-01, 2.81861094E-01, 2.45936797E-01, 2.06152923E-01,
     &   1.45901537E-01, 9.28212103E-02, 3.13168788E-02, 1.14711660E-03,
     &   3.11833165E-02, 6.21664035E-02, 1.45775997E-01, 2.06036270E-01,
     &   2.45828081E-01, 2.81761236E-01, 3.06040685E-01, 3.31422014E-01,
     &   3.50670812E-01, 3.70281185E-01, 3.84750156E-01, 3.97175825E-01,
     &   4.07684802E-01, 4.17704796E-01, 4.24534288E-01, 4.30648400E-01,
     &   4.34975138E-01, 4.37724051E-01, 4.39184857E-01, 4.39194566E-01,
     &   4.37740152E-01, 4.34917971E-01, 4.30499630E-01, 4.24432921E-01,
     &   4.16754085E-01, 4.08306143E-01, 3.97073257E-01, 3.83896520E-01,
     &   3.69076075E-01, 3.51493611E-01, 3.31105189E-01, 3.07600605E-01,
     &   2.79666328E-01, 2.46403166E-01, 2.64390764E-01, 2.04222320E-01,
     &   1.46813187E-01, 1.24705963E-01, 8.05852759E-02, 5.94227484E-02,
     &   4.70438851E-02, 3.89295803E-02, 3.32029212E-02, 2.89463154E-02,
     &   2.56586331E-02, 2.30431634E-02, 2.09130427E-02, 1.91447568E-02,
     &   1.76534031E-02, 1.63787065E-02, 1.52766829E-02, 1.43144941E-02,
     &   1.34671276E-02, 1.27152860E-02, 1.20437431E-02, 1.14401404E-02,
     &   3.68723820E-03, 2.94554894E-03, 2.63613070E-03, 2.46665432E-03,
     &   2.35773502E-03, 2.28199750E-03, 2.22564069E-03, 2.18199162E-03,
     &   2.14695191E-03, 2.11785380E-03, 2.09315683E-03, 2.07162273E-03,
     &   2.05244828E-03, 2.03505148E-03, 2.01890214E-03, 2.00371845E-03,
     &   1.98916618E-03, 1.97497920E-03, 1.96109626E-03, 1.94725514E-03,
     &   1.93332783E-03, 1.91924087E-03, 1.90488501E-03, 1.89014125E-03,
     &   1.87490324E-03, 1.85907120E-03, 1.84257715E-03, 1.82528067E-03,
     &   1.80704987E-03, 1.78768589E-03, 1.76701082E-03, 1.74464095E-03,
     &   1.71990573E-03, 1.69118023E-03, 1.64153590E-03, 1.62206186E-03/
  
         DATA ((DGS(I,J), I= 801, 900), J=2,2) /
     &   1.62174649E-03, 1.62374453E-03, 1.62646230E-03, 1.62928803E-03,
     &   1.63188592E-03, 1.63408957E-03, 1.63575655E-03, 1.63681634E-03,
     &   1.63718099E-03, 1.63682061E-03, 1.63564536E-03, 1.63366285E-03,
     &   1.63080416E-03, 1.62704374E-03, 1.62227240E-03, 1.61649061E-03,
     &   1.60967740E-03, 1.60168956E-03, 1.59248831E-03, 1.58204528E-03,
     &   1.57015251E-03, 1.55680300E-03, 1.54185494E-03, 1.52501086E-03,
     &   1.50624628E-03, 1.48507389E-03, 1.46123491E-03, 1.43428071E-03,
     &   1.40325315E-03, 1.36817887E-03, 1.32613047E-03, 1.27260846E-03,
     &   1.19937635E-03, 1.17854478E-03, 1.11764755E-03, 1.03473089E-03,
     &   1.02714239E-03, 9.75235315E-04, 9.58159707E-04, 9.46205161E-04,
     &   9.29678551E-04, 9.13833382E-04, 9.02432235E-04, 8.96481720E-04,
     &   8.90470890E-04, 8.82434950E-04, 8.68274903E-04, 8.63731802E-04,
     &   8.37147604E-04, 8.07901714E-04, 8.07735192E-04, 7.86586317E-04,
     &   7.67006425E-04, 7.66369377E-04, 7.62775159E-04, 7.69838748E-04,
     &   7.92732182E-04, 8.61478595E-04, 7.46044410E-04, 6.89778895E-04,
     &   7.03445992E-04, 7.17661780E-04, 7.42068815E-04, 7.38715289E-04,
     &   7.21464675E-04, 7.21122342E-04, 7.15251374E-04, 6.92162419E-04,
     &   6.80225000E-04, 6.50367729E-04, 6.45267300E-04, 6.45932252E-04,
     &   6.34014250E-04, 6.25812202E-04, 6.20239982E-04, 6.08361905E-04,
     &   5.98987267E-04, 5.94994568E-04, 5.93779896E-04, 5.92425107E-04,
     &   5.92571209E-04, 5.90030231E-04, 5.86951526E-04, 5.83150854E-04,
     &   5.83164192E-04, 5.85253141E-04, 5.82846463E-04, 5.70587009E-04,
     &   5.63207938E-04, 5.60197418E-04, 5.58092503E-04, 5.56837475E-04,
     &   5.55263548E-04, 5.54898492E-04, 5.53000048E-04, 5.48433921E-04,
     &   5.46335667E-04, 5.44313871E-04, 5.42267804E-04, 5.39066554E-04/
  
         DATA ((DGS(I,J), I= 901,1000), J=2,2) /
     &   5.37304994E-04, 5.36280360E-04, 5.35989858E-04, 5.35948535E-04,
     &   5.38715404E-04, 5.32510661E-04, 5.27955958E-04, 5.27175096E-04,
     &   5.26666251E-04, 5.24449590E-04, 5.24394062E-04, 5.25965706E-04,
     &   5.20708450E-04, 5.25829548E-04, 5.36942913E-04, 5.19540275E-04,
     &   5.18224343E-04, 5.13052582E-04, 5.12748846E-04, 5.18244797E-04,
     &   5.09125685E-04, 5.06839733E-04, 5.04738209E-04, 5.03084918E-04,
     &   4.99481325E-04, 5.01161113E-04, 5.01054802E-04, 5.00486523E-04,
     &   4.98096789E-04, 4.99208335E-04, 5.00113585E-04, 4.95172076E-04,
     &   4.94653654E-04, 4.94559268E-04, 4.89168459E-04, 4.89369788E-04,
     &   4.90567359E-04, 4.89236089E-04, 4.90299212E-04, 4.92100036E-04,
     &   4.84982143E-04, 4.84333741E-04, 4.84128979E-04, 4.91729202E-04,
     &   4.88348385E-04, 4.87696456E-04, 4.85351671E-04, 4.91792365E-04,
     &   5.03870707E-04, 5.10921951E-04, 5.03887290E-04, 5.03383353E-04,
     &   5.08798281E-04, 5.05177611E-04, 5.08971683E-04, 5.23358928E-04,
     &   5.72752806E-04, 5.43920416E-04, 5.45928371E-04, 5.62695985E-04,
     &   5.31805991E-04, 5.26349045E-04, 5.24337247E-04, 5.46934531E-04,
     &   5.49943173E-04, 5.94526711E-04, 5.98842285E-04, 6.73317649E-04,
     &   7.72928930E-04, 8.21598595E-04, 1.00250865E-03, 1.14691852E-03,
     &   1.13832031E-03, 9.73131890E-04, 8.14993474E-04, 6.78729458E-04,
     &   6.40112594E-04, 6.10584767E-04, 5.67791061E-04, 5.37844146E-04,
     &   5.25921090E-04, 5.15489340E-04, 5.06274248E-04, 4.98067366E-04,
     &   4.90707245E-04, 4.84063014E-04, 4.78040001E-04, 4.72552289E-04,
     &   4.67532272E-04, 4.62923969E-04, 4.58680612E-04, 4.54762819E-04,
     &   4.51137192E-04, 4.47775232E-04, 4.44652489E-04, 4.41747885E-04,
     &   4.39043173E-04, 4.36522508E-04, 4.34172087E-04, 4.31979866E-04/
  
         DATA ((DGS(I,J), I=1001,1100), J=2,2) /
     &   4.29935322E-04, 4.28029259E-04, 4.26253643E-04, 4.24601472E-04,
     &   4.23066662E-04, 4.21643957E-04, 4.20328848E-04, 4.19117517E-04,
     &   4.18006778E-04, 4.16994041E-04, 4.16077278E-04, 4.15255001E-04,
     &   4.14526241E-04, 4.13890543E-04, 4.13449009E-04, 4.12899068E-04,
     &   4.12287257E-04, 4.12070552E-04, 4.12274124E-04, 4.12934455E-04,
     &   4.14101708E-04, 4.15843199E-04, 4.18248452E-04, 4.21436692E-04,
     &   4.25568173E-04, 4.30861911E-04, 4.37627518E-04, 4.46301990E-04,
     &   4.57564103E-04, 4.72508054E-04, 4.74268463E-04, 4.93082217E-04,
     &   5.23274703E-04, 5.73260289E-04, 6.50037350E-04, 6.81244577E-04,
     &   8.12674527E-04, 1.07125917E-03, 1.19613273E-03, 1.37565468E-03,
     &   1.49766002E-03, 1.65213831E-03, 2.12424411E-03, 3.08586640E-03,
     &   5.63424111E-03, 5.74833506E-03, 5.86724879E-03, 5.99129218E-03,
     &   6.12080225E-03, 6.25614622E-03, 6.39772488E-03, 6.54597663E-03,
     &   6.70138192E-03, 6.86446844E-03, 7.03581706E-03, 7.21606874E-03,
     &   7.40593245E-03, 7.60619448E-03, 7.81772926E-03, 8.04151207E-03,
     &   8.27863401E-03, 8.53031969E-03, 8.79794836E-03, 9.08307905E-03,
     &   9.38748087E-03, 9.71316965E-03, 1.00624525E-02, 1.04379822E-02,
     &   1.08428250E-02, 1.12805438E-02, 1.17553031E-02, 1.22720020E-02,
     &   1.28364432E-02, 1.34555514E-02, 1.41376585E-02, 1.48928799E-02,
     &   1.57336191E-02, 1.66752529E-02, 1.77370790E-02, 1.89436516E-02,
     &   2.03267007E-02, 2.19279607E-02, 2.38034495E-02, 2.60301508E-02,
     &   2.87168330E-02, 3.20223377E-02, 3.61881346E-02, 4.16000608E-02,
     &   4.89151146E-02, 5.93502483E-02, 7.54396305E-02, 1.03473001E-01,
     &   1.64489734E-01, 2.02334114E-01, 3.85697852E-01, 5.37269573E-01,
     &   6.47542137E-01, 7.33191594E-01, 8.03841294E-01, 8.65931923E-01/
  
         DATA ((DGS(I,J), I=1101,1200), J=2,2) /
     &   9.23828765E-01, 9.64344642E-01, 1.00582180E+00, 1.04693464E+00,
     &   1.06691786E+00, 1.09519206E+00, 1.11236662E+00, 1.13417378E+00,
     &   1.14004377E+00, 1.14915862E+00, 1.15386715E+00, 1.15303306E+00,
     &   1.15015213E+00, 1.14539683E+00, 1.12938439E+00, 1.11780007E+00,
     &   1.10336000E+00, 1.06556265E+00, 1.04182621E+00, 1.01452632E+00,
     &   9.83117812E-01, 9.08938821E-01, 8.65111279E-01, 8.17427571E-01,
     &   7.08518981E-01, 6.47986411E-01, 5.14311893E-01, 3.66606429E-01,
     &   3.91846223E-02, 3.61748806E-01, 5.09847767E-01, 6.43343068E-01,
     &   7.61404624E-01, 8.14253325E-01, 8.62210686E-01, 9.05874413E-01,
     &   9.81010106E-01, 1.01235667E+00, 1.04019916E+00, 1.06415571E+00,
     &   1.10235412E+00, 1.11683680E+00, 1.12872333E+00, 1.14496424E+00,
     &   1.14990720E+00, 1.15419074E+00, 1.15393224E+00, 1.14938110E+00,
     &   1.14044742E+00, 1.12800001E+00, 1.11288291E+00, 1.09576287E+00,
     &   1.06754234E+00, 1.04768652E+00, 1.00648522E+00, 9.65111013E-01,
     &   9.24468832E-01, 8.66531155E-01, 8.04385688E-01, 7.33666895E-01,
     &   6.47931359E-01, 5.37553048E-01, 3.83627065E-01, 2.09216902E-01,
     &   1.19642637E-01, 8.37059837E-02, 6.43601823E-02, 5.22757985E-02,
     &   4.40119389E-02, 3.80046481E-02, 3.34409396E-02, 2.98564296E-02,
     &   2.69666029E-02, 2.45874250E-02, 2.25945867E-02, 2.09010960E-02,
     &   1.94442531E-02, 1.81777250E-02, 1.70665325E-02, 1.60837735E-02,
     &   1.52084187E-02, 1.44237923E-02, 1.37165007E-02, 1.30756630E-02,
     &   1.24923480E-02, 1.19591566E-02, 1.14699063E-02, 1.10193914E-02,
     &   1.06031973E-02, 1.02175558E-02, 9.85923070E-03, 9.52542751E-03,
     &   9.21372031E-03, 8.92199300E-03, 8.64839139E-03, 8.39128387E-03,
     &   8.14922901E-03, 7.92094859E-03, 7.70530521E-03, 7.50128336E-03/
  
         DATA ((DGS(I,J), I=1201,1300), J=2,2) /
     &   7.30797352E-03, 7.12455872E-03, 6.95030303E-03, 6.78454178E-03,
     &   6.62667311E-03, 6.47615074E-03, 6.33247771E-03, 6.19520090E-03,
     &   6.06390635E-03, 5.93821508E-03, 5.81777949E-03, 5.70228017E-03,
     &   5.59142311E-03, 3.22122341E-03, 2.72371951E-03, 2.24762574E-03,
     &   1.57818002E-03, 8.66745290E-04, 7.53908012E-04, 7.51350839E-04,
     &   9.10425856E-04, 3.50105241E-03, 7.32701010E-03, 7.71051142E-03,
     &   8.13679483E-03, 8.61340395E-03, 9.14976832E-03, 9.75783542E-03,
     &   1.04529662E-02, 1.12552479E-02, 1.21914639E-02, 1.32981372E-02,
     &   1.46263994E-02, 1.62501064E-02, 1.82800388E-02, 2.08902732E-02,
     &   2.43709190E-02, 2.92441206E-02, 3.65539917E-02, 4.87360239E-02,
     &   7.30912979E-02, 1.05749236E-01, 1.11228921E-01, 1.17306897E-01,
     &   1.24086692E-01, 1.31697148E-01, 1.40300652E-01, 1.50104796E-01,
     &   1.61379245E-01, 1.74480765E-01, 1.89891368E-01, 2.08278367E-01,
     &   2.30592489E-01, 2.58235355E-01, 2.93360686E-01, 3.39451590E-01,
     &   4.02517236E-01, 4.93831024E-01, 6.37030936E-01, 8.89452943E-01,
     &   1.40553591E+00, 1.56025992E+00, 1.66865793E+00, 1.77151011E+00,
     &   1.83744917E+00, 1.86648570E+00, 1.86585238E+00, 1.84158892E+00,
     &   1.79194888E+00, 1.71087763E+00, 1.59225241E+00, 1.49240482E+00,
     &   1.47535603E+00, 1.46665945E+00, 1.44892014E+00, 1.44169473E+00,
     &   1.43622691E+00, 1.43071721E+00, 1.37330606E+00, 1.27924973E+00,
     &   1.16358296E+00, 1.04964443E+00, 9.12622065E-01, 7.65474605E-01,
     &   6.25374812E-01, 5.44816229E-01, 4.62579441E-01, 3.78896753E-01,
     &   2.59784679E-01, 1.73663076E-01, 8.69820192E-02, 3.48228409E-02,
     &   1.74287764E-02, 6.17543168E-04, 1.73966959E-02, 8.69499567E-02,
     &   1.73631130E-01, 2.59752927E-01, 6.57049632E-01, 8.10588114E-01/
  
         DATA ((DGS(I,J), I=1301,1400), J=2,2) /
     &   1.07572587E+00, 1.18762404E+00, 1.30095410E+00, 1.40251840E+00,
     &   1.48391911E+00, 1.56350706E+00, 1.62573059E+00, 1.68495936E+00,
     &   1.72989363E+00, 1.77120152E+00, 1.80123892E+00, 1.82739696E+00,
     &   1.84505630E+00, 1.86649927E+00, 1.87040284E+00, 1.87002491E+00,
     &   1.86648758E+00, 1.85075299E+00, 1.83877123E+00, 1.81546631E+00,
     &   1.11552528E+00, 7.51732724E-01, 5.61355315E-01, 4.46650846E-01,
     &   3.70454500E-01, 3.16299575E-01, 2.75880853E-01, 2.44581633E-01,
     &   2.19638507E-01, 1.99299006E-01, 1.82399179E-01, 1.68136186E-01,
     &   1.55938581E-01, 1.45388674E-01, 1.36174147E-01, 1.28056832E-01,
     &   1.20851965E-01, 1.14414022E-01, 1.08626838E-01, 7.31153602E-02,
     &   4.87601006E-02, 3.65780814E-02, 2.92682264E-02, 2.43950459E-02,
     &   2.09144273E-02, 1.83042273E-02, 1.62743378E-02, 1.46506837E-02,
     &   1.33224863E-02, 1.22158920E-02, 1.12796856E-02, 1.04773765E-02,
     &   9.78220583E-03, 9.17408498E-03, 8.63765132E-03, 8.16095577E-03,
     &   7.73456665E-03, 7.35093805E-03, 4.85632111E-03, 1.24713761E-03,
     &   8.66747731E-04, 1.14218117E-03, 4.39823576E-03, 4.70456013E-03,
     &   5.05861435E-03, 5.47230896E-03, 5.96191662E-03, 6.55023126E-03,
     &   7.27014369E-03, 8.17101982E-03, 9.33044044E-03, 1.08776451E-02,
     &   1.30453284E-02, 1.62988897E-02, 2.17239864E-02, 2.26676956E-02,
     &   2.36971193E-02, 2.48247410E-02, 2.60651645E-02, 2.74360404E-02,
     &   2.89594298E-02, 3.06620736E-02, 3.25773517E-02, 3.47482364E-02,
     &   3.72292431E-02, 4.00915737E-02, 4.34312130E-02, 4.73778940E-02,
     &   5.21130458E-02, 5.79004304E-02, 4.39823576E-03, 6.51336092E-02,
     &   7.44302042E-02, 8.68225261E-02, 1.04161868E-01, 1.30144059E-01,
     &   1.73350129E-01, 2.59292575E-01, 5.10774217E-01, 6.05174441E-01/
  
         DATA ((DGS(I,J), I=1401,1500), J=2,2) /
     &   8.43624470E-01, 1.01812575E+00, 1.15624740E+00, 1.27179201E+00,
     &   1.36974840E+00, 1.45343296E+00, 1.52435082E+00, 1.58638886E+00,
     &   1.64121539E+00, 1.68759144E+00, 1.72489361E+00, 1.75587786E+00,
     &   1.78020754E+00, 1.79789504E+00, 1.81042971E+00, 1.81616983E+00,
     &   1.81613600E+00, 1.81052482E+00, 1.79842787E+00, 1.77885525E+00,
     &   1.75532646E+00, 1.72482819E+00, 1.68664649E+00, 1.64007006E+00,
     &   1.58441291E+00, 1.53063647E+00, 1.45674922E+00, 1.37223966E+00,
     &   1.27681893E+00, 1.15159600E+00, 1.01167654E+00, 8.34794976E-01,
     &   6.16677483E-01, 7.42769336E-04, 6.16660250E-01, 8.34778811E-01,
     &   1.01166153E+00, 1.15158213E+00, 1.27680628E+00, 1.37222808E+00,
     &   1.45673874E+00, 1.53062710E+00, 1.58440445E+00, 1.64006269E+00,
     &   1.68664017E+00, 1.72482289E+00, 1.75532214E+00, 1.77885187E+00,
     &   1.79842553E+00, 1.81052346E+00, 1.81613556E+00, 1.81617025E+00,
     &   1.81043105E+00, 1.79789731E+00, 1.78021067E+00, 1.75588186E+00,
     &   1.72489846E+00, 1.68759711E+00, 1.64122189E+00, 1.58639619E+00,
     &   1.52435892E+00, 1.45344183E+00, 1.36975801E+00, 1.27180237E+00,
     &   1.15625849E+00, 1.01813755E+00, 8.43636965E-01, 6.05187620E-01,
     &   5.10734120E-01, 2.59292301E-01, 1.73363909E-01, 1.30154308E-01,
     &   1.04173379E-01, 8.68364034E-02, 7.44440740E-02, 6.51465770E-02,
     &   5.79142654E-02, 5.21268545E-02, 4.73912286E-02, 4.34450579E-02,
     &   4.01054022E-02, 3.72427628E-02, 3.47620249E-02, 3.25911166E-02,
     &   2.17378200E-02, 1.63127106E-02, 1.30593436E-02, 1.08914431E-02,
     &   9.34452274E-03, 8.18514333E-03, 7.28394277E-03, 6.56445876E-03,
     &   5.97620782E-03, 5.48718825E-03, 5.07326370E-03, 4.71916377E-03,
     &   4.41278724E-03, 1.31111766E-03, 9.08668013E-04, 1.01520924E-03/
  
         DATA ((DGS(I,J), I=1501,1600), J=2,2) /
     &   1.14979500E-03, 1.26457842E-03, 1.36505156E-03, 1.45208774E-03,
     &   1.52870458E-03, 1.59568152E-03, 1.65445549E-03, 1.70543158E-03,
     &   1.74949119E-03, 1.78736498E-03, 1.81905594E-03, 1.84477146E-03,
     &   1.86501436E-03, 1.88008177E-03, 1.88974977E-03, 1.89420203E-03,
     &   1.89359935E-03, 1.88771278E-03, 1.87665770E-03, 1.86068036E-03,
     &   1.83858596E-03, 1.81174270E-03, 1.77841796E-03, 1.73904220E-03,
     &   1.69255742E-03, 1.63862327E-03, 1.57704063E-03, 1.50782274E-03,
     &   1.42572117E-03, 1.33198000E-03, 1.22512058E-03, 1.09642826E-03,
     &   9.41079683E-04, 7.36077520E-04, 9.32385489E-04, 1.08661592E-03,
     &   1.21513011E-03, 1.32216688E-03, 1.41625976E-03, 1.49881041E-03,
     &   1.56851226E-03, 1.63061639E-03, 1.68509091E-03, 1.73212092E-03,
     &   1.77203540E-03, 1.80589613E-03, 1.83325422E-03, 1.85586898E-03,
     &   1.87233235E-03, 1.88386467E-03, 1.89021088E-03, 1.89123866E-03,
     &   1.88717971E-03, 1.87787739E-03, 1.86313322E-03, 1.84316283E-03,
     &   1.81766464E-03, 1.78611975E-03, 1.74830645E-03, 1.70459014E-03,
     &   1.65302984E-03, 1.59386935E-03, 1.52624034E-03, 1.44854808E-03,
     &   1.35981936E-03, 1.25646642E-03, 1.13652989E-03, 9.89544595E-04,
     &   8.01507492E-04, 7.98051621E-04, 7.67394607E-04, 7.57533070E-04,
     &   7.55131447E-04, 7.55012349E-04, 7.54851711E-04, 7.55962311E-04,
     &   7.58018398E-04, 7.60220722E-04, 7.62307150E-04, 7.63947311E-04,
     &   7.65070465E-04, 7.65876515E-04, 7.66121591E-04, 7.65939455E-04,
     &   7.65166825E-04, 7.63586105E-04, 7.61419468E-04, 7.58519504E-04,
     &   7.54938164E-04, 7.50671654E-04, 7.45647785E-04, 7.39705420E-04,
     &   7.32967815E-04, 7.25378953E-04, 7.16900886E-04, 7.07519210E-04,
     &   6.97188615E-04, 6.85843496E-04, 6.73238199E-04, 6.59418825E-04/
  
         DATA ((DGS(I,J), I=1601,1700), J=2,2) /
     &   6.44743093E-04, 6.26331594E-04, 6.06362954E-04, 5.85193640E-04,
     &   5.55158560E-04, 5.66819293E-04, 5.80267819E-04, 5.92694973E-04,
     &   6.05391951E-04, 6.17155328E-04, 6.27335472E-04, 6.37070464E-04,
     &   6.46360167E-04, 6.54958649E-04, 6.62983301E-04, 6.70496735E-04,
     &   6.77484149E-04, 6.83917535E-04, 6.89845925E-04, 6.95288596E-04,
     &   7.00173623E-04, 7.04591677E-04, 7.08555853E-04, 7.12032050E-04,
     &   7.15001030E-04, 7.17482288E-04, 7.19569545E-04, 7.21256738E-04,
     &   7.22594490E-04, 7.23538245E-04, 7.24020648E-04, 7.24150199E-04,
     &   7.24103384E-04, 7.23997245E-04, 7.23891457E-04, 7.24001106E-04,
     &   7.24726511E-04, 7.22270152E-04, 7.17189388E-04, 7.11510235E-04,
     &   6.93367025E-04, 6.92924869E-04, 6.92389060E-04, 6.91742851E-04,
     &   6.90953194E-04, 6.90010651E-04, 6.88937710E-04, 6.87719776E-04,
     &   6.86340083E-04, 6.82392242E-04, 6.78334199E-04, 6.74370694E-04,
     &   6.70449290E-04, 6.66500188E-04, 6.62453488E-04, 6.58320828E-04,
     &   6.54129109E-04, 6.49782177E-04, 6.45266864E-04, 6.40590378E-04,
     &   6.35665504E-04, 6.30616822E-04, 6.25369200E-04, 6.19853466E-04,
     &   6.13948536E-04, 6.08042704E-04, 6.01721751E-04, 5.95338287E-04,
     &   5.87778796E-04, 5.79935819E-04, 5.71978829E-04, 5.63951791E-04,
     &   5.54587010E-04, 5.44463445E-04, 5.33978789E-04, 5.19492824E-04,
     &   5.22760714E-04, 5.28805806E-04, 5.35684151E-04, 5.42449845E-04,
     &   5.48232828E-04, 5.54246452E-04, 5.60349033E-04, 5.66328053E-04,
     &   5.71200852E-04, 5.76140617E-04, 5.80709174E-04, 5.85207083E-04,
     &   5.89509648E-04, 5.93582545E-04, 5.97493397E-04, 6.01284522E-04,
     &   6.04846900E-04, 6.08202634E-04, 6.11462392E-04, 6.14526525E-04,
     &   6.17498237E-04, 6.20355988E-04, 6.23073983E-04, 6.25698699E-04/
  
         DATA ((DGS(I,J), I=1701,1800), J=2,2) /
     &   6.28290699E-04, 6.30881574E-04, 6.33385735E-04, 6.34667090E-04,
     &   6.36026922E-04, 6.37563058E-04, 6.39425630E-04, 6.41785720E-04,
     &   6.43981172E-04, 6.43084178E-04, 6.42031644E-04, 6.35553619E-04,
     &   6.32722530E-04, 6.38481939E-04, 6.52943501E-04, 6.77138567E-04,
     &   7.07020794E-04, 7.43944825E-04, 7.83997270E-04, 8.30037240E-04,
     &   8.80574847E-04, 9.33285662E-04, 9.94100620E-04, 1.05866714E-03,
     &   1.08459495E-03, 1.11052275E-03, 1.06058022E-03, 1.01572215E-03,
     &   9.78311990E-04, 9.43793945E-04, 9.14371678E-04, 8.86731378E-04,
     &   8.62811233E-04, 8.40047426E-04, 8.19258813E-04, 8.00979968E-04,
     &   7.83340850E-04, 7.67711680E-04, 7.52525486E-04, 7.38985861E-04,
     &   7.25755720E-04, 7.13899228E-04, 7.02259084E-04, 6.91782138E-04,
     &   6.81455106E-04, 6.72125431E-04, 6.63288074E-04, 6.54533704E-04,
     &   6.46587893E-04, 6.38694364E-04, 6.31510750E-04, 6.24356461E-04,
     &   6.17830127E-04, 6.11315835E-04, 6.05360655E-04, 5.99404443E-04,
     &   5.93948962E-04, 5.88482538E-04, 5.83243676E-04, 5.78432835E-04,
     &   5.73600446E-04, 5.69156085E-04, 5.64685279E-04, 5.60567681E-04,
     &   5.56419988E-04, 5.52595010E-04, 5.48737258E-04, 5.45175371E-04,
     &   5.41578791E-04, 5.38254318E-04, 5.35037567E-04, 5.31784329E-04,
     &   5.28772604E-04, 5.25723730E-04, 5.22898513E-04, 5.20035813E-04,
     &   5.17380742E-04, 5.14688113E-04, 5.12188676E-04, 5.09651810E-04,
     &   5.07295087E-04, 5.04901226E-04, 5.02575971E-04, 5.00413410E-04,
     &   4.98214374E-04, 4.96110359E-04, 4.94027480E-04, 4.92087727E-04,
     &   4.90112668E-04, 4.88272170E-04, 4.86397022E-04, 4.84648582E-04,
     &   4.82866183E-04, 4.81203274E-04, 4.79507120E-04, 4.77923809E-04,
     &   4.76377415E-04, 4.74798865E-04, 4.73324198E-04, 4.71818112E-04/
  
         DATA ((DGS(I,J), I=1801,1840), J=2,2) /
     &   4.70410451E-04, 4.68972107E-04, 4.67627129E-04, 4.66252201E-04,
     &   4.64965941E-04, 4.63650457E-04, 4.62419273E-04, 4.61159580E-04,
     &   4.49057701E-04, 4.39297702E-04, 4.37909907E-04, 4.00024984E-04,
     &   3.91187927E-04, 3.77404719E-04, 3.72815242E-04, 3.72627822E-04,
     &   3.68414722E-04, 3.68403774E-04, 3.66134945E-04, 3.66059772E-04,
     &   3.66015159E-04, 3.65106379E-04, 3.64561315E-04, 3.64468241E-04,
     &   3.64319028E-04, 3.64104296E-04, 3.63957241E-04, 3.63866701E-04,
     &   3.63055500E-04, 3.63017465E-04, 3.62800961E-04, 3.62762034E-04,
     &   3.62590475E-04, 3.62524232E-04, 3.62472179E-04, 3.62424122E-04,
     &   3.62287163E-04, 3.62095197E-04, 3.62016090E-04, 3.61527294E-04/
c
      save
      RETURN
      END


      subroutine dalhad_timelike1n
      IMPLICIT NONE
      INTEGER NC,I,J
      PARAMETER(NC=200)
      REAL EMA(NC),DAM(NC,2),DGM(NC,2)
      COMMON /DATM09/EMA,DAM,DGM
  
       DATA  (EMA(I), I=   1, 100) /
     &   1.3005E+01, 1.3500E+01, 1.4000E+01, 1.4500E+01, 1.5000E+01,
     &   1.6000E+01, 1.7000E+01, 1.8000E+01, 1.9000E+01, 2.0000E+01,
     &   2.1000E+01, 2.2000E+01, 2.3000E+01, 2.4000E+01, 2.5000E+01,
     &   2.6000E+01, 2.7000E+01, 2.8000E+01, 2.9000E+01, 3.0000E+01,
     &   3.1000E+01, 3.2000E+01, 3.3000E+01, 3.4000E+01, 3.5000E+01,
     &   3.6000E+01, 3.7000E+01, 3.8000E+01, 3.9000E+01, 4.0000E+01,
     &   4.1000E+01, 4.2000E+01, 4.3000E+01, 4.4000E+01, 4.5000E+01,
     &   4.6000E+01, 4.7000E+01, 4.8000E+01, 4.9000E+01, 5.0000E+01,
     &   5.1000E+01, 5.2000E+01, 5.3000E+01, 5.4000E+01, 5.5000E+01,
     &   5.6000E+01, 5.7000E+01, 5.8000E+01, 5.9000E+01, 6.0000E+01,
     &   6.1000E+01, 6.2000E+01, 6.3000E+01, 6.4000E+01, 6.5000E+01,
     &   6.6000E+01, 6.7000E+01, 6.8000E+01, 6.9000E+01, 7.0000E+01,
     &   7.1000E+01, 7.2000E+01, 7.3000E+01, 7.4000E+01, 7.5000E+01,
     &   7.6000E+01, 7.7000E+01, 7.8000E+01, 7.9000E+01, 8.0000E+01,
     &   8.1000E+01, 8.2000E+01, 8.3000E+01, 8.4000E+01, 8.5000E+01,
     &   8.6000E+01, 8.7000E+01, 8.8000E+01, 8.9000E+01, 9.0000E+01,
     &   9.1000E+01, 9.2000E+01, 9.3000E+01, 9.4000E+01, 9.5000E+01,
     &   9.6000E+01, 9.7000E+01, 9.8000E+01, 9.9000E+01, 1.0000E+02,
     &   1.0900E+02, 1.1800E+02, 1.2700E+02, 1.3600E+02, 1.4500E+02,
     &   1.5400E+02, 1.6300E+02, 1.7200E+02, 1.8100E+02, 1.9000E+02/
  
       DATA  (EMA(I), I= 101, 200) /
     &   1.9900E+02, 2.0800E+02, 2.1700E+02, 2.2600E+02, 2.3500E+02,
     &   2.4400E+02, 2.5300E+02, 2.6200E+02, 2.7100E+02, 2.8000E+02,
     &   2.8900E+02, 2.9800E+02, 3.0700E+02, 3.1600E+02, 3.2500E+02,
     &   3.3400E+02, 3.4300E+02, 3.5200E+02, 3.6100E+02, 3.7000E+02,
     &   3.7900E+02, 3.8800E+02, 3.9700E+02, 4.0600E+02, 4.1500E+02,
     &   4.2400E+02, 4.3300E+02, 4.4200E+02, 4.5100E+02, 4.6000E+02,
     &   4.6900E+02, 4.7800E+02, 4.8700E+02, 4.9600E+02, 5.0500E+02,
     &   5.1400E+02, 5.2300E+02, 5.3200E+02, 5.4100E+02, 5.5000E+02,
     &   5.5900E+02, 5.6800E+02, 5.7700E+02, 5.8600E+02, 5.9500E+02,
     &   6.0400E+02, 6.1300E+02, 6.2200E+02, 6.3100E+02, 6.4000E+02,
     &   6.4900E+02, 6.5800E+02, 6.6700E+02, 6.7600E+02, 6.8500E+02,
     &   6.9400E+02, 7.0300E+02, 7.1200E+02, 7.2100E+02, 7.3000E+02,
     &   7.3900E+02, 7.4800E+02, 7.5700E+02, 7.6600E+02, 7.7500E+02,
     &   7.8400E+02, 7.9300E+02, 8.0200E+02, 8.1100E+02, 8.2000E+02,
     &   8.2900E+02, 8.3800E+02, 8.4700E+02, 8.5600E+02, 8.6500E+02,
     &   8.7400E+02, 8.8300E+02, 8.9200E+02, 9.0100E+02, 9.1000E+02,
     &   9.1900E+02, 9.2800E+02, 9.3700E+02, 9.4600E+02, 9.5500E+02,
     &   9.6400E+02, 9.7300E+02, 9.8200E+02, 9.9100E+02, 1.0000E+03,
     &   2.0000E+03, 3.0000E+03, 4.0000E+03, 5.0000E+03, 6.0000E+03,
     &   8.0000E+03, 1.0000E+04, 1.2000E+04, 1.4000E+04, 2.0000E+04/
  
  
         DATA ((DAM(I,J), I=   1, 100), J=1,1) /
     &   1.5790E-02, 1.6037E-02, 1.6275E-02, 1.6502E-02, 1.6718E-02,
     &   1.7126E-02, 1.7504E-02, 1.7858E-02, 1.8191E-02, 1.8505E-02,
     &   1.8803E-02, 1.9085E-02, 1.9355E-02, 1.9612E-02, 1.9859E-02,
     &   2.0095E-02, 2.0321E-02, 2.0539E-02, 2.0749E-02, 2.0952E-02,
     &   2.1148E-02, 2.1338E-02, 2.1522E-02, 2.1700E-02, 2.1873E-02,
     &   2.2041E-02, 2.2204E-02, 2.2363E-02, 2.2518E-02, 2.2668E-02,
     &   2.2815E-02, 2.2958E-02, 2.3098E-02, 2.3235E-02, 2.3368E-02,
     &   2.3498E-02, 2.3626E-02, 2.3751E-02, 2.3873E-02, 2.3993E-02,
     &   2.4110E-02, 2.4225E-02, 2.4338E-02, 2.4449E-02, 2.4558E-02,
     &   2.4664E-02, 2.4769E-02, 2.4872E-02, 2.4973E-02, 2.5073E-02,
     &   2.5171E-02, 2.5267E-02, 2.5361E-02, 2.5454E-02, 2.5546E-02,
     &   2.5636E-02, 2.5725E-02, 2.5813E-02, 2.5899E-02, 2.5984E-02,
     &   2.6068E-02, 2.6151E-02, 2.6232E-02, 2.6312E-02, 2.6392E-02,
     &   2.6470E-02, 2.6547E-02, 2.6623E-02, 2.6698E-02, 2.6773E-02,
     &   2.6846E-02, 2.6918E-02, 2.6990E-02, 2.7061E-02, 2.7130E-02,
     &   2.7199E-02, 2.7268E-02, 2.7335E-02, 2.7402E-02, 2.7468E-02,
     &   2.7533E-02, 2.7597E-02, 2.7661E-02, 2.7724E-02, 2.7786E-02,
     &   2.7848E-02, 2.7909E-02, 2.7970E-02, 2.8030E-02, 2.8089E-02,
     &   2.8597E-02, 2.9064E-02, 2.9496E-02, 2.9899E-02, 3.0276E-02,
     &   3.0630E-02, 3.0964E-02, 3.1280E-02, 3.1580E-02, 3.1865E-02/
  
         DATA ((DAM(I,J), I= 101, 200), J=1,1) /
     &   3.2136E-02, 3.2396E-02, 3.2645E-02, 3.2883E-02, 3.3112E-02,
     &   3.3333E-02, 3.3545E-02, 3.3750E-02, 3.3948E-02, 3.4140E-02,
     &   3.4325E-02, 3.4505E-02, 3.4679E-02, 3.4848E-02, 3.5013E-02,
     &   3.5172E-02, 3.5328E-02, 3.5478E-02, 3.5628E-02, 3.5772E-02,
     &   3.5914E-02, 3.6051E-02, 3.6186E-02, 3.6317E-02, 3.6446E-02,
     &   3.6572E-02, 3.6695E-02, 3.6816E-02, 3.6934E-02, 3.7050E-02,
     &   3.7164E-02, 3.7275E-02, 3.7385E-02, 3.7492E-02, 3.7597E-02,
     &   3.7702E-02, 3.7804E-02, 3.7906E-02, 3.8002E-02, 3.8100E-02,
     &   3.8195E-02, 3.8288E-02, 3.8381E-02, 3.8472E-02, 3.8562E-02,
     &   3.8650E-02, 3.8736E-02, 3.8825E-02, 3.8903E-02, 3.8990E-02,
     &   3.9075E-02, 3.9155E-02, 3.9238E-02, 3.9316E-02, 3.9394E-02,
     &   3.9473E-02, 3.9549E-02, 3.9616E-02, 3.9697E-02, 3.9769E-02,
     &   3.9831E-02, 3.9915E-02, 3.9987E-02, 4.0041E-02, 4.0134E-02,
     &   4.0224E-02, 4.0301E-02, 4.0393E-02, 4.0453E-02, 4.0501E-02,
     &   4.0526E-02, 4.0586E-02, 4.0630E-02, 4.0673E-02, 4.0716E-02,
     &   4.0779E-02, 4.0816E-02, 4.0872E-02, 4.0946E-02, 4.0982E-02,
     &   4.1041E-02, 4.1028E-02, 4.1147E-02, 4.1206E-02, 4.1265E-02,
     &   4.1359E-02, 4.1371E-02, 4.1433E-02, 4.1493E-02, 4.1487E-02,
     &   4.5662E-02, 4.8013E-02, 4.9690E-02, 5.0992E-02, 5.2054E-02,
     &   5.3724E-02, 5.5022E-02, 5.6086E-02, 5.6980E-02, 5.9051E-02/
  
         DATA ((DAM(I,J), I=   1, 100), J=2,2) /
     &   2.1016E-04, 2.0196E-04, 1.9683E-04, 1.9334E-04, 1.9081E-04,
     &   1.8736E-04, 1.8509E-04, 1.8348E-04, 1.8227E-04, 1.8133E-04,
     &   1.8058E-04, 1.7996E-04, 1.7945E-04, 1.7901E-04, 1.7864E-04,
     &   1.7832E-04, 1.7804E-04, 1.7780E-04, 1.7759E-04, 1.7739E-04,
     &   1.7722E-04, 1.7707E-04, 1.7693E-04, 1.7681E-04, 1.7670E-04,
     &   1.7660E-04, 1.7650E-04, 1.7642E-04, 1.7634E-04, 1.7627E-04,
     &   1.7620E-04, 1.7614E-04, 1.7608E-04, 1.7603E-04, 1.7598E-04,
     &   1.7593E-04, 1.7589E-04, 1.7585E-04, 1.7581E-04, 1.7578E-04,
     &   1.7574E-04, 1.7571E-04, 1.7568E-04, 1.7565E-04, 1.7563E-04,
     &   1.7560E-04, 1.7558E-04, 1.7556E-04, 1.7554E-04, 1.7552E-04,
     &   1.7550E-04, 1.7548E-04, 1.7546E-04, 1.7545E-04, 1.7543E-04,
     &   1.7542E-04, 1.7540E-04, 1.7539E-04, 1.7538E-04, 1.7537E-04,
     &   1.7535E-04, 1.7534E-04, 1.7533E-04, 1.7532E-04, 1.7531E-04,
     &   1.7530E-04, 1.7530E-04, 1.7529E-04, 1.7528E-04, 1.7527E-04,
     &   1.7526E-04, 1.7526E-04, 1.7525E-04, 1.7524E-04, 1.7523E-04,
     &   1.7523E-04, 1.7522E-04, 1.7522E-04, 1.7521E-04, 1.7521E-04,
     &   1.7520E-04, 1.7519E-04, 1.7519E-04, 1.7519E-04, 1.7518E-04,
     &   1.7518E-04, 1.7517E-04, 1.7517E-04, 1.7516E-04, 1.7516E-04,
     &   1.7513E-04, 1.7511E-04, 1.7509E-04, 1.7507E-04, 1.7506E-04,
     &   1.7505E-04, 1.7505E-04, 1.7504E-04, 1.7503E-04, 1.7503E-04/
  
         DATA ((DAM(I,J), I= 101, 200), J=2,2) /
     &   1.7503E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04,
     &   1.7502E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04,
     &   1.7502E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04,
     &   1.7502E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04,
     &   1.7502E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04,
     &   1.7502E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04, 1.7502E-04,
     &   1.7504E-04, 1.7505E-04, 1.7505E-04, 1.7506E-04, 1.7506E-04,
     &   1.7507E-04, 1.7507E-04, 1.7508E-04, 1.7508E-04, 1.7509E-04/
c deg
         DATA ((DGM(I,J), I=   1, 100), J=1,1) /
     &   3.2049E-02, 3.2586E-02, 3.3097E-02, 3.3581E-02, 3.4043E-02,
     &   3.4906E-02, 3.5703E-02, 3.6447E-02, 3.7143E-02, 3.7800E-02,
     &   3.8421E-02, 3.9010E-02, 3.9570E-02, 4.0105E-02, 4.0617E-02,
     &   4.1107E-02, 4.1574E-02, 4.2027E-02, 4.2462E-02, 4.2882E-02,
     &   4.3289E-02, 4.3681E-02, 4.4062E-02, 4.4430E-02, 4.4788E-02,
     &   4.5135E-02, 4.5472E-02, 4.5800E-02, 4.6119E-02, 4.6430E-02,
     &   4.6734E-02, 4.7030E-02, 4.7318E-02, 4.7600E-02, 4.7876E-02,
     &   4.8145E-02, 4.8408E-02, 4.8666E-02, 4.8919E-02, 4.9166E-02,
     &   4.9408E-02, 4.9646E-02, 4.9878E-02, 5.0107E-02, 5.0331E-02,
     &   5.0551E-02, 5.0767E-02, 5.0980E-02, 5.1189E-02, 5.1394E-02,
     &   5.1595E-02, 5.1794E-02, 5.1989E-02, 5.2181E-02, 5.2370E-02,
     &   5.2556E-02, 5.2740E-02, 5.2920E-02, 5.3098E-02, 5.3274E-02,
     &   5.3447E-02, 5.3617E-02, 5.3785E-02, 5.3951E-02, 5.4114E-02,
     &   5.4276E-02, 5.4435E-02, 5.4592E-02, 5.4747E-02, 5.4900E-02,
     &   5.5051E-02, 5.5201E-02, 5.5348E-02, 5.5494E-02, 5.5638E-02,
     &   5.5780E-02, 5.5921E-02, 5.6060E-02, 5.6197E-02, 5.6333E-02,
     &   5.6468E-02, 5.6601E-02, 5.6732E-02, 5.6862E-02, 5.6991E-02,
     &   5.7118E-02, 5.7244E-02, 5.7369E-02, 5.7492E-02, 5.7614E-02,
     &   5.8661E-02, 5.9624E-02, 6.0516E-02, 6.1347E-02, 6.2124E-02,
     &   6.2854E-02, 6.3542E-02, 6.4193E-02, 6.4811E-02, 6.5399E-02/
  
         DATA ((DGM(I,J), I= 101, 200), J=1,1) /
     &   6.5959E-02, 6.6495E-02, 6.7007E-02, 6.7499E-02, 6.7971E-02,
     &   6.8426E-02, 6.8863E-02, 6.9286E-02, 6.9694E-02, 7.0088E-02,
     &   7.0471E-02, 7.0841E-02, 7.1200E-02, 7.1549E-02, 7.1888E-02,
     &   7.2217E-02, 7.2538E-02, 7.2847E-02, 7.3156E-02, 7.3454E-02,
     &   7.3746E-02, 7.4029E-02, 7.4307E-02, 7.4578E-02, 7.4844E-02,
     &   7.5103E-02, 7.5357E-02, 7.5606E-02, 7.5850E-02, 7.6089E-02,
     &   7.6323E-02, 7.6553E-02, 7.6779E-02, 7.7000E-02, 7.7217E-02,
     &   7.7432E-02, 7.7643E-02, 7.7853E-02, 7.8051E-02, 7.8253E-02,
     &   7.8449E-02, 7.8641E-02, 7.8832E-02, 7.9020E-02, 7.9206E-02,
     &   7.9386E-02, 7.9565E-02, 7.9748E-02, 7.9908E-02, 8.0087E-02,
     &   8.0262E-02, 8.0427E-02, 8.0599E-02, 8.0759E-02, 8.0921E-02,
     &   8.1084E-02, 8.1240E-02, 8.1379E-02, 8.1546E-02, 8.1693E-02,
     &   8.1822E-02, 8.1995E-02, 8.2143E-02, 8.2253E-02, 8.2445E-02,
     &   8.2631E-02, 8.2790E-02, 8.2980E-02, 8.3103E-02, 8.3203E-02,
     &   8.3255E-02, 8.3378E-02, 8.3468E-02, 8.3558E-02, 8.3646E-02,
     &   8.3775E-02, 8.3853E-02, 8.3967E-02, 8.4119E-02, 8.4195E-02,
     &   8.4316E-02, 8.4289E-02, 8.4534E-02, 8.4656E-02, 8.4779E-02,
     &   8.4972E-02, 8.4996E-02, 8.5125E-02, 8.5247E-02, 8.5235E-02,
     &   9.3843E-02, 9.8688E-02, 1.0215E-01, 1.0483E-01, 1.0702E-01,
     &   1.1046E-01, 1.1314E-01, 1.1533E-01, 1.1717E-01, 1.2144E-01/
  
         DATA ((DGM(I,J), I=   1, 100), J=2,2) /
     &   4.9611E-04, 4.6116E-04, 4.3930E-04, 4.2457E-04, 4.1403E-04,
     &   4.0002E-04, 3.9119E-04, 3.8513E-04, 3.8073E-04, 3.7740E-04,
     &   3.7480E-04, 3.7272E-04, 3.7102E-04, 3.6961E-04, 3.6841E-04,
     &   3.6740E-04, 3.6653E-04, 3.6577E-04, 3.6511E-04, 3.6452E-04,
     &   3.6401E-04, 3.6355E-04, 3.6313E-04, 3.6276E-04, 3.6243E-04,
     &   3.6212E-04, 3.6185E-04, 3.6160E-04, 3.6137E-04, 3.6115E-04,
     &   3.6096E-04, 3.6078E-04, 3.6061E-04, 3.6046E-04, 3.6032E-04,
     &   3.6019E-04, 3.6006E-04, 3.5995E-04, 3.5984E-04, 3.5974E-04,
     &   3.5964E-04, 3.5956E-04, 3.5947E-04, 3.5939E-04, 3.5932E-04,
     &   3.5925E-04, 3.5918E-04, 3.5912E-04, 3.5906E-04, 3.5901E-04,
     &   3.5895E-04, 3.5890E-04, 3.5885E-04, 3.5881E-04, 3.5877E-04,
     &   3.5872E-04, 3.5869E-04, 3.5865E-04, 3.5861E-04, 3.5858E-04,
     &   3.5855E-04, 3.5851E-04, 3.5848E-04, 3.5846E-04, 3.5843E-04,
     &   3.5840E-04, 3.5838E-04, 3.5835E-04, 3.5833E-04, 3.5831E-04,
     &   3.5828E-04, 3.5826E-04, 3.5824E-04, 3.5822E-04, 3.5821E-04,
     &   3.5819E-04, 3.5817E-04, 3.5815E-04, 3.5814E-04, 3.5812E-04,
     &   3.5811E-04, 3.5809E-04, 3.5808E-04, 3.5807E-04, 3.5805E-04,
     &   3.5804E-04, 3.5803E-04, 3.5802E-04, 3.5801E-04, 3.5800E-04,
     &   3.5791E-04, 3.5784E-04, 3.5779E-04, 3.5775E-04, 3.5772E-04,
     &   3.5769E-04, 3.5767E-04, 3.5765E-04, 3.5764E-04, 3.5763E-04/
  
         DATA ((DGM(I,J), I= 101, 200), J=2,2) /
     &   3.5762E-04, 3.5761E-04, 3.5760E-04, 3.5759E-04, 3.5759E-04,
     &   3.5758E-04, 3.5758E-04, 3.5757E-04, 3.5757E-04, 3.5757E-04,
     &   3.5756E-04, 3.5756E-04, 3.5756E-04, 3.5756E-04, 3.5756E-04,
     &   3.5756E-04, 3.5756E-04, 3.5755E-04, 3.5755E-04, 3.5755E-04,
     &   3.5755E-04, 3.5755E-04, 3.5755E-04, 3.5755E-04, 3.5755E-04,
     &   3.5755E-04, 3.5755E-04, 3.5755E-04, 3.5755E-04, 3.5755E-04,
     &   3.5755E-04, 3.5755E-04, 3.5755E-04, 3.5756E-04, 3.5756E-04,
     &   3.5756E-04, 3.5756E-04, 3.5756E-04, 3.5756E-04, 3.5756E-04,
     &   3.5756E-04, 3.5756E-04, 3.5756E-04, 3.5756E-04, 3.5756E-04,
     &   3.5756E-04, 3.5756E-04, 3.5756E-04, 3.5756E-04, 3.5756E-04,
     &   3.5756E-04, 3.5756E-04, 3.5756E-04, 3.5756E-04, 3.5756E-04,
     &   3.5756E-04, 3.5757E-04, 3.5757E-04, 3.5757E-04, 3.5757E-04,
     &   3.5757E-04, 3.5757E-04, 3.5757E-04, 3.5757E-04, 3.5757E-04,
     &   3.5757E-04, 3.5757E-04, 3.5757E-04, 3.5757E-04, 3.5757E-04,
     &   3.5757E-04, 3.5757E-04, 3.5757E-04, 3.5757E-04, 3.5757E-04,
     &   3.5757E-04, 3.5757E-04, 3.5757E-04, 3.5758E-04, 3.5758E-04,
     &   3.5758E-04, 3.5758E-04, 3.5758E-04, 3.5758E-04, 3.5758E-04,
     &   3.5758E-04, 3.5758E-04, 3.5758E-04, 3.5758E-04, 3.5758E-04,
     &   3.5762E-04, 3.5764E-04, 3.5765E-04, 3.5766E-04, 3.5767E-04,
     &   3.5769E-04, 3.5770E-04, 3.5771E-04, 3.5771E-04, 3.5773E-04/
c      
      save
      RETURN
      END

       subroutine dalhad_spaceliken
       IMPLICIT NONE
       INTEGER NA,I,J
       PARAMETER(NA=979)
       REAL ETA(NA),DAT(NA,2),DGT(NA,2)
       COMMON /DATT09/ETA,DAT,DGT
  
       DATA  (ETA(I), I=   1, 100) /
     &  -1.0000E+03,-9.9000E+02,-9.8000E+02,-9.7000E+02,-9.6000E+02,
     &  -9.5000E+02,-9.4000E+02,-9.3000E+02,-9.2000E+02,-9.1000E+02,
     &  -9.0000E+02,-8.9000E+02,-8.8000E+02,-8.7000E+02,-8.6000E+02,
     &  -8.5000E+02,-8.4000E+02,-8.3000E+02,-8.2000E+02,-8.1000E+02,
     &  -8.0000E+02,-7.9000E+02,-7.8000E+02,-7.7000E+02,-7.6000E+02,
     &  -7.5000E+02,-7.4000E+02,-7.3000E+02,-7.2000E+02,-7.1000E+02,
     &  -7.0000E+02,-6.9000E+02,-6.8000E+02,-6.7000E+02,-6.6000E+02,
     &  -6.5000E+02,-6.4000E+02,-6.3000E+02,-6.2000E+02,-6.1000E+02,
     &  -6.0000E+02,-5.9000E+02,-5.8000E+02,-5.7000E+02,-5.6000E+02,
     &  -5.5000E+02,-5.4000E+02,-5.3000E+02,-5.2000E+02,-5.1000E+02,
     &  -5.0000E+02,-4.9000E+02,-4.8000E+02,-4.7000E+02,-4.6000E+02,
     &  -4.5000E+02,-4.4000E+02,-4.3000E+02,-4.2000E+02,-4.1000E+02,
     &  -4.0000E+02,-3.9000E+02,-3.8000E+02,-3.7000E+02,-3.6000E+02,
     &  -3.5000E+02,-3.4000E+02,-3.3000E+02,-3.2000E+02,-3.1000E+02,
     &  -3.0000E+02,-2.9000E+02,-2.8000E+02,-2.7000E+02,-2.6000E+02,
     &  -2.5000E+02,-2.4000E+02,-2.3000E+02,-2.2000E+02,-2.1000E+02,
     &  -2.0000E+02,-1.9000E+02,-1.8000E+02,-1.7000E+02,-1.6000E+02,
     &  -1.5000E+02,-1.4000E+02,-1.3000E+02,-1.2000E+02,-1.1000E+02,
     &  -1.0000E+02,-9.9000E+01,-9.8000E+01,-9.7000E+01,-9.6000E+01,
     &  -9.5000E+01,-9.4000E+01,-9.3000E+01,-9.2000E+01,-9.1000E+01/
  
       DATA  (ETA(I), I= 101, 200) /
     &  -9.0000E+01,-8.9000E+01,-8.8000E+01,-8.7000E+01,-8.6000E+01,
     &  -8.5000E+01,-8.4000E+01,-8.3000E+01,-8.2000E+01,-8.1000E+01,
     &  -8.0000E+01,-7.9000E+01,-7.8000E+01,-7.7000E+01,-7.6000E+01,
     &  -7.5000E+01,-7.4000E+01,-7.3000E+01,-7.2000E+01,-7.1000E+01,
     &  -7.0000E+01,-6.9000E+01,-6.8000E+01,-6.7000E+01,-6.6000E+01,
     &  -6.5000E+01,-6.4000E+01,-6.3000E+01,-6.2000E+01,-6.1000E+01,
     &  -6.0000E+01,-5.9000E+01,-5.8000E+01,-5.7000E+01,-5.6000E+01,
     &  -5.5000E+01,-5.4000E+01,-5.3000E+01,-5.2000E+01,-5.1000E+01,
     &  -5.0000E+01,-4.9000E+01,-4.8000E+01,-4.7000E+01,-4.6000E+01,
     &  -4.5000E+01,-4.4000E+01,-4.3000E+01,-4.2000E+01,-4.1000E+01,
     &  -4.0000E+01,-3.9000E+01,-3.8000E+01,-3.7000E+01,-3.6000E+01,
     &  -3.5000E+01,-3.4000E+01,-3.3000E+01,-3.2000E+01,-3.1000E+01,
     &  -3.0000E+01,-2.9000E+01,-2.8000E+01,-2.7000E+01,-2.6000E+01,
     &  -2.5000E+01,-2.4000E+01,-2.3000E+01,-2.2000E+01,-2.1000E+01,
     &  -2.0000E+01,-1.9000E+01,-1.8000E+01,-1.7000E+01,-1.6000E+01,
     &  -1.5000E+01,-1.4000E+01,-1.3000E+01,-1.2000E+01,-1.1000E+01,
     &  -1.0000E+01,-9.9100E+00,-9.8200E+00,-9.7300E+00,-9.6400E+00,
     &  -9.5500E+00,-9.4600E+00,-9.3700E+00,-9.2800E+00,-9.1900E+00,
     &  -9.1000E+00,-9.0100E+00,-8.9200E+00,-8.8300E+00,-8.7400E+00,
     &  -8.6500E+00,-8.5600E+00,-8.4700E+00,-8.3800E+00,-8.2900E+00/
  
       DATA  (ETA(I), I= 201, 300) /
     &  -8.2000E+00,-8.1100E+00,-8.0200E+00,-7.9300E+00,-7.8400E+00,
     &  -7.7500E+00,-7.6600E+00,-7.5700E+00,-7.4800E+00,-7.3900E+00,
     &  -7.3000E+00,-7.2100E+00,-7.1200E+00,-7.0300E+00,-6.9400E+00,
     &  -6.8500E+00,-6.7600E+00,-6.6700E+00,-6.5800E+00,-6.4900E+00,
     &  -6.4000E+00,-6.3100E+00,-6.2200E+00,-6.1300E+00,-6.0400E+00,
     &  -5.9500E+00,-5.8600E+00,-5.7700E+00,-5.6800E+00,-5.5900E+00,
     &  -5.5000E+00,-5.4100E+00,-5.3200E+00,-5.2300E+00,-5.1400E+00,
     &  -5.0500E+00,-4.9600E+00,-4.8700E+00,-4.7800E+00,-4.6900E+00,
     &  -4.6000E+00,-4.5100E+00,-4.4200E+00,-4.3300E+00,-4.2400E+00,
     &  -4.1500E+00,-4.0600E+00,-3.9700E+00,-3.8800E+00,-3.7900E+00,
     &  -3.7000E+00,-3.6100E+00,-3.5200E+00,-3.4300E+00,-3.3400E+00,
     &  -3.2500E+00,-3.1600E+00,-3.0700E+00,-2.9800E+00,-2.8900E+00,
     &  -2.8000E+00,-2.7100E+00,-2.6200E+00,-2.5300E+00,-2.4400E+00,
     &  -2.3500E+00,-2.2600E+00,-2.1700E+00,-2.0800E+00,-1.9900E+00,
     &  -1.9000E+00,-1.8100E+00,-1.7200E+00,-1.6300E+00,-1.5400E+00,
     &  -1.4500E+00,-1.3600E+00,-1.2700E+00,-1.1800E+00,-1.0900E+00,
     &  -1.0000E+00,-9.9100E-01,-9.8200E-01,-9.7300E-01,-9.6400E-01,
     &  -9.5500E-01,-9.4600E-01,-9.3700E-01,-9.2800E-01,-9.1900E-01,
     &  -9.1000E-01,-9.0100E-01,-8.9200E-01,-8.8300E-01,-8.7400E-01,
     &  -8.6500E-01,-8.5600E-01,-8.4700E-01,-8.3800E-01,-8.2900E-01/
  
       DATA  (ETA(I), I= 301, 400) /
     &  -8.2000E-01,-8.1100E-01,-8.0200E-01,-7.9300E-01,-7.8400E-01,
     &  -7.7500E-01,-7.6600E-01,-7.5700E-01,-7.4800E-01,-7.3900E-01,
     &  -7.3000E-01,-7.2100E-01,-7.1200E-01,-7.0300E-01,-6.9400E-01,
     &  -6.8500E-01,-6.7600E-01,-6.6700E-01,-6.5800E-01,-6.4900E-01,
     &  -6.4000E-01,-6.3100E-01,-6.2200E-01,-6.1300E-01,-6.0400E-01,
     &  -5.9500E-01,-5.8600E-01,-5.7700E-01,-5.6800E-01,-5.5900E-01,
     &  -5.5000E-01,-5.4100E-01,-5.3200E-01,-5.2300E-01,-5.1400E-01,
     &  -5.0500E-01,-4.9600E-01,-4.8700E-01,-4.7800E-01,-4.6900E-01,
     &  -4.6000E-01,-4.5100E-01,-4.4200E-01,-4.3300E-01,-4.2400E-01,
     &  -4.1500E-01,-4.0600E-01,-3.9700E-01,-3.8800E-01,-3.7900E-01,
     &  -3.7000E-01,-3.6100E-01,-3.5200E-01,-3.4300E-01,-3.3400E-01,
     &  -3.2500E-01,-3.1600E-01,-3.0700E-01,-2.9800E-01,-2.8900E-01,
     &  -2.8000E-01,-2.7100E-01,-2.6200E-01,-2.5300E-01,-2.4400E-01,
     &  -2.3500E-01,-2.2600E-01,-2.1700E-01,-2.0800E-01,-1.9900E-01,
     &  -1.9000E-01,-1.8100E-01,-1.7200E-01,-1.6300E-01,-1.5400E-01,
     &  -1.4500E-01,-1.3600E-01,-1.2700E-01,-1.1800E-01,-1.0900E-01,
     &  -1.0000E-01,-9.9100E-02,-9.8200E-02,-9.7300E-02,-9.6400E-02,
     &  -9.5500E-02,-9.4600E-02,-9.3700E-02,-9.2800E-02,-9.1900E-02,
     &  -9.1000E-02,-9.0100E-02,-8.9200E-02,-8.8300E-02,-8.7400E-02,
     &  -8.6500E-02,-8.5600E-02,-8.4700E-02,-8.3800E-02,-8.2900E-02/
  
       DATA  (ETA(I), I= 401, 500) /
     &  -8.2000E-02,-8.1100E-02,-8.0200E-02,-7.9300E-02,-7.8400E-02,
     &  -7.7500E-02,-7.6600E-02,-7.5700E-02,-7.4800E-02,-7.3900E-02,
     &  -7.3000E-02,-7.2100E-02,-7.1200E-02,-7.0300E-02,-6.9400E-02,
     &  -6.8500E-02,-6.7600E-02,-6.6700E-02,-6.5800E-02,-6.4900E-02,
     &  -6.4000E-02,-6.3100E-02,-6.2200E-02,-6.1300E-02,-6.0400E-02,
     &  -5.9500E-02,-5.8600E-02,-5.7700E-02,-5.6800E-02,-5.5900E-02,
     &  -5.5000E-02,-5.4100E-02,-5.3200E-02,-5.2300E-02,-5.1400E-02,
     &  -5.0500E-02,-4.9600E-02,-4.8700E-02,-4.7800E-02,-4.6900E-02,
     &  -4.6000E-02,-4.5100E-02,-4.4200E-02,-4.3300E-02,-4.2400E-02,
     &  -4.1500E-02,-4.0600E-02,-3.9700E-02,-3.8800E-02,-3.7900E-02,
     &  -3.7000E-02,-3.6100E-02,-3.5200E-02,-3.4300E-02,-3.3400E-02,
     &  -3.2500E-02,-3.1600E-02,-3.0700E-02,-2.9800E-02,-2.8900E-02,
     &  -2.8000E-02,-2.7100E-02,-2.6200E-02,-2.5300E-02,-2.4400E-02,
     &  -2.3500E-02,-2.2600E-02,-2.1700E-02,-2.0800E-02,-1.9900E-02,
     &  -1.9000E-02,-1.8100E-02,-1.7200E-02,-1.6300E-02,-1.5400E-02,
     &  -1.4500E-02,-1.3600E-02,-1.2700E-02,-1.1800E-02,-1.0900E-02,
     &  -1.0000E-02,-9.9100E-03,-9.8200E-03,-9.7300E-03,-9.6400E-03,
     &  -9.5500E-03,-9.4600E-03,-9.3700E-03,-9.2800E-03,-9.1900E-03,
     &  -9.1000E-03,-9.0100E-03,-8.9200E-03,-8.8300E-03,-8.7400E-03,
     &  -8.6500E-03,-8.5600E-03,-8.4700E-03,-8.3800E-03,-8.2900E-03/
  
       DATA  (ETA(I), I= 501, 600) /
     &  -8.2000E-03,-8.1100E-03,-8.0200E-03,-7.9300E-03,-7.8400E-03,
     &  -7.7500E-03,-7.6600E-03,-7.5700E-03,-7.4800E-03,-7.3900E-03,
     &  -7.3000E-03,-7.2100E-03,-7.1200E-03,-7.0300E-03,-6.9400E-03,
     &  -6.8500E-03,-6.7600E-03,-6.6700E-03,-6.5800E-03,-6.4900E-03,
     &  -6.4000E-03,-6.3100E-03,-6.2200E-03,-6.1300E-03,-6.0400E-03,
     &  -5.9500E-03,-5.8600E-03,-5.7700E-03,-5.6800E-03,-5.5900E-03,
     &  -5.5000E-03,-5.4100E-03,-5.3200E-03,-5.2300E-03,-5.1400E-03,
     &  -5.0500E-03,-4.9600E-03,-4.8700E-03,-4.7800E-03,-4.6900E-03,
     &  -4.6000E-03,-4.5100E-03,-4.4200E-03,-4.3300E-03,-4.2400E-03,
     &  -4.1500E-03,-4.0600E-03,-3.9700E-03,-3.8800E-03,-3.7900E-03,
     &  -3.7000E-03,-3.6100E-03,-3.5200E-03,-3.4300E-03,-3.3400E-03,
     &  -3.2500E-03,-3.1600E-03,-3.0700E-03,-2.9800E-03,-2.8900E-03,
     &  -2.8000E-03,-2.7100E-03,-2.6200E-03,-2.5300E-03,-2.4400E-03,
     &  -2.3500E-03,-2.2600E-03,-2.1700E-03,-2.0800E-03,-1.9900E-03,
     &  -1.9000E-03,-1.8100E-03,-1.7200E-03,-1.6300E-03,-1.5400E-03,
     &  -1.4500E-03,-1.3600E-03,-1.2700E-03,-1.1800E-03,-1.0900E-03,
     &  -1.0000E-03,-9.9100E-04,-9.8200E-04,-9.7300E-04,-9.6400E-04,
     &  -9.5500E-04,-9.4600E-04,-9.3700E-04,-9.2800E-04,-9.1900E-04,
     &  -9.1000E-04,-9.0100E-04,-8.9200E-04,-8.8300E-04,-8.7400E-04,
     &  -8.6500E-04,-8.5600E-04,-8.4700E-04,-8.3800E-04,-8.2900E-04/
  
       DATA  (ETA(I), I= 601, 700) /
     &  -8.2000E-04,-8.1100E-04,-8.0200E-04,-7.9300E-04,-7.8400E-04,
     &  -7.7500E-04,-7.6600E-04,-7.5700E-04,-7.4800E-04,-7.3900E-04,
     &  -7.3000E-04,-7.2100E-04,-7.1200E-04,-7.0300E-04,-6.9400E-04,
     &  -6.8500E-04,-6.7600E-04,-6.6700E-04,-6.5800E-04,-6.4900E-04,
     &  -6.4000E-04,-6.3100E-04,-6.2200E-04,-6.1300E-04,-6.0400E-04,
     &  -5.9500E-04,-5.8600E-04,-5.7700E-04,-5.6800E-04,-5.5900E-04,
     &  -5.5000E-04,-5.4100E-04,-5.3200E-04,-5.2300E-04,-5.1400E-04,
     &  -5.0500E-04,-4.9600E-04,-4.8700E-04,-4.7800E-04,-4.6900E-04,
     &  -4.6000E-04,-4.5100E-04,-4.4200E-04,-4.3300E-04,-4.2400E-04,
     &  -4.1500E-04,-4.0600E-04,-3.9700E-04,-3.8800E-04,-3.7900E-04,
     &  -3.7000E-04,-3.6100E-04,-3.5200E-04,-3.4300E-04,-3.3400E-04,
     &  -3.2500E-04,-3.1600E-04,-3.0700E-04,-2.9800E-04,-2.8900E-04,
     &  -2.8000E-04,-2.7100E-04,-2.6200E-04,-2.5300E-04,-2.4400E-04,
     &  -2.3500E-04,-2.2600E-04,-2.1700E-04,-2.0800E-04,-1.9900E-04,
     &  -1.9000E-04,-1.8100E-04,-1.7200E-04,-1.6300E-04,-1.5400E-04,
     &  -1.4500E-04,-1.3600E-04,-1.2700E-04,-1.1800E-04,-1.0900E-04,
     &  -1.0000E-04,-9.9100E-05,-9.8200E-05,-9.7300E-05,-9.6400E-05,
     &  -9.5500E-05,-9.4600E-05,-9.3700E-05,-9.2800E-05,-9.1900E-05,
     &  -9.1000E-05,-9.0100E-05,-8.9200E-05,-8.8300E-05,-8.7400E-05,
     &  -8.6500E-05,-8.5600E-05,-8.4700E-05,-8.3800E-05,-8.2900E-05/
  
       DATA  (ETA(I), I= 701, 800) /
     &  -8.2000E-05,-8.1100E-05,-8.0200E-05,-7.9300E-05,-7.8400E-05,
     &  -7.7500E-05,-7.6600E-05,-7.5700E-05,-7.4800E-05,-7.3900E-05,
     &  -7.3000E-05,-7.2100E-05,-7.1200E-05,-7.0300E-05,-6.9400E-05,
     &  -6.8500E-05,-6.7600E-05,-6.6700E-05,-6.5800E-05,-6.4900E-05,
     &  -6.4000E-05,-6.3100E-05,-6.2200E-05,-6.1300E-05,-6.0400E-05,
     &  -5.9500E-05,-5.8600E-05,-5.7700E-05,-5.6800E-05,-5.5900E-05,
     &  -5.5000E-05,-5.4100E-05,-5.3200E-05,-5.2300E-05,-5.1400E-05,
     &  -5.0500E-05,-4.9600E-05,-4.8700E-05,-4.7800E-05,-4.6900E-05,
     &  -4.6000E-05,-4.5100E-05,-4.4200E-05,-4.3300E-05,-4.2400E-05,
     &  -4.1500E-05,-4.0600E-05,-3.9700E-05,-3.8800E-05,-3.7900E-05,
     &  -3.7000E-05,-3.6100E-05,-3.5200E-05,-3.4300E-05,-3.3400E-05,
     &  -3.2500E-05,-3.1600E-05,-3.0700E-05,-2.9800E-05,-2.8900E-05,
     &  -2.8000E-05,-2.7100E-05,-2.6200E-05,-2.5300E-05,-2.4400E-05,
     &  -2.3500E-05,-2.2600E-05,-2.1700E-05,-2.0800E-05,-1.9900E-05,
     &  -1.9000E-05,-1.8100E-05,-1.7200E-05,-1.6300E-05,-1.5400E-05,
     &  -1.4500E-05,-1.3600E-05,-1.2700E-05,-1.1800E-05,-1.0900E-05,
     &  -1.0000E-05,-9.9100E-06,-9.8200E-06,-9.7300E-06,-9.6400E-06,
     &  -9.5500E-06,-9.4600E-06,-9.3700E-06,-9.2800E-06,-9.1900E-06,
     &  -9.1000E-06,-9.0100E-06,-8.9200E-06,-8.8300E-06,-8.7400E-06,
     &  -8.6500E-06,-8.5600E-06,-8.4700E-06,-8.3800E-06,-8.2900E-06/
  
       DATA  (ETA(I), I= 801, 900) /
     &  -8.2000E-06,-8.1100E-06,-8.0200E-06,-7.9300E-06,-7.8400E-06,
     &  -7.7500E-06,-7.6600E-06,-7.5700E-06,-7.4800E-06,-7.3900E-06,
     &  -7.3000E-06,-7.2100E-06,-7.1200E-06,-7.0300E-06,-6.9400E-06,
     &  -6.8500E-06,-6.7600E-06,-6.6700E-06,-6.5800E-06,-6.4900E-06,
     &  -6.4000E-06,-6.3100E-06,-6.2200E-06,-6.1300E-06,-6.0400E-06,
     &  -5.9500E-06,-5.8600E-06,-5.7700E-06,-5.6800E-06,-5.5900E-06,
     &  -5.5000E-06,-5.4100E-06,-5.3200E-06,-5.2300E-06,-5.1400E-06,
     &  -5.0500E-06,-4.9600E-06,-4.8700E-06,-4.7800E-06,-4.6900E-06,
     &  -4.6000E-06,-4.5100E-06,-4.4200E-06,-4.3300E-06,-4.2400E-06,
     &  -4.1500E-06,-4.0600E-06,-3.9700E-06,-3.8800E-06,-3.7900E-06,
     &  -3.7000E-06,-3.6100E-06,-3.5200E-06,-3.4300E-06,-3.3400E-06,
     &  -3.2500E-06,-3.1600E-06,-3.0700E-06,-2.9800E-06,-2.8900E-06,
     &  -2.8000E-06,-2.7100E-06,-2.6200E-06,-2.5300E-06,-2.4400E-06,
     &  -2.3500E-06,-2.2600E-06,-2.1700E-06,-2.0800E-06,-1.9900E-06,
     &  -1.9000E-06,-1.8100E-06,-1.7200E-06,-1.6300E-06,-1.5400E-06,
     &  -1.4500E-06,-1.3600E-06,-1.2700E-06,-1.1800E-06,-1.0900E-06,
     &  -1.0000E-06, 0.0000E+00, 1.0000E-06, 3.0010E-03, 6.0010E-03,
     &   9.0010E-03, 1.2000E-02, 1.5000E-02, 1.8000E-02, 2.1000E-02,
     &   2.4000E-02, 2.7000E-02, 3.0000E-02, 3.3000E-02, 3.6000E-02,
     &   3.9000E-02, 4.2000E-02, 4.5000E-02, 4.8000E-02, 5.1000E-02/
  
       DATA  (ETA(I), I= 901, 979) /
     &   5.4000E-02, 5.7000E-02, 6.0000E-02, 6.3000E-02, 6.6000E-02,
     &   6.9000E-02, 7.2000E-02, 7.5000E-02, 7.8000E-02, 8.1000E-02,
     &   8.4000E-02, 8.7000E-02, 9.0000E-02, 9.3000E-02, 9.6000E-02,
     &   9.9000E-02, 1.0200E-01, 1.0500E-01, 1.0800E-01, 1.1100E-01,
     &   1.1400E-01, 1.1700E-01, 1.2000E-01, 1.2300E-01, 1.2600E-01,
     &   1.2900E-01, 1.3200E-01, 1.3500E-01, 1.3800E-01, 1.4100E-01,
     &   1.4400E-01, 1.4700E-01, 1.5000E-01, 1.5300E-01, 1.5600E-01,
     &   1.5900E-01, 1.6200E-01, 1.6500E-01, 1.6800E-01, 1.7100E-01,
     &   1.7400E-01, 1.7700E-01, 1.8000E-01, 1.8300E-01, 1.8600E-01,
     &   1.8900E-01, 1.9200E-01, 1.9500E-01, 1.9800E-01, 2.0100E-01,
     &   2.0400E-01, 2.0700E-01, 2.1000E-01, 2.1300E-01, 2.1600E-01,
     &   2.1900E-01, 2.2200E-01, 2.2500E-01, 2.2800E-01, 2.3100E-01,
     &   2.3400E-01, 2.3700E-01, 2.4000E-01, 2.4300E-01, 2.4600E-01,
     &   2.4900E-01, 2.5200E-01, 2.5500E-01, 2.5800E-01, 2.6100E-01,
     &   2.6400E-01, 2.6700E-01, 2.7000E-01, 2.7100E-01, 2.7300E-01,
     &   0.27370000D0, 0.27599999D0, 0.27630001D0, 0.27700001D0/
  
         DATA ((DAT(I,J), I=   1, 100), J=1,1) /
     &   4.1572E-02, 4.1514E-02, 4.1454E-02, 4.1394E-02, 4.1334E-02,
     &   4.1272E-02, 4.1210E-02, 4.1148E-02, 4.1085E-02, 4.1021E-02,
     &   4.0956E-02, 4.0891E-02, 4.0825E-02, 4.0758E-02, 4.0690E-02,
     &   4.0622E-02, 4.0552E-02, 4.0482E-02, 4.0411E-02, 4.0340E-02,
     &   4.0267E-02, 4.0193E-02, 4.0119E-02, 4.0043E-02, 3.9967E-02,
     &   3.9889E-02, 3.9811E-02, 3.9731E-02, 3.9650E-02, 3.9568E-02,
     &   3.9485E-02, 3.9401E-02, 3.9315E-02, 3.9229E-02, 3.9141E-02,
     &   3.9051E-02, 3.8960E-02, 3.8868E-02, 3.8775E-02, 3.8679E-02,
     &   3.8582E-02, 3.8484E-02, 3.8384E-02, 3.8282E-02, 3.8178E-02,
     &   3.8073E-02, 3.7965E-02, 3.7856E-02, 3.7744E-02, 3.7630E-02,
     &   3.7514E-02, 3.7396E-02, 3.7275E-02, 3.7151E-02, 3.7025E-02,
     &   3.6896E-02, 3.6765E-02, 3.6630E-02, 3.6492E-02, 3.6350E-02,
     &   3.6206E-02, 3.6057E-02, 3.5905E-02, 3.5748E-02, 3.5587E-02,
     &   3.5422E-02, 3.5252E-02, 3.5077E-02, 3.4896E-02, 3.4710E-02,
     &   3.4517E-02, 3.4318E-02, 3.4112E-02, 3.3898E-02, 3.3677E-02,
     &   3.3446E-02, 3.3206E-02, 3.2956E-02, 3.2695E-02, 3.2421E-02,
     &   3.2134E-02, 3.1833E-02, 3.1514E-02, 3.1178E-02, 3.0821E-02,
     &   3.0441E-02, 3.0035E-02, 2.9598E-02, 2.9126E-02, 2.8613E-02,
     &   2.8051E-02, 2.7992E-02, 2.7932E-02, 2.7871E-02, 2.7810E-02,
     &   2.7748E-02, 2.7686E-02, 2.7623E-02, 2.7559E-02, 2.7494E-02/
  
         DATA ((DAT(I,J), I= 101, 200), J=1,1) /
     &   2.7429E-02, 2.7363E-02, 2.7296E-02, 2.7229E-02, 2.7161E-02,
     &   2.7092E-02, 2.7022E-02, 2.6951E-02, 2.6879E-02, 2.6807E-02,
     &   2.6734E-02, 2.6659E-02, 2.6584E-02, 2.6508E-02, 2.6431E-02,
     &   2.6352E-02, 2.6273E-02, 2.6193E-02, 2.6111E-02, 2.6029E-02,
     &   2.5945E-02, 2.5860E-02, 2.5773E-02, 2.5686E-02, 2.5597E-02,
     &   2.5507E-02, 2.5415E-02, 2.5322E-02, 2.5227E-02, 2.5131E-02,
     &   2.5034E-02, 2.4934E-02, 2.4833E-02, 2.4730E-02, 2.4626E-02,
     &   2.4519E-02, 2.4411E-02, 2.4300E-02, 2.4187E-02, 2.4073E-02,
     &   2.3956E-02, 2.3836E-02, 2.3714E-02, 2.3590E-02, 2.3462E-02,
     &   2.3332E-02, 2.3199E-02, 2.3063E-02, 2.2924E-02, 2.2782E-02,
     &   2.2636E-02, 2.2486E-02, 2.2332E-02, 2.2175E-02, 2.2013E-02,
     &   2.1846E-02, 2.1675E-02, 2.1498E-02, 2.1316E-02, 2.1129E-02,
     &   2.0935E-02, 2.0735E-02, 2.0527E-02, 2.0313E-02, 2.0090E-02,
     &   1.9859E-02, 1.9618E-02, 1.9367E-02, 1.9106E-02, 1.8832E-02,
     &   1.8545E-02, 1.8244E-02, 1.7928E-02, 1.7593E-02, 1.7239E-02,
     &   1.6863E-02, 1.6463E-02, 1.6034E-02, 1.5572E-02, 1.5073E-02,
     &   1.4530E-02, 1.4478E-02, 1.4426E-02, 1.4374E-02, 1.4322E-02,
     &   1.4268E-02, 1.4215E-02, 1.4161E-02, 1.4106E-02, 1.4051E-02,
     &   1.3996E-02, 1.3940E-02, 1.3883E-02, 1.3826E-02, 1.3769E-02,
     &   1.3710E-02, 1.3652E-02, 1.3592E-02, 1.3533E-02, 1.3472E-02/
  
         DATA ((DAT(I,J), I= 201, 300), J=1,1) /
     &   1.3411E-02, 1.3350E-02, 1.3287E-02, 1.3224E-02, 1.3161E-02,
     &   1.3097E-02, 1.3032E-02, 1.2966E-02, 1.2900E-02, 1.2833E-02,
     &   1.2765E-02, 1.2697E-02, 1.2628E-02, 1.2558E-02, 1.2487E-02,
     &   1.2415E-02, 1.2343E-02, 1.2270E-02, 1.2196E-02, 1.2121E-02,
     &   1.2045E-02, 1.1968E-02, 1.1890E-02, 1.1811E-02, 1.1731E-02,
     &   1.1650E-02, 1.1568E-02, 1.1485E-02, 1.1401E-02, 1.1316E-02,
     &   1.1229E-02, 1.1141E-02, 1.1052E-02, 1.0962E-02, 1.0870E-02,
     &   1.0777E-02, 1.0683E-02, 1.0587E-02, 1.0490E-02, 1.0391E-02,
     &   1.0291E-02, 1.0189E-02, 1.0085E-02, 9.9797E-03, 9.8724E-03,
     &   9.7634E-03, 9.6524E-03, 9.5394E-03, 9.4243E-03, 9.3071E-03,
     &   9.1878E-03, 9.0661E-03, 8.9420E-03, 8.8154E-03, 8.6862E-03,
     &   8.5544E-03, 8.4197E-03, 8.2820E-03, 8.1413E-03, 7.9973E-03,
     &   7.8499E-03, 7.6990E-03, 7.5442E-03, 7.3855E-03, 7.2225E-03,
     &   7.0550E-03, 6.8827E-03, 6.7052E-03, 6.5222E-03, 6.3332E-03,
     &   6.1378E-03, 5.9353E-03, 5.7252E-03, 5.5066E-03, 5.2787E-03,
     &   5.0406E-03, 4.7911E-03, 4.5288E-03, 4.2525E-03, 3.9604E-03,
     &   3.6511E-03, 3.6192E-03, 3.5871E-03, 3.5547E-03, 3.5222E-03,
     &   3.4895E-03, 3.4566E-03, 3.4235E-03, 3.3902E-03, 3.3567E-03,
     &   3.3230E-03, 3.2891E-03, 3.2550E-03, 3.2207E-03, 3.1862E-03,
     &   3.1515E-03, 3.1165E-03, 3.0814E-03, 3.0461E-03, 3.0106E-03/
  
         DATA ((DAT(I,J), I= 301, 400), J=1,1) /
     &   2.9748E-03, 2.9389E-03, 2.9027E-03, 2.8664E-03, 2.8298E-03,
     &   2.7930E-03, 2.7561E-03, 2.7189E-03, 2.6815E-03, 2.6440E-03,
     &   2.6062E-03, 2.5682E-03, 2.5301E-03, 2.4917E-03, 2.4532E-03,
     &   2.4145E-03, 2.3756E-03, 2.3365E-03, 2.2972E-03, 2.2578E-03,
     &   2.2182E-03, 2.1785E-03, 2.1386E-03, 2.0985E-03, 2.0583E-03,
     &   2.0180E-03, 1.9776E-03, 1.9370E-03, 1.8963E-03, 1.8555E-03,
     &   1.8147E-03, 1.7737E-03, 1.7327E-03, 1.6916E-03, 1.6505E-03,
     &   1.6093E-03, 1.5682E-03, 1.5270E-03, 1.4858E-03, 1.4447E-03,
     &   1.4036E-03, 1.3625E-03, 1.3216E-03, 1.2807E-03, 1.2400E-03,
     &   1.1994E-03, 1.1589E-03, 1.1187E-03, 1.0786E-03, 1.0388E-03,
     &   9.9921E-04, 9.5993E-04, 9.2095E-04, 8.8232E-04, 8.4405E-04,
     &   8.0618E-04, 7.6874E-04, 7.3176E-04, 6.9527E-04, 6.5931E-04,
     &   6.2390E-04, 5.8909E-04, 5.5491E-04, 5.2140E-04, 4.8859E-04,
     &   4.5652E-04, 4.2522E-04, 3.9474E-04, 3.6512E-04, 3.3638E-04,
     &   3.0857E-04, 2.8173E-04, 2.5590E-04, 2.3111E-04, 2.0740E-04,
     &   1.8481E-04, 1.6337E-04, 1.4311E-04, 1.2408E-04, 1.0630E-04,
     &   8.9804E-05, 8.8227E-05, 8.6662E-05, 8.5111E-05, 8.3572E-05,
     &   8.2047E-05, 8.0536E-05, 7.9037E-05, 7.7552E-05, 7.6081E-05,
     &   7.4623E-05, 7.3178E-05, 7.1746E-05, 7.0328E-05, 6.8924E-05,
     &   6.7533E-05, 6.6155E-05, 6.4792E-05, 6.3441E-05, 6.2105E-05/
  
         DATA ((DAT(I,J), I= 401, 500), J=1,1) /
     &   6.0782E-05, 5.9472E-05, 5.8177E-05, 5.6895E-05, 5.5627E-05,
     &   5.4372E-05, 5.3132E-05, 5.1905E-05, 5.0692E-05, 4.9493E-05,
     &   4.8308E-05, 4.7136E-05, 4.5979E-05, 4.4836E-05, 4.3706E-05,
     &   4.2591E-05, 4.1489E-05, 4.0402E-05, 3.9328E-05, 3.8269E-05,
     &   3.7224E-05, 3.6193E-05, 3.5176E-05, 3.4173E-05, 3.3184E-05,
     &   3.2210E-05, 3.1249E-05, 3.0303E-05, 2.9372E-05, 2.8454E-05,
     &   2.7551E-05, 2.6662E-05, 2.5787E-05, 2.4927E-05, 2.4081E-05,
     &   2.3250E-05, 2.2432E-05, 2.1630E-05, 2.0841E-05, 2.0067E-05,
     &   1.9308E-05, 1.8563E-05, 1.7832E-05, 1.7116E-05, 1.6415E-05,
     &   1.5728E-05, 1.5055E-05, 1.4397E-05, 1.3754E-05, 1.3125E-05,
     &   1.2511E-05, 1.1911E-05, 1.1326E-05, 1.0756E-05, 1.0200E-05,
     &   9.6591E-06, 9.1326E-06, 8.6208E-06, 8.1237E-06, 7.6412E-06,
     &   7.1735E-06, 6.7204E-06, 6.2821E-06, 5.8585E-06, 5.4496E-06,
     &   5.0554E-06, 4.6760E-06, 4.3114E-06, 3.9615E-06, 3.6263E-06,
     &   3.3060E-06, 3.0004E-06, 2.7096E-06, 2.4336E-06, 2.1724E-06,
     &   1.9260E-06, 1.6945E-06, 1.4777E-06, 1.2757E-06, 1.0886E-06,
     &   9.1628E-07, 8.9986E-07, 8.8360E-07, 8.6748E-07, 8.5151E-07,
     &   8.3569E-07, 8.2001E-07, 8.0449E-07, 7.8911E-07, 7.7388E-07,
     &   7.5880E-07, 7.4387E-07, 7.2908E-07, 7.1445E-07, 6.9996E-07,
     &   6.8562E-07, 6.7143E-07, 6.5739E-07, 6.4349E-07, 6.2975E-07/
  
         DATA ((DAT(I,J), I= 501, 600), J=1,1) /
     &   6.1615E-07, 6.0270E-07, 5.8940E-07, 5.7625E-07, 5.6324E-07,
     &   5.5039E-07, 5.3768E-07, 5.2512E-07, 5.1271E-07, 5.0045E-07,
     &   4.8833E-07, 4.7637E-07, 4.6455E-07, 4.5288E-07, 4.4136E-07,
     &   4.2999E-07, 4.1877E-07, 4.0769E-07, 3.9676E-07, 3.8598E-07,
     &   3.7535E-07, 3.6487E-07, 3.5454E-07, 3.4435E-07, 3.3432E-07,
     &   3.2443E-07, 3.1469E-07, 3.0510E-07, 2.9566E-07, 2.8636E-07,
     &   2.7721E-07, 2.6822E-07, 2.5937E-07, 2.5067E-07, 2.4211E-07,
     &   2.3371E-07, 2.2545E-07, 2.1735E-07, 2.0939E-07, 2.0158E-07,
     &   1.9392E-07, 1.8640E-07, 1.7904E-07, 1.7182E-07, 1.6475E-07,
     &   1.5783E-07, 1.5106E-07, 1.4444E-07, 1.3796E-07, 1.3164E-07,
     &   1.2546E-07, 1.1943E-07, 1.1355E-07, 1.0782E-07, 1.0224E-07,
     &   9.6800E-08, 9.1513E-08, 8.6375E-08, 8.1385E-08, 7.6543E-08,
     &   7.1850E-08, 6.7305E-08, 6.2909E-08, 5.8662E-08, 5.4562E-08,
     &   5.0612E-08, 4.6809E-08, 4.3155E-08, 3.9650E-08, 3.6293E-08,
     &   3.3084E-08, 3.0024E-08, 2.7113E-08, 2.4350E-08, 2.1735E-08,
     &   1.9269E-08, 1.6951E-08, 1.4782E-08, 1.2761E-08, 1.0889E-08,
     &   9.1647E-09, 9.0005E-09, 8.8377E-09, 8.6765E-09, 8.5167E-09,
     &   8.3584E-09, 8.2016E-09, 8.0463E-09, 7.8925E-09, 7.7401E-09,
     &   7.5893E-09, 7.4399E-09, 7.2920E-09, 7.1456E-09, 7.0007E-09,
     &   6.8573E-09, 6.7153E-09, 6.5748E-09, 6.4358E-09, 6.2984E-09/
  
         DATA ((DAT(I,J), I= 601, 700), J=1,1) /
     &   6.1623E-09, 6.0278E-09, 5.8948E-09, 5.7632E-09, 5.6331E-09,
     &   5.5045E-09, 5.3774E-09, 5.2518E-09, 5.1277E-09, 5.0050E-09,
     &   4.8839E-09, 4.7642E-09, 4.6460E-09, 4.5293E-09, 4.4140E-09,
     &   4.3003E-09, 4.1880E-09, 4.0773E-09, 3.9680E-09, 3.8602E-09,
     &   3.7539E-09, 3.6490E-09, 3.5457E-09, 3.4438E-09, 3.3434E-09,
     &   3.2445E-09, 3.1471E-09, 3.0512E-09, 2.9568E-09, 2.8638E-09,
     &   2.7723E-09, 2.6823E-09, 2.5938E-09, 2.5068E-09, 2.4213E-09,
     &   2.3372E-09, 2.2547E-09, 2.1736E-09, 2.0940E-09, 2.0159E-09,
     &   1.9393E-09, 1.8641E-09, 1.7905E-09, 1.7183E-09, 1.6476E-09,
     &   1.5784E-09, 1.5107E-09, 1.4444E-09, 1.3797E-09, 1.3164E-09,
     &   1.2546E-09, 1.1944E-09, 1.1355E-09, 1.0782E-09, 1.0224E-09,
     &   9.6802E-10, 9.1515E-10, 8.6376E-10, 8.1386E-10, 7.6545E-10,
     &   7.1851E-10, 6.7307E-10, 6.2910E-10, 5.8662E-10, 5.4563E-10,
     &   5.0612E-10, 4.6810E-10, 4.3156E-10, 3.9650E-10, 3.6293E-10,
     &   3.3085E-10, 3.0024E-10, 2.7113E-10, 2.4350E-10, 2.1735E-10,
     &   1.9269E-10, 1.6951E-10, 1.4782E-10, 1.2761E-10, 1.0889E-10,
     &   9.1647E-11, 9.0005E-11, 8.8377E-11, 8.6765E-11, 8.5167E-11,
     &   8.3584E-11, 8.2016E-11, 8.0463E-11, 7.8925E-11, 7.7402E-11,
     &   7.5893E-11, 7.4399E-11, 7.2920E-11, 7.1456E-11, 7.0007E-11,
     &   6.8573E-11, 6.7153E-11, 6.5748E-11, 6.4359E-11, 6.2984E-11/
  
         DATA ((DAT(I,J), I= 701, 800), J=1,1) /
     &   6.1623E-11, 6.0278E-11, 5.8948E-11, 5.7632E-11, 5.6331E-11,
     &   5.5046E-11, 5.3774E-11, 5.2518E-11, 5.1277E-11, 5.0050E-11,
     &   4.8839E-11, 4.7642E-11, 4.6460E-11, 4.5293E-11, 4.4141E-11,
     &   4.3003E-11, 4.1881E-11, 4.0773E-11, 3.9680E-11, 3.8602E-11,
     &   3.7539E-11, 3.6490E-11, 3.5457E-11, 3.4438E-11, 3.3434E-11,
     &   3.2445E-11, 3.1471E-11, 3.0512E-11, 2.9568E-11, 2.8638E-11,
     &   2.7723E-11, 2.6823E-11, 2.5938E-11, 2.5068E-11, 2.4213E-11,
     &   2.3372E-11, 2.2547E-11, 2.1736E-11, 2.0940E-11, 2.0159E-11,
     &   1.9393E-11, 1.8641E-11, 1.7905E-11, 1.7183E-11, 1.6476E-11,
     &   1.5784E-11, 1.5107E-11, 1.4444E-11, 1.3797E-11, 1.3164E-11,
     &   1.2546E-11, 1.1944E-11, 1.1355E-11, 1.0782E-11, 1.0224E-11,
     &   9.6802E-12, 9.1515E-12, 8.6376E-12, 8.1386E-12, 7.6545E-12,
     &   7.1851E-12, 6.7307E-12, 6.2910E-12, 5.8662E-12, 5.4563E-12,
     &   5.0612E-12, 4.6810E-12, 4.3156E-12, 3.9650E-12, 3.6293E-12,
     &   3.3085E-12, 3.0024E-12, 2.7113E-12, 2.4350E-12, 2.1735E-12,
     &   1.9269E-12, 1.6951E-12, 1.4782E-12, 1.2761E-12, 1.0889E-12,
     &   9.1647E-13, 9.0005E-13, 8.8377E-13, 8.6765E-13, 8.5167E-13,
     &   8.3584E-13, 8.2016E-13, 8.0463E-13, 7.8925E-13, 7.7402E-13,
     &   7.5893E-13, 7.4399E-13, 7.2920E-13, 7.1456E-13, 7.0007E-13,
     &   6.8573E-13, 6.7153E-13, 6.5748E-13, 6.4359E-13, 6.2984E-13/
  
         DATA ((DAT(I,J), I= 801, 900), J=1,1) /
     &   6.1623E-13, 6.0278E-13, 5.8948E-13, 5.7632E-13, 5.6331E-13,
     &   5.5046E-13, 5.3774E-13, 5.2518E-13, 5.1277E-13, 5.0050E-13,
     &   4.8839E-13, 4.7642E-13, 4.6460E-13, 4.5293E-13, 4.4141E-13,
     &   4.3003E-13, 4.1881E-13, 4.0773E-13, 3.9680E-13, 3.8602E-13,
     &   3.7539E-13, 3.6490E-13, 3.5457E-13, 3.4438E-13, 3.3434E-13,
     &   3.2445E-13, 3.1471E-13, 3.0512E-13, 2.9568E-13, 2.8638E-13,
     &   2.7723E-13, 2.6823E-13, 2.5938E-13, 2.5068E-13, 2.4213E-13,
     &   2.3372E-13, 2.2547E-13, 2.1736E-13, 2.0940E-13, 2.0159E-13,
     &   1.9393E-13, 1.8641E-13, 1.7905E-13, 1.7183E-13, 1.6476E-13,
     &   1.5784E-13, 1.5107E-13, 1.4444E-13, 1.3797E-13, 1.3164E-13,
     &   1.2546E-13, 1.1944E-13, 1.1355E-13, 1.0782E-13, 1.0224E-13,
     &   9.6802E-14, 9.1515E-14, 8.6376E-14, 8.1386E-14, 7.6545E-14,
     &   7.1851E-14, 6.7306E-14, 6.2910E-14, 5.8662E-14, 5.4563E-14,
     &   5.0612E-14, 4.6810E-14, 4.3156E-14, 3.9650E-14, 3.6293E-14,
     &   3.3085E-14, 3.0024E-14, 2.7113E-14, 2.4350E-14, 2.1735E-14,
     &   1.9269E-14, 1.6951E-14, 1.4782E-14, 1.2761E-14, 1.0889E-14,
     &   9.1647E-15, 0.0000E+00,-9.7278E-15,-8.2539E-08,-3.3006E-07,
     &  -7.4263E-07,-1.3201E-06,-2.0630E-06,-2.9714E-06,-4.0453E-06,
     &  -5.2852E-06,-6.6912E-06,-8.2637E-06,-1.0003E-05,-1.1910E-05,
     &  -1.3984E-05,-1.6226E-05,-1.8637E-05,-2.1217E-05,-2.3967E-05/
  
         DATA ((DAT(I,J), I= 901, 979), J=1,1) /
     &  -2.6888E-05,-2.9979E-05,-3.3242E-05,-3.6678E-05,-4.0288E-05,
     &  -4.4071E-05,-4.8030E-05,-5.2165E-05,-5.6477E-05,-6.0967E-05,
     &  -6.5636E-05,-7.0486E-05,-7.5517E-05,-8.0731E-05,-8.6129E-05,
     &  -9.1713E-05,-9.7483E-05,-1.0344E-04,-1.0959E-04,-1.1593E-04,
     &  -1.2246E-04,-1.2919E-04,-1.3612E-04,-1.4324E-04,-1.5057E-04,
     &  -1.5809E-04,-1.6583E-04,-1.7376E-04,-1.8191E-04,-1.9027E-04,
     &  -1.9884E-04,-2.0763E-04,-2.1664E-04,-2.2588E-04,-2.3533E-04,
     &  -2.4502E-04,-2.5493E-04,-2.6508E-04,-2.7547E-04,-2.8610E-04,
     &  -2.9698E-04,-3.0810E-04,-3.1948E-04,-3.3111E-04,-3.4301E-04,
     &  -3.5517E-04,-3.6761E-04,-3.8032E-04,-3.9331E-04,-4.0660E-04,
     &  -4.2017E-04,-4.3405E-04,-4.4824E-04,-4.6274E-04,-4.7756E-04,
     &  -4.9272E-04,-5.0821E-04,-5.2405E-04,-5.4026E-04,-5.5683E-04,
     &  -5.7379E-04,-5.9115E-04,-6.0891E-04,-6.2711E-04,-6.4575E-04,
     &  -6.6486E-04,-6.8447E-04,-7.0459E-04,-7.2527E-04,-7.4654E-04,
     &  -7.6847E-04,-7.9110E-04,-8.1455E-04,-8.2257E-04,-8.3896E-04,
     &  -8.44818570E-04,-8.64600435E-04,-8.67252456E-04,-8.73520305E-04/
  
         DATA ((DAT(I,J), I=   1, 100), J=2,2) /
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04,
     &   1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7501E-04, 1.7500E-04,
     &   1.7500E-04, 1.7500E-04, 1.7500E-04, 1.7500E-04, 1.7500E-04,
     &   1.7500E-04, 1.7500E-04, 1.7500E-04, 1.7500E-04, 1.7500E-04,
     &   1.7500E-04, 1.7500E-04, 1.7500E-04, 1.7500E-04, 1.7500E-04,
     &   1.7500E-04, 1.7500E-04, 1.7500E-04, 1.7500E-04, 1.7500E-04,
     &   1.7499E-04, 1.7499E-04, 1.7499E-04, 1.7499E-04, 1.7499E-04,
     &   1.7499E-04, 1.7499E-04, 1.7499E-04, 1.7499E-04, 1.7499E-04,
     &   1.7499E-04, 1.7499E-04, 1.7499E-04, 1.7499E-04, 1.7498E-04,
     &   1.7498E-04, 1.7498E-04, 1.7498E-04, 1.7498E-04, 1.7498E-04,
     &   1.7498E-04, 1.7498E-04, 1.7498E-04, 1.7497E-04, 1.7497E-04,
     &   1.7497E-04, 1.7497E-04, 1.7497E-04, 1.7497E-04, 1.7497E-04,
     &   1.7496E-04, 1.7496E-04, 1.7496E-04, 1.7496E-04, 1.7496E-04,
     &   1.7495E-04, 1.7495E-04, 1.7495E-04, 1.7495E-04, 1.7494E-04,
     &   1.7494E-04, 1.7493E-04, 1.7493E-04, 1.7492E-04, 1.7492E-04,
     &   1.7491E-04, 1.7491E-04, 1.7490E-04, 1.7489E-04, 1.7488E-04,
     &   1.7486E-04, 1.7485E-04, 1.7483E-04, 1.7481E-04, 1.7478E-04,
     &   1.7474E-04, 1.7473E-04, 1.7473E-04, 1.7472E-04, 1.7472E-04,
     &   1.7471E-04, 1.7471E-04, 1.7470E-04, 1.7470E-04, 1.7469E-04/
  
         DATA ((DAT(I,J), I= 101, 200), J=2,2) /
     &   1.7469E-04, 1.7468E-04, 1.7468E-04, 1.7467E-04, 1.7466E-04,
     &   1.7466E-04, 1.7465E-04, 1.7464E-04, 1.7463E-04, 1.7463E-04,
     &   1.7462E-04, 1.7461E-04, 1.7460E-04, 1.7459E-04, 1.7458E-04,
     &   1.7457E-04, 1.7456E-04, 1.7455E-04, 1.7454E-04, 1.7453E-04,
     &   1.7452E-04, 1.7451E-04, 1.7449E-04, 1.7448E-04, 1.7447E-04,
     &   1.7445E-04, 1.7444E-04, 1.7442E-04, 1.7440E-04, 1.7439E-04,
     &   1.7437E-04, 1.7435E-04, 1.7433E-04, 1.7431E-04, 1.7428E-04,
     &   1.7426E-04, 1.7424E-04, 1.7421E-04, 1.7418E-04, 1.7415E-04,
     &   1.7412E-04, 1.7409E-04, 1.7405E-04, 1.7402E-04, 1.7398E-04,
     &   1.7394E-04, 1.7389E-04, 1.7385E-04, 1.7380E-04, 1.7374E-04,
     &   1.7368E-04, 1.7362E-04, 1.7355E-04, 1.7348E-04, 1.7341E-04,
     &   1.7332E-04, 1.7323E-04, 1.7313E-04, 1.7303E-04, 1.7291E-04,
     &   1.7278E-04, 1.7264E-04, 1.7249E-04, 1.7232E-04, 1.7213E-04,
     &   1.7192E-04, 1.7169E-04, 1.7143E-04, 1.7114E-04, 1.7081E-04,
     &   1.7044E-04, 1.7001E-04, 1.6953E-04, 1.6897E-04, 1.6832E-04,
     &   1.6756E-04, 1.6667E-04, 1.6560E-04, 1.6432E-04, 1.6276E-04,
     &   1.6083E-04, 1.6064E-04, 1.6044E-04, 1.6023E-04, 1.6003E-04,
     &   1.5981E-04, 1.5960E-04, 1.5937E-04, 1.5915E-04, 1.5891E-04,
     &   1.5868E-04, 1.5844E-04, 1.5819E-04, 1.5793E-04, 1.5768E-04,
     &   1.5741E-04, 1.5714E-04, 1.5686E-04, 1.5658E-04, 1.5629E-04/
  
         DATA ((DAT(I,J), I= 201, 300), J=2,2) /
     &   1.5599E-04, 1.5569E-04, 1.5537E-04, 1.5505E-04, 1.5473E-04,
     &   1.5439E-04, 1.5405E-04, 1.5370E-04, 1.5333E-04, 1.5296E-04,
     &   1.5258E-04, 1.5219E-04, 1.5179E-04, 1.5138E-04, 1.5096E-04,
     &   1.5053E-04, 1.5008E-04, 1.4963E-04, 1.4916E-04, 1.4868E-04,
     &   1.4818E-04, 1.4767E-04, 1.4715E-04, 1.4661E-04, 1.4606E-04,
     &   1.4549E-04, 1.4490E-04, 1.4429E-04, 1.4367E-04, 1.4303E-04,
     &   1.4237E-04, 1.4169E-04, 1.4098E-04, 1.4026E-04, 1.3951E-04,
     &   1.3874E-04, 1.3794E-04, 1.3712E-04, 1.3627E-04, 1.3539E-04,
     &   1.3449E-04, 1.3355E-04, 1.3258E-04, 1.3157E-04, 1.3053E-04,
     &   1.2945E-04, 1.2833E-04, 1.2717E-04, 1.2597E-04, 1.2472E-04,
     &   1.2342E-04, 1.2208E-04, 1.2067E-04, 1.1921E-04, 1.1769E-04,
     &   1.1611E-04, 1.1446E-04, 1.1274E-04, 1.1094E-04, 1.0905E-04,
     &   1.0709E-04, 1.0503E-04, 1.0287E-04, 1.0061E-04, 9.8239E-05,
     &   9.5750E-05, 9.3137E-05, 9.0392E-05, 8.7507E-05, 8.4476E-05,
     &   8.1291E-05, 7.7946E-05, 7.4434E-05, 7.0753E-05, 6.6899E-05,
     &   6.2873E-05, 5.8678E-05, 5.4322E-05, 4.9818E-05, 4.5186E-05,
     &   4.0452E-05, 3.9975E-05, 3.9496E-05, 3.9017E-05, 3.8538E-05,
     &   3.8058E-05, 3.7578E-05, 3.7097E-05, 3.6615E-05, 3.6134E-05,
     &   3.5652E-05, 3.5170E-05, 3.4688E-05, 3.4205E-05, 3.3723E-05,
     &   3.3240E-05, 3.2758E-05, 3.2275E-05, 3.1793E-05, 3.1311E-05/
  
         DATA ((DAT(I,J), I= 301, 400), J=2,2) /
     &   3.0829E-05, 3.0347E-05, 2.9866E-05, 2.9385E-05, 2.8905E-05,
     &   2.8425E-05, 2.7946E-05, 2.7467E-05, 2.6990E-05, 2.6513E-05,
     &   2.6037E-05, 2.5562E-05, 2.5087E-05, 2.4614E-05, 2.4143E-05,
     &   2.3672E-05, 2.3203E-05, 2.2735E-05, 2.2268E-05, 2.1803E-05,
     &   2.1340E-05, 2.0878E-05, 2.0418E-05, 1.9961E-05, 1.9504E-05,
     &   1.9051E-05, 1.8599E-05, 1.8149E-05, 1.7702E-05, 1.7257E-05,
     &   1.6814E-05, 1.6375E-05, 1.5937E-05, 1.5503E-05, 1.5072E-05,
     &   1.4643E-05, 1.4218E-05, 1.3796E-05, 1.3377E-05, 1.2962E-05,
     &   1.2550E-05, 1.2142E-05, 1.1737E-05, 1.1337E-05, 1.0941E-05,
     &   1.0549E-05, 1.0161E-05, 9.7774E-06, 9.3986E-06, 9.0245E-06,
     &   8.6553E-06, 8.2912E-06, 7.9322E-06, 7.5786E-06, 7.2305E-06,
     &   6.8880E-06, 6.5514E-06, 6.2209E-06, 5.8965E-06, 5.5785E-06,
     &   5.2670E-06, 4.9623E-06, 4.6646E-06, 4.3740E-06, 4.0908E-06,
     &   3.8152E-06, 3.5473E-06, 3.2874E-06, 3.0357E-06, 2.7925E-06,
     &   2.5579E-06, 2.3322E-06, 2.1156E-06, 1.9083E-06, 1.7105E-06,
     &   1.5226E-06, 1.3446E-06, 1.1768E-06, 1.0194E-06, 8.7264E-07,
     &   7.3670E-07, 7.2371E-07, 7.1082E-07, 6.9805E-07, 6.8539E-07,
     &   6.7284E-07, 6.6040E-07, 6.4808E-07, 6.3586E-07, 6.2375E-07,
     &   6.1176E-07, 5.9988E-07, 5.8811E-07, 5.7645E-07, 5.6491E-07,
     &   5.5347E-07, 5.4215E-07, 5.3095E-07, 5.1985E-07, 5.0887E-07/
  
         DATA ((DAT(I,J), I= 401, 500), J=2,2) /
     &   4.9800E-07, 4.8725E-07, 4.7661E-07, 4.6608E-07, 4.5567E-07,
     &   4.4537E-07, 4.3519E-07, 4.2512E-07, 4.1516E-07, 4.0532E-07,
     &   3.9560E-07, 3.8599E-07, 3.7649E-07, 3.6711E-07, 3.5784E-07,
     &   3.4870E-07, 3.3966E-07, 3.3075E-07, 3.2194E-07, 3.1326E-07,
     &   3.0469E-07, 2.9624E-07, 2.8790E-07, 2.7968E-07, 2.7158E-07,
     &   2.6359E-07, 2.5573E-07, 2.4798E-07, 2.4034E-07, 2.3283E-07,
     &   2.2543E-07, 2.1815E-07, 2.1098E-07, 2.0394E-07, 1.9701E-07,
     &   1.9020E-07, 1.8351E-07, 1.7694E-07, 1.7048E-07, 1.6415E-07,
     &   1.5793E-07, 1.5183E-07, 1.4585E-07, 1.3999E-07, 1.3425E-07,
     &   1.2863E-07, 1.2312E-07, 1.1774E-07, 1.1248E-07, 1.0733E-07,
     &   1.0231E-07, 9.7401E-08, 9.2615E-08, 8.7949E-08, 8.3403E-08,
     &   7.8976E-08, 7.4670E-08, 7.0484E-08, 6.6418E-08, 6.2473E-08,
     &   5.8647E-08, 5.4942E-08, 5.1358E-08, 4.7894E-08, 4.4551E-08,
     &   4.1328E-08, 3.8225E-08, 3.5244E-08, 3.2383E-08, 2.9643E-08,
     &   2.7024E-08, 2.4526E-08, 2.2149E-08, 1.9893E-08, 1.7757E-08,
     &   1.5743E-08, 1.3850E-08, 1.2078E-08, 1.0427E-08, 8.8978E-09,
     &   7.4893E-09, 7.3551E-09, 7.2222E-09, 7.0904E-09, 6.9599E-09,
     &   6.8306E-09, 6.7024E-09, 6.5755E-09, 6.4498E-09, 6.3254E-09,
     &   6.2021E-09, 6.0800E-09, 5.9592E-09, 5.8396E-09, 5.7211E-09,
     &   5.6039E-09, 5.4879E-09, 5.3732E-09, 5.2596E-09, 5.1472E-09/
  
         DATA ((DAT(I,J), I= 501, 600), J=2,2) /
     &   5.0361E-09, 4.9262E-09, 4.8175E-09, 4.7100E-09, 4.6037E-09,
     &   4.4986E-09, 4.3947E-09, 4.2921E-09, 4.1906E-09, 4.0904E-09,
     &   3.9914E-09, 3.8936E-09, 3.7970E-09, 3.7016E-09, 3.6074E-09,
     &   3.5145E-09, 3.4228E-09, 3.3322E-09, 3.2429E-09, 3.1548E-09,
     &   3.0679E-09, 2.9823E-09, 2.8978E-09, 2.8146E-09, 2.7325E-09,
     &   2.6517E-09, 2.5721E-09, 2.4937E-09, 2.4165E-09, 2.3405E-09,
     &   2.2658E-09, 2.1922E-09, 2.1199E-09, 2.0488E-09, 1.9789E-09,
     &   1.9102E-09, 1.8427E-09, 1.7765E-09, 1.7114E-09, 1.6476E-09,
     &   1.5850E-09, 1.5235E-09, 1.4633E-09, 1.4044E-09, 1.3466E-09,
     &   1.2900E-09, 1.2347E-09, 1.1806E-09, 1.1276E-09, 1.0759E-09,
     &   1.0254E-09, 9.7616E-10, 9.2810E-10, 8.8125E-10, 8.3561E-10,
     &   7.9118E-10, 7.4797E-10, 7.0597E-10, 6.6519E-10, 6.2561E-10,
     &   5.8726E-10, 5.5011E-10, 5.1418E-10, 4.7946E-10, 4.4596E-10,
     &   4.1367E-10, 3.8259E-10, 3.5272E-10, 3.2407E-10, 2.9663E-10,
     &   2.7041E-10, 2.4540E-10, 2.2160E-10, 1.9902E-10, 1.7765E-10,
     &   1.5749E-10, 1.3855E-10, 1.2082E-10, 1.0430E-10, 8.8996E-11,
     &   7.4906E-11, 7.3564E-11, 7.2234E-11, 7.0916E-11, 6.9610E-11,
     &   6.8316E-11, 6.7035E-11, 6.5765E-11, 6.4508E-11, 6.3263E-11,
     &   6.2030E-11, 6.0809E-11, 5.9600E-11, 5.8403E-11, 5.7219E-11,
     &   5.6047E-11, 5.4886E-11, 5.3738E-11, 5.2602E-11, 5.1479E-11/
  
         DATA ((DAT(I,J), I= 601, 700), J=2,2) /
     &   5.0367E-11, 4.9267E-11, 4.8180E-11, 4.7105E-11, 4.6041E-11,
     &   4.4990E-11, 4.3952E-11, 4.2925E-11, 4.1910E-11, 4.0908E-11,
     &   3.9917E-11, 3.8939E-11, 3.7973E-11, 3.7019E-11, 3.6077E-11,
     &   3.5148E-11, 3.4230E-11, 3.3325E-11, 3.2432E-11, 3.1551E-11,
     &   3.0682E-11, 2.9825E-11, 2.8980E-11, 2.8147E-11, 2.7327E-11,
     &   2.6519E-11, 2.5722E-11, 2.4938E-11, 2.4167E-11, 2.3407E-11,
     &   2.2659E-11, 2.1924E-11, 2.1200E-11, 2.0489E-11, 1.9790E-11,
     &   1.9103E-11, 1.8428E-11, 1.7765E-11, 1.7115E-11, 1.6476E-11,
     &   1.5850E-11, 1.5236E-11, 1.4634E-11, 1.4044E-11, 1.3466E-11,
     &   1.2901E-11, 1.2347E-11, 1.1806E-11, 1.1277E-11, 1.0760E-11,
     &   1.0255E-11, 9.7618E-12, 9.2812E-12, 8.8126E-12, 8.3562E-12,
     &   7.9120E-12, 7.4798E-12, 7.0598E-12, 6.6520E-12, 6.2562E-12,
     &   5.8726E-12, 5.5012E-12, 5.1419E-12, 4.7947E-12, 4.4596E-12,
     &   4.1367E-12, 3.8259E-12, 3.5273E-12, 3.2407E-12, 2.9664E-12,
     &   2.7041E-12, 2.4540E-12, 2.2160E-12, 1.9902E-12, 1.7765E-12,
     &   1.5749E-12, 1.3855E-12, 1.2082E-12, 1.0430E-12, 8.8996E-13,
     &   7.4906E-13, 7.3564E-13, 7.2234E-13, 7.0916E-13, 6.9610E-13,
     &   6.8316E-13, 6.7035E-13, 6.5765E-13, 6.4508E-13, 6.3263E-13,
     &   6.2030E-13, 6.0809E-13, 5.9600E-13, 5.8403E-13, 5.7219E-13,
     &   5.6047E-13, 5.4886E-13, 5.3738E-13, 5.2602E-13, 5.1479E-13/
  
         DATA ((DAT(I,J), I= 701, 800), J=2,2) /
     &   5.0367E-13, 4.9267E-13, 4.8180E-13, 4.7105E-13, 4.6042E-13,
     &   4.4990E-13, 4.3952E-13, 4.2925E-13, 4.1910E-13, 4.0908E-13,
     &   3.9917E-13, 3.8939E-13, 3.7973E-13, 3.7019E-13, 3.6077E-13,
     &   3.5148E-13, 3.4230E-13, 3.3325E-13, 3.2432E-13, 3.1551E-13,
     &   3.0682E-13, 2.9825E-13, 2.8980E-13, 2.8147E-13, 2.7327E-13,
     &   2.6519E-13, 2.5722E-13, 2.4938E-13, 2.4167E-13, 2.3407E-13,
     &   2.2659E-13, 2.1924E-13, 2.1200E-13, 2.0489E-13, 1.9790E-13,
     &   1.9103E-13, 1.8428E-13, 1.7765E-13, 1.7115E-13, 1.6476E-13,
     &   1.5850E-13, 1.5236E-13, 1.4634E-13, 1.4044E-13, 1.3466E-13,
     &   1.2901E-13, 1.2347E-13, 1.1806E-13, 1.1277E-13, 1.0760E-13,
     &   1.0255E-13, 9.7618E-14, 9.2812E-14, 8.8126E-14, 8.3562E-14,
     &   7.9120E-14, 7.4798E-14, 7.0598E-14, 6.6520E-14, 6.2562E-14,
     &   5.8726E-14, 5.5012E-14, 5.1419E-14, 4.7947E-14, 4.4596E-14,
     &   4.1367E-14, 3.8259E-14, 3.5273E-14, 3.2407E-14, 2.9664E-14,
     &   2.7041E-14, 2.4540E-14, 2.2160E-14, 1.9902E-14, 1.7765E-14,
     &   1.5749E-14, 1.3855E-14, 1.2082E-14, 1.0430E-14, 8.8996E-15,
     &   7.4906E-15, 7.3564E-15, 7.2234E-15, 7.0916E-15, 6.9610E-15,
     &   6.8316E-15, 6.7035E-15, 6.5765E-15, 6.4508E-15, 6.3263E-15,
     &   6.2030E-15, 6.0809E-15, 5.9600E-15, 5.8403E-15, 5.7219E-15,
     &   5.6047E-15, 5.4886E-15, 5.3738E-15, 5.2602E-15, 5.1479E-15/
  
         DATA ((DAT(I,J), I= 801, 900), J=2,2) /
     &   5.0367E-15, 4.9267E-15, 4.8180E-15, 4.7105E-15, 4.6041E-15,
     &   4.4990E-15, 4.3952E-15, 4.2925E-15, 4.1910E-15, 4.0908E-15,
     &   3.9917E-15, 3.8939E-15, 3.7973E-15, 3.7019E-15, 3.6077E-15,
     &   3.5148E-15, 3.4230E-15, 3.3325E-15, 3.2432E-15, 3.1551E-15,
     &   3.0682E-15, 2.9825E-15, 2.8980E-15, 2.8147E-15, 2.7327E-15,
     &   2.6519E-15, 2.5722E-15, 2.4938E-15, 2.4167E-15, 2.3407E-15,
     &   2.2659E-15, 2.1924E-15, 2.1200E-15, 2.0489E-15, 1.9790E-15,
     &   1.9103E-15, 1.8428E-15, 1.7765E-15, 1.7115E-15, 1.6476E-15,
     &   1.5850E-15, 1.5236E-15, 1.4634E-15, 1.4044E-15, 1.3466E-15,
     &   1.2901E-15, 1.2347E-15, 1.1806E-15, 1.1277E-15, 1.0760E-15,
     &   1.0255E-15, 9.7618E-16, 9.2812E-16, 8.8126E-16, 8.3562E-16,
     &   7.9120E-16, 7.4798E-16, 7.0598E-16, 6.6520E-16, 6.2562E-16,
     &   5.8726E-16, 5.5012E-16, 5.1419E-16, 4.7947E-16, 4.4596E-16,
     &   4.1367E-16, 3.8259E-16, 3.5273E-16, 3.2407E-16, 2.9664E-16,
     &   2.7041E-16, 2.4540E-16, 2.2160E-16, 1.9902E-16, 1.7765E-16,
     &   1.5749E-16, 1.3855E-16, 1.2082E-16, 1.0430E-16, 8.8996E-17,
     &   7.4906E-17, 0.0000E+00, 7.9779E-17, 6.7462E-10, 2.6977E-09,
     &   6.0696E-09, 1.0789E-08, 1.6860E-08, 2.4283E-08, 3.3059E-08,
     &   4.3189E-08, 5.4675E-08, 6.7520E-08, 8.1726E-08, 9.7296E-08,
     &   1.1423E-07, 1.3254E-07, 1.5222E-07, 1.7328E-07, 1.9571E-07/
  
         DATA ((DAT(I,J), I= 901, 979), J=2,2) /
     &   2.1954E-07, 2.4475E-07, 2.7136E-07, 2.9937E-07, 3.2879E-07,
     &   3.5961E-07, 3.9186E-07, 4.2553E-07, 4.6063E-07, 4.9718E-07,
     &   5.3516E-07, 5.7461E-07, 6.1551E-07, 6.5789E-07, 7.0175E-07,
     &   7.4711E-07, 7.9396E-07, 8.4232E-07, 8.9221E-07, 9.4363E-07,
     &   9.9660E-07, 1.0511E-06, 1.1072E-06, 1.1649E-06, 1.2242E-06,
     &   1.2851E-06, 1.3477E-06, 1.4118E-06, 1.4777E-06, 1.5452E-06,
     &   1.6145E-06, 1.6854E-06, 1.7581E-06, 1.8326E-06, 1.9089E-06,
     &   1.9869E-06, 2.0668E-06, 2.1486E-06, 2.2323E-06, 2.3179E-06,
     &   2.4054E-06, 2.4949E-06, 2.5865E-06, 2.6800E-06, 2.7757E-06,
     &   2.8735E-06, 2.9735E-06, 3.0757E-06, 3.1802E-06, 3.2870E-06,
     &   3.3961E-06, 3.5077E-06, 3.6218E-06, 3.7384E-06, 3.8577E-06,
     &   3.9796E-06, 4.1044E-06, 4.2320E-06, 4.3626E-06, 4.4963E-06,
     &   4.6332E-06, 4.7733E-06, 4.9170E-06, 5.0642E-06, 5.2152E-06,
     &   5.3702E-06, 5.5293E-06, 5.6928E-06, 5.8610E-06, 6.0341E-06,
     &   6.2126E-06, 6.3968E-06, 6.5872E-06, 6.6521E-06, 6.7844E-06,
     &   6.83142760E-06, 6.98914060E-06, 7.01007560E-06, 7.05926463E-06/
c deg
         DATA ((DGT(I,J), I=   1, 100), J=1,1) /
     &   8.5411E-02, 8.5290E-02, 8.5168E-02, 8.5044E-02, 8.4919E-02,
     &   8.4793E-02, 8.4665E-02, 8.4536E-02, 8.4406E-02, 8.4274E-02,
     &   8.4141E-02, 8.4006E-02, 8.3870E-02, 8.3732E-02, 8.3593E-02,
     &   8.3451E-02, 8.3309E-02, 8.3164E-02, 8.3018E-02, 8.2870E-02,
     &   8.2720E-02, 8.2568E-02, 8.2415E-02, 8.2259E-02, 8.2101E-02,
     &   8.1941E-02, 8.1779E-02, 8.1615E-02, 8.1449E-02, 8.1280E-02,
     &   8.1109E-02, 8.0935E-02, 8.0759E-02, 8.0580E-02, 8.0398E-02,
     &   8.0214E-02, 8.0027E-02, 7.9837E-02, 7.9644E-02, 7.9447E-02,
     &   7.9248E-02, 7.9045E-02, 7.8838E-02, 7.8628E-02, 7.8414E-02,
     &   7.8197E-02, 7.7975E-02, 7.7749E-02, 7.7519E-02, 7.7285E-02,
     &   7.7045E-02, 7.6801E-02, 7.6552E-02, 7.6298E-02, 7.6038E-02,
     &   7.5772E-02, 7.5500E-02, 7.5223E-02, 7.4938E-02, 7.4647E-02,
     &   7.4348E-02, 7.4042E-02, 7.3728E-02, 7.3405E-02, 7.3074E-02,
     &   7.2733E-02, 7.2382E-02, 7.2021E-02, 7.1649E-02, 7.1264E-02,
     &   7.0867E-02, 7.0457E-02, 7.0032E-02, 6.9592E-02, 6.9135E-02,
     &   6.8660E-02, 6.8165E-02, 6.7649E-02, 6.7111E-02, 6.6547E-02,
     &   6.5955E-02, 6.5333E-02, 6.4678E-02, 6.3984E-02, 6.3248E-02,
     &   6.2465E-02, 6.1627E-02, 6.0727E-02, 5.9755E-02, 5.8698E-02,
     &   5.7539E-02, 5.7417E-02, 5.7293E-02, 5.7168E-02, 5.7042E-02,
     &   5.6915E-02, 5.6786E-02, 5.6656E-02, 5.6525E-02, 5.6392E-02/
  
         DATA ((DGT(I,J), I= 101, 200), J=1,1) /
     &   5.6257E-02, 5.6121E-02, 5.5984E-02, 5.5845E-02, 5.5704E-02,
     &   5.5562E-02, 5.5418E-02, 5.5272E-02, 5.5125E-02, 5.4975E-02,
     &   5.4824E-02, 5.4671E-02, 5.4516E-02, 5.4359E-02, 5.4200E-02,
     &   5.4039E-02, 5.3875E-02, 5.3710E-02, 5.3542E-02, 5.3372E-02,
     &   5.3199E-02, 5.3024E-02, 5.2846E-02, 5.2666E-02, 5.2483E-02,
     &   5.2297E-02, 5.2108E-02, 5.1916E-02, 5.1721E-02, 5.1523E-02,
     &   5.1322E-02, 5.1117E-02, 5.0909E-02, 5.0697E-02, 5.0482E-02,
     &   5.0262E-02, 5.0039E-02, 4.9811E-02, 4.9579E-02, 4.9343E-02,
     &   4.9102E-02, 4.8856E-02, 4.8604E-02, 4.8348E-02, 4.8086E-02,
     &   4.7819E-02, 4.7545E-02, 4.7265E-02, 4.6979E-02, 4.6685E-02,
     &   4.6385E-02, 4.6077E-02, 4.5761E-02, 4.5436E-02, 4.5103E-02,
     &   4.4760E-02, 4.4408E-02, 4.4045E-02, 4.3671E-02, 4.3286E-02,
     &   4.2888E-02, 4.2476E-02, 4.2051E-02, 4.1610E-02, 4.1153E-02,
     &   4.0678E-02, 4.0185E-02, 3.9671E-02, 3.9134E-02, 3.8574E-02,
     &   3.7987E-02, 3.7371E-02, 3.6724E-02, 3.6041E-02, 3.5318E-02,
     &   3.4552E-02, 3.3736E-02, 3.2863E-02, 3.1926E-02, 3.0915E-02,
     &   2.9815E-02, 2.9712E-02, 2.9607E-02, 2.9501E-02, 2.9395E-02,
     &   2.9288E-02, 2.9180E-02, 2.9070E-02, 2.8960E-02, 2.8849E-02,
     &   2.8737E-02, 2.8624E-02, 2.8510E-02, 2.8395E-02, 2.8279E-02,
     &   2.8162E-02, 2.8044E-02, 2.7924E-02, 2.7804E-02, 2.7682E-02/
  
         DATA ((DGT(I,J), I= 201, 300), J=1,1) /
     &   2.7559E-02, 2.7435E-02, 2.7310E-02, 2.7183E-02, 2.7056E-02,
     &   2.6927E-02, 2.6796E-02, 2.6664E-02, 2.6531E-02, 2.6397E-02,
     &   2.6260E-02, 2.6123E-02, 2.5984E-02, 2.5843E-02, 2.5701E-02,
     &   2.5557E-02, 2.5412E-02, 2.5265E-02, 2.5116E-02, 2.4965E-02,
     &   2.4813E-02, 2.4659E-02, 2.4502E-02, 2.4344E-02, 2.4184E-02,
     &   2.4022E-02, 2.3857E-02, 2.3691E-02, 2.3522E-02, 2.3351E-02,
     &   2.3177E-02, 2.3001E-02, 2.2823E-02, 2.2642E-02, 2.2458E-02,
     &   2.2272E-02, 2.2082E-02, 2.1890E-02, 2.1695E-02, 2.1497E-02,
     &   2.1295E-02, 2.1091E-02, 2.0882E-02, 2.0671E-02, 2.0455E-02,
     &   2.0236E-02, 2.0013E-02, 1.9786E-02, 1.9554E-02, 1.9318E-02,
     &   1.9078E-02, 1.8833E-02, 1.8582E-02, 1.8327E-02, 1.8066E-02,
     &   1.7799E-02, 1.7527E-02, 1.7248E-02, 1.6963E-02, 1.6671E-02,
     &   1.6371E-02, 1.6064E-02, 1.5749E-02, 1.5425E-02, 1.5091E-02,
     &   1.4748E-02, 1.4395E-02, 1.4030E-02, 1.3654E-02, 1.3264E-02,
     &   1.2860E-02, 1.2440E-02, 1.2004E-02, 1.1549E-02, 1.1074E-02,
     &   1.0577E-02, 1.0055E-02, 9.5048E-03, 8.9244E-03, 8.3104E-03,
     &   7.6594E-03, 7.5922E-03, 7.5245E-03, 7.4565E-03, 7.3880E-03,
     &   7.3191E-03, 7.2498E-03, 7.1801E-03, 7.1100E-03, 7.0395E-03,
     &   6.9685E-03, 6.8971E-03, 6.8253E-03, 6.7531E-03, 6.6804E-03,
     &   6.6074E-03, 6.5338E-03, 6.4599E-03, 6.3855E-03, 6.3107E-03/
  
         DATA ((DGT(I,J), I= 301, 400), J=1,1) /
     &   6.2355E-03, 6.1598E-03, 6.0837E-03, 6.0072E-03, 5.9302E-03,
     &   5.8529E-03, 5.7751E-03, 5.6968E-03, 5.6182E-03, 5.5392E-03,
     &   5.4597E-03, 5.3799E-03, 5.2996E-03, 5.2190E-03, 5.1379E-03,
     &   5.0565E-03, 4.9747E-03, 4.8926E-03, 4.8101E-03, 4.7272E-03,
     &   4.6441E-03, 4.5605E-03, 4.4767E-03, 4.3926E-03, 4.3082E-03,
     &   4.2235E-03, 4.1386E-03, 4.0534E-03, 3.9681E-03, 3.8825E-03,
     &   3.7967E-03, 3.7108E-03, 3.6248E-03, 3.5386E-03, 3.4524E-03,
     &   3.3661E-03, 3.2798E-03, 3.1934E-03, 3.1072E-03, 3.0210E-03,
     &   2.9348E-03, 2.8489E-03, 2.7631E-03, 2.6775E-03, 2.5922E-03,
     &   2.5072E-03, 2.4225E-03, 2.3382E-03, 2.2544E-03, 2.1711E-03,
     &   2.0883E-03, 2.0061E-03, 1.9246E-03, 1.8437E-03, 1.7637E-03,
     &   1.6845E-03, 1.6062E-03, 1.5289E-03, 1.4526E-03, 1.3775E-03,
     &   1.3035E-03, 1.2307E-03, 1.1593E-03, 1.0892E-03, 1.0207E-03,
     &   9.5366E-04, 8.8827E-04, 8.2459E-04, 7.6269E-04, 7.0266E-04,
     &   6.4457E-04, 5.8850E-04, 5.3453E-04, 4.8275E-04, 4.3322E-04,
     &   3.8603E-04, 3.4124E-04, 2.9893E-04, 2.5918E-04, 2.2204E-04,
     &   1.8758E-04, 1.8429E-04, 1.8102E-04, 1.7778E-04, 1.7457E-04,
     &   1.7138E-04, 1.6822E-04, 1.6509E-04, 1.6199E-04, 1.5892E-04,
     &   1.5587E-04, 1.5285E-04, 1.4986E-04, 1.4690E-04, 1.4397E-04,
     &   1.4106E-04, 1.3819E-04, 1.3534E-04, 1.3252E-04, 1.2972E-04/
  
         DATA ((DGT(I,J), I= 401, 500), J=1,1) /
     &   1.2696E-04, 1.2423E-04, 1.2152E-04, 1.1884E-04, 1.1619E-04,
     &   1.1357E-04, 1.1098E-04, 1.0842E-04, 1.0589E-04, 1.0338E-04,
     &   1.0091E-04, 9.8459E-05, 9.6041E-05, 9.3653E-05, 9.1294E-05,
     &   8.8964E-05, 8.6663E-05, 8.4391E-05, 8.2149E-05, 7.9937E-05,
     &   7.7753E-05, 7.5600E-05, 7.3475E-05, 7.1381E-05, 6.9315E-05,
     &   6.7280E-05, 6.5274E-05, 6.3298E-05, 6.1352E-05, 5.9435E-05,
     &   5.7549E-05, 5.5692E-05, 5.3865E-05, 5.2068E-05, 5.0301E-05,
     &   4.8564E-05, 4.6857E-05, 4.5180E-05, 4.3534E-05, 4.1917E-05,
     &   4.0331E-05, 3.8775E-05, 3.7249E-05, 3.5753E-05, 3.4288E-05,
     &   3.2853E-05, 3.1448E-05, 3.0074E-05, 2.8730E-05, 2.7416E-05,
     &   2.6133E-05, 2.4881E-05, 2.3659E-05, 2.2467E-05, 2.1306E-05,
     &   2.0176E-05, 1.9076E-05, 1.8007E-05, 1.6969E-05, 1.5961E-05,
     &   1.4984E-05, 1.4038E-05, 1.3122E-05, 1.2237E-05, 1.1383E-05,
     &   1.0560E-05, 9.7674E-06, 9.0057E-06, 8.2748E-06, 7.5748E-06,
     &   6.9056E-06, 6.2673E-06, 5.6599E-06, 5.0834E-06, 4.5378E-06,
     &   4.0232E-06, 3.5394E-06, 3.0866E-06, 2.6648E-06, 2.2739E-06,
     &   1.9140E-06, 1.8797E-06, 1.8457E-06, 1.8120E-06, 1.7787E-06,
     &   1.7456E-06, 1.7129E-06, 1.6804E-06, 1.6483E-06, 1.6165E-06,
     &   1.5850E-06, 1.5538E-06, 1.5229E-06, 1.4924E-06, 1.4621E-06,
     &   1.4321E-06, 1.4025E-06, 1.3732E-06, 1.3441E-06, 1.3154E-06/
  
         DATA ((DGT(I,J), I= 501, 600), J=1,1) /
     &   1.2870E-06, 1.2589E-06, 1.2312E-06, 1.2037E-06, 1.1765E-06,
     &   1.1497E-06, 1.1231E-06, 1.0969E-06, 1.0710E-06, 1.0453E-06,
     &   1.0200E-06, 9.9505E-07, 9.7037E-07, 9.4599E-07, 9.2193E-07,
     &   8.9817E-07, 8.7473E-07, 8.5159E-07, 8.2877E-07, 8.0626E-07,
     &   7.8405E-07, 7.6216E-07, 7.4057E-07, 7.1930E-07, 6.9833E-07,
     &   6.7768E-07, 6.5733E-07, 6.3730E-07, 6.1757E-07, 5.9816E-07,
     &   5.7905E-07, 5.6026E-07, 5.4178E-07, 5.2360E-07, 5.0574E-07,
     &   4.8818E-07, 4.7094E-07, 4.5400E-07, 4.3738E-07, 4.2106E-07,
     &   4.0506E-07, 3.8936E-07, 3.7398E-07, 3.5891E-07, 3.4414E-07,
     &   3.2969E-07, 3.1554E-07, 3.0171E-07, 2.8819E-07, 2.7497E-07,
     &   2.6207E-07, 2.4947E-07, 2.3719E-07, 2.2522E-07, 2.1355E-07,
     &   2.0220E-07, 1.9116E-07, 1.8042E-07, 1.7000E-07, 1.5989E-07,
     &   1.5008E-07, 1.4059E-07, 1.3141E-07, 1.2253E-07, 1.1397E-07,
     &   1.0572E-07, 9.7776E-08, 9.0144E-08, 8.2822E-08, 7.5810E-08,
     &   6.9108E-08, 6.2716E-08, 5.6634E-08, 5.0862E-08, 4.5401E-08,
     &   4.0249E-08, 3.5408E-08, 3.0876E-08, 2.6655E-08, 2.2744E-08,
     &   1.9143E-08, 1.8800E-08, 1.8461E-08, 1.8124E-08, 1.7790E-08,
     &   1.7459E-08, 1.7132E-08, 1.6807E-08, 1.6486E-08, 1.6168E-08,
     &   1.5853E-08, 1.5541E-08, 1.5232E-08, 1.4926E-08, 1.4623E-08,
     &   1.4324E-08, 1.4027E-08, 1.3734E-08, 1.3443E-08, 1.3156E-08/
  
         DATA ((DGT(I,J), I= 601, 700), J=1,1) /
     &   1.2872E-08, 1.2591E-08, 1.2313E-08, 1.2038E-08, 1.1767E-08,
     &   1.1498E-08, 1.1233E-08, 1.0970E-08, 1.0711E-08, 1.0455E-08,
     &   1.0202E-08, 9.9516E-09, 9.7047E-09, 9.4609E-09, 9.2202E-09,
     &   8.9826E-09, 8.7481E-09, 8.5167E-09, 8.2884E-09, 8.0633E-09,
     &   7.8412E-09, 7.6222E-09, 7.4063E-09, 7.1935E-09, 6.9839E-09,
     &   6.7773E-09, 6.5738E-09, 6.3734E-09, 6.1762E-09, 5.9820E-09,
     &   5.7909E-09, 5.6029E-09, 5.4181E-09, 5.2363E-09, 5.0576E-09,
     &   4.8821E-09, 4.7096E-09, 4.5402E-09, 4.3740E-09, 4.2108E-09,
     &   4.0508E-09, 3.8938E-09, 3.7400E-09, 3.5892E-09, 3.4415E-09,
     &   3.2970E-09, 3.1555E-09, 3.0172E-09, 2.8819E-09, 2.7498E-09,
     &   2.6207E-09, 2.4948E-09, 2.3720E-09, 2.2522E-09, 2.1356E-09,
     &   2.0220E-09, 1.9116E-09, 1.8043E-09, 1.7000E-09, 1.5989E-09,
     &   1.5009E-09, 1.4059E-09, 1.3141E-09, 1.2254E-09, 1.1397E-09,
     &   1.0572E-09, 9.7777E-10, 9.0145E-10, 8.2822E-10, 7.5810E-10,
     &   6.9108E-10, 6.2716E-10, 5.6634E-10, 5.0862E-10, 4.5401E-10,
     &   4.0249E-10, 3.5408E-10, 3.0877E-10, 2.6655E-10, 2.2744E-10,
     &   1.9144E-10, 1.8800E-10, 1.8461E-10, 1.8124E-10, 1.7790E-10,
     &   1.7459E-10, 1.7132E-10, 1.6807E-10, 1.6486E-10, 1.6168E-10,
     &   1.5853E-10, 1.5541E-10, 1.5232E-10, 1.4926E-10, 1.4623E-10,
     &   1.4324E-10, 1.4027E-10, 1.3734E-10, 1.3443E-10, 1.3156E-10/
  
         DATA ((DGT(I,J), I= 701, 800), J=1,1) /
     &   1.2872E-10, 1.2591E-10, 1.2313E-10, 1.2038E-10, 1.1767E-10,
     &   1.1498E-10, 1.1233E-10, 1.0970E-10, 1.0711E-10, 1.0455E-10,
     &   1.0202E-10, 9.9516E-11, 9.7047E-11, 9.4609E-11, 9.2202E-11,
     &   8.9826E-11, 8.7481E-11, 8.5167E-11, 8.2884E-11, 8.0633E-11,
     &   7.8412E-11, 7.6222E-11, 7.4063E-11, 7.1935E-11, 6.9839E-11,
     &   6.7773E-11, 6.5738E-11, 6.3734E-11, 6.1762E-11, 5.9820E-11,
     &   5.7909E-11, 5.6029E-11, 5.4181E-11, 5.2363E-11, 5.0576E-11,
     &   4.8821E-11, 4.7096E-11, 4.5402E-11, 4.3740E-11, 4.2108E-11,
     &   4.0508E-11, 3.8938E-11, 3.7400E-11, 3.5892E-11, 3.4415E-11,
     &   3.2970E-11, 3.1555E-11, 3.0172E-11, 2.8819E-11, 2.7498E-11,
     &   2.6207E-11, 2.4948E-11, 2.3720E-11, 2.2522E-11, 2.1356E-11,
     &   2.0220E-11, 1.9116E-11, 1.8043E-11, 1.7000E-11, 1.5989E-11,
     &   1.5009E-11, 1.4059E-11, 1.3141E-11, 1.2254E-11, 1.1397E-11,
     &   1.0572E-11, 9.7777E-12, 9.0145E-12, 8.2822E-12, 7.5810E-12,
     &   6.9108E-12, 6.2716E-12, 5.6634E-12, 5.0862E-12, 4.5401E-12,
     &   4.0249E-12, 3.5408E-12, 3.0877E-12, 2.6655E-12, 2.2744E-12,
     &   1.9144E-12, 1.8800E-12, 1.8461E-12, 1.8124E-12, 1.7790E-12,
     &   1.7459E-12, 1.7132E-12, 1.6807E-12, 1.6486E-12, 1.6168E-12,
     &   1.5853E-12, 1.5541E-12, 1.5232E-12, 1.4926E-12, 1.4623E-12,
     &   1.4324E-12, 1.4027E-12, 1.3734E-12, 1.3443E-12, 1.3156E-12/
  
         DATA ((DGT(I,J), I= 801, 900), J=1,1) /
     &   1.2872E-12, 1.2591E-12, 1.2313E-12, 1.2038E-12, 1.1767E-12,
     &   1.1498E-12, 1.1233E-12, 1.0970E-12, 1.0711E-12, 1.0455E-12,
     &   1.0202E-12, 9.9516E-13, 9.7047E-13, 9.4609E-13, 9.2202E-13,
     &   8.9826E-13, 8.7481E-13, 8.5167E-13, 8.2884E-13, 8.0633E-13,
     &   7.8412E-13, 7.6222E-13, 7.4063E-13, 7.1935E-13, 6.9839E-13,
     &   6.7773E-13, 6.5738E-13, 6.3734E-13, 6.1762E-13, 5.9820E-13,
     &   5.7909E-13, 5.6029E-13, 5.4181E-13, 5.2363E-13, 5.0576E-13,
     &   4.8821E-13, 4.7096E-13, 4.5402E-13, 4.3740E-13, 4.2108E-13,
     &   4.0508E-13, 3.8938E-13, 3.7400E-13, 3.5892E-13, 3.4415E-13,
     &   3.2970E-13, 3.1555E-13, 3.0172E-13, 2.8819E-13, 2.7498E-13,
     &   2.6207E-13, 2.4948E-13, 2.3720E-13, 2.2522E-13, 2.1356E-13,
     &   2.0220E-13, 1.9116E-13, 1.8043E-13, 1.7000E-13, 1.5989E-13,
     &   1.5009E-13, 1.4059E-13, 1.3141E-13, 1.2254E-13, 1.1397E-13,
     &   1.0572E-13, 9.7777E-14, 9.0145E-14, 8.2822E-14, 7.5810E-14,
     &   6.9108E-14, 6.2716E-14, 5.6634E-14, 5.0862E-14, 4.5401E-14,
     &   4.0249E-14, 3.5408E-14, 3.0877E-14, 2.6655E-14, 2.2744E-14,
     &   1.9144E-14, 0.0000E+00,-1.9144E-14,-1.7241E-07,-6.8945E-07,
     &  -1.5512E-06,-2.7575E-06,-4.3093E-06,-6.2067E-06,-8.4500E-06,
     &  -1.1040E-05,-1.3977E-05,-1.7261E-05,-2.0895E-05,-2.4877E-05,
     &  -2.9210E-05,-3.3894E-05,-3.8930E-05,-4.4319E-05,-5.0064E-05/
  
         DATA ((DGT(I,J), I= 901, 979), J=1,1) /
     &  -5.6164E-05,-6.2622E-05,-6.9438E-05,-7.6616E-05,-8.4155E-05,
     &  -9.2059E-05,-1.0033E-04,-1.0897E-04,-1.1797E-04,-1.2735E-04,
     &  -1.3711E-04,-1.4724E-04,-1.5775E-04,-1.6864E-04,-1.7991E-04,
     &  -1.9158E-04,-2.0363E-04,-2.1608E-04,-2.2893E-04,-2.4217E-04,
     &  -2.5582E-04,-2.6988E-04,-2.8434E-04,-2.9923E-04,-3.1453E-04,
     &  -3.3025E-04,-3.4640E-04,-3.6299E-04,-3.8001E-04,-3.9748E-04,
     &  -4.1539E-04,-4.3376E-04,-4.5258E-04,-4.7187E-04,-4.9163E-04,
     &  -5.1186E-04,-5.3258E-04,-5.5379E-04,-5.7550E-04,-5.9771E-04,
     &  -6.2044E-04,-6.4369E-04,-6.6746E-04,-6.9178E-04,-7.1665E-04,
     &  -7.4207E-04,-7.6806E-04,-7.9463E-04,-8.2180E-04,-8.4957E-04,
     &  -8.7795E-04,-9.0697E-04,-9.3663E-04,-9.6695E-04,-9.9795E-04,
     &  -1.0296E-03,-1.0620E-03,-1.0952E-03,-1.1291E-03,-1.1638E-03,
     &  -1.1992E-03,-1.2356E-03,-1.2728E-03,-1.3108E-03,-1.3499E-03,
     &  -1.3899E-03,-1.4310E-03,-1.4731E-03,-1.5164E-03,-1.5610E-03,
     &  -1.6070E-03,-1.6545E-03,-1.7037E-03,-1.7205E-03,-1.7550E-03,
     &  -1.76726462E-03,-1.80884495E-03,-1.81442226E-03,-1.82760723E-03/
  
         DATA ((DGT(I,J), I=   1, 100), J=2,2) /
     &   3.5755E-04, 3.5755E-04, 3.5755E-04, 3.5755E-04, 3.5755E-04,
     &   3.5755E-04, 3.5755E-04, 3.5755E-04, 3.5755E-04, 3.5755E-04,
     &   3.5755E-04, 3.5754E-04, 3.5754E-04, 3.5754E-04, 3.5754E-04,
     &   3.5754E-04, 3.5754E-04, 3.5754E-04, 3.5754E-04, 3.5754E-04,
     &   3.5754E-04, 3.5753E-04, 3.5753E-04, 3.5753E-04, 3.5753E-04,
     &   3.5753E-04, 3.5753E-04, 3.5753E-04, 3.5753E-04, 3.5753E-04,
     &   3.5752E-04, 3.5752E-04, 3.5752E-04, 3.5752E-04, 3.5752E-04,
     &   3.5752E-04, 3.5752E-04, 3.5751E-04, 3.5751E-04, 3.5751E-04,
     &   3.5751E-04, 3.5751E-04, 3.5751E-04, 3.5750E-04, 3.5750E-04,
     &   3.5750E-04, 3.5750E-04, 3.5750E-04, 3.5750E-04, 3.5749E-04,
     &   3.5749E-04, 3.5749E-04, 3.5749E-04, 3.5748E-04, 3.5748E-04,
     &   3.5748E-04, 3.5748E-04, 3.5747E-04, 3.5747E-04, 3.5747E-04,
     &   3.5746E-04, 3.5746E-04, 3.5746E-04, 3.5745E-04, 3.5745E-04,
     &   3.5744E-04, 3.5744E-04, 3.5743E-04, 3.5743E-04, 3.5742E-04,
     &   3.5741E-04, 3.5741E-04, 3.5740E-04, 3.5739E-04, 3.5738E-04,
     &   3.5737E-04, 3.5736E-04, 3.5735E-04, 3.5734E-04, 3.5732E-04,
     &   3.5731E-04, 3.5729E-04, 3.5727E-04, 3.5724E-04, 3.5721E-04,
     &   3.5718E-04, 3.5713E-04, 3.5708E-04, 3.5702E-04, 3.5694E-04,
     &   3.5684E-04, 3.5682E-04, 3.5681E-04, 3.5680E-04, 3.5679E-04,
     &   3.5677E-04, 3.5676E-04, 3.5674E-04, 3.5673E-04, 3.5671E-04/
  
         DATA ((DGT(I,J), I= 101, 200), J=2,2) /
     &   3.5670E-04, 3.5668E-04, 3.5667E-04, 3.5665E-04, 3.5663E-04,
     &   3.5661E-04, 3.5659E-04, 3.5657E-04, 3.5655E-04, 3.5653E-04,
     &   3.5651E-04, 3.5649E-04, 3.5646E-04, 3.5644E-04, 3.5641E-04,
     &   3.5639E-04, 3.5636E-04, 3.5633E-04, 3.5630E-04, 3.5627E-04,
     &   3.5624E-04, 3.5621E-04, 3.5617E-04, 3.5614E-04, 3.5610E-04,
     &   3.5606E-04, 3.5602E-04, 3.5597E-04, 3.5593E-04, 3.5588E-04,
     &   3.5583E-04, 3.5578E-04, 3.5573E-04, 3.5567E-04, 3.5561E-04,
     &   3.5555E-04, 3.5548E-04, 3.5541E-04, 3.5534E-04, 3.5526E-04,
     &   3.5518E-04, 3.5509E-04, 3.5500E-04, 3.5490E-04, 3.5480E-04,
     &   3.5469E-04, 3.5457E-04, 3.5444E-04, 3.5431E-04, 3.5417E-04,
     &   3.5402E-04, 3.5385E-04, 3.5368E-04, 3.5349E-04, 3.5329E-04,
     &   3.5307E-04, 3.5284E-04, 3.5258E-04, 3.5231E-04, 3.5201E-04,
     &   3.5169E-04, 3.5133E-04, 3.5094E-04, 3.5052E-04, 3.5005E-04,
     &   3.4954E-04, 3.4897E-04, 3.4835E-04, 3.4765E-04, 3.4687E-04,
     &   3.4600E-04, 3.4502E-04, 3.4392E-04, 3.4267E-04, 3.4125E-04,
     &   3.3962E-04, 3.3774E-04, 3.3556E-04, 3.3300E-04, 3.2996E-04,
     &   3.2630E-04, 3.2594E-04, 3.2557E-04, 3.2519E-04, 3.2480E-04,
     &   3.2441E-04, 3.2400E-04, 3.2359E-04, 3.2318E-04, 3.2275E-04,
     &   3.2232E-04, 3.2187E-04, 3.2142E-04, 3.2096E-04, 3.2049E-04,
     &   3.2001E-04, 3.1951E-04, 3.1901E-04, 3.1850E-04, 3.1798E-04/
  
         DATA ((DGT(I,J), I= 201, 300), J=2,2) /
     &   3.1744E-04, 3.1689E-04, 3.1633E-04, 3.1576E-04, 3.1517E-04,
     &   3.1457E-04, 3.1396E-04, 3.1333E-04, 3.1269E-04, 3.1203E-04,
     &   3.1135E-04, 3.1066E-04, 3.0994E-04, 3.0922E-04, 3.0847E-04,
     &   3.0770E-04, 3.0691E-04, 3.0610E-04, 3.0527E-04, 3.0442E-04,
     &   3.0354E-04, 3.0264E-04, 3.0171E-04, 3.0075E-04, 2.9977E-04,
     &   2.9875E-04, 2.9771E-04, 2.9663E-04, 2.9553E-04, 2.9438E-04,
     &   2.9320E-04, 2.9198E-04, 2.9073E-04, 2.8943E-04, 2.8808E-04,
     &   2.8669E-04, 2.8526E-04, 2.8377E-04, 2.8223E-04, 2.8063E-04,
     &   2.7898E-04, 2.7726E-04, 2.7548E-04, 2.7363E-04, 2.7171E-04,
     &   2.6971E-04, 2.6763E-04, 2.6547E-04, 2.6321E-04, 2.6086E-04,
     &   2.5841E-04, 2.5585E-04, 2.5318E-04, 2.5038E-04, 2.4746E-04,
     &   2.4439E-04, 2.4118E-04, 2.3782E-04, 2.3428E-04, 2.3057E-04,
     &   2.2667E-04, 2.2257E-04, 2.1825E-04, 2.1369E-04, 2.0890E-04,
     &   2.0384E-04, 1.9850E-04, 1.9286E-04, 1.8691E-04, 1.8064E-04,
     &   1.7401E-04, 1.6703E-04, 1.5966E-04, 1.5192E-04, 1.4378E-04,
     &   1.3525E-04, 1.2633E-04, 1.1705E-04, 1.0742E-04, 9.7498E-05,
     &   8.7329E-05, 8.6302E-05, 8.5273E-05, 8.4242E-05, 8.3210E-05,
     &   8.2177E-05, 8.1143E-05, 8.0107E-05, 7.9071E-05, 7.8033E-05,
     &   7.6995E-05, 7.5956E-05, 7.4916E-05, 7.3876E-05, 7.2836E-05,
     &   7.1795E-05, 7.0754E-05, 6.9713E-05, 6.8673E-05, 6.7632E-05/
  
         DATA ((DGT(I,J), I= 301, 400), J=2,2) /
     &   6.6592E-05, 6.5552E-05, 6.4513E-05, 6.3475E-05, 6.2438E-05,
     &   6.1401E-05, 6.0366E-05, 5.9332E-05, 5.8300E-05, 5.7269E-05,
     &   5.6240E-05, 5.5213E-05, 5.4188E-05, 5.3166E-05, 5.2146E-05,
     &   5.1128E-05, 5.0113E-05, 4.9101E-05, 4.8092E-05, 4.7086E-05,
     &   4.6084E-05, 4.5086E-05, 4.4091E-05, 4.3100E-05, 4.2113E-05,
     &   4.1131E-05, 4.0154E-05, 3.9181E-05, 3.8213E-05, 3.7251E-05,
     &   3.6294E-05, 3.5342E-05, 3.4396E-05, 3.3457E-05, 3.2524E-05,
     &   3.1597E-05, 3.0677E-05, 2.9764E-05, 2.8858E-05, 2.7960E-05,
     &   2.7070E-05, 2.6188E-05, 2.5314E-05, 2.4448E-05, 2.3592E-05,
     &   2.2744E-05, 2.1906E-05, 2.1078E-05, 2.0260E-05, 1.9452E-05,
     &   1.8654E-05, 1.7868E-05, 1.7093E-05, 1.6330E-05, 1.5578E-05,
     &   1.4839E-05, 1.4113E-05, 1.3400E-05, 1.2700E-05, 1.2014E-05,
     &   1.1342E-05, 1.0685E-05, 1.0044E-05, 9.4172E-06, 8.8068E-06,
     &   8.2128E-06, 7.6356E-06, 7.0758E-06, 6.5337E-06, 6.0098E-06,
     &   5.5046E-06, 5.0186E-06, 4.5522E-06, 4.1059E-06, 3.6803E-06,
     &   3.2757E-06, 2.8926E-06, 2.5315E-06, 2.1929E-06, 1.8771E-06,
     &   1.5847E-06, 1.5567E-06, 1.5290E-06, 1.5015E-06, 1.4743E-06,
     &   1.4473E-06, 1.4205E-06, 1.3940E-06, 1.3677E-06, 1.3417E-06,
     &   1.3159E-06, 1.2903E-06, 1.2650E-06, 1.2399E-06, 1.2151E-06,
     &   1.1905E-06, 1.1661E-06, 1.1420E-06, 1.1182E-06, 1.0945E-06/
  
         DATA ((DGT(I,J), I= 401, 500), J=2,2) /
     &   1.0712E-06, 1.0480E-06, 1.0251E-06, 1.0025E-06, 9.8010E-07,
     &   9.5795E-07, 9.3604E-07, 9.1438E-07, 8.9296E-07, 8.7179E-07,
     &   8.5087E-07, 8.3020E-07, 8.0978E-07, 7.8960E-07, 7.6967E-07,
     &   7.4999E-07, 7.3056E-07, 7.1138E-07, 6.9245E-07, 6.7377E-07,
     &   6.5533E-07, 6.3715E-07, 6.1922E-07, 6.0154E-07, 5.8412E-07,
     &   5.6694E-07, 5.5002E-07, 5.3334E-07, 5.1692E-07, 5.0076E-07,
     &   4.8484E-07, 4.6918E-07, 4.5378E-07, 4.3862E-07, 4.2372E-07,
     &   4.0908E-07, 3.9468E-07, 3.8055E-07, 3.6667E-07, 3.5304E-07,
     &   3.3967E-07, 3.2655E-07, 3.1369E-07, 3.0108E-07, 2.8874E-07,
     &   2.7664E-07, 2.6481E-07, 2.5323E-07, 2.4191E-07, 2.3084E-07,
     &   2.2003E-07, 2.0948E-07, 1.9919E-07, 1.8915E-07, 1.7938E-07,
     &   1.6986E-07, 1.6059E-07, 1.5159E-07, 1.4285E-07, 1.3436E-07,
     &   1.2613E-07, 1.1816E-07, 1.1046E-07, 1.0301E-07, 9.5815E-08,
     &   8.8883E-08, 8.2211E-08, 7.5799E-08, 6.9646E-08, 6.3754E-08,
     &   5.8121E-08, 5.2748E-08, 4.7635E-08, 4.2783E-08, 3.8191E-08,
     &   3.3859E-08, 2.9787E-08, 2.5976E-08, 2.2426E-08, 1.9136E-08,
     &   1.6107E-08, 1.5819E-08, 1.5533E-08, 1.5249E-08, 1.4968E-08,
     &   1.4690E-08, 1.4415E-08, 1.4142E-08, 1.3872E-08, 1.3604E-08,
     &   1.3339E-08, 1.3076E-08, 1.2816E-08, 1.2559E-08, 1.2304E-08,
     &   1.2052E-08, 1.1803E-08, 1.1556E-08, 1.1312E-08, 1.1070E-08/
  
         DATA ((DGT(I,J), I= 501, 600), J=2,2) /
     &   1.0831E-08, 1.0595E-08, 1.0361E-08, 1.0130E-08, 9.9010E-09,
     &   9.6750E-09, 9.4516E-09, 9.2308E-09, 9.0127E-09, 8.7971E-09,
     &   8.5842E-09, 8.3738E-09, 8.1661E-09, 7.9610E-09, 7.7585E-09,
     &   7.5585E-09, 7.3613E-09, 7.1666E-09, 6.9745E-09, 6.7850E-09,
     &   6.5981E-09, 6.4139E-09, 6.2322E-09, 6.0532E-09, 5.8768E-09,
     &   5.7030E-09, 5.5317E-09, 5.3631E-09, 5.1971E-09, 5.0338E-09,
     &   4.8730E-09, 4.7148E-09, 4.5593E-09, 4.4063E-09, 4.2560E-09,
     &   4.1082E-09, 3.9631E-09, 3.8206E-09, 3.6807E-09, 3.5434E-09,
     &   3.4087E-09, 3.2766E-09, 3.1472E-09, 3.0203E-09, 2.8961E-09,
     &   2.7744E-09, 2.6554E-09, 2.5390E-09, 2.4252E-09, 2.3140E-09,
     &   2.2054E-09, 2.0994E-09, 1.9960E-09, 1.8953E-09, 1.7971E-09,
     &   1.7016E-09, 1.6086E-09, 1.5183E-09, 1.4306E-09, 1.3455E-09,
     &   1.2630E-09, 1.1831E-09, 1.1058E-09, 1.0312E-09, 9.5911E-10,
     &   8.8966E-10, 8.2282E-10, 7.5859E-10, 6.9697E-10, 6.3796E-10,
     &   5.8156E-10, 5.2777E-10, 4.7659E-10, 4.2802E-10, 3.8206E-10,
     &   3.3871E-10, 2.9797E-10, 2.5984E-10, 2.2431E-10, 1.9140E-10,
     &   1.6110E-10, 1.5821E-10, 1.5535E-10, 1.5252E-10, 1.4971E-10,
     &   1.4693E-10, 1.4417E-10, 1.4144E-10, 1.3874E-10, 1.3606E-10,
     &   1.3341E-10, 1.3078E-10, 1.2818E-10, 1.2561E-10, 1.2306E-10,
     &   1.2054E-10, 1.1804E-10, 1.1557E-10, 1.1313E-10, 1.1071E-10/
  
         DATA ((DGT(I,J), I= 601, 700), J=2,2) /
     &   1.0832E-10, 1.0596E-10, 1.0362E-10, 1.0131E-10, 9.9020E-11,
     &   9.6760E-11, 9.4526E-11, 9.2317E-11, 9.0135E-11, 8.7979E-11,
     &   8.5849E-11, 8.3746E-11, 8.1668E-11, 7.9616E-11, 7.7591E-11,
     &   7.5591E-11, 7.3618E-11, 7.1671E-11, 6.9750E-11, 6.7855E-11,
     &   6.5986E-11, 6.4143E-11, 6.2326E-11, 6.0536E-11, 5.8771E-11,
     &   5.7033E-11, 5.5321E-11, 5.3634E-11, 5.1974E-11, 5.0340E-11,
     &   4.8732E-11, 4.7151E-11, 4.5595E-11, 4.4065E-11, 4.2562E-11,
     &   4.1084E-11, 3.9633E-11, 3.8208E-11, 3.6808E-11, 3.5435E-11,
     &   3.4088E-11, 3.2768E-11, 3.1473E-11, 3.0204E-11, 2.8962E-11,
     &   2.7745E-11, 2.6555E-11, 2.5391E-11, 2.4252E-11, 2.3140E-11,
     &   2.2054E-11, 2.0995E-11, 1.9961E-11, 1.8953E-11, 1.7972E-11,
     &   1.7016E-11, 1.6087E-11, 1.5183E-11, 1.4306E-11, 1.3455E-11,
     &   1.2630E-11, 1.1831E-11, 1.1058E-11, 1.0312E-11, 9.5912E-12,
     &   8.8967E-12, 8.2283E-12, 7.5860E-12, 6.9698E-12, 6.3797E-12,
     &   5.8157E-12, 5.2778E-12, 4.7659E-12, 4.2802E-12, 3.8206E-12,
     &   3.3871E-12, 2.9797E-12, 2.5984E-12, 2.2431E-12, 1.9140E-12,
     &   1.6110E-12, 1.5821E-12, 1.5535E-12, 1.5252E-12, 1.4971E-12,
     &   1.4693E-12, 1.4417E-12, 1.4144E-12, 1.3874E-12, 1.3606E-12,
     &   1.3341E-12, 1.3078E-12, 1.2818E-12, 1.2561E-12, 1.2306E-12,
     &   1.2054E-12, 1.1804E-12, 1.1557E-12, 1.1313E-12, 1.1071E-12/
  
         DATA ((DGT(I,J), I= 701, 800), J=2,2) /
     &   1.0832E-12, 1.0596E-12, 1.0362E-12, 1.0131E-12, 9.9020E-13,
     &   9.6760E-13, 9.4526E-13, 9.2317E-13, 9.0135E-13, 8.7979E-13,
     &   8.5850E-13, 8.3746E-13, 8.1668E-13, 7.9616E-13, 7.7591E-13,
     &   7.5592E-13, 7.3618E-13, 7.1671E-13, 6.9750E-13, 6.7855E-13,
     &   6.5986E-13, 6.4143E-13, 6.2327E-13, 6.0536E-13, 5.8771E-13,
     &   5.7033E-13, 5.5321E-13, 5.3634E-13, 5.1974E-13, 5.0340E-13,
     &   4.8732E-13, 4.7151E-13, 4.5595E-13, 4.4065E-13, 4.2562E-13,
     &   4.1084E-13, 3.9633E-13, 3.8208E-13, 3.6808E-13, 3.5435E-13,
     &   3.4088E-13, 3.2768E-13, 3.1473E-13, 3.0204E-13, 2.8962E-13,
     &   2.7745E-13, 2.6555E-13, 2.5391E-13, 2.4252E-13, 2.3140E-13,
     &   2.2054E-13, 2.0995E-13, 1.9961E-13, 1.8953E-13, 1.7972E-13,
     &   1.7016E-13, 1.6087E-13, 1.5183E-13, 1.4306E-13, 1.3455E-13,
     &   1.2630E-13, 1.1831E-13, 1.1058E-13, 1.0312E-13, 9.5912E-14,
     &   8.8967E-14, 8.2283E-14, 7.5860E-14, 6.9698E-14, 6.3797E-14,
     &   5.8157E-14, 5.2778E-14, 4.7659E-14, 4.2802E-14, 3.8206E-14,
     &   3.3871E-14, 2.9797E-14, 2.5984E-14, 2.2431E-14, 1.9140E-14,
     &   1.6110E-14, 1.5821E-14, 1.5535E-14, 1.5252E-14, 1.4971E-14,
     &   1.4693E-14, 1.4417E-14, 1.4144E-14, 1.3874E-14, 1.3606E-14,
     &   1.3341E-14, 1.3078E-14, 1.2818E-14, 1.2561E-14, 1.2306E-14,
     &   1.2054E-14, 1.1804E-14, 1.1557E-14, 1.1313E-14, 1.1071E-14/
  
         DATA ((DGT(I,J), I= 801, 900), J=2,2) /
     &   1.0832E-14, 1.0596E-14, 1.0362E-14, 1.0131E-14, 9.9020E-15,
     &   9.6760E-15, 9.4526E-15, 9.2317E-15, 9.0135E-15, 8.7979E-15,
     &   8.5849E-15, 8.3746E-15, 8.1668E-15, 7.9616E-15, 7.7591E-15,
     &   7.5592E-15, 7.3618E-15, 7.1671E-15, 6.9750E-15, 6.7855E-15,
     &   6.5986E-15, 6.4143E-15, 6.2327E-15, 6.0536E-15, 5.8771E-15,
     &   5.7033E-15, 5.5321E-15, 5.3634E-15, 5.1974E-15, 5.0340E-15,
     &   4.8732E-15, 4.7151E-15, 4.5595E-15, 4.4065E-15, 4.2562E-15,
     &   4.1084E-15, 3.9633E-15, 3.8208E-15, 3.6808E-15, 3.5435E-15,
     &   3.4088E-15, 3.2768E-15, 3.1473E-15, 3.0204E-15, 2.8962E-15,
     &   2.7745E-15, 2.6555E-15, 2.5391E-15, 2.4252E-15, 2.3140E-15,
     &   2.2054E-15, 2.0995E-15, 1.9961E-15, 1.8953E-15, 1.7972E-15,
     &   1.7016E-15, 1.6087E-15, 1.5183E-15, 1.4306E-15, 1.3455E-15,
     &   1.2630E-15, 1.1831E-15, 1.1058E-15, 1.0312E-15, 9.5912E-16,
     &   8.8967E-16, 8.2283E-16, 7.5860E-16, 6.9698E-16, 6.3797E-16,
     &   5.8157E-16, 5.2778E-16, 4.7659E-16, 4.2802E-16, 3.8206E-16,
     &   3.3871E-16, 2.9797E-16, 2.5984E-16, 2.2431E-16, 1.9140E-16,
     &   1.6110E-16, 0.0000E+00, 1.6110E-16, 1.4509E-09, 5.8018E-09,
     &   1.3054E-08, 2.3204E-08, 3.6261E-08, 5.2225E-08, 7.1098E-08,
     &   9.2884E-08, 1.1759E-07, 1.4521E-07, 1.7576E-07, 2.0925E-07,
     &   2.4567E-07, 2.8504E-07, 3.2736E-07, 3.7264E-07, 4.2090E-07/
  
         DATA ((DGT(I,J), I= 901, 979), J=2,2) /
     &   4.7213E-07, 5.2635E-07, 5.8357E-07, 6.4380E-07, 7.0706E-07,
     &   7.7335E-07, 8.4269E-07, 9.1509E-07, 9.9057E-07, 1.0691E-06,
     &   1.1508E-06, 1.2356E-06, 1.3236E-06, 1.4147E-06, 1.5090E-06,
     &   1.6065E-06, 1.7073E-06, 1.8112E-06, 1.9185E-06, 2.0290E-06,
     &   2.1429E-06, 2.2601E-06, 2.3807E-06, 2.5047E-06, 2.6322E-06,
     &   2.7631E-06, 2.8975E-06, 3.0355E-06, 3.1771E-06, 3.3222E-06,
     &   3.4710E-06, 3.6235E-06, 3.7798E-06, 3.9399E-06, 4.1038E-06,
     &   4.2715E-06, 4.4433E-06, 4.6190E-06, 4.7988E-06, 4.9827E-06,
     &   5.1708E-06, 5.3631E-06, 5.5598E-06, 5.7609E-06, 5.9665E-06,
     &   6.1766E-06, 6.3914E-06, 6.6110E-06, 6.8355E-06, 7.0649E-06,
     &   7.2994E-06, 7.5391E-06, 7.7842E-06, 8.0348E-06, 8.2910E-06,
     &   8.5530E-06, 8.8210E-06, 9.0952E-06, 9.3757E-06, 9.6629E-06,
     &   9.9569E-06, 1.0258E-05, 1.0567E-05, 1.0883E-05, 1.1207E-05,
     &   1.1540E-05, 1.1882E-05, 1.2234E-05, 1.2595E-05, 1.2967E-05,
     &   1.3351E-05, 1.3746E-05, 1.4156E-05, 1.4295E-05, 1.4580E-05,
     &   1.46808911E-05, 1.50200047E-05, 1.50650222E-05, 1.51707989E-05/
c
      save
      RETURN
      END
