cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 27 FEB 01  O.Veretin
c 
c  This is the funaction  R(s) computed in perturbations
c  theory up to O(alpha_S^3) and with mass corretions
c  up to O(mm^2/s^2)
c
c   Based on:
c   K.G. Chetyrkin, R.V. Harlander, J.H. K\"uhn,
c   "Quartic mass corrections to R_had at O(\alpha_s^3)",
c    hep-ph/0005139,
c
c
c  Arguments:
c     sqrts = sqrt(s)  (in GeV)
c             must be > 10.0 GeV
c     alpha = alpha(s)
c             note that mu=s in the following
c             therefore alpha should be taken as alpha(s)
c     loops = 1,...,4
c             1 correspond to parton model (no QCD corr.)
c             4 QCD corr. to O(alpha^3)
c
c      R2, R4 = 0 or 1
c        we have further two parameter R2 and R4 to 
c        swich on/off mm and mm^2 corections correspondingly.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      function Rpt( sqrts, alpha, loops, R2, R4 )
      implicit none
      integer loops, R2, R4
      real*8 Rpt, sqrts, alpha

      real*8  r0, rq2, rq4c, rq4b, rq4t, rvQ2, rvQ4c, rvQ4b, rvQ4t,
     $        Zc, Zb, Zt, lms, s, api, api1, api2, api3, factor

      integer nc, nf
      parameter ( nc = 3 )

      real*8 mc, mb, mt
      real *8 mq(6),mp(6),th(6)
      common /quama/mq,mp,th
c      parameter ( mc = 1.6d0 )
c      parameter ( mb = 4.7d0 )
c      parameter ( mt = 175.d0 )
      mc = mp(4)
      mb = mp(5) 
      mt = mp(6)

      s = sqrts**2

c  Include 4 flavours
c      if ( ( sqrts .GT. 5.d0 ) .AND. ( sqrts .LT. 9.6d0 ) ) then
      if ( ( sqrts .GT. 3.1d0 ) .AND. ( sqrts .LT. 9.6d0 ) ) then
      nf = 4
      factor = 6.d0/9.d0
      Zc = mc**2/s
      Zb = 0.d0
      Zt = 0.d0
      goto 10
      endif

c  Include five flavours
c      if ( ( sqrts .GT. 10.d0 ) .AND. ( sqrts .LT. 2.d0*mt ) ) then
      if ( ( sqrts .GT. 9.6d0 ) .AND. ( sqrts .LT. 2.d0*mt ) ) then
      nf = 5
      factor = 10.d0/9.d0
      Zc = mc**2/s
      Zb = mb**2/s
      Zt = 0.d0
      goto 10
      endif

c  Include six flavours
      if ( sqrts .GT. 2.d0*mt ) then
      nf = 6
      factor = 11.d0/9.d0
      Zc = mc**2/s
      Zb = mb**2/s
      Zt = mt**2/s
      goto 10
      endif

c Else -> error
      write (6,*) "Function R: to low value of sqrts." 
      stop

 10   continue

c Compute required number of loops ( loops = 1,...,4 )
      api = alpha/3.1415926d0
      api1 = 0.d0
      api2 = 0.d0
      api3 = 0.d0
      if ( loops .GT. 1 ) then 
      api1 = api
      endif
      if ( loops .GT. 2 ) then 
      api2 = api**2
      endif
      if ( loops .GT. 3 ) then 
      api3 = api**3
      endif

      r0 = 1.d0 
     $   + api1 
     $   + api2 * ( 1.98571d0 - 0.115295d0*nf)
     $   + api3 * ( -6.63694d0 - 1.20013d0*nf - 0.00517836d0*nf**2)

      rq2 = api3 * ( -7.87659d0 + 0.35007d0*nf )

      rvQ2 = ( api1 * 12.d0
     $     + api2 * ( 126.5d0 - 4.333333d0*nf )
     $     + api3 * ( 1032.14d0 - 104.167d0*nf + 1.21819d0*nf**2 ) )

      lms = log( mc**2/s )
      rq4c = ( api2 * ( -0.474894d0 - lms )
     $    + api3 * ( 4.59784d0 - 22.8619d0*lms + 2.d0*lms**2
     $        + ( 0.196497d0 + 0.88052d0*lms )*nf ) )

      lms = log( mb**2/s )
      rq4b = ( api2 * ( -0.474894d0 - lms )
     $    + api3 * ( 4.59784d0 - 22.8619d0*lms + 2.d0*lms**2
     $        + ( 0.196497d0 + 0.88052d0*lms )*nf ) )

      lms = log( mt**2/s )
      rq4t = ( api2 * ( -0.474894d0 - lms )
     $    + api3 * ( 4.59784d0 - 22.8619d0*lms + 2.d0*lms**2
     $        + ( 0.196497d0 + 0.88052d0*lms )*nf ) )

      lms = log( mc**2/s )
      rvQ4c = ( -6.d0 - api1 * 22.d0
     $     + api2 * ( 148.218d0 - 6.5d0*lms 
     $          + ( -1.84078d0 + 0.333333d0*lms)*nf )
     $     + api3 * ( 4776.95d0 -244.612d0*lms + 13.d0*lms**2
     $          + (-275.898d0+18.1861d0*lms-0.666667d0*lms**2)*nf
     $          + (4.97396d0 - 0.185185d0*lms)*nf**2 ) )

      lms = log( mb**2/s )
      rvQ4b = ( -6.d0 - api1 * 22.d0
     $     + api2 * ( 148.218d0 - 6.5d0*lms 
     $          + ( -1.84078d0 + 0.333333d0*lms)*nf )
     $     + api3 * ( 4776.95d0 -244.612d0*lms + 13.d0*lms**2
     $          + (-275.898d0+18.1861d0*lms-0.666667d0*lms**2)*nf
     $          + (4.97396d0 - 0.185185d0*lms)*nf**2 ) )
      
      lms = log( mt**2/s )
      rvQ4t = ( -6.d0 - api1 * 22.d0
     $     + api2 * ( 148.218d0 - 6.5d0*lms 
     $          + ( -1.84078d0 + 0.333333d0*lms)*nf )
     $     + api3 * ( 4776.95d0 -244.612d0*lms + 13.d0*lms**2
     $          + (-275.898d0+18.1861d0*lms-0.666667d0*lms**2)*nf
     $          + (4.97396d0 - 0.185185d0*lms)*nf**2 ) )


c  111111111111111111111111
c  Contribution of light quarks
      Rpt = nc * factor *( r0 
     $              + R2*(Zc + Zb + Zt) * rq2 
     $              + R4*(rq4c*Zc**2 + rq4b*Zb**2 + rq4t*Zt**2) )

c  +++++++
c  c quark + singlet
      if ( nf .EQ. 4 ) then
      Rpt = Rpt + nc * 4.d0/9.d0 *( r0 
     $      + Zc * rvQ2 *R2 + Zc**2 * rvQ4c * R4)      
      Rpt = Rpt + api**3 * ( -0.413180d0*(2.d0/3.d0)**2   ! singlet
     $       + Zc**2 * 17.8121*(2.d0/3.d0)*(2.d0/3.d0) )  ! singlet
      endif

c  +++++++
c  b quark
      if ( nf .EQ. 5 ) then
      Rpt = Rpt + nc * 1.d0/9.d0 *( r0 
     $      + Zc * rvQ2 * R2 + Zc**2 * rvQ4b *R4)      
      Rpt = Rpt + api**3 * ( -0.413180d0*(1.d0/3.d0)**2   ! singlet
     $       + Zb**2 * 17.8121*(-1.d0/3.d0)*(1.d0/3.d0) ) ! singlet
      endif

c  +++++++
c  t quark
      if ( nf .EQ. 6 ) then
      Rpt = Rpt + nc * 4.d0/9.d0 *( r0 
     $      + Zt * rvQ2 * R2 + Zt**2 * rvQ4t * R4 )      
      Rpt = Rpt + api**3 * ( -0.413180d0*(3.d0/3.d0)**2   ! singlet
     $       + Zt**2 * 17.8121*(2.d0/3.d0)*(3.d0/3.d0) ) ! singlet
      endif

      return
      END
