c;;;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Fortran -*-
c     ;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; intRdatx.f ---
c;; Author : Friedrich Jegerlehner
c;; Created On : Tue Sep 22 11:20:44 2009
c;; Last Modified By: Friedrich Jegerlehner
c;; Last Modified On: Fri Jan  3 17:54:18 2020
c;; RCS: $Id$
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;; Copyright (C) 2009 Friedrich Jegerlehner
c;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c;;
c
C 05/10/2012 - PI3,KKC,KKN data versusPDG resonance parameters for omega and phi region:
C              new flag iomegaphicut to be set in this main program
C              upgraded pi^0 gamma and eta gamma dataset (incomplete and R replaced by sigma)
C 23/03/2013 Adler slope at zero: IKE=10 D(Q^2)/Q^2 at Q=0
c 27/09/2013 EC (Euclidean Correlator) IKE=8 t^3*G_2(t), IKE=9 EC
c     t^4*G_3(t) G_n(t)= int_a^infty d omega omega^n rho(omega^2) exp
c     -omega*t [calculated by H. Meyer in lattice QCD]
c
C 22/05/2013 - flavorseparation revisited: I=1 N_F=2 weight is 9/10 --> 10/10
C            - KLOE 12 included
c 13/10/2013 -- new flavorspliting and recombination implemented (after
c     tests in lattice QCD by Harvey Meyer et al.). Previous
c     implementation based on assuming OZI violation in light sector to
c     be small is not tenable [very bad approximation]; now "old" recipe
c     reinstalled based on approximate isovector symmetry affects
c     RS3G0321..f and RS330321.f substantially [preliminary
c     implementation in hadr5n13.f]
c
C 20/11/2014 - updating exclusive-datasets.f
c            - CMD-3 3(pi^+pi^-) PLB 723 (2013)
c            - SND13 omega pi^- -> pi^0 pi^0 gamma PRD 88 (2013)
c            - BaBar pp arXiv 1302.0055 [PRD 87 (2013)]
c            - BaBar K+K- arXiv 1306.3600 [PRD 88 (2013)]
c            - BaBar KLKS,KLKSpi+pi-, arXiv 1403.7593 [PRD 89 (2014)]
c            - BaBar KSKSpi+pi-,KSKSK+K-
c
c 07/01/2015 -- improved writing fortran array tables for hadr5n.f
c     routines in alphaQED and alphaSM (automated via shell script
c     intRdatx.sh)
c -- all datasets used [accessed via Rdat_fun.f, RS3G_fun.f, RS33_fun.f,
c     RSn2_fun.f and RSGG_fun.f from alphaQED package]] represent bare
c     data (VP subtracted), so all renormalization is disabled except
c     for narrow resonance contributions baser on PDG physical (dressed)
c     masses width and branching fractions i..e. reno=1 in FUNCTION FACT
c -- resonance contributions renormalized point by point of xRdat
c     -extended.f raster [common renormBW], via BW function in package alphaQEDcomplex
c -- implemented test for NaN and +Infinity/-Infinity results via
c     logical isnan and isinf (own routine at end of this program), if
c     true result at point is replaced by interpolation of the two
c     neighboring points
c 29/09/2015 Rdat_all updated incl BESIII;
c     try implementing scan not incl KLOE data for alphacomplex measurement by KLOE
c     new option flag; IERK3C=1,3 for error analysis of triple VP insertion
c     IERK3C=1 integrate VP, IERK3C=3 integrate error
c 05/02/2016   2016 update together with new KEDR data PLB 753 (2016) 533 via epemrdata1n.f
c 04/07/2017 new Tables listing stat and syst error separately (in place of total error only)
c     affected is subroutine printscan, new arrays UV for stst and VV for sysy error
c 10/07/2017 new error analysis for resonances performed with vapo1n.f ==> FSTA,FSYS arrays in resonances_dat.f
c 15/06/2019 implemented Delta alpha_had for Nf=3,4,5 writing plotfiles to intRdatx_n3,4,5.dap
c            same for Delta alpha_2,had for Nf=3,4,5 writing plotfiles to intRdat3g_n3,4,5.dap 
c            in the spacelike regime for comparison with lattice data.
c            For crosscheck "der" and "deg" of hadr5n17.f are written to plotfiles fort.66 and fort.67 respectively; these 
c            must agree with intRdatx_n5.dap and intRdatg3_n5.dap aswell as with intRdatx_gg.dap and intRdatx_33g.dap,
c            respectively. Also FORTRAN arrays written to dalhads.dat can be obtained for Nf=3,4,5 
c            by specifying NF in intRdatx.inp (or elsewhere) as 8th entry on 2nd line
c 22/08/2019 undo rho-gamma mixing option IUNMIX=1 [0] for usual VP subtracted e^+e^- data
c            =1 option as LQCD bookkeeping, where it is considered as a HO QED effect
c            set in main, affects pipi channel below 1 GeV via function FACT
c 03/12/2019 new flag inf5 for skipping NF=3,4,5 option in timelike deg channel      
c 30/12/2019 New flag IUNMIX=1 for subtraction of rho^--gamma mixing 
c            in order to better disentangle QED from strong interaction 1PI blob 
c            for crosscheck: der on fort.66, deg on fort.67 
      program intRdatx
c link routine vapotail
      implicit none
      LOGICAL lwrite,lmessage,NFlog,isinf,lprintdeg
      character*1 yesno
      character*2 year
      character*22 aaa
      character*4 DSET
      CHARACTER*11 tex4(15)
      CHARACTER*10 tex4x(5)
      CHARACTER*27 KERN(16)
      CHARACTER*10 UPDATEVERSION
      CHARACTER*3 XAXIS
      CHARACTER*5 xxx,yyy
      integer time,stime,icount,jcount,iwarn,irennew,
     &     itestflavorsplitting,itest,icomplement,i3gnew,inf2on,
     &     iresonances_sav,iomegaphidat,ierrcountY,ierrcountZ,
     &     Nrep2,Nrep3,irever,ignu,inf5,iWarnOption
      integer inde(10)
      parameter(Nrep2=12,Nrep3=7)
      integer iYerr(10),iZerr(10),Ireplisttwo(Nrep2),Ireplithree(Nrep3)
      character*25 ctime,uhrzeit
      integer j,jp,jep,ipm,Nall,NV,II,jj,ier,jer,iplot,
     &     IER1,IER_SAV,IERK3C,
     &     NH,NR,IM,IR,IECUT,Ioldsubdiv,ISU2,NFL,INAG,Irep,jrep,
     &     itail,inotail
c taken out 4 points 0.32, 1.45, 2.20, 3.685632, 3.7
      parameter(Nall=2187,NV=2187)
      INTEGER NA,NB,NC,UNIT
      PARAMETER(NA=979,NB=200,NC=200)
      REAL ETX(NA),ESX(NB),EHI(NC)
      INTEGER     LW,       LIW
      PARAMETER  (LW=320000,LIW=LW/4)
      INTEGER IFAIL,KOUNT,IW(LIW)
      INTEGER IPV,IKE,JKE,NP,NPin,N,IE,NPM,NE,NER,NI,NS,ISPLINE,
     &     IREN,IRENBW,IRENO,IREN_SAV,IRENO_SAV,
     &     IGEV,NF,ICHK,IOR,ISCAN,NMI,ILOG,IINV,NM,K,I,
     &     Ismooth,IGS,iso,ifuture,ICHK_SAV,IANOMALY,NFfixed,ICHKfix
      character*3 INL
      integer IEx,NMx,NHx,NRx,IMx,IRx
      REAL *8 Emin,Emax,Lmin,Lmax,EPSREL,EPPMAX,MPOL,EPM,EPMc,W(LW)
      REAL *8 E,S,RS40,RS40x,RSpQCD,STA2R,SYS2R,ABSERR,RSDAT,
     &     FACT,RP,RM,DMP,DMPSAV,RMP,XABS,FK,TERE,VZ,LL,DL
      REAL *8 ML1(3),EL(15),EH(15),TB(15,5),TBL(15,3),TBH(15,3),
     &     TG(15,5),L3G(15,3),H3G(15,3),T3(15,5),L33(15,3),H33(15,3),
     &     T2(15,5),N2L(15,3),N2H(15,3),TT(15,5),LGG(15,3),HGG(15,3),
     &     RE(3),RD(3),RG(3),R3(3),R2(3),RESR(6),STAR(6),SYSR(6)
      REAL *8 X(Nall)
      REAL *8 CHPTCUT,EC,ECSAV,ECUT,ECUTS,ERHORES,ERHOPLU,ETAILCUT,
     &     N2pQCDcut,ETAILCUT_SAV,EINFTY,DHL,URI,
     &     ANS1,ANS2,ANS3,EPC,RESOLD,ETOOLD,RESNEW,ETONEW,
     &     STAOLD,SYSOLD,STANEW,SYSNEW,RESN2OLD,ETON2OLD,
     &     RESN2NEW,ETON2NEW,STAN2OLD,SYSN2OLD,STAN2NEW,SYSN2NEW,fm
      REAL *8 RESAV(3),RDSAV(3),RGSAV(3),RCSAV(3),R3SAV(3),R2SAV(3),
     &     ELR(5),EHR(5)
      REAL *8 XL,XH,SL,SH,RES,STA,SYS,STA2,SYS2,ETOT,REX,REY,
     &     RGS,RGY,RGX,R2S,R2Y,R2X,T2tail1,T2tail2,RN2tailp,RN2tailm,
     &     POLP,POLM,POLP1,POLM1,MP2,M2,M2_1,X931,ZT
      REAL *8 SUM1,STA1,SYS1,SUper,STper,SYper,ETOT1,EREL1
      REAL *8 SU3G,ST3G,SY3G,S3GUper,S3GTper,S3GYper,w3g(5)
      REAL *8 RES3Gn,STA3Gn,SYS3Gn,STA23Gn,SYS23Gn,ETOT3Gn,
     &     RESN2,STA2N2,SYS2N2,STAN2,SYSN2,ETOTN2
      REAL *8 NULL,ALPI,ALPx,FAC,NAGFAC,PIx,g2atMW,
     &     facg2dive2,e2inv,sin2ell,MELE,MU,MTAU,
     &     F0,MPLUS,E0,E2KCUT,MU2,FA,MU2I,RSBGI,OMREI,FIREI,RSB3G,RSB33
      REAL *8 ALS,EST,ESY,MZINP,MTOP,E1,E2,S1,S2,S1t,S2t,
     &     RpQCDtest,RESU1,RESU2,alerr,ELOW,EHIGH,ddk3c,derrdk3c
      REAL *8 EMI,EMA,EP,DE,EFIMM,EFIMX,EFIPX,EFIPP
      REAL *8 XV(NV),YV(NV),ZV(NV),UV(NV),VV(NV)
      REAL *8 MAXMOD1,MAXMOD2,Test1,Test2,tiny,
     &     MOMGeV,DEGOMGeV,GOMGeV,EOMPx,MFIGeV,GFIGeV,DEGFIGeV
      real *8 pRESUDS,pRESUDSP,pRESUDSM,pSYSUDS,
     &     pRESUDS1,pRESUDS1P,pRESUDS1M,pSYSUDS1,
     &     pRESUDSC,pRESUDSCP,pRESUDSCM,pSYSUDSC,
     &     resuds,stauds,sysuds,resudsc,staudsc,sysudsc,
     &     resudscb,staudscb,sysudscb,
     &     resuds_rem,stauds_rem,sysuds_rem,eto3_rem,resuds_n
      real *8 pr3,pR3p,pR3m,pe3,pr4,pR4p,pR4m,pe4,pr5,pR5p,pR5m,pe5
      real *8 rromega,stomega,syomega,rrphi,stphi,syphi,rrpsi,stpsi,
     &     reomega,rephi,repsi,reups,
     &     sypsi,rrups,stups,syups,stback,streso,syback,syreso,
     &     prc,prb,pec,peb,repQCD,erpQCD,resc,resb,stac,sysyc,
     &     stab,sysb,eto3,eto4,eto5,etoc,etob
      real *8 deg,errdeg,stadeg,sysdeg,der,errder,stader,sysder,
     &     rereso,erreso,sysc,reback,erback,
     &     retes,rete1,rete2,ertes,erte1,erte2
      real *8 cud_3g,cud_33,cuds,cc,cb,crho,comega,cphi,cpsi,cups
     &     ,efuture,futureprecision
      real *8 xud_3g,xuds,xc,xb,xrho,xomega,xphi,xpsi,xups
      real *8 cuds_33,cc_33,cb_33,crho_33,comega_33,cphi_33,cpsi_33,
     &     cups_33,repQCD_33,erpQCD_33,reback_33,stback_33,syback_33,
     &     erback_33,rereso_33,streso_33,syreso_33,erreso_33,rrrho,
     &     strho,syrho,resuds_33,stauds_33,sysuds_33,deg_33,stadeg_33,
     &     sysdeg_33,errdeg_33
      real *8 reback_new,stback_new,syback_new,erback_new,
     &     resuds_33new,stauds_33new,sysuds_33new,
     &     reback_33new,stback_33new,syback_33new,erback_33new,
     &     deg_new,stadeg_new,sysdeg_new,errdeg_new,
     &     deg_33new,stadeg_33new,sysdeg_33new,errdeg_33new
      real *8 reback_tes,stback_tes,syback_tes,erback_tes,
     &     repQCD_tes,erpQCD_tes,
     &     rereso_tes,streso_tes,syreso_tes,erreso_tes,
     &     deg_tes,stadeg_tes,sysdeg_tes,errdeg_tes,
     &     rereso_33new,streso_33new,syreso_33new,erreso_33new
      real *8 resud_rem,staud_rem,sysud_rem,eto3u_rem,eto3s_rem
      real *8 RS3GDAT,RS33DAT,RSN2DAT,RSBN2,RSN2,
     &     RES3G,ETOT3G,STA3G,SYS3G,RES33,ETOT33,STA33,SYS33,
     &     RESOLD3G,STAOLD3G,SYSOLD3G,ETOOLD3G,
     &     RESOLD33,STAOLD33,SYSOLD33,ETOOLD33,
     &     RESNEW3G,STANEW3G,SYSNEW3G,ETONEW3G,
     &     RESNEW33,STANEW33,SYSNEW33,ETONEW33
      real *8 RESG33,STAG33,SYSG33,ETOTG33,
     &     RESG34,STAG34,SYSG34,ETOTG34,
     &     RESG35,STAG35,SYSG35,ETOTG35
      real *8 RSGGDAT,
     &     RESGG,ETOTGG,STAGG,SYSGG,
     &     RESOLDGG,STAOLDGG,SYSOLDGG,ETOOLDGG,
     &     RESNEWGG,STANEWGG,SYSNEWGG,ETONEWGG
      real *8 resgg3,stagg3,sysgg3,resgg4,stagg4,sysgg4,
     &     resgg5,stagg5,sysgg5,ETOTGG3,ETOTGG4,ETOTGG5
      real *8 dder,derrdersta,derrdersys,
     &            ddeg,derrdegsta,derrdegsys
      integer ixAdler,Nadler
      parameter(Nadler=410)
      real *8 xAdler(Nadler)
      real *8 PSI(6),PSINWA(6),YPI(6),YPINWA(6),
     &     TBL1(15),TBH1(15),TB1(15),TB2(15),
     &     TGL1(15),TGH1(15),TG1(15),TG2(15),
     &     T3L1(15),T3H1(15),T31(15),T32(15),
     &     T2L1(15),T2H1(15),T21(15),T22(15)
c      real *8 ETA(20,2),TAB(20,4)
c      common /ASTO/ETA,TAB
      integer NPPS(6),NPYP(6)
      real *8 EMAT,EMIT,DET,ECIfun,ECJfun,t,loss,ADPx,UMR
      integer icountps(6),icountyp(6)
      common /icountre/icountps,icountyp
      common /N2CUT/N2pQCDcut
      COMMON/MFPI/MP2,M2/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
      COMMON/POLE/MPOL
      COMMON/NAR/NI,NS,ISPLINE
      COMMON/MFK/MU2I
      COMMON/lema/ml1,IGEV
      COMMON/CSTS/PIx,ALPx,FA
      common/warnings/iwarn
      common/pri/lwrite
      COMMON /messages/lmessage
      common /rennew/irennew
c error fag ier=1,2,3 for result,staterr, syserr
      COMMON/ERR/IER
      COMMON/ERRK3C/IERK3C
      COMMON/NPOINTS/NP
c resonsnces flag iresonances=1 include resonances as data here; 0 include them elsewhere
      COMMON /OMEPHIDAT/iomegaphidat
      COMMON /renoBW/IRENBW
c individual results for PSI and YPS family if IKE different from 1 or 2 compare
c numerical BW integration with analytic NW approximation
      COMMON /NWA/PSI,PSINWA,YPI,YPINWA
c      FUNCTION RSpQCD(s,IER) in Rdat_all.f requires QCD paremeters incl errors
c provided via the following common block:
      COMMON/QCDPA/ALS,EST,ESY,MZINP,MTOP,NF,IOR,ICHK
c to set QCD paramaters alpha_s with stat and sys error
c (one may be zero the other the tot error), scale sqrt(s) usually = M_Z top mass
c number of external flavos e.g 5 no top, 6 incl top, ior=number of loops in R(s)
c to be calculated, ICHK is dummy here (in some programs used for alternative routines calculating R(s))
      real *8 mq(6),mp(6),th(6)
      REAL *8 PI1,ALINP,ERRAL,EINP,MTOP1,st2,als2,mtop2
      integer nx,ny,NNLO,IUNMIX
      parameter(nx=400,ny=15)
      integer nres(ny)
      double precision renpts(nx,ny),renofa(nx,ny),fracerr(nx,ny)
c
      include 'common.h'
c
      common/var1/pi1,ALINP,ERRAL,EINP,MTOP1
      COMMON /RCUTS/CHPTCUT,EC,ECUT
      COMMON/GUSA/IGS,iso
      common /quama/mq,mp,th
      common /parm/st2,als2,mtop2
      COMMON/SMOO/Ismooth
      common/QUAER/alerr,IER1
      COMMON/TELNUM/KOUNT
      COMMON /DASE/DSET ! used in fpi2rnew Gounaris-Sakurai fit
      common /future/efuture,futureprecision,ifuture
      COMMON /NAGRELERR/EPSREL
      COMMON /TAIL/ETAILCUT,EINFTY ! common for vapotail the very tail
      COMMON /RANGES/EL,EH,inde
      common /adp/nagfac
      common /spole/M2_1
      COMMON /REGG/TBL1,TBH1,TB1,TB2
      COMMON /RE3G/TGL1,TGH1,TG1,TG2
      COMMON /RE33/T3L1,T3H1,T31,T32
      COMMON /REN2/T2L1,T2H1,T21,T22
      common /fermi/fm
      common /flavor/NFL
      common /testflavor/itestflavorsplitting,icomplement
      common /forcedNF/NFlog
      common /renormBW/renpts,renofa,fracerr,nres
      common /energyindex/IE
      COMMON /NNLO/ADPx,NNLO
      COMMON /DERDEG/ISU2
      common/aux/IEx,NMx,NHx,NRx,IMx,IRx
      COMMON /RGMIX/IUNMIX
      DATA NPPS /359,213,222,325,341, 81/
      DATA NPYP /202,155,163, 71,388,160/
      DATA tex4/'$\\rho    $','$\\omega  $','$\\phi    $','$J/\\psi  $',
     &'$\\Upsilon$','  had     ','  had     ','  had     ','  had     ',
     &'  had     ','  had     ',' pQCD     ',' data     ',' pQCD     ',
     &' total    '/
       DATA KERN /
     &     '"Muon anomaly"             ','"Delta alpha"              ',
     .     '"Adler function"           ','"Tau anomaly"              ',
     .     '"Electron anomaly"         ',
     &     '"Muon anomaly" HO 2a)      ','"Muon anomaly" HO 2b)      ',
     &     '"Electron anomaly" HO 2a)  ','"Harveys-correlator EC"    ',
     &     '"Adler D(Q^2)/Q^2 at Q=0"  ','"amuh(K3c) 1st integral    ',
     &     '"amuh(K3c) 2nd integral"   ','"Electron g-2" NNLO 3a)    ',
     &     '"Muon g-2" NNLO 3a)        ','"Muon g-2" NNLO 3b)        ',
     &     '"Muon g-2" NNLO 3bLbL)     '/
c         --123456789012345678901234567---123456789012345678901234567
c repair pairs and triples in PV intergral pionts
       data Ireplisttwo /92,95,97,107,112,154,244,502,1067,1078,1223,
     &      1903/
       data Ireplithree /97,106,112,1108,1225,1833,1892/
C CONSTANTS AND INPUT DATA
      external RSBGI,OMREI,FIREI,RS40,RS40x,RSpQCD,RSDAT,FACT,XABS,FK,
     &     RS3GDAT,RS33DAT,RSN2DAT,RSB3G,RSB33,RSBN2,RSN2,ECIfun,ECJfun

      include 'xnewAdler.f'

      include 'xRdat-extended.f' ! just DATA X/.../ statement

      call constants()
      call constants_qcd()      ! fills commons var and RCUTS
      call Rdata()
      call resonances_renocomplex() ! fill common renormBW VP subtraction on physical resonances
c fill common QCDPA
      URI=1.D-3                 ! convert MeV into GeV
c      MPLUS=139.56995D0*URI     ! pi^+ mass
c      MP2=MPLUS**2
      ALS =ALINP
      ALS2=ALINP
      EST=ERRAL
      ESY=0.D0
      MZINP=EINP
      MTOP =MTOP1
      MTOP2=MTOP1
      PIx =pi
      ALPx=alp
      ADPx=adp
      do j=1,3
         ml1(j)=ml(j)
      enddo
      fm=0.1973269631d0         ! fermi in GeV^{-1}

      ixAdler=0
      IER=1
c      IPV=0
c      POLP=0.d0
c      POLM=0.d0
c      N2pQCDcut=2.d0
c      IKE=-1
c       do =i=1,Nall
c          s=X(i)**2
c          write (14,'(1x,i4,3x,f12.6,4(2x,1pe11.4))')
c     &         i,X(i),FACT(s),RSN2(s,IER),RSN2DAT(s),RSBN2(s)
c       enddo
      UPDATEVERSION="30/12/2019"
      iWarnOption=0
c      inf2on=0  ! switch of vapotail call for NF=2 --- testing option default =1
c return zero for tail above HL(6)[=3.2] in R3G and R33 data and fit routines
      inf2on=1
      if (inf2on.eq.0) then
         write (*,*) ' Warning: --- NF2 in tails switched off ---- '
         call WarnOption(' intRdatx, MAIN     ' ,323,'inf2on')
         iWarnOption=1
      endif
      ignu=1  ! data format for gnuplot
c      ignu=0  ! data format for graphx
c test of future precision
c real data
      ifuture=0
c test data
c      ifuture=1      ! now on test
      efuture=2.5d0
      futureprecision=0.01
      DSET='WAVE'  ! for fit of world avgerage
      DSET='CMD2'  ! for standard CMD-2 fit
      NULL=0.0D0
c ' irennew=0[old],1[new] reno'
c     in this program irennew only affects BW resonances in analytic
c     form via bwintana.f, switched off in FACT kernel
      irennew=1
      iwarn=0                   ! warnings for applying RG in rhad.f HS package
      lmessage=.true.
      lwrite=.true.
      lwrite=.false.
      lprintdeg=.false.         ! do not write <gamma 3> correlator results deg to table
      IGEV=0                    ! flag for switching units MeV to GeV in function FACT
      NNLO=0                    ! switch to NNLO for muon (g-2)
      IUNMIX=0                  ! =0 standard e+e- data
c      IUNMIX=1                  ! =1 subtract gamma-rho mixing from e+e- data
C ' Renormalization of data by vacuum polarization: '
C What do we want?
************************************************************************
c# ./qed/old/renormalize2.f acts on FACT kernel
c# Summary: IRENO (what to calculate) =0 raw data ,
c#                                    -1 undressed,
c#                                     1 physical dressed,
c#          IREN (type of data)       =0 undressed data (full VP subtracted)
c#                                     1 partially corrected (e VP only),
c#                                     2 physical (e.g., PDG widths),
c#                                     3 undressed pQCD (perturbative tail)
c#                                     4 do not renormalize; set reno=1
C ' IRENO=0 raw data ,-1 Born, 1 physical '
C What type of data?
C   IREN =0 undressed, 1 only e loop corrected, 2 dressed, 3 undressed pQCD, 4 do not renormalize
      IREN=0
      IRENO=-1
c      itestflavorsplitting=1
      itestflavorsplitting=0
      if (itestflavorsplitting.eq.1) then
         call WarnOption(' intRdatx, MAIN     ' ,365,
     &        'itestflavorsplitting')
         iWarnOption=1
      endif
      itest=0
c      itest=1
c new flag i3gnew: 3g in tables intRdat1.tab using nagdriven tab entries 1,..,9 8 and 10
c with option ICHK=4 which yields R3G via rqcdHS3x
      i3gnew=1
c      i3gnew=0
      inf5  =0  ! use NF decomposition as new method also for NFL=5; reset below
C######  energy units : GeV ;scale 1 GeV
      IGEV=1
      ISU2=0         ! switch to calculate and print delta g SM SU(2) coupling in place of alpha
      IRESON=0       ! switch on resonances in fits via RS40smoothed(s)
      iresonances=0  ! switch on resonances in RS40(s)
c      iresonances=1  ! switch on resonances in RS40(s)
c iresonances=1 include narrow resonances; =0 background only
      iso=2          ! e^+ e^- data GS fit; iso=0,1 yields isospin 0, isovector part
c Note: only this option makes sense as option iomegaphidat=1 is not implemented in Rdat_fit.f
      iomegaphidat=0 ! use BW with PDG parameters for omega and phi
c      iomegaphidat=1 ! use omega and phi data contribution to R
      if (iomegaphidat.eq.1)
     &     write (*,*) ' Error: option iomegaphidat=1 not implemented!'
      XAXIS='LIN'
      NFL=5
      INAG=1
      inotail=0
      write (*,*) ' CUTs CHPTCUT,EC,ECUT (if null use Standard Cuts)'
      READ(*,*) CHPTCUT,EC,ECUT,ETAILCUT
      EINFTY=1.D5    ! numerically infinite energy in GeV
      if (CHPTCUT.eq.NULL) CHPTCUT=0.318d0
c      EC=9.5d0
      if (EC.eq.NULL) EC=5.2d0
      if (ECUT.eq.NULL) ECUT=13.d0
      IF (EC.GT.9.5d0) EC=9.5d0        ! here starts non-perturbative Upsilon region
      IF (EC.LT.4.5d0) EC=4.5d0        ! don't use pQCD in resonance region
      IF (ECUT.LT.11.2d0) ECUT=11.2d0  ! recommended HS threshold
      IF (ECUT.GT.40.d0) ECUT=40.d0    ! no good data available beyond this
      IF (ETAILCUT.LT.1.d3) ETAILCUT=1.d3
      ETAILCUT_SAV=ETAILCUT
C cut boundaries CHPTCUT,EC,ECUT
      Ioldsubdiv=0  ! old subdivision for tests
c 2m_pi - CHPTCUT (standard=0.318)
        EL(1)=  0.27914036d0
        EH(1)=  CHPTCUT ! XCPT(NCHP)
c CHPTCUT - 0.61
        EL(2)=  EH(1) ! XLOW(1)
c        EH(2)= 0.61d0
        EH(2)= 0.63d0
        if (Ioldsubdiv.eq.1) EH(2)= 0.81d0  ! old subdivision
c omega  .41871054d0,.810d0 [MOM=782.65D0,GOM=8.49d0]
c 0.61 - 1.4
        EL(3)=   EH(2)
        EH(3)=   1.4d0
c omega  .41871054d0,.810d0
c phi   1.00D0,1.04D0
c 1.4 - 2.0
        EL(4)=   EH(3)
        EH(4)=   2.0d0
c 2.0 - 3.2
        EL(5)=  EH(4)
        EH(5)=   3.2d0
        if (Ioldsubdiv.eq.1) EH(5)=   3.1d0  ! old subdivision
c psi1  3.08587D0 3.10787D0
c 3.2 - 3.6
        EL(6)=   EH(5)
        EH(6)=   3.6d0
c 3.6 - 5.2
        if (EC.EQ.9.5d0) then  ! there is no pQCD cut effective (data used)
           EL(7)= EH(6)
           EH(7)= 5.2d0
c psi2  3.67496D0 3.69696D0
c psi3  3.7000D0  3.8400D0
c psi4  3.960D0   4.098D0
c psi5  4.102D0   4.275D0
c psi6  4.315D0   4.515D0
c 5.2 - 9.5
           EL(8)=  EH(7)
           EH(8)=  9.5d0
        else  ! there is an effective cut
           EL(7)= EH(6)
           EH(7)= EC
           EL(8)=  EH(7)
           EH(8)=  9.5d0
        endif
        if (Ioldsubdiv.eq.1) EH(8)=  9.46d0 ! old subdivision
c yps1  9.44937D0 9.47137D0
c 9.5 - 13 [40]
        EL(9)=   EH(8)
        EH(9)=   ECUT ! U7A(NU7A)
c yps2  10.01226D010.03426D0
c yps3  10.3442D0 10.3662D0
c yps4  10.473D0  10.687D0
c yps5  10.690D0  10.950D0
c yps6  10.975D0  11.063D0
c QCD_1 and QCD_2 not used
c        EXQCD1m= XQCD1(1)
c        EXQCD1p= XQCD1(NQCD1)
c        EXQCD2m= XQCD2(1)
c        EXQCD2p= XQCD2(NQCD3)
c QCD_3 used for region EC (>=5.2) to 9.46 GeV
        EL(10)=   EH(9)
        EH(10)=   EINFTY
        do j=1,10
           call getindex(EL(j),Nall,X,inde(j))
           write (*,*) EL(j),inde(j)
        enddo
c above ECUT analytic pQCD
c        write (*,*) ' Test ECUT [main --- 116]:',ECUT
c make an array
C DATA MASSES M.. and WIDTHS G..in MeV;P.. branching fraction to ee
        write (*,*) ' 1) ISCAN=0: prompt for input energy[range],',
     &       ' ISCAN=1,2,-1,-2: use X arrays'
        write (*,*) ' 2) Ismooth?[0=R data,1=R fits]'
        write (*,*) ' 3) IGS=0: Rho Chebyshev fit,',
     &       ' =1 Gounaris-Sakurai fit'
        write (*,*) ' 4) DSET=WAVE,CMD2 GSfit WA or CMD2 based'
        write (*,*) ' 5) table/plot output: iplot=0/1'
        write (*,*) ' 6) alpha/alpha_2[old,new]/Pi33[old,new] tables:',
     &       'ISU2=0,[1,2],[3,4]?'
        write (*,*) ' 7) re-renormalize irennew=0/1[old/new]?'
        write (*,*) ' 8) number of flavors NFL=2,3,4,5?'
        write (*,*) ' 9) SIMPSON OR NAG? INAG=0,1'
        write (*,*) '10) pQCD tail for NF=2 inotail=0,1,-1'
        READ(*,*) ISCAN,Ismooth,IGS,DSET,iplot,ISU2,irennew,NFL,
     &       INAG,inotail
 101    FORMAT(3X,' Kernel type IKE='
     .,'0 (g-2) muon,1 (delta r),2 Adler function,'
     .,/,20X,'3 (g-2) tau,4 (g-2) ele'
     .,/,20X,'5 (g-2) muon HOa,6 (g-2) muon HOb,7 (g-2) ele HOa'
     .,/,20X,'8 EC:t^3*G_2(t),9 EC: t^4*G_3(t),10 Adler slope at 0'
     .,/,20X,'11 NNLO K3c double insertion 1st integral,12 amu(K3c)'
     .,/,3X,' momentum transfer: E>0 time-like , E<0 space-like '
     .,/,3X,' if NP =1 : table for E=Emin (Emax redundant) '
     .,/,3X,' if E=Emin < Emax and NP > 1 data file for graph '
     .,/,3X,' N number of subintervals for SIMPSON-Rule integration '
     .,/,3X,' EPS relative error demanded for NAG-Routine integrals'
     .,/,3X,' x-axis "LIN" or "LOG"')
        if (NFL.GE.5) inf5=1 ! use original method for calculating 5 flavor deg
        inf5=0
        WRITE(*,101)
        READ (*,*) IKE,Emin,Emax,NP,N,EPSREL,XAXIS
        write (*,*) '@@@@@@@@@@@@@@@@@@@@@@'
        write (*,*) ' READ:',IKE,Emin,Emax,NP
        write (*,*) '@@@@@@@@@@@@@@@@@@@@@@'
        if ((IKE.eq.2).and.(NP.EQ.Nadler)) then
           ixAdler=1
           EMIN=xAdler(1)
           EMAX=xAdler(NP)
           lmessage=.false.
           lwrite=.false.
        endif
c        INL=CHAR(NFL+1)
        write (INL,10) NP+1
        write (*,*) ' --',INL,'--',NP+1
 10     format(i3)
        NPin=NP
        IANOMALY=0
c  get from input ALS,EST,ESY,MZINP,MTOP,NF,IOR
        WRITE(*,*) ' Enter: ALS(M), EST, ESY, M, MTOP,',
     .       ' NF=5(no top), 6(with top) and IOR=1,2,3,4 order of pQCD '
        WRITE(*,*) ' ICHK=0,1,2   pQCD: 0(m=0), 1 (95), 2 (00) [HS] '
        READ (*,*) ALS,EST,ESY,MZINP,MTOP,NF,IOR,ICHK
        write (*,*) ' Four loop hadronic cantribution: NNLO=0,1 |
     &       IERK3C=1,3'
        READ (*,*)  NNLO,IERK3C
c if ISCAN not 0  and run via intRdatx.sh overwrite ISCAN and ISU2 as selected by script
        IF (ISCAN.EQ.-3) THEN
           write (*,*) ' preset NFL=?',NFL
           IF (NFL.EQ.0) THEN
              write (*,*) ' ISCAN,ISU2,NFL=?'
              READ (*,*) ISCAN,ISU2,NFL
           ELSE 
              write (*,*) ' ISCAN,ISU2=?'
              READ (*,*) ISCAN,ISU2
           ENDIF
        ENDIF
        IF (IKE.EQ.-8) THEN
           write (*,*) ' IKE,ISU2,NFL,AXIS=?'
           READ (*,*) IKE,ISU2,NFL,XAXIS
        ENDIF
        if (ISU2.EQ.1) then
           write (*,*) ' option outdated: choosing ISU2=2 <3g> '
           ISU2=2
        endif
        if (ISU2.EQ.3) then
           write (*,*) ' option outdated: choosing ISU2=4 for <33> '
           ISU2=4
        endif
        if ((IKE.EQ.0).or.(IKE.EQ.10).or.(IKE.EQ.12).or.
     &       (IKE.EQ.13).or.
     &       ((IKE.GE.3).and.(IKE.LE.7)))
     &       then
           EMIN=null
           EMAX=null
           NP=1
           IGEV=1
           ISU2=0               ! calculate and print delta g SM SU(2) coupling in place of alpha
           IRESON=0             ! switch on resonances in fits via RS40smoothed(s)
           iresonances=0        ! switch on resonances in RS40(s)
           iso=2                ! e^+ e^- data GS fit; iso=0,1 yields isospin 0, isovector part
           XAXIS='LIN'
           NFL=5
           INAG=1
           ISCAN=0
           IANOMALY=1
           write (*,*) '@@@@@@@@@@@@@@@@@@@@@@'
           write (*,*) ' E=0 quantity:',IKE,EMIN,EMAX,NP
           write (*,*) '@@@@@@@@@@@@@@@@@@@@@@'
        endif
        IF (ISU2.EQ.10) THEN
c like timelike low scan with kernel K3c IKE=10 1st  integral
           ISCAN=1
           ISU2=0
           IKE =11
c amuhad(K3c) with kernel R/s and scan result from 1st  integral as data
        ENDIF
        IF (NNLO.EQ.1) THEN
           IF (IKE.EQ.5) JKE=14
           IF (IKE.EQ.6) JKE=15
           IF (IKE.EQ.7) JKE=16
        ELSE
           JKE=IKE+1
        ENDIF
        IF (IKE.EQ.11) JKE=11
        IF (IKE.EQ.12) JKE=12
        if (IKE.EQ.11) then
           do ii=1,100
              e=x(ii)
              s=e*e
              write (127,'(2x,f12.4,2x,1pe11.4)') e,rsdat(s)
           enddo
        endif
        if ((ISCAN.NE.0).OR.(iplot.eq.1)) then
           lmessage=.false.
           lwrite=.false.
        endif
        if (lwrite) then
           write (*,*)  ' lwrite is true.'
        else
           write (*,*)  ' lwrite is false.'
        endif
       alerr=EST
c       IKE=1
       icount=0
c       iplot=1
       ALPI=137.0359991657D0    ! pm (51) June 2012 Aoyama et al.
       facbw=9.d0/4.d0*alpi**2  ! in common RESFIT
       ALP=1D0/ALPI
       PI=4.D0*DATAN(1.D0)
       FAC=ALP/PI/3.d0
       GMU=1.16637D-5    ! fermi constant
       MW=80.399D0       ! pm 0.023 PDG 2010
       MZ=91.1876D0      ! pm 0.0021
       sin2ell=0.23153d0 ! pm 0.00016 LEPEEWG Phys Rep 427 (2006) 257 -- value utilized in hadr5n in calculating deg
c set via common parm
       st2=0.23153d0            ! Reference value for weak mixing parameter
c SU(2) coupling at MW mass scale
       g2atMW=4.d0*sqrt(2.d0)*GMU*MW*MW
c e^2=4 pi alpha [delta alpha   = - e^2 (Pi_{gamma gamma}(s)-Pi_{gamma gamma}(0))]
c                 delta alpha_2 = - g^2 (Pi_{3 gamma}(s)-Pi_{3 gamma}(0))
c will tabulate/plot Pi_{3 gamma}(s)-Pi_{3 gamma}(s), since g^2 depends on specific definition
c which should be choosen in main program
       e2inv =1.d0/(4.d0*PI*ALP)
       facg2dive2=g2atMW*e2inv
       MELE=ML(1)
       MU  =ML(2)
       MTAU=ML(3)
C Tau anomaly:  set  MU=MTAU
       IF (IKE.EQ.3) MU=MTAU
       IF ((IKE.EQ.4).OR.(IKE.EQ.13)) MU=MELE
       IF ((NNLO.EQ.0).AND.(IKE.EQ.7)) MU=MELE
       F0=1.D0/12.D0/PI**2
       MPLUS=0.13957018D0   ! PDG 2004
c       FJ 20/05/2002 E0 -> 2E0, EPPMAX=0.81
       E0=2.d0*MPLUS
C New cut 2 M_K threshold
c       mka=493.677D0            ! +/- 0.016  PDG 2006
       E2KCUT=0.987354D0
C IPV=flag for PV-integration
       MU2=MU**2
       MP2=MPLUS**2
       MU2I=1.D0/MU2
c this routine calculates running alpha in the time-like low-energy region between
c 2 mpi and 40 GeV. works with an adapted grid of energy points
       FA=-F0
C Go to new normalization 10 e^2 times old
       IF(IKE.EQ.1)   FA=4.D0*PI*ALP*10.D0*FA*1.D3
       IF(IKE.EQ.2)   FA=12.D0*PI**2*FA
       IF(IKE.EQ.0)   FA=4.D0/3.D0*ALP**2*MU2*1.D10*F0
       IF(IKE.EQ.3)   FA=4.D0/3.D0*ALP**2*MU2*1.D8 *F0
       IF(IKE.EQ.4)   FA=4.D0/3.D0*ALP**2*MU2*1.D14*F0
       IF (NNLO.EQ.1) THEN
          IF(IKE.EQ.5)   FA=4.D0*ALP**2*1.D12*F0*ADP**2
          IF(IKE.EQ.6)   FA=4.D0*ALP**2*1.D12*F0*ADP**2
          IF(IKE.EQ.7)   FA=4.D0*ALP**2*1.D12*F0*ADP**2
       ELSE
          IF(IKE.EQ.5)   FA=2.D0/3.D0*ADP**3*1.D11
          IF(IKE.EQ.6)   FA=2.D0/3.D0*ADP**3*1.D11
          IF(IKE.EQ.7)   FA=2.D0/3.D0*ADP**3*1.D13
       ENDIF
       IF(IKE.EQ.8)   FA=F0
       IF(IKE.EQ.9)   FA=F0
       IF(IKE.EQ.10)   FA=1.d0  ! D(Q^2)/Q^2 at Q=0 ==> int ds R(s)/s^2
       IF(IKE.EQ.11)   FA=1.d0/9.d0*ADP**4*1.D12  ! amhhad K3c 1st integral scan
       IF(IKE.EQ.12)   FA=1.d0  ! amhhad K3c 2nd integral
       IF(IKE.EQ.13)   FA=4.D0*ALP**2*1.D14*F0*ADP**2
c       IF(IKE.EQ.8)   THEN
cc          IF ((ISU2.EQ.3).OR.(ISU2.EQ.4)) THEN
ccc here I=1 isospin part considered as an overall normalization factor
cc             if (itestflavorsplitting.eq.0) then
cc                FA=F0*3.2d0     ! Pi^{I=1}= 9/10 Pi^{em}; Pi_33= 9/32 Pi^{em} => 32/10=3.2
cc             else
cc                FA=F0
cc             endif
cc          ELSE
c          FA=F0
cc          ENDIF
c       ENDIF
       NAGFAC=FA
       write (*,*) ' -FA-FA-FA-:',FA,'(IKE=',IKE,')'
       OPEN (UNIT=1,FILE='intRdatx_gg.dap',STATUS='NEW')
       OPEN (UNIT=2,FILE='intRdatx_3g.dap',STATUS='NEW')
       OPEN (UNIT=3,FILE='intRdatx_33.dap',STATUS='NEW')
       OPEN (UNIT=51,FILE='intRdatx_n2.dap',STATUS='NEW')
       if ((IKE.EQ.1).and.(ISCAN.EQ.-1).and.
     &         (ISU2.EQ.0)) then
          OPEN (UNIT=11,FILE='intRdatx_n3.dap',STATUS='NEW')
          OPEN (UNIT=12,FILE='intRdatx_n4.dap',STATUS='NEW')
          OPEN (UNIT=13,FILE='intRdatx_n5.dap',STATUS='NEW')
       endif
       if ((IKE.EQ.1).and.(ISCAN.EQ.-1).and.
     &         (ISU2.EQ.2)) then
          OPEN (UNIT=11,FILE='intRdatg3_n3.dap',STATUS='NEW')
          OPEN (UNIT=12,FILE='intRdatg3_n4.dap',STATUS='NEW')
          OPEN (UNIT=13,FILE='intRdatg3_n5.dap',STATUS='NEW')
       endif
       if (iplot.eq.0) then
          OPEN (UNIT=7,FILE='dalhads.dat',STATUS='NEW')
          OPEN (UNIT=8,FILE='intRdatx.tab',STATUS='NEW')
c          OPEN (UNIT=4,FILE='flavors.dat',STATUS='NEW')
          OPEN (UNIT=70,FILE='EClabel',STATUS='NEW')
       else
       endif
c test K3c 1st integral data
       write (*,*) ' test K3c 1st integral data'
       de=1.d0
       call dhadrK3cdist15(de,ddk3c,derrdk3c)
       write (*,*) de,ddk3c,derrdk3c
c
C fort.3 is copied by intRdatx.sh to intRdatx.tab
c
       STIME=time()
       uhrzeit=CTime(STIME)
       read(uhrzeit,'(A22,A2)') aaa,year

       write (*,*) ' Uhrzeit: ',uhrzeit
       IF (ISCAN.EQ.0) THEN
          write (8,*) ' Timestamp:',uhrzeit
       ENDIF
C
c       if (iplot.eq.0) then
c       else
c       endif
       DO I=1,3
          WRITE(I,*) '      16.0000'
          WRITE(I,*) '       1.0000'
       ENDDO
       if ((IKE.EQ.1).and.(ISCAN.NE.0).and.
     &     ((ISU2.EQ.0).or.(ISU2.EQ.2))) then
          DO I=1,3
             WRITE(10+I,*) '      16.0000'
             WRITE(10+I,*) '       1.0000'
          ENDDO
c write NFL to EClabel used in intRdatx.sh to label file to which dalshad.dat is copied 
          IF (NFL.EQ.2) THEN
             write (70,'(a1)') '2'
          ELSE IF (NFL.EQ.3) THEN
             write (70,'(a1)') '3'
          ELSE IF (NFL.EQ.4) THEN
             write (70,'(a1)') '4'
          ELSE IF (NFL.EQ.5) THEN
             write (70,'(a1)') '5'
          ELSE 
          ENDIF
       endif
       WRITE(50,*) '      16.0000'
       WRITE(50,*) '       1.0000'
       WRITE(51,*) '      16.0000'
       WRITE(51,*) '       1.0000'
       WRITE(22,*) '       2.0000'
       WRITE(22,*) '       1.0000'
       IF (ISCAN.EQ.0) THEN
c Test *XXX* 19/03/2010 SU2 had shift at rho
          WRITE(44,*) '       2.0000'
          WRITE(44,*) '       1.0000'
          WRITE(44,*) '     ',float(NP)
          WRITE(45,*) '       2.0000'
          WRITE(45,*) '       1.0000'
          WRITE(45,*) '     ',float(NP)
          WRITE(46,*) '       2.0000'
          WRITE(46,*) '       1.0000'
          WRITE(46,*) '     ',float(NP)
          WRITE(47,*) '       2.0000'
          WRITE(47,*) '       1.0000'
          WRITE(47,*) '     ',float(NP)
          WRITE(48,*) '       2.0000'
          WRITE(48,*) '       1.0000'
          WRITE(48,*) '     ',float(NP)
          WRITE(49,*) '       2.0000'
          WRITE(49,*) '       1.0000'
          WRITE(49,*) '     ',float(NP)
c end test *XXX*
          WRITE(98,*) '       2.0000'
          WRITE(98,*) '       1.0000'
          WRITE(98,*) '     ',float(NP)
       ENDIF
       IF (ISCAN.EQ.1) THEN
          do j=1,6
             icountps(j)=0
             write (110+j,*) '       3.0000'
             write (110+j,*) '       1.0000'
             write (110+j,*) '     ',float(NPPS(j))
             write (110+j,*) '    PSI(',j,')'
          enddo
          do j=1,6
             icountyp(j)=0
             write (120+j,*) '       3.0000'
             write (120+j,*) '       1.0000'
             write (120+j,*) '     ',float(NPYP(j))
             write (120+j,*) '    YPS(',j,')'
          enddo
       ENDIF
c      write (*,*) ' main FA:',FA
       IRESON=0                  ! =1 switch on resonances in fits via RSBGI(s)
       iresonances=0             ! =1 switch on resonances in RS40(s)
c      write (*,*) ' Test ECUT [main --- 206]:',ECUT
C High energy cut ECUT on data read from vapo1n.inp
      DO I=1,NV
         XV(I)=NULL
         YV(I)=NULL
         ZV(I)=NULL
         UV(I)=NULL
         VV(I)=NULL
      ENDDO
      ILOG=0
      IINV=0
      IF (XAXIS.EQ.'LOG') ILOG=1
      IF (XAXIS.EQ.'INV') IINV=1
      IF (ISCAN.EQ.-2) ILOG=1
      IF ((EMAX.EQ.EMIN)) NP=1
      IF ((EMAX.NE.EMIN).AND.(NP.EQ.1)) EMAX=EMIN
      IF ((NP.GT.1).AND.(EMAX.NE.EMIN)) THEN
         DE=(EMAX-EMIN)/NP
         IF ((EMIN.GE.13.D0).OR.(ILOG.EQ.1)) THEN
c use logarithmic sacle
            ILOG=1
            LMIN=LOG(EMIN)
            LMAX=LOG(EMAX)
            DE=(LMAX-LMIN)/NP
         ENDIF
         IF (IINV.EQ.1) THEN
c use logarithmic sacle
            IINV=1
            LMIN=1.d0/EMIN
            LMAX=1.d0/EMAX
            DE=(LMAX-LMIN)/NP
         ENDIF
         NPM=NP+1
      ELSE
         NPM=1
      ENDIF
c      write (*,*) ' Main -- L216:',NP,EMIN,EMAX
      E=EMIN
c      write (*,*) ' Test ECUT [main --- 218]:',ECUT
c treating resonance contributions in integration: calculated by subroutione resonances
      EPPMAX=0.81d0
      ERHORES=0.77703500000000003d0
      ERHOPLU=0.78783499999999995d0
      EFIMM=0.95899999999999996d0
      EFIMX=1.00000000000000000d0
      EFIPX=1.0400000000000000d0
      EFIPP=1.1120000000000001d0
      EPM =0.050d0 !  50 MeV
      EPMc=0.150d0 ! 150 MeV
      EPC =0.200d0 ! 200 MeV
      IF (EPMc.GE.EPC) EPMc=EPC-EPM
      DMP =0.100/10.d0
c      DMP =0.100d0/10.d0
      RMP =1.05d0                     ! must be > 1.0
      IF (RMP.LE.1.0d0) RMP=1.1d0
c omega  .41871054d0,.810d0 [MOM=782.65D0,GOM=8.49d0]
c 0.61 - 1.4
      MOMGeV=0.78265D0
      GOMGeV=0.00849D0
      DEGOMGeV=4.d0*GOMGeV
      MFIGeV=1.019460D0
      GFIGeV=0.00426D0
      DEGFIGeV=4.d0*GFIGeV
      if ((IKE.EQ.2).OR.(IKE.GE.8).OR.(IKE.EQ.0))
     &     sin2ell=1.d0
C Begin energy loop for running alpha etc
c      NMI=20
c      IF (ISCAN.EQ.1) NMI=374
c      IF (ISCAN.EQ.0) NMI=1
      NMI=1
c      NMI=690
c      lwrite=.true.
      IF (ISCAN.EQ.1) NPM=Nall-1
      IF (ISCAN.EQ.-1) NPM=NA
c      IF (ISCAN.EQ.-1) NPM=10 ! use for test runs
      IF (ISCAN.EQ.-2) NPM=NC
      IF (ISCAN.EQ.2) NPM=NB
      IF (ISCAN.EQ.3) NPM=NC
      if (ixAdler.eq.1) then
         NMI=1
         NPM=NAdler
      endif
c
c a test of direct rho exchange
c see F.J & R. Szafron 2011
c e = 0.302822, g_{\rho ee} = 0.01826
      S=(MOMGeV)**2
      TERE=ALP/PI*MU2/3.d0/S*FK(S)*(0.01826d0/0.302822d0)**2
      write (*,*) ' Direct Rho:',TERE
      IECUT=0
      ECSAV=EC
      ECUTS=ECUT
      DMPSAV=DMP
      icount=1 ! counting calculated points
      ZT=1.d0
      if (IKE.EQ.1) ZT=10000.d0
c Compare flavor separated versions of RS(s) --> check fort.99 --> plot dase0821_Fsplit.par
      if ((itestflavorsplitting.eq.1).and.(itest.eq.1)) then
         call testRSflavorsplitting
      endif
      if (((NFL.EQ.3).or.(NFL.EQ.4)).and.(ISU2.EQ.0))
     &     itestflavorsplitting=1
      write (*,*) ' itestflavorsplitting:',itestflavorsplitting
c
c Test J/psi region
c      itest=1
c      NMI=3       ! 0.312
c      NPM=232     ! 1.000
c      NMI=820
c      NPM=840
c CHICUT
       ! 0.312
c      NPM=10      ! 1.000
c 0.61 region
c      NMI=70      ! 0.312
c      NPM=100      ! 1.000
c 2.00 region
c      NMI=460      ! 1.750
c      NMI=7       ! first after 0.312 which crashes
c      NPM=510      ! 2.600
c      NMI=1939
c      NMI=1670
c      NPM=1680
c      NPM=340
c test vew versions of R3G.f R3Gdat_fun.f from 3.19 to 3.201
c      NMI=830
c      NMI=855
cc      NPM=850
c      NPM=870
c test high energy tail
c      NMI=1230
c      NPM=2177
c
      DO I=1,3
         WRITE(I,*)  '    ',float(npm-nmi+1)
      ENDDO
      if ((IKE.EQ.1).and.(ISCAN.EQ.-1).and.
     &    ((ISU2.EQ.0).or.(ISU2.EQ.2))) then
         DO I=1,3
            WRITE(10+I,*)  '    ',float(npm-nmi+1)
         ENDDO
      endif
c      if ((IKE.EQ.1).and.(ISCAN.EQ.-1).and.
c     &     (ISU2.EQ.0)) then
c         WRITE(66,*) '       2.0000'
c         WRITE(66,*) '       1.0000'
c         WRITE(66,*)  '    ',float(npm-nmi+1)
c      endif
c      if ((IKE.EQ.1).and.(ISCAN.EQ.-1).and.
c     &     (ISU2.EQ.2)) then
c         WRITE(67,*) '       2.0000'
c         WRITE(67,*) '       1.0000'
c         WRITE(67,*)  '    ',float(npm-nmi+1)
c      endif
      WRITE(50,*)  '    ',float(npm-nmi+1)
      WRITE(51,*)  '    ',float(npm-nmi+1)
      if (itest.eq.1) then
c      WRITE(15,*) '       2.0000'
c      WRITE(15,*) '       1.0000'
c      WRITE(15,*) '     ',float(npm-nmi+1)
         WRITE(16,*) '       2.0000'
         WRITE(16,*) '       1.0000'
         WRITE(16,*) '     ',float(npm-nmi+1)
         WRITE(17,*) '       2.0000'
         WRITE(17,*) '       1.0000'
         WRITE(17,*) '     ',float(npm-nmi+1)
         WRITE(57,*) '       2.0000'
         WRITE(57,*) '       1.0000'
         WRITE(57,*) '     ',float(npm-nmi+1)
         if (iplot.eq.1) then
            WRITE(98,*) '       2.0000'
            WRITE(98,*) '       1.0000'
            WRITE(98,*) '     ',float(npm-nmi+1)
         else
         endif
         do i=1,9
            WRITE(80+i,*) '       2.0000'
            WRITE(80+i,*) '       1.0000'
            WRITE(80+i,*) '     ',float(npm-nmi+1)
         enddo
         IF ((IKE.NE.1).AND.(IKE.NE.2)) THEN
            do ir=1,6
               write (90+ir,*) '       2.0000'
               write (90+ir,*) '       1.0000'
               write (90+ir,*) '     ',float(npm-nmi+1)
            enddo
         ENDIF
      endif
      DO I=1,3
         WRITE(I,102) uhrzeit
         WRITE(I,103)
         WRITE(I,104) KERN(JKE),UPDATEVERSION
         WRITE(I,103)
         WRITE(I,105) ISCAN,Ismooth,IGS,DSET,iplot,itestflavorsplitting
         WRITE(I,107) IKE,Emin,Emax,NP,N,EPSREL,XAXIS
         WRITE(I,108) ALS,EST,ESY,MZINP,MTOP
         WRITE(I,208) NF,IOR,ICHK,ISU2,NFL,INAG,inotail,sin2ell
         WRITE(I,109) CHPTCUT,EC,ECUT,st2,sin2ell/st2
         WRITE(I,103)
         if ((IKE.EQ.1).and.(ISCAN.EQ.-1).and.
     &       ((ISU2.EQ.0).or.(ISU2.EQ.2))) then
            WRITE(10+I,102) uhrzeit
            WRITE(10+I,103)
            WRITE(10+I,104) KERN(JKE),UPDATEVERSION
            WRITE(10+I,103)
            WRITE(10+I,105) ISCAN,Ismooth,IGS,DSET,iplot,
     &           itestflavorsplitting
            WRITE(10+I,107) IKE,Emin,Emax,NP,N,EPSREL,XAXIS
            WRITE(10+I,108) ALS,EST,ESY,MZINP,MTOP
            WRITE(10+I,208) NF,IOR,ICHK,ISU2,NFL,INAG,inotail,sin2ell
            WRITE(10+I,109) CHPTCUT,EC,ECUT,st2,sin2ell/st2
            WRITE(10+I,103)
         endif
      ENDDO
      DO I=50,51
         WRITE(I,102) uhrzeit
         WRITE(I,103)
         WRITE(I,104) KERN(JKE),UPDATEVERSION
         WRITE(I,103)
         WRITE(I,105) ISCAN,Ismooth,IGS,DSET,iplot,itestflavorsplitting
         WRITE(I,107) IKE,Emin,Emax,NP,N,EPSREL,XAXIS
         WRITE(I,108) ALS,EST,ESY,MZINP,MTOP
         WRITE(I,208) NF,IOR,ICHK,ISU2,NFL,INAG,inotail,sin2ell
         WRITE(I,109) CHPTCUT,EC,ECUT,st2,sin2ell/st2
         WRITE(I,103)
      ENDDO
      I=1
      WRITE(*,102) uhrzeit
      WRITE(*,103)
      WRITE(*,104) KERN(JKE),UPDATEVERSION
      WRITE(*,103)
      WRITE(*,105) ISCAN,Ismooth,IGS,DSET,iplot,itestflavorsplitting
      WRITE(*,107) IKE,Emin,Emax,NP,N,EPSREL,XAXIS
      WRITE(*,108) ALS,EST,ESY,MZINP,MTOP
      WRITE(*,208) NF,IOR,ICHK,ISU2,NFL,INAG,inotail,sin2ell
      WRITE(*,109) CHPTCUT,EC,ECUT,st2,sin2ell/st2
      WRITE(*,103)
      write (*,*) ' Main energy loop:',NMI,NPM
C     Begin energy loop for running alpha etc
      if (IANOMALY.EQ.1) then
         NMI=1
         NPM=1
      endif
      N2pQCDcut=2.d0
c      N2pQCDcut=1.d11
c      itest=1
      if (itest.eq.1) then
         call testRSBGI(0.32d0,2.5d0)
         itest=0
      endif
      ierrcountY=0
      ierrcountZ=0
c      NMI=1490
c      NPM=1600
c      NMI=1820
c      NPM=1890
c      NMI=500
c      NPM=600
c      NMI=640
c      ELOW =3.092d0
c      EHIGH=3.102d0
c      DE=(EHIGH-ELOW)/500.
c      E=ELOW
c      if (IKE.EQ.11) NMI=10
c      if (IKE.EQ.11) NPM=14
      IF (ISCAN.EQ.-2) THEN
         EMIN=-20000.d0
         EMAX= ETX(1)
         LMIN=LOG(abs(EMIN))
         LMAX=LOG(abs(EMAX))
         DL=(LMAX-LMIN)/(NC-1)
         E=EMIN
         NMI=1
         NPM=NC
      ENDIF
      if ((IKE.EQ.0).or.(IKE.EQ.10).or.(IKE.EQ.12).or.
     &     (IKE.EQ.13).or.
     &     ((IKE.GE.3).and.(IKE.LE.7)))
     &     then
         EMIN=null
         EMAX=null
         NP=1
         IGEV=1
         ISU2=0                 ! calculate and print delta g SM SU(2) coupling in place of alpha
         IRESON=0               ! switch on resonances in fits via RS40smoothed(s)
         iresonances=0          ! switch on resonances in RS40(s)
         iso=2                  ! e^+ e^- data GS fit; iso=0,1 yields isospin 0, isovector part
         XAXIS='LIN'
         NFL=5
         INAG=1
         ISCAN=0
         IANOMALY=1
         NMI=1
         NPM=1
         E=0.d0
         write (*,*) '@@@@@@@@@@@@@@@@@@@@@@'
         write (*,*) ' E=0 quantity:',IKE,EMIN,EMAX,NP
         write (*,*) '@@@@@@@@@@@@@@@@@@@@@@'
      endif
c      NER=1
c      IF (IKE.EQ.12) NER=3
      DO IE=NMI,NPM
c      do NE=1,NER  ! error loop for 2nd integration of double integral k3c
c      DO IE=1,500
         if (ixAdler.eq.1) E=xAdler(IE)
c There is a numerical stability problem in a narrow range here; skip it!
c         IF ((ISCAN.EQ.1).AND.((IE.GT.370).AND.(IE.LT.374))) GOTO 60
C######energy units : MeV ;scale MPLUS (Pionmass)
c         write (*,*) ' E0,E,ECUT',E0,E,ECUT
c     if E at critical point do loop about the two neighboring points
         IF (ISCAN.EQ.1) THEN
c     E=(X(IE)+X(IE+1))/2.d0
            E=X(IE)
            if (IE.LT.6) E=(X(IE)+X(IE+1))/2.d0
c            E=E+DE
c resonance scan: take X coordinates itself
c         IF (((X(IE).GE.0.79d0).AND.(X(IE).LE.0.830d0)).OR.
c     &     ((X(IE).GE.1.00D0).AND.(X(IE).LE.1.04D0)).OR.
c     &     ((X(IE).GE.3.08587D0).AND.(X(IE).LE.3.10787D0)).OR.
c     &     ((X(IE).GE.3.67496D0).AND.(X(IE).LE.3.69696D0)).OR.
c     &     ((X(IE).GE.3.7000D0).AND.(X(IE).LE.3.8400D0)).OR.
c     &     ((X(IE).GE.9.44937D0).AND.(X(IE).LE.9.47137D0)).OR.
c     &     ((X(IE).GE.10.01226D0).AND.(X(IE).LE.10.03426D0)).OR.
c     &     ((X(IE).GE.10.3442D0).AND.(X(IE).LE.10.3662D0)).OR.
c     &     ((X(IE).GE.10.473D0).AND.(X(IE).LE.10.687D0)).OR.
c     &     ((X(IE).GE.10.690D0).AND.(X(IE).LE.10.950D0)).OR.
c     &     ((X(IE).GE.10.975D0).AND.(X(IE).LE.11.063D0)))THEN
c            E=X(IE)
c         ENDIF
         ENDIF
         IF (ISCAN.EQ.-1) E =ETX(IE)
         IF (ISCAN.EQ.-2) LL=LOG(-E)
         IF (ISCAN.EQ. 2) E =ESX(IE)
c omega  .41871054d0,.810d0
c phi   1.00D0,1.04D0
c psi masses 3096.92D0,3686.09D0,3770.0D0
c psi widthes 0.091D0,0.281D0,23.6D0
c psi1  3.08587D0 3.10787D0
c psi2  3.67496D0 3.69696D0
c psi3  3.7000D0  3.8400D0
c Upsilon masses 9460.30D0,10023.26D0,10355.2D0,10580D0,10865D0,11019D0
c Upsilon widthes 5.30D-2  ,4.3D-2   ,2.63D-2  ,20.0D0 ,110D0  ,79D0
c yps1  9.44937D0 9.47137D0
c yps2  10.01226D010.03426D0
c yps3  10.3442D0 10.3662D0
c yps4  10.473D0  10.687D0
c yps5  10.690D0  10.950D0
c yps6  10.975D0  11.063D0
         if (IANOMALY.EQ.1) E=null
         if (IE.EQ.1) then
            write (*,*) ' Main E loop: (IE,E)',IE,E
         else
            write (*,*) ' Main E loop:       ',IE,E
         endif
c
         EP=E
         IF (EC.NE.ECSAV) EC=ECSAV
         IF (ECUT.NE.ECUTS) ECUT=ECUTS
         ETAILCUT=ETAILCUT_SAV
         IF (DMP.NE.DMPSAV) DMP=DMPSAV
         EOMPx=DMIN1(0.80d0,MOMGeV+DEGOMGeV)
         IF ((EP.GE.MOMGeV-DEGOMGeV).AND.(EP.LE.EOMPx)) THEN
c Shifting the CUT here is actually not a good idea, don't do it!
c            ECSAV=EC
c            EC=DMIN1(EC+EPC,9.5d0)
            DMP=GOMGeV/4.d0  ! try this
c            GOTO 60
         ENDIF
         IF ((EP.GE.MFIGeV-DEGFIGeV).AND.(EP.LE.MFIGeV+DEGFIGeV)) THEN
c Shifting the CUT here is actually not a good idea, don't do it!
c            ECSAV=EC
c            EC=DMIN1(EC+EPC,9.5d0)
            DMP=GFIGeV/4.d0  ! try this
c            GOTO 60
         ENDIF
         IF ((EP.GE.EC-EPMc).AND.(EP.LE.EC+EPMc)) THEN
c Shifting the CUT here is actually not a good idea, don't do it!
c            ECSAV=EC
c            EC=DMIN1(EC+EPC,9.5d0)
            DMP=DMP/2.d0  ! try this
c            GOTO 60
         ENDIF
         IF ((EP.GE.ECUT-EPMc).AND.(EP.LE.ECUT+EPMc)) THEN
c Shifting the CUT here is actually not a good idea, don't do it!
c            ECUTS=ECUT
c            ECUT=ECUT-EPC
c Just skip the point and later take average of the previous and the procceding one;
c set a mark for that
            IECUT=1
            GOTO 60
         ENDIF
c avoid region boundary points in taking principal values
         if ((EP.eq.0.312d0).or.(EP.eq.0.61d0)
     &        .or.(EP.eq.0.81d0).or.(EP.eq.0.818d0)
     &        .or.(EP.eq.0.9d0).or.(EP.eq.1.05d0).or.(EP.eq.1.4d0)
     &        .or.(EP.eq.ERHORES).or.(EP.eq.1.85d0)
     &        .or.(EP.eq.ERHOPLU).or.(EP.eq.EFIMM).or.(EP.eq.EFIMX)
     &        .or.(EP.eq.EFIPX).or.(EP.eq.EFIPP)
     &        .or.(EP.eq.1.45d0).or.(EP.eq.2.00d0).or.(EP.eq.2.15d0)
     &        .or.(EP.eq.2.20d0)
     &        .or.(EP.eq.3.2d0).or.(EP.eq.3.4d0)
     &        .or.(EP.eq.3.6d0).or.(EP.eq.5.2d0).or.(EP.eq.9.46d0)
     &        .or.(EP.eq.10.473d0).OR.(IE.EQ.1833)
     &        .or.(EP.eq.10.687d0).OR.(IE.EQ.1893)
     &        .or.(EP.eq.11.5d0).or.(EP.eq.13.d0)) then
            call getindex(EP,Nall,X,jep)
            ipm=2
         else
            ipm=1
         endif
         do jj=1,ipm
            if (ipm.eq.2) E=X(jep+jj-1)
            S=E**2
            IF (E.LT.0D0) S=-S
            M2=S
            M2_1=M2
            IER=1
            DO jer=1,3
               RE(jer)=NULL
               RD(jer)=NULL
               RG(jer)=NULL
               R3(jer)=NULL
               R2(jer)=NULL
               RESAV(jer)=NULL
               RDSAV(jer)=NULL
               RGSAV(jer)=NULL
               R3SAV(jer)=NULL
               R2SAV(jer)=NULL
            ENDDO
            if ((IKE.GE.8).OR.(IKE.EQ.0)) then
               IPV=0
               MPOL=0.D0
               POLP=0.D0
               POLM=0.D0
               POLP1=0.D0
               POLM1=0.D0
            else
               IF ((E.LT.E0).OR.(E.GT.ECUT)) THEN
                  IPV=0
               ELSE
                  IPV=1
               ENDIF
C     IPV=flag for PV-integration
               IF ((E.GT.E0).AND.(E.LT.ECUT)) THEN
                  MPOL=E
                  POLP1=DMIN1(ECUT ,MPOL*RMP,MPOL+DMP)
                  POLM1=DMAX1(E0,MPOL/RMP,MPOL-DMP)
                  POLP =POLP1**2
                  POLM =POLM1**2
c using NAG PV integration routine keep intervals reasonably large
c               IF (E.LT.0.4d0) THEN
c                  POLM=E0
c                  POLP=0.5d0
c                  ELSE IF (E.LT.0.62d0) THEN
c                  POLM=0.3d0
c                  POLP=0.76d0
c                  ELSE IF (E.LT.0.78d0) THEN
c                  POLM=0.57d0
c                  POLP=0.792d0
c                  ELSE IF (E.LT.EPPMAX) THEN
c                  POLM=0.77d0
c                  POLP=EFIMM
c                  ELSE
c                  ENDIF
               ELSE
                  MPOL=0.D0
                  POLP=0.D0
                  POLM=0.D0
                  POLP1=0.D0
                  POLM1=0.D0
               ENDIF
C     first do PV integral for s in [POLM,POLP]
               EMI=POLM1
               EMA=POLP1
               IF (IPV.EQ.1) THEN
                  IER_SAV=IER
                  DO jer=1,3
                     IER=jer
                     RE(jer)=NULL
                     RD(jer)=NULL
                     RG(jer)=NULL
                     R3(jer)=NULL
                     R2(jer)=NULL

         if (lwrite)
     &        write (*,*) ' first do PV integral for s in [POLM,POLP]'
c                  write (*,*) ' D01AQF RSBGI -- 1: sqrt(m2',sqrt(m2),
c     &                 sqrt(POLM),sqrt(POLP),RSBGI(m2)
c                  write (104,124) sqrt(m2),RSBGI(m2)
                     IFAIL=-1
                     CALL D01AQF(RSBGI,POLM,POLP,M2,0D0,EPSREL,RE(jer),
     &                    ABSERR,W,LW,IW,LIW,IFAIL)
                     IFAIL=-1
                     CALL D01AQF(RSB3G,POLM,POLP,M2,0D0,EPSREL,RG(jer),
     &                    ABSERR,W,LW,IW,LIW,IFAIL)
                     IFAIL=-1
                     CALL D01AQF(RSB33,POLM,POLP,M2,0D0,EPSREL,R3(jer),
     &                    ABSERR,W,LW,IW,LIW,IFAIL)
C
                     IF (IFAIL.NE.0) WRITE (*,99996) 'RSBGI -- 1:',IFAIL
c                  write (*,*)   ' POLE result  smoothed:', RE(jer)
c                  CALL D01AQF(RSDAT,POLM,POLP,M2,0D0,EPSREL,RD(jer),
c                     CALL D01AQF(RSBGI,POLM,POLP,M2,0D0,EPSREL,RD(jer),
c     &                    ABSERR,W,LW,IW,LIW,IFAIL)
c                     IF (IFAIL.NE.0) WRITE (*,99996) 'RSDAT -- 1:',IFAIL
c                  write (*,*)   ' POLE result data raw:', RD(jer)
                     RE(ier)=RE(ier)*FA
c                     RD(ier)=RD(ier)*FA
                     RD(ier)=RE(ier)
                     RG(ier)=RG(ier)*FA
                     R3(ier)=R3(ier)*FA
                     R2(ier)=R2(ier)*FA
c                     if (ier.eq.1) then
c                        write(22,124) MPOL,RE(ier),RD(ier),
c     &                       RG(ier),RC(ier)
c                     endif
                     if (jj.eq.1) then
                        RESAV(ier)=RE(ier)
                        RDSAV(ier)=RD(ier)
                        RGSAV(ier)=RG(ier)
                        R3SAV(ier)=R3(ier)
                        R2SAV(ier)=R2(ier)
                     endif
                  ENDDO
                  IER=IER_SAV
                  if (ipm.eq.2) then
                     do jer=1,3
                        RE(jer)=(RESAV(jer)+RE(jer))/2.d0
                        RD(jer)=(RDSAV(jer)+RD(jer))/2.d0
                        RG(jer)=(RGSAV(jer)+RG(jer))/2.d0
                        R3(jer)=(R3SAV(jer)+R3(jer))/2.d0
                        R2(jer)=(R2SAV(jer)+R2(jer))/2.d0
                     enddo
                     E=EP
                     M2=E*E
                     M2_1=M2
                  endif
               ENDIF
            endif               ! IKE=0,8,9,10 option (zero momentum)
         enddo
c         write(22,124) MPOL,RE(1),RD(1),RE(1)/RD(1)-1.d0
C     Integrate background by trapezoidal rule
         if (lwrite)
     &        write (*,*) ' Integrate background by trapezoidal rule'
c     931 is 10 GeV
         N=10
         IPV=0                  ! use data non-resonant background in complement of pole region
         IRESON=0               ! =1 switch on resonances in fits via RSBGI(s)
         iresonances=0          ! =1 switch on resonances in RS40(s)
         iresonances_sav=iresonances
         DO j=1,9
            do jer=1,3
               TB(j,jer) =NULL
               TBL(j,jer)=NULL
               TBH(j,jer)=NULL
               TG(j,jer) =NULL
               L3G(j,jer)=NULL
               H3G(j,jer)=NULL
               T3(j,jer) =NULL
               L33(j,jer)=NULL
               H33(j,jer)=NULL
               T2(j,jer) =NULL
               N2L(j,jer)=NULL
               N2H(j,jer)=NULL
            enddo
         ENDDO
c find in which interval the pole lies
         jp=8
         DO WHILE (MPOL.LT.EL(jp))
            jp=jp-1
            if (jp.eq.0) goto 1011
         ENDDO
 1011    continue
c         write (86,*) IE,jp,E,RE(jp),RG(jp),R3(jp)
c         write (*,*) ' Main --- 2180:FA',FA
         if (lwrite) write (*,*) ' Main: Calculating background now'
c         IKE=1
         DO II=1,Nall-1
C     adapt interval subdivision to width of interval:
C     N input parameter -> N used for Simpson rule
            XL=X(II)
            XH=X(II+1)
            DHL=XH-XL
c            write (*,*) ' +++>> xl,xh:', xl,xh
c     element 931 is at 10 GeV
c            if (II.GT.931) then
c               if ((XL.LT.ECUT).and.(XH.GT.ECUT)) XH=ECUT
c               if ((XL.GE.ECUT).and.(XH.GT.ECUT)) goto 99
c            endif
            if ((XL.GE.EL(1)).and.(XH.LE.EH(1))) j=1
            if ((XL.GE.EL(2)).and.(XH.LE.EH(2))) j=2
            if ((XL.GE.EL(3)).and.(XH.LE.EH(3))) j=3
            if ((XL.GE.EL(4)).and.(XH.LE.EH(4))) j=4
            if ((XL.GE.EL(5)).and.(XH.LE.EH(5))) j=5
            if ((XL.GE.EL(6)).and.(XH.LE.EH(6))) j=6
            if ((XL.GE.EL(7)).and.(XH.LE.EH(7))) j=7
            if ((XL.GE.EL(8)).and.(XH.LE.EH(8))) j=8
            if ((XL.GE.EL(9)).and.(XH.LE.EH(9))) j=9
c             IF (DHL.GT.0.2D0) N=10
            SL=XL**2
            SH=XH**2
c             write (*,*) ' Main --- 613:j',j,II
c
c             write (*,*) ' before SIMPS:',POLM,POLP,XL,XH
c integrate complement of pole range
c first collect contributions below pole region
            IF (SH.LE.POLM) THEN
c pole region below intergration range
c            --|---|------|-------|--
c              SL  SH    POLM    POLP
               IER=1
               CALL SIMPS(RSDAT,SL,SH,N,ANS1)
               TBL(j,ier)=TBL(j,ier)+ANS1
               CALL SIMPS(RS3GDAT,SL,SH,N,ANS1)
               L3G(j,ier)=L3G(j,ier)+ANS1
               CALL SIMPS(RS33DAT,SL,SH,N,ANS1)
               L33(j,ier)=L33(j,ier)+ANS1
               CALL SIMPS(RSN2DAT,SL,SH,N,ANS1)
               N2L(j,ier)=N2L(j,ier)+ANS1
               IER=2
               CALL SIMPS(RSDAT,SL,SH,N,ANS2)
               TBL(j,ier)=SQRT(TBL(j,ier)**2+ANS2**2)
               CALL SIMPS(RS3GDAT,SL,SH,N,ANS2)
               L3G(j,ier)=SQRT(L3G(j,ier)**2+ANS2**2)
               CALL SIMPS(RS33DAT,SL,SH,N,ANS2)
               L33(j,ier)=SQRT(L33(j,ier)**2+ANS2**2)
               CALL SIMPS(RSN2DAT,SL,SH,N,ANS2)
               N2L(j,ier)=SQRT(N2L(j,ier)**2+ANS2**2)
               IER=3
               CALL SIMPS(RSDAT,SL,SH,N,ANS3)
               TBL(j,ier)=TBL(j,ier)+XABS(ANS3)
               CALL SIMPS(RS3GDAT,SL,SH,N,ANS3)
               L3G(j,ier)=L3G(j,ier)+XABS(ANS3)
               CALL SIMPS(RS33DAT,SL,SH,N,ANS3)
               L33(j,ier)=L33(j,ier)+XABS(ANS3)
               CALL SIMPS(RSN2DAT,SL,SH,N,ANS3)
               N2L(j,ier)=N2L(j,ier)+XABS(ANS3)
            ELSE IF (SL.GE.POLP) THEN
c pole region above intergration range
c            --|-------|----|---|--
c             POLM    POLP  SL  SH
c                write (14,*) ' --->', sl,rsn2dat(sl),sh,rsn2dat(sh)
               IER=1
               CALL SIMPS(RSDAT,SL,SH,N,ANS1)
               TBH(j,ier)=TBH(j,ier)+ANS1
               CALL SIMPS(RS3GDAT,SL,SH,N,ANS1)
               H3G(j,ier)=H3G(j,ier)+ANS1
               CALL SIMPS(RS33DAT,SL,SH,N,ANS1)
               H33(j,ier)=H33(j,ier)+ANS1
               CALL SIMPS(RSN2DAT,SL,SH,N,ANS1)
               N2H(j,ier)=N2H(j,ier)+ANS1
               IER=2
               CALL SIMPS(RSDAT,SL,SH,N,ANS2)
               TBH(j,ier)=SQRT(TBH(j,ier)**2+ANS2**2)
               CALL SIMPS(RS3GDAT,SL,SH,N,ANS2)
               H3G(j,ier)=SQRT(H3G(j,ier)**2+ANS2**2)
               CALL SIMPS(RS33DAT,SL,SH,N,ANS2)
               H33(j,ier)=SQRT(H33(j,ier)**2+ANS2**2)
               CALL SIMPS(RSN2DAT,SL,SH,N,ANS2)
               N2H(j,ier)=SQRT(N2H(j,ier)**2+ANS2**2)
               IER=3
               CALL SIMPS(RSDAT,SL,SH,N,ANS3)
               TBH(j,ier)=TBH(j,ier)+XABS(ANS3)
               CALL SIMPS(RS3GDAT,SL,SH,N,ANS3)
               H3G(j,ier)=H3G(j,ier)+XABS(ANS3)
               CALL SIMPS(RS33DAT,SL,SH,N,ANS3)
               H33(j,ier)=H33(j,ier)+XABS(ANS3)
               CALL SIMPS(RSN2DAT,SL,SH,N,ANS3)
               N2H(j,ier)=N2H(j,ier)+XABS(ANS3)
            ELSE IF ((SL.LT.POLM).AND.(SH.GT.POLM)) THEN
c POLM  contained in integration range
c            --|-------|----|---|--   or  --|-------|-----|----|---
c              SL     POLM  SH  POLP        SL     POLM  POLP SH
               IER=1
               CALL SIMPS(RSDAT,SL,POLM,N,ANS1)
               TBL(j,ier)=TBL(j,ier)+ANS1
               CALL SIMPS(RS3GDAT,SL,POLM,N,ANS1)
               L3G(j,ier)=L3G(j,ier)+ANS1
               CALL SIMPS(RS33DAT,SL,POLM,N,ANS1)
               L33(j,ier)=L33(j,ier)+ANS1
               CALL SIMPS(RSN2DAT,SL,POLM,N,ANS1)
               N2L(j,ier)=N2L(j,ier)+ANS1
               IER=2
               CALL SIMPS(RSDAT,SL,POLM,N,ANS2)
               TBL(j,ier)=SQRT(TBL(j,ier)**2+ANS2**2)
               CALL SIMPS(RS3GDAT,SL,POLM,N,ANS2)
               L3G(j,ier)=SQRT(L3G(j,ier)**2+ANS2**2)
               CALL SIMPS(RS33DAT,SL,POLM,N,ANS2)
               L33(j,ier)=SQRT(L33(j,ier)**2+ANS2**2)
               CALL SIMPS(RSN2DAT,SL,POLM,N,ANS2)
               N2L(j,ier)=SQRT(N2L(j,ier)**2+ANS2**2)
               IER=3
               CALL SIMPS(RSDAT,SL,POLM,N,ANS3)
               TBL(j,ier)=TBL(j,ier)+XABS(ANS3)
               CALL SIMPS(RS3GDAT,SL,POLM,N,ANS3)
               L3G(j,ier)=L3G(j,ier)+XABS(ANS3)
               CALL SIMPS(RS33DAT,SL,POLM,N,ANS3)
               L33(j,ier)=L33(j,ier)+XABS(ANS3)
               CALL SIMPS(RSN2DAT,SL,POLM,N,ANS3)
               N2L(j,ier)=N2L(j,ier)+XABS(ANS3)
c in addition POLP in integration range
c            --|-------|-----|----|---
c              SL     POLM  POLP SH
               IF (SH.GT.POLP) THEN
                  IER=1
                  CALL SIMPS(RSDAT,POLP,SH,N,ANS1)
                  TBH(j,ier)=TBH(j,ier)+ANS1
                  CALL SIMPS(RS3GDAT,POLP,SH,N,ANS1)
                  H3G(j,ier)=H3G(j,ier)+ANS1
                  CALL SIMPS(RS33DAT,POLP,SH,N,ANS1)
                  H33(j,ier)=H33(j,ier)+ANS1
                  CALL SIMPS(RSN2DAT,POLP,SH,N,ANS1)
                  N2H(j,ier)=N2H(j,ier)+ANS1
                  IER=2
                  CALL SIMPS(RSDAT,POLP,SH,N,ANS2)
                  TBH(j,ier)=SQRT(TBH(j,ier)**2+ANS2**2)
                  CALL SIMPS(RS3GDAT,POLP,SH,N,ANS2)
                  H3G(j,ier)=SQRT(H3G(j,ier)**2+ANS2**2)
                  CALL SIMPS(RS33DAT,POLP,SH,N,ANS2)
                  H33(j,ier)=SQRT(H33(j,ier)**2+ANS2**2)
                  CALL SIMPS(RSN2DAT,POLP,SH,N,ANS2)
                  N2H(j,ier)=SQRT(N2H(j,ier)**2+ANS2**2)
                  IER=3
                  CALL SIMPS(RSDAT,POLP,SH,N,ANS3)
                  TBH(j,ier)=TBH(j,ier)+XABS(ANS3)
                  CALL SIMPS(RS3GDAT,POLP,SH,N,ANS3)
                  H3G(j,ier)=H3G(j,ier)+XABS(ANS3)
                  CALL SIMPS(RS33DAT,POLP,SH,N,ANS3)
                  H33(j,ier)=H33(j,ier)+XABS(ANS3)
                  CALL SIMPS(RSN2DAT,POLP,SH,N,ANS3)
                  N2H(j,ier)=N2H(j,ier)+XABS(ANS3)
               ENDIF
            ELSE IF ((SL.GE.POLM).AND.(SH.GT.POLP)) THEN
c remaining case
c            ---|----|---|-----|---
c              POLM  SL  POLP  SH
               IER=1
               CALL SIMPS(RSDAT,POLP,SH,N,ANS1)
               TBH(j,ier)=TBH(j,ier)+ANS1
               CALL SIMPS(RS3GDAT,POLP,SH,N,ANS1)
               H3G(j,ier)=H3G(j,ier)+ANS1
               CALL SIMPS(RS33DAT,POLP,SH,N,ANS1)
               H33(j,ier)=H33(j,ier)+ANS1
               CALL SIMPS(RSN2DAT,POLP,SH,N,ANS1)
               N2H(j,ier)=N2H(j,ier)+ANS1
               IER=2
               CALL SIMPS(RSDAT,POLP,SH,N,ANS2)
               TBH(j,ier)=SQRT(TBH(j,ier)**2+ANS2**2)
               CALL SIMPS(RS3GDAT,POLP,SH,N,ANS2)
               H3G(j,ier)=SQRT(H3G(j,ier)**2+ANS2**2)
               CALL SIMPS(RS33DAT,POLP,SH,N,ANS2)
               H33(j,ier)=SQRT(H33(j,ier)**2+ANS2**2)
               CALL SIMPS(RSN2DAT,POLP,SH,N,ANS2)
               N2H(j,ier)=SQRT(N2H(j,ier)**2+ANS2**2)
               IER=3
               CALL SIMPS(RSDAT,POLP,SH,N,ANS3)
               TBH(j,ier)=TBH(j,ier)+XABS(ANS3)
               CALL SIMPS(RS3GDAT,POLP,SH,N,ANS3)
               H3G(j,ier)=H3G(j,ier)+XABS(ANS3)
               CALL SIMPS(RS33DAT,POLP,SH,N,ANS3)
               H33(j,ier)=H33(j,ier)+XABS(ANS3)
               CALL SIMPS(RSN2DAT,POLP,SH,N,ANS3)
               N2H(j,ier)=N2H(j,ier)+XABS(ANS3)
            ELSE
            ENDIF
c             write (*,*) ' SIMPS loop',RSN2DAT(SH),N2H(j,1),N2L(j,1)
c     write (*,*) ' trapezoidal summation:',II
c             if ((j.ge.1).and.(j.le.5)) then
c                write (84,*) IE,j,E,TBL(j,1),L3G(j,1),L33(j,1)
c                write (85,*) IE,j,E,TBH(j,1),H3G(j,1),H33(j,1)
c             endif
         ENDDO
 99      CONTINUE
         if (np.le.10) then
             write (14,*) ' '
             write (14,*) ' +++>>',(H3G(j,1), j=2,4)
             write (14,*) ' +++>>',(N2H(j,1), j=2,4)
             write (14,*) ' +++>>',(H3G(j,2), j=2,4)
             write (14,*) ' +++>>',(N2H(j,2), j=2,4)
             write (14,*) ' +++>>',(H3G(j,3), j=2,4)
             write (14,*) ' +++>>',(N2H(j,3), j=2,4)
             write (14,*) ' '
             write (14,*) ' +++>>',(L3G(j,1), j=2,4)
             write (14,*) ' +++>>',(N2L(j,1), j=2,4)
             write (14,*) ' +++>>',(L3G(j,2), j=2,4)
             write (14,*) ' +++>>',(N2L(j,2), j=2,4)
             write (14,*) ' +++>>',(L3G(j,3), j=2,4)
             write (14,*) ' +++>>',(N2L(j,3), j=2,4)
          endif
c

          if (INAG.NE.0) then
          if (lwrite) write (*,*) ' calling nagdrivenGG '
             call nagdrivenGG(m2)
             if (NFL.EQ.2) then
                if (lwrite) write (*,*) ' calling nagdrivenNF2 '
                call nagdrivenNF2(m2)
             endif
             if (ISU2.EQ.2) then
                if (lwrite) write (*,*) ' calling nagdriven3G '
                call nagdriven3G(m2)
             else if (ISU2.EQ.4) then
                if (lwrite) write (*,*) ' calling nagdriven33 '
                call nagdriven33(m2)
             endif
          endif
c          call nagdrivenGG(m2)
c          call nagdrivenNF2(m2)
c          call nagdriven3G(m2)
c          call nagdriven33(m2)
c
c
          tiny=1.0d-10
c gg
          DO j=1,9
             do jer=1,3
                TBL(j,jer)=TBL(j,jer)*FA
                if (jer.gt.1) TBL(j,jer)=XABS(TBL(j,jer))
                TBH(j,jer)=TBH(j,jer)*FA
                if (jer.gt.1) TBH(j,jer)=XABS(TBH(j,jer))
                if (j.eq.jp) then
                   if (jer.eq.2) then
                      TB(j,jer)=sqrt(TBL(j,jer)**2+TBH(j,jer)**2
     &                     +RE(jer)**2)
                   else
                      TB(j,jer)=TBL(j,jer)+TBH(j,jer)+RE(jer)
                      if (jer.eq.1) then
                         write (13,123) EP,TBL(j,jer),TBH(j,jer),RE(jer)
                         MAXMOD1=DMAX1(ABS(TBL(j,jer)),ABS(TBH(j,jer)))
                         Test1=ABS((TBL(j,jer)+TBH(j,jer)))/MAXMOD1
                         if (Test1.lt.tiny) write (14,*) ' EPOL,Test1',
     &                        EP,Test1
                         MAXMOD2=DMAX1(ABS(TBL(j,jer)),ABS(TBH(j,jer)),
     &                        ABS(RE(jer)))
                         Test2=ABS((TBL(j,jer)+TBH(j,jer)+RE(jer)))/
     &                        MAXMOD2
                         if (Test2.lt.tiny) then
                            write (14,*) ' EPOL,Test2',EP,Test2
                         endif
                      endif
                   endif
                else
                   if (jer.eq.2) then
                      TB(j,jer)=sqrt(TBL(j,jer)**2+TBH(j,jer)**2)
                   else
                      TB(j,jer)=TBL(j,jer)+TBH(j,jer)
                   endif
                endif
c                write (84,*) IE,j,E,TBL(j,jer)
             enddo
          ENDDO
c 3g
          DO j=1,9
             do jer=1,3
                L3G(j,jer)=L3G(j,jer)*FA
                if (jer.gt.1) L3G(j,jer)=XABS(L3G(j,jer))
                H3G(j,jer)=H3G(j,jer)*FA
                if (jer.gt.1) H3G(j,jer)=XABS(H3G(j,jer))
                if (j.eq.jp) then
                   if (jer.eq.2) then
                      TG(j,jer)=sqrt(L3G(j,jer)**2+H3G(j,jer)**2
     &                     +RG(jer)**2)
                   else
                      TG(j,jer)=L3G(j,jer)+H3G(j,jer)+RG(jer)
                      if (jer.eq.1) then
                         write (13,123) EP,L3G(j,jer),H3G(j,jer),RG(jer)
                         MAXMOD1=DMAX1(ABS(L3G(j,jer)),ABS(H3G(j,jer)))
                         Test1=ABS((L3G(j,jer)+H3G(j,jer)))/MAXMOD1
                         if (Test1.lt.tiny) write (14,*) ' EPOL,Test1',
     &                        EP,Test1
                         MAXMOD2=DMAX1(ABS(L3G(j,jer)),ABS(H3G(j,jer)),
     &                        ABS(RG(jer)))
                         Test2=ABS((L3G(j,jer)+H3G(j,jer)+RG(jer)))/
     &                        MAXMOD2
                         if (Test2.lt.tiny) then
                            write (14,*) ' EPOL,Test2',EP,Test2
                         endif
                      endif
                   endif
                else
                   if (jer.eq.2) then
                      TG(j,jer)=sqrt(L3G(j,jer)**2+H3G(j,jer)**2)
                   else
                      TG(j,jer)=L3G(j,jer)+H3G(j,jer)
                   endif
                endif
             enddo
          ENDDO
c 33
          DO j=1,9
             do jer=1,3
                L33(j,jer)=L33(j,jer)*FA
                if (jer.gt.1) L33(j,jer)=XABS(L33(j,jer))
                H33(j,jer)=H33(j,jer)*FA
                if (jer.gt.1) H33(j,jer)=XABS(H33(j,jer))
                if (j.eq.jp) then
                   if (jer.eq.2) then
                      T3(j,jer)=sqrt(L33(j,jer)**2+H33(j,jer)**2
     &                     +R3(jer)**2)
                   else
                      T3(j,jer)=L33(j,jer)+H33(j,jer)+R3(jer)
                      if (jer.eq.1) then
                         write (13,123) EP,L33(j,jer),H33(j,jer),R3(jer)
                         MAXMOD1=DMAX1(ABS(L33(j,jer)),ABS(H33(j,jer)))
                         Test1=ABS((L33(j,jer)+H33(j,jer)))/MAXMOD1
                         if (Test1.lt.tiny) write (14,*) ' EPOL,Test1',
     &                        EP,Test1
                         MAXMOD2=DMAX1(ABS(L33(j,jer)),ABS(H33(j,jer)),
     &                        ABS(R3(jer)))
                         Test2=ABS((L33(j,jer)+H33(j,jer)+R3(jer)))/
     &                        MAXMOD2
                         if (Test2.lt.tiny) then
                            write (14,*) ' EPOL,Test2',EP,Test2
                         endif
                      endif
                   endif
                else
                   if (jer.eq.2) then
                      T3(j,jer)=sqrt(L33(j,jer)**2+H33(j,jer)**2)
                   else
                      T3(j,jer)=L33(j,jer)+H33(j,jer)
                   endif
                endif
             enddo
          ENDDO
c N2
          DO j=1,9
             do jer=1,3
                N2L(j,jer)=N2L(j,jer)*FA
                if (jer.gt.1) N2L(j,jer)=XABS(N2L(j,jer))
                N2H(j,jer)=N2H(j,jer)*FA
                if (jer.gt.1) N2H(j,jer)=XABS(N2H(j,jer))
                if (j.eq.jp) then
                   if (jer.eq.2) then
                      T2(j,jer)=sqrt(N2L(j,jer)**2+N2H(j,jer)**2
     &                     +R2(jer)**2)
                   else
                      T2(j,jer)=N2L(j,jer)+N2H(j,jer)+R2(jer)
                      if (jer.eq.1) then
                         write (13,123) EP,N2L(j,jer),N2H(j,jer),R2(jer)
                         MAXMOD1=DMAX1(ABS(N2L(j,jer)),ABS(N2H(j,jer)))
                         Test1=ABS((N2L(j,jer)+N2H(j,jer)))/MAXMOD1
                         if (Test1.lt.tiny) write (14,*) ' EPOL,Test1',
     &                        EP,Test1
                         MAXMOD2=DMAX1(ABS(N2L(j,jer)),ABS(N2H(j,jer)),
     &                        ABS(R2(jer)))
                         Test2=ABS((N2L(j,jer)+N2H(j,jer)+R2(jer)))/
     &                        MAXMOD2
                         if (Test2.lt.tiny) then
                            write (14,*) ' EPOL,Test2',EP,Test2
                         endif
                      endif
                   endif
                else
                   if (jer.eq.2) then
                      T2(j,jer)=sqrt(N2L(j,jer)**2+N2H(j,jer)**2)
                   else
                      T2(j,jer)=N2L(j,jer)+N2H(j,jer)
                   endif
                endif
             enddo
          ENDDO

c test integral over region 0.61 to 3.2
          if (itest.eq.1) then
             RES=0.d0
             REY=0.d0
             REX=0.d0
             do jj=2,5
                RES=RES+TB1(jj)
                REY=REY+TB2(jj)
                REX=REX+TB(jj,1)
             enddo
             write (16,124) e,res/ZT,rey/ZT,rex/ZT
C end test
c test integral over region 0.61 to 3.2
c          RGS=(9.d0/20.d0*TG1(2)+TG1(3)+TG1(4)+0.5d0*TG1(5))/ZT/sin2ell
c          RGY=(9.d0/20.d0*TG2(2)+TG2(3)+TG2(4)+0.5d0*TG2(5))/ZT/sin2ell
c          RGX=(9.d0/20.d0*TG(2,1)+TG(3,1)+TG(4,1)+0.5d0*TG(5,1))
c     &         /ZT/sin2ell
c weight factors now included in integrands
c             if (NFL.EQ.2) then
                R2S=(T21(2)+T21(3)+T21(4)+T21(5))/ZT/sin2ell ! NAG in one step
                R2Y=(T22(2)+T22(3)+T22(4)+T22(5))/ZT/sin2ell ! NAG in ranges
                R2X=(T2(2,1)+T2(3,1)+T2(4,1)+T2(5,1)) ! SIMPSON in ranges
     &               /ZT/sin2ell
                write (57,124) fm/E,R2S,R2Y,R2X
c             else
                RGS=(TG1(2)+TG1(3)+TG1(4)+TG1(5))/ZT/sin2ell ! NAG in one step
                RGY=(TG2(2)+TG2(3)+TG2(4)+TG2(5))/ZT/sin2ell ! NAG in ranges
                RGX=(TG(2,1)+TG(3,1)+TG(4,1)+TG(5,1)) ! SIMPSON in ranges
     &               /ZT/sin2ell
                write (17,124) fm/E,RGS,RGY,RGX
c             endif
          endif
c
          if (itest.eq.1) then
             write (14,'(1x,i4,3x,f8.4)') IE,E
             write (14,'(1x,5(2x,1pe11.4))')
     &            T2(1,1),T2(2,1),T2(3,1),T2(4,1),T2(5,1)
             write (14,'(1x,5(2x,1pe11.4))')
     &            T22(1),T22(2),T22(3),T22(4),T22(5)
          endif
c
          if (INAG.EQ.1) then
             TB(2,1)=TB2(2)
             TB(3,1)=TB2(3)
             TB(4,1)=TB2(4)
             TB(5,1)=TB2(5)
             if (NFL.EQ.2) then
                T2(2,1)=T22(2)
                T2(3,1)=T22(3)
                T2(4,1)=T22(4)
                T2(5,1)=T22(5)
             endif
             if (ISU2.EQ.2) then
                TG(2,1)=TG2(2)
                TG(3,1)=TG2(3)
                TG(4,1)=TG2(4)
                TG(5,1)=TG2(5)
                if (i3gnew.eq.1) then
                   TG(1,1)=TG2(1)
                   TG(6,1)=TG2(6)
                   TG(7,1)=TG2(7)
                   TG(8,1)=TG2(8)
                   TG(9,1)=TG2(9)
                endif
             else if (ISU2.EQ.4) then
                T3(2,1)=T32(2)
                T3(3,1)=T32(3)
                T3(4,1)=T32(4)
                T3(5,1)=T32(5)
             endif
          endif
C end test
          if (itest.eq.1) then
             write (81,124) E,TB(2,1),TB(3,1),TB(2,1)+TB(3,1)
             write (82,124) E,TB(4,1),TB(5,1),TB(4,1)+TB(5,1)
             write (83,124) E,TG(2,1),TG(3,1),TG(2,1)+TG(3,1)
             write (84,124) E,TG(4,1),TG(5,1),TG(4,1)+TG(5,1)
             write (87,*) IE,E
             write (88,*) IE,E
             write(87,'(1x,4(1pe11.4,2x))') (TB(i,1),i=1,6)
             write(88,'(1x,4(1pe11.4,2x))') (TG(i,1),i=1,6)
          endif
          if (lwrite) write (*,*) ' Main: Calculating resonances now'
c using physical widths requires subtraction of full VP
          IRENO_SAV=IRENO
          IREN_SAV=IREN
          IRENO=-1
          IREN=2                ! with physical (dressed) width from PDG
          IRENBW=IREN
          IRENBW=2
          IRENBW=1
          if (IRENBW.EQ.1) then
             write (*,*) ' BW reno space-like'
          endif
          if (IRENBW.EQ.2) then
             write (*,*) ' BW reno time-like'
          endif
c          write (*,*) ' Main: Calculating resonances now'
c RESR(1) is isoscalar part of rho only [iso=0 flag]
c          lwrite=.true.
          if (lwrite) write (*,*) ' calling resonances '
          if (lwrite) write (*,*) ' IKE,E,FA ',IKE,E,FA
          IGEV=0
          call resonances(E,RESR,STAR,SYSR,STA2R,SYS2R)
          IGEV=1
          if (lwrite) write (*,*) ' Main: Calculating resonances done'
c Test -- Add up resonances
          RES=0.d0
          STA=0.d0
          SYS=0.d0
          STA2=0.d0
          SYS2=0.d0
          write (*,*) ' ##### resonances 2,...,5 ##### '
          UMR=1.d0
          IF ((IKE.EQ.0).OR.(IKE.EQ.3).OR.(IKE.EQ.4)) UMR=1.d6
          do j=2,5
             RESR(j)=RESR(j)*UMR
             STAR(j)=STAR(j)*UMR
             SYSR(j)=SYSR(j)*UMR
             RES=RES+RESR(j)
             STA2=STA2+STAR(j)**2
             SYS2=SYS2+SYSR(j)**2
             write (*,*) j,RESR(j)
          enddo
          write (*,*) ' ##### resonances end ##### '
          if (itest.eq.1) then
             write (98,124) E,RES,sqrt(STA2),sqrt(STA2),sqrt(SYS2)
c Test individual Psi and Ypsilon resonances numerically integrated BW versus analytic NW approximation
             IF ((IKE.NE.1).AND.(IKE.NE.2)) THEN
                do ir=1,6
                   write (90+ir,124) E,PSI(ir),PSINWA(ir),
     &                  YPI(ir),YPINWA(ir)
                enddo
             ENDIF
          endif
c subtract I=0 (rho-omega mixing) for ISU2=1 and 3 only done below
c          TG(3,1)=TG(3,1)-RESR(1)
c reset renormalization to none
          IREN =IREN_SAV
          IRENO=IRENO_SAV
c the routine resonances changes M2 in common FPI (to units MEV^2)
c therefore reset M2
          M2=S
          M2_1=M2
          IER=1
c          write (*,*) ' resonances:'
c          write (*,'(1x,6(2x,1pe11.3))') resr
c          write (*,'(1x,6(2x,1pe11.3))') star
c          write (*,'(1x,6(2x,1pe11.3))') sysr
c calculate perturbative tail
c###    Parameters for perturbative QCD       ###'
          ICHK_SAV=ICHK
c          Standard INPUT: Enter 1=yes,0=no '
c          EST=0.002D0                                     '
c          EST=0.002D0                                     '
c          ESY=0.000D0                                     '
c          MZINP=91.1867D0       !  PDG 2004                   '
c          MTOP=172.5D0          !  PDG 2006                   '
c          NF=5                  ! (no top), 6(with top)       '
c          ICHK=0,1,2            !pQCD: 0(m=0), 1 (95), 2 (00) '
C if there is a pQCD CUT below Upsilon resonances
c BEG: NEW -- NEW -- NEW -- NEW -- NEW -- NEW -- NEW -- NEW
          if (inf2on.eq.1) then
c NF=2 perturbative tail
c             inotail=1
c             inotail=0
             ICHK_SAV=ICHK
             NFlog=.true.
             if (inotail.eq.1) then
                T2tail1=null
                T2tail2=null
             else
                NFfixed=2
c                ICHKfix=4    ! 5= 3G channel <3g> rqcdHS3x
c                ICHKfix=5    ! 5= 33 channel <33> rqcdHS3x
c                ICHKfix=6    ! 5= gg channel <gg> rqcdHS3x
                ICHKfix=2    ! <gg> rqcdHSn
                RN2tailp=null
                RN2tailm=null
                if (lwrite) write (*,*) ' calling vapotail '
                do itail=5,10
                   call vapotail(EL(itail),EH(itail),RP,ALS+EST,MZINP,
     &                  MTOP,2,ior,NFfixed,ICHKfix)
                   call vapotail(EL(itail),EH(itail),RM,ALS-EST,MZINP,
     &                  MTOP,2,ior,NFfixed,ICHKfix)
                   RN2tailp=RN2tailp+RP
                   RN2tailm=RN2tailm+RM
                enddo
                RP=RN2tailp
                RM=RN2tailm
c Overwrite data result for this region by pQCD result
c normalize to <3g>= 9/20 <gg>
                S1t=EL(5)**2
                S2t=EH(10)**2
                t=1.d0/sqrt(M2)
c                RpQCDtest=5.D0/3.D0*0.45d0*FA*ECIfun(t,EL(5),EH(10),loss)
c 29/06/2013 new normalization to I_1=4<n2>=4<33>=9/10<gg>
                if (IKE.EQ.8) then
                   RpQCDtest=5.D0/3.D0*0.9d0*FA*ECIfun(t,2.d0,2.d3,loss)
                endif
                if (IKE.EQ.9) then
                   RpQCDtest=5.D0/3.D0*0.9d0*FA*ECJfun(t,2.d0,2.d3,loss)
                endif
                if (ICHKfix.eq.5) then                ! R =R33  from RSt in vapotail
                   T2tail1=(RP+RM)/2.d0*FA*2.d0
                   T2tail2=ABS((RP-RM)/2.d0*FA)*2.d0
                else if (ICHKfix.eq.4) then           ! R =R3G  from RSt in vapotail
                   T2tail1=(RP+RM)/2.d0*FA*2.d0
                   T2tail2=ABS((RP-RM)/2.d0*FA)*2.d0
                else
                   T2tail1=(RP+RM)/2.d0*FA*0.9d0      ! R =RGG  from RSt in vapotail
                   T2tail2=ABS((RP-RM)/2.d0*FA)*0.9d0
                endif
                write (101,124) sqrt(M2),T2(5,1),T2tail1,RpQCDtest,loss
                if (inotail.eq.-1) T2tail1=RpQCDtest
c          write (*,*) ' Warning: intRdatx NF2 tail LO only'
             endif
             T2(5,1)=T2tail1
             T2(5,2)=null
             T2(5,3)=T2tail2
             NFlog=.false.
             ICHK=ICHK_SAV
          endif
c END: NEW -- NEW -- NEW -- NEW -- NEW -- NEW -- NEW -- NEW
          if (lwrite) write (*,*) ' Main: Calculating perturbative part'
          if (EL(8).LT.EH(8)) then
             if (lwrite) write (*,*) ' Main: EC to M(Upsilon)'
             call vapotail(EL(8),EH(8),RP,ALS+EST,MZINP,
     &            MTOP,2,ior,NF,ICHK)
             call vapotail(EL(8),EH(8),RM,ALS-EST,MZINP,
     &            MTOP,2,ior,NF,ICHK)
c Overwrite data result for this region by pQCD result
             TB(8,1)=(RP+RM)/2.d0*FA
             TB(8,2)=0.d0
             TB(8,3)=ABS((RP-RM)/2.d0*FA)
             if (i3gnew.eq.1) then
                ICHK=4
                call vapotail(EL(8),EH(8),RP,ALS+EST,MZINP,
     &               MTOP,2,ior,NF,ICHK)
                call vapotail(EL(8),EH(8),RM,ALS-EST,MZINP,
     &               MTOP,2,ior,NF,ICHK)
c Overwrite data result for this region by pQCD result
                TG(8,1)=(RP+RM)/2.d0*FA
                TG(8,2)=0.d0
                TG(8,3)=ABS((RP-RM)/2.d0*FA)
                ICHK=ICHK_SAV
             endif
             E1=EC
             E2=EINFTY            !(> 1.D3=INFINITY)
             S1=E1**2
             S2=(2.D0*MTOP)**2
             RpQCDtest=11.D0/3.D0*(LOG(ABS((S2-M2)/(S1-M2)))-LOG(S2/S1))
     .            +      5.d0*(LOG(ABS(S2/(S2-M2))))
          endif
c          write (*,*) ' EL(9),E2,ALS+EST,MZINP,MTOP,2,ior,NF,ICHK'
c          write (*,*) EL(9),E2,ALS+EST,MZINP,MTOP,2,ior,NF,ICHK
          if (lwrite) write (*,*) ' Main: high energy tail'
         call vapotail(EL(10),E2,RESU1,ALS+EST,MZINP,MTOP,2,ior,NF,ICHK)
         call vapotail(EL(10),E2,RESU2,ALS-EST,MZINP,MTOP,2,ior,NF,ICHK)
          TB(10,1)=(RESU1+RESU2)/2.d0*FA
          TB(10,2)=0.d0
          TB(10,3)=ABS((RESU1-RESU2)/2.d0*FA)
          if (i3gnew.eq.1) then
             ICHK=4
         call vapotail(EL(10),E2,RESU1,ALS+EST,MZINP,MTOP,2,ior,NF,ICHK)
         call vapotail(EL(10),E2,RESU2,ALS-EST,MZINP,MTOP,2,ior,NF,ICHK)
             TG(10,1)=(RESU1+RESU2)/2.d0*FA
             TG(10,2)=0.d0
             TG(10,3)=ABS((RESU1-RESU2)/2.d0*FA)
             ICHK=ICHK_SAV
          endif
          ICHK=ICHK_SAV
          if (lwrite) write (*,*)
     &         ' Main: Calculating perturbative part done'
c          WRITE(*,*) ' pQCD tail: LO vs HS',E1,sqrt(M2),
c     &         RpQCDtest*FA,TB(9,1)
c
c Compare results from vapo1n program:
c Perturbative ranges/tails
c   5.2  -  9.46 GeV:    33.511158315794333       3.06088995023281150E-002
c  13.0  - 40.00 GeV:    72.941622555664665       3.84194686870500471E-002
c
c 3g resonance wights
          w3g(2)=0.d0
          w3g(3)=0.75d0
          w3g(4)=0.375d0
          w3g(5)=0.75d0
c
          RES=0.d0
          STA=0.d0
          SYS=0.d0
          STA2=0.d0
          SYS2=0.d0
          RESN2=0.d0
          STAN2=0.d0
          SYSN2=0.d0
          STA2N2=0.d0
          SYS2N2=0.d0
          RES3Gn=0.d0
          STA3Gn=0.d0
          SYS3Gn=0.d0
          STA23Gn=0.d0
          SYS23Gn=0.d0

          do jj=1,10
             RES=RES+TB(jj,1)
             STA2=STA2+TB(jj,2)**2
             SYS2=SYS2+TB(jj,3)**2
c             write (*,*) ' RES(jj):',jj,RES
             RES3Gn=RES3Gn+TG(jj,1)
             STA23Gn=STA23Gn+TG(jj,2)**2
             SYS23Gn=SYS23Gn+TG(jj,3)**2
          enddo
          do jj=1,5
             RESN2=RESN2+T2(jj,1)
             STA2N2=STA2N2+T2(jj,2)**2
             SYS2N2=SYS2N2+T2(jj,3)**2
          enddo
          write (14,'(1x,f12.6,1(2x,1pe11.4))') E,RESN2
c Add up resonances
          do j=2,5
             RES=RES+RESR(j)
             STA2=STA2+STAR(j)**2
             SYS2=SYS2+SYSR(j)**2
             RES3Gn=RES3Gn+w3g(j)*RESR(j)
             STA23Gn=STA23Gn+(w3g(j)*STAR(j))**2
             SYS23Gn=SYS23Gn+(w3g(j)*SYSR(j))**2
          enddo
          STA=SQRT(STA2)
          SYS=SQRT(SYS2)
          ETOT=SQRT(STA2+SYS2)
          STAN2=SQRT(STA2N2)
          SYSN2=SQRT(SYS2N2)
          ETOTN2=SQRT(STA2N2+SYS2N2)
          STA3Gn=SQRT(STA23Gn)
          SYS3Gn=SQRT(SYS23Gn)
          ETOT3Gn=SQRT(STA23Gn+SYS23Gn)
C print to file
          tex4x(1)=' Chpt     '
          tex4x(2)=tex4(1)
          tex4x(3)=' data/QCD '
          ELR(1)=0.32d0
          EHR(1)=0.81d0
          ELR(2)=0.42d0
          EHR(2)=0.81d0
          ELR(3)=1.00d0
          EHR(3)=1.04d0
          ELR(4)=3.1d0
          EHR(4)=4.5d0
          ELR(5)=9.4d0
          EHR(5)=11.1d0
          do j=1,14
             TB(j,4)=sqrt(TB(j,2)**2+TB(j,3)**2)
             TB(j,5)=abs(TB(j,4)/TB(j,1))*100.
             TG(j,4)=sqrt(TG(j,2)**2+TG(j,3)**2)
             TG(j,5)=abs(TG(j,4)/TG(j,1))*100.
          enddo
          do j=1,5
             T2(j,4)=sqrt(T2(j,2)**2+T2(j,3)**2)
             T2(j,5)=abs(T2(j,4)/T2(j,1))*100.
          enddo
c recalculate sums and errors
          SUM1=0.d0
          STA1=0.d0
          SYS1=0.d0
          SU3G=0.d0
          ST3G=0.d0
          SY3G=0.d0
          do j=1,7
             write (40,*) TB(j,1)
             SUM1=SUM1+TB(j,1)
             STA1=sqrt(STA1**2+TB(j,2)**2)
             SYS1=sqrt(SYS1**2+TB(j,3)**2)
             SU3G=SU3G+TG(j,1)
             ST3G=sqrt(ST3G**2+TG(j,2)**2)
             SY3G=sqrt(SY3G**2+TG(j,3)**2)
          enddo
c resonances except rho
          do j=2,5
             SUM1=SUM1+RESR(j)
             STA1=sqrt(STA1**2+STAR(j)**2)
             SYS1=sqrt(SYS1**2+SYSR(j)**2)
             SU3G=SU3G+w3g(j)*RESR(j)
             ST3G=sqrt(ST3G**2+(w3g(j)*STAR(j))**2)
             SY3G=sqrt(SY3G**2+(w3g(j)*SYSR(j))**2)
          enddo
c add had 9.5 -13.0
          SUM1=SUM1+TB(9,1)
          STA1=sqrt(STA1**2+TB(9,2)**2)
          SYS1=sqrt(SYS1**2+TB(9,3)**2)
          SU3G=SU3G+TG(9,1)
          ST3G=sqrt(ST3G**2+TG(9,2)**2)
          SY3G=sqrt(SY3G**2+TG(9,3)**2)
c all data
          TB(11,1)=SUM1
          TB(11,2)=STA1
          TB(11,3)=SYS1
          TB(11,4)=sqrt(STA1**2+SYS1**2)
          TB(11,5)=abs(TB(11,4)/TB(11,1))*100.
          TG(11,1)=SU3G
          TG(11,2)=ST3G
          TG(11,3)=SY3G
          TG(11,4)=sqrt(ST3G**2+SY3G**2)
          TG(11,5)=abs(TG(11,4)/TG(11,1))*100.
c add the two pQCD terms
          SUper=TB(8,1)+TB(10,1)
          STper=sqrt(TB(8,2)**2+TB(10,2)**2)
          SYper=sqrt(TB(8,3)**2+TB(10,3)**2)
          S3GUper=TG(8,1)+TG(10,1)
          S3GTper=sqrt(TG(8,2)**2+TG(10,2)**2)
          S3GYper=sqrt(TG(8,3)**2+TG(10,3)**2)
c all together data+pQCD
          TB(12,1)=SUM1+SUper
          TB(12,2)=sqrt(STA1**2+STper**2)
          TB(12,3)=sqrt(SYS1**2+SYper**2)
          TB(12,4)=sqrt(TB(12,2)**2+TB(12,3)**2)
          TB(12,5)=abs(TB(12,4)/TB(12,1))*100.
          TG(12,1)=SU3G+S3GUper
          TG(12,2)=sqrt(ST3G**2+S3GTper**2)
          TG(12,3)=sqrt(SY3G**2+S3GYper**2)
          TG(12,4)=sqrt(TG(12,2)**2+TG(12,3)**2)
          TG(12,5)=abs(TG(12,4)/TG(12,1))*100.
c
c          if (NFL.EQ.2) then
c             der   =RESN2
c             stader=STAN2
c             sysder=SYSN2
c             errder=ETOTN2
c          else
             der   =RES
             stader=STA
             sysder=SYS
             errder=ETOT
c          endif

c          deg   =RES3Gn
c          stader=STA3Gn
c          sysder=S3Gn
c          errder=ETOT3Gn

c Overview ranges and tables [TB(i,k),... arrays]
c
c 2m_pi ... CHPTCUT ... 0.61 .... 1.4 ... 2.0 ... 3.2 ... 3.6 ... 5.2 ... 9.5 ... 13. ... infty
c   |----1-----|----2----|----3----|---4---|---5---|---6---|---7---|---8---|---9---|---10---
c   <-----------ud------------->|<-------+s------>|<-----+c-------------->|<----+b------...
c                        |--- Flavor sep ---|
c
c          IF ((ISCAN.NE.0).AND.(ISU2.EQ.0)) THEN
c          ELSE     ! calculation of SU2 coupling
c ##################################################
c flavor separation continuum in SU(3) limit u=d=s
c gamma gamma: uds + c + b
c 3 gamma    : 1/2 uds + 3/8 c + 3/4 b
c 01/10/2013 new weight for ud component 9/20 --> 10/20
c         SU2: 10/20 ud + 3/4 s up tp 2 GeV
          cud_3g=10.d0/20.d0
          cuds  =1.d0/2.d0
          cc    =3.d0/8.d0
          cb    =3.d0/4.d0
          crho  =cuds
          comega=0.d0
          cphi  =cb
          cpsi  =cc
          cups  =cb
c save all gamma 3 coeffs as the c's are reset dependng on external flavor specification
          xud_3g=cud_3g
          xuds  =cuds
          xc    =cc
          xb    =cb
          xrho  =crho
          xomega=comega
          xphi  =cphi
          xpsi  =cpsi
          xups  =cups
c new FJ 13/08/2011
c 3 3    : 9/32 uds + 9/64 c + 9/16 b
c 01/10/2013 new weight for ud component 9/40 --> 10/40
c     SU2: 10/40 ud  + 9/16 s up to 2 GeV
          cud_33   =10.d0/40.d0
          cuds_33  =9.d0/32.d0
          cc_33    =9.d0/64.d0
          cb_33    =9.d0/16.d0
          crho_33  =1.d0/4.d0
          comega_33=0.d0
          cphi_33  =cb_33
          cpsi_33  =cc_33
          cups_33  =cb_33
c test
          if (itestflavorsplitting.eq.1) then
             cud_3g=1.0d0
             cuds  =1.0d0
             cc    =1.0d0
             cb    =1.0d0
             crho  =1.0d0
             comega=1.0d0
             cphi  =1.0d0
             cpsi  =1.0d0
             cups  =1.0d0
             cud_33   =1.0d0
             cuds_33  =1.0d0
             cc_33    =1.0d0
             cb_33    =1.0d0
             crho_33  =1.0d0
             comega_33=1.0d0
             cphi_33  =1.0d0
             cpsi_33  =1.0d0
             cups_33  =1.0d0
             sin2ell=1.d0
             do j=2,6
                do jj=1,3
                   TG(j,jj)=TB(j,jj)
                   T3(j,jj)=TB(j,jj)
                enddo
             enddo
          endif
          IF (NFL.LE.4) THEN
c 4 flavor Correlators
             cb       =null
             cb_33    =null
             cups     =null
             cups_33  =null
          ENDIF
          IF (NFL.LE.3) THEN
c 3 flavor Correlators
             cc       =null
             cc_33    =null
             cpsi     =null
             cpsi_33  =null
          ENDIF
c with present standard energy range splitting [Ioldsubdiv=0] rho is in TB(3,1)
c in new ([us] and [s] separate) flavor splitting isoscalar part subtracted in data table
c arrays TG and T3
c for old SU(3) splitting [with new separate SU(2) splitting I=0 component subtracted from data]
          IF ((ISU2.EQ.1).OR.(ISU2.EQ.3)) THEN
             write (4,*) ' NEW NEW NEW -- subtract I=0 from rho'
             write (4,*)  ' Energy:',sqrt(M2)
             write (4,1111) ' rhoI=2,0',TB(3,1),RESR(1)
             TB(3,1)=TB(3,1)-RESR(1)
             write (4,1111) ' rho I=1 ',TB(3,1)
             IF (ISCAN.EQ.0) write (4,*) ' background '
          ENDIF
c uds
          resuds=TB(1,1)+TB(2,1)
     &          +TB(3,1)+TB(4,1)+TB(5,1)+TB(6,1)
          resuds_n=TG(1,1)+TG(2,1)
     &          +TG(3,1)+TG(4,1)+TG(5,1)+TG(6,1)
          stauds=sqrt(TB(1,2)**2+TB(2,2)**2
     &          +TB(3,2)**2+TB(4,2)**2+TB(5,2)**2+TB(6,2)**2)
          sysuds=sqrt(TB(1,3)**2+TB(2,3)**2
     &          +TB(3,3)**2+TB(4,3)**2+TB(5,3)**2+TB(6,3)**2)
          eto3=sqrt(stauds**2+sysuds**2)
          write (*,*) ' main: uds', resuds/2.d0,resuds_n
c uds without region 0.61 to 2.0 i.e. Tab elements 2 + 3 & 4 + 5
c ud up to 0.61
          resud_rem=TB(1,1)
          staud_rem=sqrt(TB(1,2)**2)
          sysud_rem=sqrt(TB(1,3)**2)
          eto3u_rem=sqrt(staud_rem**2+sysud_rem**2)
c uds above 2.0
          resuds_rem=TB(6,1)
          stauds_rem=sqrt(TB(6,2)**2)
          sysuds_rem=sqrt(TB(6,3)**2)
          eto3s_rem=sqrt(stauds_rem**2+sysuds_rem**2)
 1111     format (1X,A9,4(1x,1pe11.3))
          IF (ISCAN.EQ.0)
     &         write (4,1111) ' resuds  ',resuds,stauds,sysuds,eto3
c udsc
          resudsc=TB(7,1)+TB(8,1)
          staudsc=sqrt(TB(7,2)**2+TB(8,2)**2)
          sysudsc=sqrt(TB(7,3)**2+TB(8,3)**2)
          eto4=sqrt(staudsc**2+sysudsc**2)
          IF (ISCAN.EQ.0)
     &         write (4,1111) ' resudsc ',resudsc,staudsc,sysudsc,eto4
c udscb
          resudscb=TB(9,1)
          staudscb=TB(9,2)
          sysudscb=TB(9,3)
          eto5=sqrt(staudscb**2+sysudscb**2)
          IF (ISCAN.EQ.0)
     &        write (4,1111) ' resudscb',resudscb,staudscb,sysudscb,eto5
          IF (ISCAN.EQ.0) write (4,*) ' pQCD '
cc uds pQCD from 2.0 GeV=EL(5) to 3.6 GeV=EH(6)
c          call vapotail(EL(5),EH(6),pRESUDSP56,ALS+EST,
c     &         MZINP,MTOP,2,ior,3,ICHK)
c          call vapotail(EL(7),EH(8),pRESUDSM56,ALS+EST,
c     &         MZINP,MTOP,2,ior,3,ICHK)
cc ud  pQCD from 2.0 GeV=EL(5) to 3.6 GeV=EH(6)
c          call vapotail(EL(5),EH(6),pRESUDP56,ALS+EST,
c     &         MZINP,MTOP,2,ior,2,ICHK)
c          call vapotail(EL(7),EH(8),pRESUDM56,ALS+EST,
c     &         MZINP,MTOP,2,ior,2,ICHK)
c uds pQCD from c threshold to b threshold
          call vapotail(EL(7),EH(8),pRESUDSP,ALS+EST,
     &         MZINP,MTOP,2,ior,3,ICHK)
          call vapotail(EL(7),EH(8),pRESUDSM,ALS+EST,
     &         MZINP,MTOP,2,ior,3,ICHK)
c udsc pQCD from b threshold to ECUT
          call vapotail(EL(9),EH(9),pRESUDSCP,ALS+EST,
     &         MZINP,MTOP,2,ior,4,ICHK)
          call vapotail(EL(9),EH(9),pRESUDSCM,ALS-EST,
     &         MZINP,MTOP,2,ior,4,ICHK)
          call vapotail(EL(9),EH(9),pRESUDS1P,ALS+EST,
     &         MZINP,MTOP,2,ior,3,ICHK)
          call vapotail(EL(9),EH(9),pRESUDS1M,ALS-EST,
     &         MZINP,MTOP,2,ior,3,ICHK)
c udscb pQCD above ECUT
          call vapotail(EL(10),EH(10),pR3p,ALS+EST,
     &         MZINP,MTOP,2,ior,3,ICHK)
          call vapotail(EL(10),EH(10),pR3m,ALS-EST,
     &         MZINP,MTOP,2,ior,3,ICHK)
          call vapotail(EL(10),EH(10),pR4p,ALS+EST,
     &         MZINP,MTOP,2,ior,4,ICHK)
          call vapotail(EL(10),EH(10),pR4m,ALS-EST,
     &         MZINP,MTOP,2,ior,4,ICHK)
          call vapotail(EL(10),EH(10),pR5p,ALS+EST,
     &         MZINP,MTOP,2,ior,5,ICHK)
          call vapotail(EL(10),EH(10),pR5m,ALS-EST,
     &         MZINP,MTOP,2,ior,5,ICHK)
c error as sys
c new split uds below quark threshold
c          pRESUDS56=(pRESUDSP56+pRESUDSM56)/2.d0*FA
c          pSYSUDS56=(pRESUDSP56-pRESUDSM56)/2.d0*FA
c          pRESUD56 =(pRESUDP56 +pRESUDM56 )/2.d0*FA
c          pSYSUD56 =(pRESUDP56 -pRESUDM56 )/2.d0*FA
c
          pRESUDS=(pRESUDSP+pRESUDSM)/2.d0*FA
          pSYSUDS=(pRESUDSP-pRESUDSM)/2.d0*FA
          pRESUDS1=(pRESUDS1P+pRESUDS1M)/2.d0*FA
          pSYSUDS1=(pRESUDS1P-pRESUDS1M)/2.d0*FA
          pRESUDSC=(pRESUDSCP+pRESUDSCM)/2.d0*FA
          pSYSUDSC=(pRESUDSCP-pRESUDSCM)/2.d0*FA
          IF (ISCAN.EQ.0) THEN
             write (4,1111) ' pRESUDS ',pRESUDS,null,pSYSUDS
             write (4,1111) ' pRESUDSC',pRESUDSC,null,pSYSUDSC
             write (4,*) ' background subtracted'
          ENDIF
c background c
          resc=resudsc-pRESUDS
          stac=staudsc
          sysc=sqrt(abs(sysudsc**2-pSYSUDS**2))
          etoc=sqrt(stac**2+sysc**2)
          IF (ISCAN.EQ.0)
     &         write (4,1111) ' resc    ',resc,stac,sysc,etoc
c background b
          resb=resudscb-pRESUDSC
          stab=staudscb
          sysb=sqrt(abs(sysudscb**2-pSYSUDSC**2))
          etob=sqrt(stab**2+sysb**2)
          IF (ISCAN.EQ.0)
     &         write (4,1111) ' resb    ',resb,stab,sysb,etob
c perturbative tail
          IF (ISCAN.EQ.0) write (4,*) ' pQCD tail '
c uds
          pr3=(pR3p+pR3m)/2.d0*FA
          pe3=(pR3p-pR3m)/2.d0*FA
c udsc
          pr4=(pR4p+pR4m)/2.d0*FA
          pe4=(pR4p-pR4m)/2.d0*FA
c udscb
          pr5=(pR5p+pR5m)/2.d0*FA
          pe5=(pR5p-pR5m)/2.d0*FA
c c
          prc=pr4-pr3
          pec=pe4-pe3
c b
          prb=pr5-pr4
          peb=pe5-pe4

c summary pQCD tails
c                    c           b            Ecut
c         -----------|-----------|------------|-------->
c  uds:    resuds(BG)    pRESUDS     pRESUDS1     pr3
c                    c           b            Ecut
c         -----------|-----------|------------|-------->
c  udsc:                resc(BG)     pRESUDSC     pr4
c                    c           b            Ecut
c         -----------|-----------|------------|-------->
c  udscb:                            resb(BG)     pr5
c
c  NF=3: pRESUDS+pRESUDS1+pr3
c  NF=4: NF=3 + (pRESUDSC-pRESUDS1)+prc[=pr4-pr3]
c  NF=5: NF=4 + prb[=pr5-pr4]
c
          IF (ISCAN.EQ.0) write (4,1111) ' pr3     ',pr3,null,pe3
          IF (ISCAN.EQ.0) write (4,1111) ' prc,prb ',prc,prb
c gamma gamma flavor recombination test
c perturbative error here 100 % correlated
          repQCD_tes=pr3+prc+prb  ! tail
     &        +(pRESUDS+pRESUDS1)+(pRESUDSC-pRESUDS1)
          erpQCD_tes=(pe3+pec+peb  ! tail
     &        +(pSYSUDS+pSYSUDS1)+(pSYSUDSC-pSYSUDS1))**2
c 3 gamma combination
c perturbative error here 100 % correlated
          repQCD=cuds*pr3+cc*prc+cb*prb  ! tail
     &        +cuds*(pRESUDS+pRESUDS1)+cc*(pRESUDSC-pRESUDS1)
          erpQCD=(cuds*pe3+cc*pec+cb*peb  ! tail
     &        +cuds*(pSYSUDS+pSYSUDS1)+cc*(pSYSUDSC-pSYSUDS1))**2
c 3 3 combination
c perturbative error here 100 % correlated
          repQCD_33=cuds_33*pr3+cc_33*prc+cb_33*prb  ! tail
     &        +cuds_33*(pRESUDS+pRESUDS1)+cc_33*(pRESUDSC-pRESUDS1)
          erpQCD_33=(cuds_33*pe3+cc_33*pec+cb_33*peb  ! tail
     &        +cuds_33*(pSYSUDS+pSYSUDS1)+cc_33*(pSYSUDSC-pSYSUDS1))**2
          IF (ISCAN.EQ.0) THEN
             write (4,1111) ' repQCD  ',repQCD,null,sqrt(erpQCD)
             write (4,1111) ' same gg ',pr5,null,pe5
             write (4,1111) ' repQCD33',repQCD_33,null,sqrt(erpQCD_33)
          ENDIF
c flavor contributions resonances
c gamma gamma: rho + omega + phi + psi + ups
c 3 gamma    : 1/2 rho + 3/4 phi + 3/8 psi + 3/4 ups
c rho may be included in background (same weight factor 1/2); omega not contributing to 3gamma
c
          rrrho=null
          strho=null
          syrho=null
          rromega=RESR(2)
          stomega=STAR(2)
          syomega=SYSR(2)
          reomega=stomega**2+syomega**2
          rrphi=RESR(3)
          stphi=STAR(3)
          syphi=SYSR(3)
          rephi=stphi**2+syphi**2
          rrpsi=RESR(4)
          stpsi=STAR(4)
          sypsi=SYSR(4)
          repsi=stpsi**2+sypsi**2
          rrups=RESR(5)
          stups=STAR(5)
          syups=SYSR(5)
          reups=stups**2+syups**2
c "background" 3 gamma
          reback=cuds*resuds+cc*resc+cb*resb
          stback=(cuds*stauds)**2+(cc*stac)**2+(cb*stab)**2
          syback=(cuds*sysuds)**2+(cc*sysc)**2+(cb*sysb)**2
          erback=stback+syback
c
          reback_new=cud_3g*resud_rem+cuds*resuds_rem
     &         +cc*resc+cb*resb+TG(2,1)+TG(3,1)+TG(4,1)+TG(5,1)
          stback_new=(cud_3g*staud_rem)**2+(cuds*stauds_rem)**2
     &         +(cc*stac)**2+(cb*stab)**2+TG(2,2)**2+TG(3,2)**2
     &         +TG(4,2)**2+TG(5,2)**2
          syback_new=(cud_3g*sysud_rem)**2+(cuds*sysuds_rem)**2
     &         +(cc*sysc)**2+(cb*sysb)**2+TG(2,3)**2+TG(3,3)**2
     &         +TG(4,3)**2+TG(5,3)**2
          erback_new=stback_new+syback_new
c "background" 3 3
          resuds_33=resuds
          stauds_33=sqrt(stauds**2)
          sysuds_33=sqrt(sysuds**2)
          reback_33=cuds_33*resuds_33+cc_33*resc+cb_33*resb
          stback_33=(cuds_33*stauds_33)**2+(cc_33*stac)**2
     &         +(cb_33*stab)**2
          syback_33=(cuds_33*sysuds_33)**2+(cc_33*sysc)**2
     &         +(cb_33*sysb)**2
          erback_33=stback_33+syback_33
c
          resuds_33new=resuds_rem
          stauds_33new=sqrt(stauds_rem**2)
          sysuds_33new=sqrt(sysuds_rem**2)
          reback_33new=cud_33*resud_rem+cuds_33*resuds_rem
     &         +cc_33*resc+cb_33*resb
     &         +T3(2,1)+T3(3,1)+T3(4,1)+T3(5,1)
          stback_33new=(cud_33*staud_rem)**2+(cuds_33*stauds_rem)**2
     &         +(cc_33*stac)**2+(cb_33*stab)**2
     &         +T3(2,2)**2+T3(3,2)**2+T3(4,2)**2+T3(5,2)**2
          syback_33new=(cud_33*sysud_rem)**2+(cuds_33*sysuds_rem)**2
     &         +(cc_33*sysc)**2+(cb_33*sysb)**2
     &         +T3(2,3)**2+T3(3,3)**2+T3(4,3)**2+T3(5,3)**2
          erback_33new=stback_33new+syback_33new
          IF (ISCAN.EQ.0) THEN
             write (4,1111) ' reback  ',reback,null,sqrt(erback)
             write (4,1111) ' [new]   ',reback_new,null,sqrt(erback_new)
             write (4,1111) ' reback33',reback_33,null,sqrt(erback_33)
             write (4,1111) ' [new]   ',reback_33new,null,
     &            sqrt(erback_33new)
          ENDIF
c resonances phi, psi, ups gamma gamma flavor recombination test
          rereso_tes=rrphi+rrpsi+rrups+rromega
          streso_tes=(stphi)**2+(stpsi)**2+
     &         (stups)**2+(stomega)**2
          syreso_tes=(syphi)**2+(sypsi)**2+
     &         (syups)**2+(syomega)**2
          erreso_tes=streso_tes+syreso_tes
c resonances phi, psi, ups 3 gamma
          rereso=cphi*rrphi+cpsi*rrpsi+cups*rrups+comega*rromega
          streso=(cphi*stphi)**2+(cpsi*stpsi)**2+
     &         (cups*stups)**2+(comega*stomega)**2
          syreso=(cphi*syphi)**2+(cpsi*sypsi)**2+
     &         (cups*syups)**2+(comega*syomega)**2
          erreso=streso+syreso
c resonances phi, psi, ups 3 3
c first without rho
          rereso_33new=cphi_33*rrphi+cpsi_33*rrpsi+cups_33*rrups
     &         +comega_33*rromega
          streso_33new=(cphi_33*stphi)**2+(cpsi_33*stpsi)**2+
     &         (cups_33*stups)**2+(comega_33*stomega)**2
          syreso_33new=(cphi_33*syphi)**2+(cpsi_33*sypsi)**2+
     &         (cups_33*syups)**2+(comega_33*syomega)**2
          erreso_33new=streso_33+syreso_33
c add properly weighted rho  [old case SU(3) flavor u=d=s assumed in flaver separation]
c in current version rrrho=0 rho taken together with its background
          rereso_33=rereso_33new+crho_33*rrrho
          streso_33=streso_33new+(crho_33*strho)**2
          syreso_33=syreso_33new+(crho_33*syrho)**2
          erreso_33=streso_33+syreso_33
          IF (ISCAN.EQ.0) THEN
             write (4,1111) ' rereso  ',rereso,null,sqrt(erreso)
             write (4,1111) ' rereso33',rereso_33,null,sqrt(erreso_33)
          ENDIF
c test gamma gamma
          deg_tes   =rereso_tes+reback_tes+repQCD_tes
          stadeg_tes=sqrt(streso_tes+stback_tes)
          sysdeg_tes=sqrt(syreso_tes+syback_tes+erpQCD_tes)
          errdeg_tes=sqrt(erreso_tes+erback_tes+erpQCD_tes)
c
          deg   =rereso+reback+repQCD
          stadeg=sqrt(streso+stback)
          sysdeg=sqrt(syreso+syback+erpQCD)
          errdeg=sqrt(erreso+erback+erpQCD)
c
          deg_33   =rereso_33+reback_33+repQCD_33
          stadeg_33=sqrt(streso_33+stback_33)
          sysdeg_33=sqrt(syreso_33+syback_33+erpQCD_33)
          errdeg_33=sqrt(erreso_33+erback_33+erpQCD_33)
c
          deg_new   =rereso+reback_new+repQCD
          stadeg_new=sqrt(streso+stback_new)
          sysdeg_new=sqrt(syreso+syback_new+erpQCD)
          errdeg_new=sqrt(erreso+erback_new+erpQCD)
c
          deg_33new   =rereso_33new+reback_33new+repQCD_33
          stadeg_33new=sqrt(streso_33new+stback_33new)
          sysdeg_33new=sqrt(syreso_33new+syback_33new+erpQCD_33)
          errdeg_33new=sqrt(erreso_33new+erback_33new+erpQCD_33)
c tests
          retes=resuds+resc+resb+rrphi+rrpsi+rrups+pr3+prc+prb+
     &         pRESUDS+pRESUDSC+rromega+RESR(1)
c          rete1=resuds+resudsc+resudscb+rrphi+rrpsi+rrups+pr3+prc+prb+
c     &         rromega
c          rete2=resuds+resudsc+resudscb+rrphi+rrpsi+rrups+pr5+rromega
c error
          ertes=eto3**2+etoc**2+etob**2+
     &         rephi+repsi+reups+reomega+
     &         pe3**2+pec**2+peb**2
     &         +pSYSUDS**2+pSYSUDSC**2
c reproduces correct error
c          ertes=stauds**2+staudsc**2+staudscb**2+
c     &         sysuds**2+sysudsc**2+sysudscb**2+
c     &         rephi+repsi+reups+reomega+
c     &         pe3**2+pec**2+peb**2
cc     &         pe5**2
          ertes=sqrt(ertes)
c ##################################################
          IF (ISCAN.EQ.0) THEN
c Test *XXX* 19/03/2010 SU2 had shift at rho
c          reback=cuds*resuds+cc*resc+cb*resb
c          rereso=cphi*rrphi+cpsi*rrpsi+cups*rrups+comega*rromega
             WRITE(44,124) E,cuds*resuds,cc*resc,cb*resb
             WRITE(45,124) E,cphi*rrphi,comega*rromega,
     &            cpsi*rrpsi+cups*rrups
c             WRITE(44,124) E,resuds,resc,resb
c             WRITE(45,124) E,rrphi,rromega,
c     &            rrpsi+rrups
             WRITE(46,124) E,reback,rereso,deg,retes
             WRITE(47,124) E,reback_new,rereso,deg_new,retes
             WRITE(48,124) E,reback_33new,rereso_33,deg_33new,retes
c end test *XXX*
c          ENDIF   ! end calculation of SU2 coupling
             if (iplot.eq.0) then
c here we write the table to intRdatx.tab
             write (8,*) ' '
             write (8,*) ' E:',E
             write (8,'(/)')
             write (8,*) ' der contributions tab'
             write (8,'(/)')
             do j=1,2           ! chpt,rho
                write (8,410) tex4x(j),EL(j),EH(j),TB(j,1),TB(j,2),
     &               TB(j,3),TB(j,4),TB(j,5)
             enddo
             do j=2,5           ! omega,phi,J/psi,Upsilon
                ETOT1=sqrt(STAR(j)**2+SYSR(j)**2)
                EREL1=abs(ETOT1/RESR(j))*100.
                write (8,410) tex4(j),ELR(j),EHR(j),RESR(j),STAR(j),
     &               SYSR(j),ETOT1,EREL1
             enddo
             do j=3,8
                write (8,410) tex4(j+4),EL(j),EH(j),TB(j,1),TB(j,2),
     &               TB(j,3),TB(j,4),TB(j,5)
             enddo
             j=9
             write (8,410) tex4(j+2),EL(j),EH(j),TB(j,1),TB(j,2),
     &            TB(j,3),TB(j,4),TB(j,5)
             j=10
             write (8,409) tex4(j+2),EL(j),TB(j,1),TB(j,2),
     &            TB(j,3),TB(j,4),TB(j,5)
             j=11
             write (8,410) tex4(j+2),EL(1),EH(9),TB(j,1),TB(j,2),
     &            TB(j,3),TB(j,4),TB(j,5)
             j=12
             write (8,409) tex4(j+3),EL(1),TB(j,1),TB(j,2),
     &            TB(j,3),TB(j,4),TB(j,5)
c 3g stuff to intRdatx.tab
             if (lprintdeg) then
             write (8,'(/)')
             write (8,*) ' deg contributions tab'
             write (8,'(/)')
             do j=1,2           ! chpt,rho
                write (8,410) tex4x(j),EL(j),EH(j),TG(j,1),TG(j,2),
     &               TG(j,3),TG(j,4),TG(j,5)
             enddo
             do j=2,5           ! omega,phi,J/psi,Upsilon
                ETOT1=sqrt(STAR(j)**2+SYSR(j)**2)
                EREL1=abs(ETOT1/RESR(j))*100.
                write (8,410) tex4(j),ELR(j),EHR(j),RESR(j),STAR(j),
     &               SYSR(j),ETOT1,EREL1
             enddo
             do j=3,8
                write (8,410) tex4(j+4),EL(j),EH(j),TG(j,1),TG(j,2),
     &               TG(j,3),TG(j,4),TG(j,5)
             enddo
             j=9
             write (8,410) tex4(j+2),EL(j),EH(j),TG(j,1),TG(j,2),
     &            TG(j,3),TG(j,4),TG(j,5)
             j=10
             write (8,409) tex4(j+2),EL(j),TG(j,1),TG(j,2),
     &            TG(j,3),TG(j,4),TG(j,5)
             j=11
             write (8,410) tex4(j+2),EL(1),EH(9),TG(j,1),TG(j,2),
     &            TG(j,3),TG(j,4),TG(j,5)
             j=12
             write (8,409) tex4(j+3),EL(1),TG(j,1),TG(j,2),
     &            TG(j,3),TG(j,4),TG(j,5)
          endif
          endif
          ENDIF

c summary pQCD tails
c                    c           b            Ecut
c         -----------|-----------|------------|-------->
c  uds:    resuds(BG)    pRESUDS     pRESUDS1     pr3
c                    c           b            Ecut
c         -----------|-----------|------------|-------->
c  udsc:               resc(c-BG)    pRESUDSC     pr4
c                    c           b            Ecut
c         -----------|-----------|------------|-------->
c  udscb:                          resb(b-BG)     pr5
c
c  NF=3: pRESUDS+pRESUDS1+pr3
c  NF=4: NF=3 + (pRESUDSC-pRESUDS1)+prc[=pr4-pr3]
c  NF=5: NF=4 + prb[=pr5-pr4]
c

c gamma gamma
          if ((IKE.EQ.1).and.(ISCAN.EQ.-1).and.
     &         (ISU2.EQ.0)) then
             resgg3=RESR(1)+resuds+rromega+rrphi+pRESUDS+pRESUDS1+pr3
             stagg3=sqrt(stauds**2+stomega**2+stphi**2)
             sysgg3=sqrt(sysuds**2+syomega**2+syphi**2+
     &            (pSYSUDS+pSYSUDS1+pe3)**2)
             ETOTGG3=sqrt(STAGG3**2+SYSGG3**2)
             resgg4=resgg3+resc+rrpsi+(pRESUDSC-pRESUDS1)+prc
             stagg4=sqrt(stagg3**2+stac**2+stpsi**2)
             sysgg4=sqrt(sysgg3**2+sysc**2 +sypsi**2+
     &            ((pSYSUDSC-pSYSUDS1)+pec)**2)
             ETOTGG4=sqrt(STAGG4**2+SYSGG4**2)
             resgg5=resgg4+resb+rrups+prb
             stagg5=sqrt(stagg4**2+stab**2+stups**2)
             sysgg5=sqrt(sysgg4**2+sysb**2+syups**2+peb**2)
             ETOTGG5=sqrt(STAGG5**2+SYSGG5**2)
             WRITE(11,124) E,RESGG3/ZT,STAGG3/ZT,SYSGG3/ZT,ETOTGG3/ZT
             WRITE(12,124) E,RESGG4/ZT,STAGG4/ZT,SYSGG4/ZT,ETOTGG4/ZT
             WRITE(13,124) E,RESGG5/ZT,STAGG5/ZT,SYSGG5/ZT,ETOTGG5/ZT
c             call dhadr5x(e,sin2ell,dder,derrdersta,derrdersys,
c     &            ddeg,derrdegsta,derrdegsys)
c             WRITE (66,124) e,dder,derrdersta,derrdersys
          endif
c 3 gamma
          if ((IKE.EQ.1).and.(ISCAN.EQ.-1).and.
     &         (ISU2.EQ.2)) then
          resg33=xud_3g*resud_rem+xuds*resuds_rem
     &          +TG(2,1)+TG(3,1)+TG(4,1)+TG(5,1)
     &          +xphi*rrphi+xomega*rromega
     &          +xuds*(pr3+pRESUDS+pRESUDS1)
          stag33=sqrt((xud_3g*staud_rem)**2+(xuds*stauds_rem)**2
     &          +TG(2,2)**2+TG(3,2)**2+TG(4,2)**2+TG(5,2)**2
     &          +(xphi*stphi)**2+(xomega*stomega)**2)
          sysg33=sqrt((xud_3g*sysud_rem)**2+(xuds*sysuds_rem)**2
     &         +TG(2,3)**2+TG(3,3)**2+TG(4,3)**2+TG(5,3)**2
     &         +(xphi*syphi)**2+(xomega*syomega)**2
     &         +(xuds*(pe3+pSYSUDS+pSYSUDS1))**2)
          ETOTG33=sqrt(STAG33**2+SYSG33**2)
          resg34=resg33+xc*resc+xpsi*rrpsi
     &         +xc*(prc+pRESUDSC-pRESUDS1)
          stag34=sqrt(stag33**2+(xc*stac)**2+(xpsi*stpsi)**2)
          sysg34=sqrt(sysg33**2+(xc*sysc)**2+(xpsi*sypsi)**2
     &           +(xc*pec)**2+(xc*(pSYSUDSC-pSYSUDS1))**2)
          ETOTG34=sqrt(STAG34**2+SYSG34**2)
          resg35=resg34+xb*resb+xups*rrups+xb*prb
          stag35=sqrt(stag34**2+(xb*stab)**2+(xups*stups)**2)
          sysg35=sqrt(sysg34**2+(xb*sysb)**2+(xups*syups)**2
     &           +(xb*peb)**2)
          ETOTG35=sqrt(STAG35**2+SYSG35**2)
             WRITE(11,124) E,RESG33/sin2ell/ZT,STAG33/sin2ell/ZT,
     &            SYSG33/sin2ell/ZT,ETOTG33/sin2ell/ZT
             WRITE(12,124) E,RESG34/sin2ell/ZT,STAG34/sin2ell/ZT,
     &            SYSG34/sin2ell/ZT,ETOTG34/sin2ell/ZT
             WRITE(13,124) E,RESG35/sin2ell/ZT,STAG35/sin2ell/ZT,
     &            SYSG35/sin2ell/ZT,ETOTG35/sin2ell/ZT
c             call dhadr5x(e,sin2ell,dder,derrdersta,derrdersys,
c     &            ddeg,derrdegsta,derrdegsys)
c             WRITE (67,124) e,ddeg,derrdegsta,derrdegsys
          endif
c test flavor recombination
          if (NFL.EQ.2) then
             RESGG =RESN2 /ZT
             ETOTGG=STAN2 /ZT
             STAGG =SYSN2 /ZT
             SYSGG =ETOTN2/ZT
          else if ((NFL.EQ.4).or.(NFL.EQ.3)) then
             RESGG =deg_tes/ZT
             ETOTGG=stadeg_tes/ZT
             STAGG =sysdeg_tes/ZT
             SYSGG =errdeg_tes/ZT
          else
             if ((IKE.EQ.1).and.(ISCAN.EQ.-1).and.
     &            (ISU2.EQ.0).and.(inf5.eq.0)) then
                RESGG =RES
                ETOTGG=ETOT
                STAGG =STA
                SYSGG =SYS
             else
                RESGG =RES/ZT
                ETOTGG=ETOT/ZT
                STAGG =STA/ZT
                SYSGG =SYS/ZT
             endif
          endif
c
          RES3G =deg_new    /sin2ell/ZT
          ETOT3G=errdeg_new/sin2ell/ZT
          STA3G =stadeg_new /sin2ell/ZT
          SYS3G =sysdeg_new /sin2ell/ZT
c check with i3gnew see intRdatx.tab for details from regions
          RES3Gn =RES3Gn /sin2ell/ZT
          STA3Gn =STA3Gn /sin2ell/ZT
          SYS3Gn =SYS3Gn /sin2ell/ZT
          ETOT3Gn=ETOT3Gn/sin2ell/ZT
c
          RES33 =deg_33new/sin2ell/ZT
          ETOT33=errdeg_33new/sin2ell/ZT
          STA33 =stadeg_33new/sin2ell/ZT
          SYS33 =sysdeg_33new/sin2ell/ZT
c test complement of uds=TG(1,)+...+TG(6)
          write (*,*) ' main uds complement:',
     &         deg-resuds/2.d0,deg_new-resuds_n
          if (ISU2.EQ.1) then
c a better choice: alpha_2=alpha/sin2ell
c sin2ell from LEP NC Z coupling sin2ell(M_Z) vs sin2(0) nu_mu e scattering: sin2(0)/sin2ell=1.002
             RES3G =deg/sin2ell/ZT
             ETOT3G=errdeg/sin2ell/ZT
             STA3G =stadeg/sin2ell/ZT
             SYS3G =sysdeg/sin2ell/ZT
c
             RES33 =deg_33/sin2ell/ZT
             ETOT33=errdeg_33/sin2ell/ZT
             STA33 =stadeg_33/sin2ell/ZT
             SYS33 =sysdeg_33/sin2ell/ZT
c for tables
             RES =deg/sin2ell
             ETOT=errdeg/sin2ell
             STA =stadeg/sin2ell
             SYS =sysdeg/sin2ell
          else if (ISU2.EQ.2) then
             RES3G =deg_new/sin2ell/ZT
             ETOT3G=errdeg_new/sin2ell/ZT
             STA3G =stadeg_new/sin2ell/ZT
             SYS3G =sysdeg_new/sin2ell/ZT
c
             RES33 =deg_33new/sin2ell/ZT
             ETOT33=errdeg_33new/sin2ell/ZT
             STA33 =stadeg_33new/sin2ell/ZT
             SYS33 =sysdeg_33new/sin2ell/ZT
c for tables
             RES =deg_new/sin2ell
             ETOT=errdeg_new/sin2ell
             STA =stadeg_new/sin2ell
             SYS =sysdeg_new/sin2ell
          else if (ISU2.EQ.3) then
c for tables
             RES =deg_33/sin2ell
             ETOT=errdeg_33/sin2ell
             STA =stadeg_33/sin2ell
             SYS =sysdeg_33/sin2ell
          else if (ISU2.EQ.4) then
c for tables
             RES =deg_33new/sin2ell
             ETOT=errdeg_33new/sin2ell
             STA =stadeg_33new/sin2ell
             SYS =sysdeg_33new/sin2ell
          endif
          if (iplot.eq.0) then
             IF (IECUT.EQ.0) THEN
                RESOLD=RES
                STAOLD=STA
                SYSOLD=SYS
                ETOOLD=ETOT
                RESN2OLD=RESN2
                STAN2OLD=STAN2
                SYSN2OLD=SYSN2
                ETON2OLD=ETOTN2
                RESOLDGG=RESGG
                STAOLDGG=STAGG
                SYSOLDGG=SYSGG
                ETOOLDGG=ETOTGG
                RESOLD3G=RES3G
                STAOLD3G=STA3G
                SYSOLD3G=SYS3G
                ETOOLD3G=ETOT3G
                RESOLD33=RES33
                STAOLD33=STA33
                SYSOLD33=SYS33
                ETOOLD33=ETOT33
                IF ((IKE.EQ.8).OR.(IKE.EQ.9)) THEN
                   irever=NPin+1-icount+1
                   WRITE(50,124) fm/E,RES,STA,SYS,ETOT
                   WRITE(51,124) fm/E,RESN2,STAN2,SYSN2,ETOTN2
                   WRITE(1,124) fm/E,RESGG,STAGG,SYSGG,ETOTGG
                   WRITE(2,124) fm/E,RES3G,STA3G,SYS3G,ETOT3G
                   WRITE(3,124) fm/E,RES33,STA33,SYS33,ETOT33
                   IF (NFL.EQ.2) THEN
                      XV(irever)=fm/E
                      YV(irever)=RESN2/ZT
                      ZV(irever)=ETOTN2/ZT
                      UV(irever)=STAN2/ZT
                      VV(irever)=SYSN2/ZT
                   ELSE IF (NFL.LT.5) THEN
                      XV(irever)=fm/E
                      YV(irever)=RES3G/ZT
                      ZV(irever)=ETOT3G/ZT
                      UV(irever)=STA3G/ZT
                      VV(irever)=SYS3G/ZT
                   ELSE
                      XV(irever)=fm/E
                      YV(irever)=RES/ZT
                      ZV(irever)=ETOT/ZT
                      UV(irever)=STA/ZT
                      VV(irever)=SYS/ZT
                   ENDIF
                ELSE
                   WRITE(50,124) E,RES,STA,SYS,ETOT
                   WRITE(51,124) E,RESN2/ZT,STAN2/ZT,SYSN2/ZT,ETOTN2/ZT
                   WRITE(1,124) E,RESGG,STAGG,SYSGG,ETOTGG
                   WRITE(2,124) E,RES3G,STA3G,SYS3G,ETOT3G
                   WRITE(3,124) E,RES33,STA33,SYS33,ETOT33
                   if ((IKE.EQ.1).and.(ISCAN.EQ.-1).and.
     &                  (ISU2.EQ.0)) then
                      IF (NFL.EQ.5) THEN
                         if (inf5.eq.0) then
                         XV(icount)=E
                         YV(icount)=RESGG5/ZT
                         ZV(icount)=ETOTGG5/ZT
                         UV(icount)=STAGG5/ZT
                         VV(icount)=SYSGG5/ZT
                      else 
                         XV(icount)=E
                         YV(icount)=RES/ZT
                         ZV(icount)=ETOT/ZT
                         UV(icount)=STA/ZT
                         VV(icount)=SYS/ZT
                      endif
                      ELSE IF (NFL.EQ.4) THEN
                         XV(icount)=E
                         YV(icount)=RESGG4/ZT
                         ZV(icount)=ETOTGG4/ZT
                         UV(icount)=STAGG4/ZT
                         VV(icount)=SYSGG4/ZT
                      ELSE IF (NFL.EQ.3) THEN
                         XV(icount)=E
                         YV(icount)=RESGG3/ZT
                         ZV(icount)=ETOTGG3/ZT
                         UV(icount)=STAGG3/ZT
                         VV(icount)=SYSGG3/ZT
                      ELSE IF (NFL.EQ.2) THEN
                         XV(icount)=E
                         YV(icount)=RESN2/ZT
                         ZV(icount)=ETOTN2/ZT
                         UV(icount)=STAN2/ZT
                         VV(icount)=SYSN2/ZT
                      ENDIF
                   else if ((IKE.EQ.1).and.(ISCAN.EQ.-1).and.
     &                     (ISU2.EQ.2)) then
                      IF (NFL.EQ.5) THEN
                         if (inf5.eq.0) then
                            XV(icount)=E
                            YV(icount)=RESG35/ZT/sin2ell
                            ZV(icount)=ETOTG35/ZT/sin2ell
                            UV(icount)=STAG35/ZT/sin2ell
                            VV(icount)=SYSG35/ZT/sin2ell
                         else 
                            XV(icount)=E
                            YV(icount)=RES/ZT
                            ZV(icount)=ETOT/ZT
                            UV(icount)=STA/ZT
                            VV(icount)=SYS/ZT
                         endif
                      ELSE IF (NFL.EQ.4) THEN
                         XV(icount)=E
                         YV(icount)=RESG34/ZT/sin2ell
                         ZV(icount)=ETOTG34/ZT/sin2ell
                         UV(icount)=STAG34/ZT/sin2ell
                         VV(icount)=SYSG34/ZT/sin2ell
                      ELSE IF (NFL.EQ.3) THEN
                         XV(icount)=E
                         YV(icount)=RESG33/ZT/sin2ell
                         ZV(icount)=ETOTG33/ZT/sin2ell
                         UV(icount)=STAG33/ZT/sin2ell
                         VV(icount)=SYSG33/ZT/sin2ell
                      ELSE IF (NFL.EQ.2) THEN
                         XV(icount)=E
                         YV(icount)=RESN2/ZT
                         ZV(icount)=ETOTN2/ZT
                         UV(icount)=STAN2/ZT
                         VV(icount)=SYSN2/ZT
                      ENDIF
                   else
                      XV(icount)=E
                      YV(icount)=RES/ZT
                      ZV(icount)=ETOT/ZT
                      UV(icount)=STA/ZT
                      VV(icount)=SYS/ZT
                   endif
                ENDIF
             ELSE  ! i.e. if IECUT=1
                IECUT=0
                icount=icount+1
                RESNEW=RES
                STANEW=STA
                SYSNEW=SYS
                ETONEW=ETOT
                RESOLD=(RESOLD+RESNEW)/2.d0
                STAOLD=(STAOLD+STANEW)/2.d0
                SYSOLD=(SYSOLD+SYSNEW)/2.d0
                ETOOLD=(ETOOLD+ETONEW)/2.d0
                RESN2NEW=RESN2
                STAN2NEW=STAN2
                SYSN2NEW=SYSN2
                ETON2NEW=ETOTN2
                RESN2OLD=(RESN2OLD+RESN2NEW)/2.d0
                STAN2OLD=(STAN2OLD+STAN2NEW)/2.d0
                SYSN2OLD=(SYSN2OLD+SYSN2NEW)/2.d0
                ETON2OLD=(ETON2OLD+ETON2NEW)/2.d0
                RESNEWGG=RESGG
                STANEWGG=STAGG
                SYSNEWGG=SYSGG
                ETONEWGG=ETOTGG
                RESOLDGG=(RESOLDGG+RESNEWGG)/2.d0
                STAOLDGG=(STAOLDGG+STANEWGG)/2.d0
                SYSOLDGG=(SYSOLDGG+SYSNEWGG)/2.d0
                ETOOLDGG=(ETOOLDGG+ETONEWGG)/2.d0
                RESNEW3G=RES3G
                STANEW3G=STA3G
                SYSNEW3G=SYS3G
                ETONEW3G=ETOT3G
                RESOLD3G=(RESOLD3G+RESNEW3G)/2.d0
                STAOLD3G=(STAOLD3G+STANEW3G)/2.d0
                SYSOLD3G=(SYSOLD3G+SYSNEW3G)/2.d0
                ETOOLD3G=(ETOOLD3G+ETONEW3G)/2.d0
                RESNEW33=RES33
                STANEW33=STA33
                SYSNEW33=SYS33
                ETONEW33=ETOT33
                RESOLD33=(RESOLD33+RESNEW33)/2.d0
                STAOLD33=(STAOLD33+STANEW33)/2.d0
                SYSOLD33=(SYSOLD33+SYSNEW33)/2.d0
                ETOOLD33=(ETOOLD33+ETONEW33)/2.d0
                IF ((IKE.EQ.8).OR.(IKE.EQ.9)) THEN
                   WRITE(50,124) fm/ECUT,RESOLD,STAOLD,SYSOLD,ETOOLD
                   WRITE(50,124) fm/E,RESNEW,STANEW,SYSNEW,ETONEW
                   WRITE(51,124) fm/ECUT,RESN2OLD,STAN2OLD,SYSN2OLD,
     &                  ETON2OLD
                   WRITE(51,124) fm/E,RESN2NEW,STAN2NEW,SYSN2NEW,
     &                  ETON2NEW
                   WRITE(1,124)fm/ECUT,RESOLDGG,STAOLDGG,SYSOLDGG,
     &                  ETOOLDGG
                   WRITE(1,124)fm/E,RESNEWGG,STANEWGG,SYSNEWGG,ETONEWGG
                   WRITE(2,124) fm/ECUT,RESOLD3G,STAOLD3G,SYSOLD3G,
     &                  ETOOLD3G
                   WRITE(2,124) fm/E,RESNEW3G,STANEW3G,SYSNEW3G,ETONEW3G
                   WRITE(3,124) fm/ECUT,RESOLD33,STAOLD33,SYSOLD33,
     &                  ETOOLD33
                   WRITE(3,124) fm/E,RESNEW33,STANEW33,SYSNEW33,ETONEW33
                ELSE
                   WRITE(50,124) ECUT,RESOLD,STAOLD,SYSOLD,ETOOLD
                   WRITE(50,124) E,RESNEW,STANEW,SYSNEW,ETONEW
                   WRITE(51,124) ECUT,RESN2OLD,STAN2OLD,SYSN2OLD,
     &                  ETON2OLD
                   WRITE(51,124) E,RESN2NEW,STAN2NEW,SYSN2NEW,
     &                  ETON2NEW
                   WRITE(1,124)ECUT,RESOLDGG,STAOLDGG,SYSOLDGG,
     &                  ETOOLDGG
                   WRITE(1,124)E,RESNEWGG,STANEWGG,SYSNEWGG,ETONEWGG
                   WRITE(2,124) ECUT,RESOLD3G,STAOLD3G,SYSOLD3G,
     &                  ETOOLD3G
                   WRITE(2,124) E,RESNEW3G,STANEW3G,SYSNEW3G,ETONEW3G
                   WRITE(3,124) ECUT,RESOLD33,STAOLD33,SYSOLD33,
     &                  ETOOLD33
                   WRITE(3,124) E,RESNEW33,STANEW33,SYSNEW33,ETONEW33
                ENDIF
                IF ((IKE.EQ.8).OR.(IKE.EQ.9)) THEN
                   irever=NPin+1-icount+1
                   IF (NFL.EQ.2) THEN
                      XV(irever)=fm/E
                      YV(irever)=RESN2/ZT
                      ZV(irever)=ETOTN2/ZT
                      UV(irever)=STAN2/ZT
                      VV(irever)=SYSN2/ZT
                   ELSE IF (NFL.LT.5) THEN
                      XV(irever)=fm/E
                      YV(irever)=RES3G/ZT
                      ZV(irever)=ETOT3G/ZT
                      UV(irever)=STA3G/ZT
                      VV(irever)=SYS3G/ZT
                   ELSE
                      XV(irever)=fm/E
                      YV(irever)=RES/ZT
                      ZV(irever)=ETOT/ZT
                      UV(irever)=STA/ZT
                      VV(irever)=SYS/ZT
                   ENDIF
                   if (icount.gt.1) then
                      XV(irever+1)=fm/ECUT
                      if (icount.gt.2) then
                         YV(irever+1)=(YV(irever)+YV(irever+2))/2.d0
                         ZV(irever+1)=(ZV(irever)+ZV(irever+2))/2.d0
                         UV(irever+1)=(UV(irever)+UV(irever+2))/2.d0
                         VV(irever+1)=(VV(irever)+VV(irever+2))/2.d0
                      endif
                   endif
                ELSE
                   XV(icount)=E
                   YV(icount)=RES/ZT
                   ZV(icount)=ETOT/ZT
                   UV(icount)=STA/ZT
                   VV(icount)=SYS/ZT
                   if (icount.gt.1) then
                      XV(icount-1)=ECUT
                      if (icount.gt.2) then
                         YV(icount-1)=(YV(icount)+YV(icount-2))/2.d0
                         ZV(icount-1)=(ZV(icount)+ZV(icount-2))/2.d0
                         UV(icount-1)=(UV(icount)+UV(icount-2))/2.d0
                         VV(icount-1)=(VV(icount)+VV(icount-2))/2.d0
                      endif
                   endif
                ENDIF
             ENDIF
          else
             IF (IECUT.EQ.0) THEN
                RESOLD=RES/ZT
                ETOOLD=ETOT/ZT
                RESN2OLD=RESN2/ZT
                ETON2OLD=ETOTN2/ZT
                RESOLDGG=RESGG/ZT
                ETOOLDGG=ETOTGG/ZT
                RESOLD3G=RES3G/ZT
                ETOOLD3G=ETOT3G/ZT
                RESOLD33=RES33/ZT
                ETOOLD33=ETOT33/ZT
                call skipprint(50,icount,E,RESOLD,ETOOLD,ETOOLD)
                call skipprint(51,icount,E,RESN2OLD,ETON2OLD,ETON2OLD)
                call skipprint(1,icount,E,RESOLDGG,ETOOLDGG,ETOOLDGG)
                call skipprint(2,icount,E,RESOLD3G,ETOOLD3G,ETOOLD3G)
                call skipprint(3,icount,E,RESOLD33,ETOOLD33,ETOOLD33)
c                WRITE(1,123) E,RESOLD,ETOOLD,ETOOLD
             ELSE
                IECUT=0
                icount=icount+1
                RESNEW=RES/ZT
                ETONEW=ETOT/ZT
                RESOLD=(RESOLD+RESNEW)/2.d0
                ETOOLD=(ETOOLD+ETONEW)/2.d0
                call skipprint(50,icount,ECUT,RESOLD,ETOOLD,ETOOLD)
                call skipprint(50,icount,E,RESNEW,ETONEW,ETONEW)
                RESN2NEW=RESN2/ZT
                ETON2NEW=ETOTN2/ZT
                RESN2OLD=(RESN2OLD+RESN2NEW)/2.d0
                ETON2OLD=(ETON2OLD+ETON2NEW)/2.d0
                call skipprint(51,icount,ECUT,RESN2OLD,ETON2OLD,
     &               ETON2OLD)
                call skipprint(51,icount,E,RESN2NEW,ETON2NEW,ETON2NEW)
                RESNEWGG=RESGG/ZT
                ETONEWGG=ETOTGG/ZT
                RESOLDGG=(RESOLDGG+RESNEWGG)/2.d0
                ETOOLDGG=(ETOOLDGG+ETONEWGG)/2.d0
                call skipprint(1,icount,ECUT,RESOLDGG,ETOOLDGG,ETOOLDGG)
                call skipprint(1,icount,E,RESNEWGG,ETONEWGG,ETONEWGG)
                RESNEW3G=RES3G/ZT
                ETONEW3G=ETOT3G/ZT
                RESOLD3G=(RESOLD3G+RESNEW3G)/2.d0
                ETOOLD3G=(ETOOLD3G+ETONEW3G)/2.d0
                call skipprint(2,icount,ECUT,RESOLD3G,ETOOLD3G,ETOOLD3G)
                call skipprint(2,icount,E,RESNEW3G,ETONEW3G,ETONEW3G)
                RESNEW33=RES33/ZT
                ETONEW33=ETOT33/ZT
                RESOLD33=(RESOLD33+RESNEW33)/2.d0
                ETOOLD33=(ETOOLD33+ETONEW33)/2.d0
                call skipprint(3,icount,ECUT,RESOLD33,ETOOLD33,ETOOLD33)
                call skipprint(3,icount,E,RESNEW33,ETONEW33,ETONEW33)
c                WRITE(1,123) ECUT,RESOLD,ETOOLD,ETOOLD
c                WRITE(1,123) E,RESNEW,ETONEW,ETONEW
             ENDIF
          endif
          if ((isinf(YV(icount))).or.(isnan(YV(icount)))) then
             ierrcountY=ierrcountY+1
             iYerr(ierrcountY)=icount
          endif
          if ((isinf(ZV(icount))).or.(isnan(ZV(icount)))) then
             ierrcountZ=ierrcountZ+1
             iZerr(ierrcountZ)=icount
          endif
          icount=icount+1
c
          IF (ISCAN.EQ.0) THEN
             if (iplot.eq.0) then
c intRdatx.tab summary
c             write (8,409) tex4(15),EL(1),RES,STA,SYS,ETOT,ETOT/RES*100.
             write (8,'(/)')
             write (8,*) ' tab summary der contributions '
             write (8,'(/)')
             write (8,*) ' der (errder):',der,stader,sysder
     &            ,'(',errder,')'
             write (8,*) ' test der:',retes,'(',ertes,')'
             write (8,*) ' '
             if ((IKE.EQ.0).or.(IKE.EQ.10).or.(IKE.EQ.12).or.
     &            (IKE.EQ.13).or.
     &            ((IKE.GE.3).and.(IKE.LE.7)))
     &            then
             else
              write (8,*) ' tab summary deg contributions '
              write (8,'(a14,3(2x,f8.3),a1,f8.3,a1)') ' deg (errdeg):',
     &             RES3Gn,STA3Gn,SYS3Gn,'(',ETOT3Gn,')'
              write (8,*) ' test deg:',RES3Gn,'(',RES3G,')'
c                write (8,*) ' Result :',YV(icount-1),'(',ZV(icount-1),')'
             endif
             write (8,*) ' Result :',YV(icount-1),'(',ZV(icount-1),')'
          endif
          ENDIF
c
 60       continue
c       enddo       ! end error loop for 2nd intergration of k3c kernel
          IF ((ILOG.EQ.0).AND.(IINV.EQ.0)) E=E+DE
          IF (ILOG.EQ.1) E=EXP(LOG(E)+DE)
          IF (IINV.EQ.1) E=1.d0/(1.d0/E+DE)
          IF (ISCAN.EQ.-2) E=-DEXP(LL+DL)
c          call dhadr5x(e,sin2ell,dder,derrdersta,derrdersys,
c     &         ddeg,derrdegsta,derrdegsys)
c          WRITE (66,124) e,dder,derrdersta,derrdersys
c          WRITE (67,124) e,ddeg,derrdegsta,derrdegsys
       enddo    ! end loop IE=NMI,NPM
c end energy scan
       IE=icount
       xxx='XXX  '
       yyy='YYY  '
       if (iplot.eq.0) then
          IF ((IKE.EQ.8).and.(ISU2.EQ.0)) THEN
             IF (NFL.EQ.2) THEN
             write (7,*) '      subroutine ECgg2oftdata'
             write (7,*) '      IMPLICIT NONE'
             write (7,*) '      INTEGER NB,I,J'
             write (7,*) '      PARAMETER(NB=',INL,')'
             write (7,*) '      REAL TGG2(NB),YGG2(NB,3)'
             write (7,*) '      COMMON /ECGG2/TGG2,YGG2'
             write (70,'(a1)') '2'
             ELSE IF (NFL.EQ.3) THEN
             write (7,*) '      subroutine ECgg3oftdata'
             write (7,*) '      IMPLICIT NONE'
             write (7,*) '      INTEGER NB,I,J'
             write (7,*) '      PARAMETER(NB=',INL,')'
             write (7,*) '      REAL TGG3(NB),YGG3(NB,3)'
             write (7,*) '      COMMON /ECGG3/TGG3,YGG3'
             write (70,'(a1)') '3'
             ELSE IF (NFL.EQ.4) THEN
             write (7,*) '      subroutine ECgg4oftdata'
             write (7,*) '      IMPLICIT NONE'
             write (7,*) '      INTEGER NB,I,J'
             write (7,*) '      PARAMETER(NB=',INL,')'
             write (7,*) '      REAL TGG4(NB),YGG4(NB,3)'
             write (7,*) '      COMMON /ECGG4/TGG4,YGG4'
             write (70,'(a1)') '4'
             ELSE
             write (7,*) '      subroutine ECgg5oftdata'
             write (7,*) '      IMPLICIT NONE'
             write (7,*) '      INTEGER NB,I,J'
             write (7,*) '      PARAMETER(NB=',INL,')'
             write (7,*) '      REAL TGG5(NB),YGG5(NB,3)'
             write (7,*) '      COMMON /ECGG5/TGG5,YGG5'
             write (70,'(a1)') '5'
             ENDIF
          ENDIF
          IF (ISU2.EQ.0) THEN
             IF (ISCAN.EQ.1) THEN
c there are Nall-1 = 2137 data point interval centers used in the evaluation
c write results as a fortran data statement
                if (IGS.EQ.0) DSET='    '
                IF (IKE.EQ.11) THEN
             write (7,*) '      subroutine k3chad_timelike'//year
             write (7,*) '      IMPLICIT NONE'
             write (7,*) '      INTEGER NB,I,J'
             write (7,*) '      PARAMETER(NB=2174)'
             write (7,*) '      REAL ESA(NB),DAS(NB,3)'
             write (7,*) '      COMMON /DK3C'//year//'/ESA,DAS'
                ELSE
             write (7,*) '      subroutine dalhad_timelike'//year
             write (7,*) '      IMPLICIT NONE'
             write (7,*) '      INTEGER NB,I,J'
             write (7,*) '      PARAMETER(NB=2174)'
             write (7,*) '      REAL ESA(NB),DAS(NB,3),DGS(NB,3)'
             write (7,*) '      COMMON /DATS'//year//'/ESA,DAS,DGS'
                ENDIF
             ELSE IF (ISCAN.EQ.-1) THEN
          write (7,*) '      subroutine dalhad_spacelike'//year
          write (7,*) '      IMPLICIT NONE'
          write (7,*) '      INTEGER NA,NC,I,J'
          write (7,*) '      PARAMETER(NA=979,NC=200)'
          write (7,*) '      REAL ETA(NA),DAT(NA,3),DGT(NA,3)'
          write (7,*) '      REAL XXX(NC),YGG(NC,3),Y3G(NC,3)'
          write (7,*) '      COMMON /DATT'//year//'/ETA,DAT,DGT'
          write (7,*) '      COMMON /DATH'//year//'/XXX,YGG,Y3G'
             ELSE IF (ISCAN.EQ.2) THEN
          write (7,*) '      subroutine dalhad_timelike1h'//year
          write (7,*) '      IMPLICIT NONE'
          write (7,*) '      INTEGER NC,I,J'
          write (7,*) '      PARAMETER(NC=200)'
          write (7,*) '      REAL EMA(NC),DAM(NC,3),DGM(NC,3)'
          write (7,*) '      COMMON /DATM'//year//'/EMA,DAM,DGM'
             ELSE
             ENDIF
          ENDIF
          WRITE(7,103)
          WRITE(7,102) uhrzeit
          WRITE(7,103)
          WRITE(7,104) KERN(JKE),UPDATEVERSION
          WRITE(7,103)
          WRITE(7,105) ISCAN,Ismooth,IGS,DSET,iplot,itestflavorsplitting
          WRITE(7,107) IKE,Emin,Emax,NP,N,EPSREL
          WRITE(7,108) ALS,EST,ESY,MZINP,MTOP
          WRITE(7,208) NF,IOR,ICHK,ISU2,NFL,INAG,inotail,sin2ell
          WRITE(7,109) CHPTCUT,EC,ECUT,st2,sin2ell/st2
          WRITE(7,103)
        if ((IKE.EQ.0).or.(IKE.EQ.10).or.(IKE.EQ.12).or.
     &       (IKE.EQ.13).or.
     &       ((IKE.GE.3).and.(IKE.LE.7)))
     &  then
           write (7,'(a9,4(2x,f7.2))') ' Result :',YV(icount-1),
     &          UV(icount-1),VV(icount-1),ZV(icount-1)
           if ((IKE.EQ.12).or.(IKE.EQ.13)) write (7,*) ' in 10^{-12} '
        endif
          IF (ISCAN.EQ.1) THEN
             xxx='ESA  '
             yyy='DAS  '
             IF ((ISU2.EQ.1).OR.(ISU2.EQ.2)) yyy='DGS  '
             IF ((ISU2.EQ.3).OR.(ISU2.EQ.4)) yyy='DGS3 '
          ENDIF
          IF (ISCAN.EQ.2) THEN
             xxx='EMA  '
             yyy='DAM  '
             IF ((ISU2.EQ.1).OR.(ISU2.EQ.2)) yyy='DGM  '
             IF ((ISU2.EQ.3).OR.(ISU2.EQ.4)) yyy='DGM3 '
          ENDIF
          IF (ISCAN.EQ.-1) THEN
             xxx='ETA  '
             yyy='DAT  '
             IF ((ISU2.EQ.1).OR.(ISU2.EQ.2)) yyy='DGT  '
             IF ((ISU2.EQ.3).OR.(ISU2.EQ.4)) yyy='DGT3 '
          ENDIF
          IF (ISCAN.EQ.-2) THEN
             xxx='XXX  '
             yyy='YGG  '
             IF ((ISU2.EQ.1).OR.(ISU2.EQ.2)) yyy='Y3G  '
             IF ((ISU2.EQ.3).OR.(ISU2.EQ.4)) yyy='Y33  '
          ENDIF
          IF ((IKE.EQ.8).and.(ISU2.EQ.0)) THEN
             IF (NFL.EQ.2) THEN
             xxx='TGG2 '
             yyy='YGG2 '
             ELSE IF (NFL.EQ.3) THEN
             xxx='TGG3 '
             yyy='YGG3 '
             ELSE IF (NFL.EQ.4) THEN
             xxx='TGG4 '
             yyy='YGG4 '
             ELSE
             xxx='TGG5 '
             yyy='YGG5 '
             ENDIF
          ENDIF
          IF (ISCAN.EQ.1) THEN
c PV integral convergence problems: points 95,97, 107,112, 154, 244, 1223; replace by linerar interpolation of neighboring points
             do jrep=1,Nrep2
                Irep=Ireplisttwo(jrep)
                YV(Irep)=YV(Irep-1)+(YV(Irep+1)-YV(Irep-1))/
     &               (XV(Irep+1)-XV(Irep-1))*(XV(Irep)-XV(Irep-1))
                UV(Irep)=UV(Irep-1)+(UV(Irep+1)-UV(Irep-1))/
     &               (XV(Irep+1)-XV(Irep-1))*(XV(Irep)-XV(Irep-1))
                VV(Irep)=VV(Irep-1)+(VV(Irep+1)-VV(Irep-1))/
     &               (XV(Irep+1)-XV(Irep-1))*(XV(Irep)-XV(Irep-1))
             enddo
c on top of the previous 2014 update required: 3 adjacent pairs
             do jrep=1,Nrep3
                Irep=Ireplithree(jrep)
                YV(Irep)=YV(Irep-1)+(YV(Irep+2)-YV(Irep-1))/
     &               (XV(Irep+2)-XV(Irep-1))*(XV(Irep)-XV(Irep-1))
                UV(Irep)=UV(Irep-1)+(UV(Irep+2)-UV(Irep-1))/
     &               (XV(Irep+2)-XV(Irep-1))*(XV(Irep)-XV(Irep-1))
                VV(Irep)=VV(Irep-1)+(VV(Irep+2)-VV(Irep-1))/
     &               (XV(Irep+2)-XV(Irep-1))*(XV(Irep)-XV(Irep-1))
c
                YV(Irep+1)=YV(Irep-1)+(YV(Irep+2)-YV(Irep-1))/
     &               (XV(Irep+2)-XV(Irep-1))*(XV(Irep+1)-XV(Irep-1))
                UV(Irep+1)=UV(Irep-1)+(UV(Irep+2)-UV(Irep-1))/
     &               (XV(Irep+2)-XV(Irep-1))*(XV(Irep+1)-XV(Irep-1))
                VV(Irep+1)=VV(Irep-1)+(VV(Irep+2)-VV(Irep-1))/
     &               (XV(Irep+2)-XV(Irep-1))*(XV(Irep+1)-XV(Irep-1))
             enddo
             if (ierrcountY.gt.0) then
                do jj=1,ierrcountY
                   Irep=iYerr(jj)
                   YV(Irep)=YV(Irep-1)+(YV(Irep+1)-YV(Irep-1))/
     &                  (XV(Irep+1)-XV(Irep-1))*(XV(Irep)-XV(Irep-1))
                   UV(Irep)=UV(Irep-1)+(UV(Irep+1)-UV(Irep-1))/
     &                  (XV(Irep+1)-XV(Irep-1))*(XV(Irep)-XV(Irep-1))
                   VV(Irep)=VV(Irep-1)+(VV(Irep+1)-VV(Irep-1))/
     &                  (XV(Irep+1)-XV(Irep-1))*(XV(Irep)-XV(Irep-1))
                enddo
             endif
             if (ierrcountZ.gt.0) then
                do jj=1,ierrcountZ
                   Irep=iZerr(jj)
                   ZV(Irep)=ZV(Irep-1)+(ZV(Irep+1)-ZV(Irep-1))/
     &                  (XV(Irep+1)-XV(Irep-1))*(XV(Irep)-XV(Irep-1))
                enddo
             endif
          ENDIF
          UNIT=7
          NP=IE-1
          IF ((IKE.EQ.8).and.(ISU2.EQ.0)) ISCAN=2
c changed 5 juli 2017  x,y,sta,sys in place of x,y,tot
          if ((IKE.EQ.0).or.(IKE.EQ.10).or.(IKE.EQ.12).or.
     &         (IKE.EQ.13).or.
     &         ((IKE.GE.3).and.(IKE.LE.7)))
     &         then
          else
             call printscan(UNIT,XV,YV,UV,VV,NP,ISCAN,xxx,yyy)
          endif
       endif
       write (*,*) ' icountps=',icountps
       write (*,*) ' icountyp=',icountyp
       write (*,*) ' -- printscan ranges -- '
       write (*,'(a18,2x,i4,5(2x,i3))') ' IE,NM,NH,NR,IM,IR',
     &         IEx,NMx,NHx,NRx,IMx,IRx
*
       if (iplot.eq.1) then
          write (*,*) ' polot data files:'
          write (*,*) ' <gg> on intRdatx_gg.dap'
          write (*,*) ' <3g> on intRdatx_3g.dap'
          write (*,*) ' <33> on intRdatx_33.dap'
          write (*,*) ' NF=2 on intRdatx_n2.dap'
       endif
*
       if (iWarnOption.eq.1) then
          write (*,*) ' Warning *** test options have been used *** '
          write (*,*) ' check flags: inf2on,itest,itestflavorseparation'
       endif
*      
 102   FORMAT('c',78('*'),/,'c','*',76X,'*',
     .      /,'c','*    --- intRdatx ---     '
     .      ,'                         ',A24,'  *'
     .      ,/,'c','*',76X,'*',/,'c',27('*')
     .      ,'# FJ@HU Berlin #',35('*'))
103    FORMAT('  ')
104    FORMAT('c',2X,A27,' Update version (',A10,') ')
105    FORMAT('c',' ISCAN: ',I2,', Ismooth: ',I1,', IGS: ',I1,
     &      ', DSET: ',A4,', iplot: ',I1,' itestflavorspl: ',I1)
107    FORMAT('c',' IKE,Emin,Emax,NP,N,EPSREL: ',I2,',',
     &      1pe11.4,',',1pe11.4,',',I4,',',I3,',',1pe9.2,', ',A3)
108    FORMAT('c',' ALS(sta)(sys): ',f7.4,'(',f7.4,')(',f7.4,
     &      '), MZINP: ',f9.4,', MTOP: ',f9.2)
208    FORMAT('c',' NF,IOR,ICHK: ',I1,',',I1,',',I1,',ISU2: ',I1
     &    ,',NFL: ',I1,',INAG: ',I1,',inotail:',I2,', sin2W_ref:',f8.5)
 109   FORMAT('c',' CHPTCUT,EC,ECUT:',f7.4,',',f7.2,',',f7.2,
     &      '  sin2W_eff:',f8.5,' sin2W_ref/sin2W_eff:',f8.5)
 123   format(1x,1pe15.8,3(2x,1pe13.6))
 124   format(1x,1pe15.8,4(2x,1pe13.6))
 409  FORMAT(1X,2X,A10,3X,'&(',F4.1,',$\infty$)&',3X,F7.2,
     .     2(1X,'(',F5.2,')'),'[',F5.2,']&',
     &     F5.1,'\% &',F5.1,'\% \\')
 410  FORMAT(1X,2X,A10,3X,'& (',F5.2,',',F5.2,') &',3X,F7.2,
     .     2(1X,'(',F5.2,')'),'[',F5.2,']&',
     &     F5.1,'\% &',F5.1,'\% \\')
c 410   format(1X,2X,A10,3X,'& (',F4.2,',',F5.2,') &',3X,F7.2,
c     .     1X,'(',F5.2,')'1X,'(',F5.2,')','&[',F5.2,']&',
c     &     F5.1,'\\%',F5.1,'\\% \\\\')
99996 FORMAT (1X,A,I4)
c      call abort()
       STOP
       END
C
       FUNCTION RSDAT(S)
C      -----------------
C Background: Integrand
c In case of PV pole in data set (time-like energies below ECUT) we need
c spline or least square fit in interval (POLM,POLP) around the pole: in
c that interval RSBGI=0 if IPV=0 and RSBGI=spline (ISPLINE=1) or fit if
c IPV=1; outside that interval RSBGI=linear interpolation of data
       IMPLICIT NONE
       INTEGER ICH,IER,IKE,IPV,IREN,IRENO,Ismooth
       REAL *8 E,S,RSDAT,RS40,RS40x,RS40smoothed,FACT,RSBGI,
     &      CHPTCUT,EC,ECUT,POLP,POLM,RS40tree
       EXTERNAL RS40,RS40x,RS40smoothed,RSBGI
       COMMON/SMOO/Ismooth
       COMMON/CHAN/ICH
       COMMON/RCUTS/CHPTCUT,EC,ECUT
       COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
c this is required for trapezoidal rule intergration of data to avoid double counting with pQCD
c parametrized regions
       E=SQRT(S)
       RSDAT=0.d0
       IF (((E.GT.EC).and.(E.LT.9.5d0)).or.(E.GT.ECUT)) THEN
          RSDAT=0.d0
       ELSE
          IF (((S.GT.POLM).AND.(S.LT.POLP)).AND.(IPV.EQ.0)) THEN
             RSDAT=0.D0
          ELSE
             if (Ismooth.eq.1) then
                RSDAT=RS40smoothed(s,IER)*FACT(s)
             else
c                RSDAT=RS40tree(s,IER)*FACT(s)
                RSDAT=RS40(s,IER)*FACT(s)
             endif
          ENDIF
       ENDIF
c       write (*,*) ' RSDAT:',sqrt(s),RSDAT,RSBGI(S),ICH
       RETURN
       END
C
      FUNCTION RS40tree(s,IER)
      implicit none
      integer IER
      double precision s,RS40tree
      RS40tree=11.d0/3.d0
      if (IER.GT.1) then
         RS40tree=11.d0/3.d0*0.01d0
      endif
      RETURN
      END
C
       FUNCTION RSBGI(S)
C      -----------------
C Background: Integrand
c In case of PV pole in data set (time-like energies below ECUT) we need
c spline or least square fit in interval (POLM,POLP) around the pole: in
c that interval RSBGI=0 if IPV=0 and RSBGI=spline (ISPLINE=1) or fit if
c IPV=1; outside that interval RSBGI=linear interpolation of data
c zero in pQCD regions
c zero in pole region [POLM,POLP] and IPV=0
c non-zero represended by fit in pole region [POLM,POLP] when IPV=1
c non-zero represented by data outside pole region
       IMPLICIT NONE
       INTEGER IER,IKE,IPV,IREN,IRENO,ICH
       REAL *8 RSBGI,RS40,RS40x,RS40smoothed,FACT
       REAL *8 S,POLP,POLM,E,CHPTCUT,EC,ECUT
       COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
       COMMON/CHAN/ICH
       COMMON/RCUTS/CHPTCUT,EC,ECUT
       EXTERNAL RS40,RS40x,RS40smoothed,FACT
       E=DSQRT(S)
       RSBGI=0.d0
       IF (((E.GT.EC).and.(E.LT.9.5d0)).or.(E.GT.ECUT)) THEN
          RSBGI=0.d0
          RETURN
       ENDIF
       IF ((S.GT.POLM).AND.(S.LT.POLP)) THEN
          IF (IPV.EQ.0) THEN
             RSBGI=0.D0
          ELSE IF (IPV.EQ.1) THEN
             RSBGI=RS40smoothed(s,IER)*FACT(s)
          ENDIF
       ELSE
          ICH=8
          RSBGI=RS40(s,IER)*FACT(s)
       ENDIF
c       write (*,*) ' RSBGI: RS40x',E,RSBGI/FACT(S)
       RETURN
       END
C
       FUNCTION RSB3G(S)
C      -----------------
C Background: Integrand
c 3g version of RSBGI i.e. in range 0.6 - 2.0 RS3G data
c nonzero in range 0.61 to 2.0
c to be used in pole interval [POLM,POLP] and when IPV=1 where RS3GDAT is set zero
       IMPLICIT NONE
       INTEGER IER,IKE,IPV,IREN,IRENO,Ismooth,
     &      itestflavorsplitting,icomplement
       integer inde(10)
       REAL *8 RS3G,FACT,RSB3G,RS40,RS40smoothed,RS3Gsmoothed
       REAL *8 S,E,CHPTCUT,EC,ECUT,POLP,POLM,MPOL,ELOW,EHIGH
       REAL *8 EL(15),EH(15)
       COMMON /RANGES/EL,EH,inde
       COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
       COMMON/RCUTS/CHPTCUT,EC,ECUT
       COMMON/POLE/MPOL
       COMMON/SMOO/Ismooth
       common /testflavor/itestflavorsplitting,icomplement
       EXTERNAL RS3G,FACT,RS40,RS40smoothed,RS3Gsmoothed
       E=DSQRT(S)
       ELOW =EL(2)
       EHIGH=EL(6)
       RSB3G=0.d0
       IF (((E.GT.EC).and.(E.LT.9.5d0)).or.(E.GT.ECUT)) THEN
          RSB3G=0.d0
          RETURN
       ENDIF
       IF ((S.GT.POLM).AND.(S.LT.POLP)) THEN
          IF (IPV.EQ.0) THEN
             RSB3G=0.D0
          ELSE IF (IPV.EQ.1) THEN
c             IF ((MPOL.GT.ELOW).and.(MPOL.LT.EHIGH)) THEN
c extrapolate to full pole region [POLM,POLP]
             RSB3G=RS3Gsmoothed(s,IER)*FACT(s)
c             ELSE
c                RSB3G=RS3Gsmoothed(s,IER)*FACT(s)
c             ENDIF
          ENDIF
       ELSE
          if (Ismooth.eq.1) then
             RSB3G=RS3Gsmoothed(s,IER)*FACT(s)
          else
             RSB3G=RS3G(s,IER)*FACT(s)
          endif
c          ICH=8
c          IF ((E.GT.ELOW).and.(E.LT.EHIGH)) THEN
c          ELSE
c             RSB3G=RS3G(s,IER)*FACT(s)
c          ENDIF
       ENDIF
       RETURN
       END
C
       FUNCTION RSB33(S)
C      -----------------
C Background: Integrand
c 3g version of RSBGI i.e. in range 0.6 - 2.0 RS33 data
c nonzero in range 0.61 to 2.0
c to be used in pole interval [POLM,POLP] and when IPV=1 where RS33DAT is set zero
       IMPLICIT NONE
       INTEGER IER,IKE,IPV,IREN,IRENO,Ismooth,
     &      itestflavorsplitting,icomplement
       integer inde(10)
       REAL *8 RS33,FACT,RSB33,RS40,RS40smoothed,RS33smoothed
       REAL *8 S,E,CHPTCUT,EC,ECUT,POLP,POLM,MPOL,ELOW,EHIGH
       REAL *8 EL(15),EH(15)
       COMMON /RANGES/EL,EH,inde
       COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
       COMMON/RCUTS/CHPTCUT,EC,ECUT
       COMMON/POLE/MPOL
       COMMON/SMOO/Ismooth
       common /testflavor/itestflavorsplitting,icomplement
       EXTERNAL RS33,FACT,RS40,RS40smoothed,RS33smoothed
       E=DSQRT(S)
       ELOW =EL(2)
       EHIGH=EL(6)
       RSB33=0.d0
       IF (((E.GT.EC).and.(E.LT.9.5d0)).or.(E.GT.ECUT)) THEN
          RSB33=0.d0
          RETURN
       ENDIF
       IF ((S.GT.POLM).AND.(S.LT.POLP)) THEN
          IF (IPV.EQ.0) THEN
             RSB33=0.D0
          ELSE IF (IPV.EQ.1) THEN
c             IF ((MPOL.GT.ELOW).and.(MPOL.LT.EHIGH)) THEN
c extrapolate to full pole region [POLM,POLP]
             RSB33=RS33smoothed(s,IER)*FACT(s)
c             ELSE
c                RSB33=RS33smoothed(s,IER)*FACT(s)
c             ENDIF
          ENDIF
       ELSE
          if (Ismooth.eq.1) then
             RSB33=RS33smoothed(s,IER)*FACT(s)
          else
             RSB33=RS33(s,IER)*FACT(s)
          endif
c          ICH=8
c          IF ((E.GT.ELOW).and.(E.LT.EHIGH)) THEN
c          ELSE
c             RSB33=RS33(s,IER)*FACT(s)
c          ENDIF
       ENDIF
       RETURN
       END
C
       FUNCTION FACT(S)
C      ----------------
C      INTEGRAL KERNELS (for PV-NAG-Routine D01AQF = Kernel*Polefactor)
C Note RENORMALIZEc via some subprogram changes IER into IER=3; here fixed by IER_SAV trick
C as a function in intRdatx.f  which uses data collected in Rdat_all.f which all
C are VP subtractsd via hadr5n09 which largely agrees with hadr5n14, so only parametric BW resonances
C need be renormalized. This now is done in the BW function routine
C hence: in FACT no renormalization at all is the be done
       IMPLICITREAL*8(A-Z)
       INTEGER IER,IER_SAV,IKE,IPV,IREN,IRENO,IGEV,irennew,iresonances,
     &      iresonances_sav,IRESON,IRESON_SAV,NNLO,IBWR,NFL
       REAL*8 ML(3)
       real*8 FK,k2a,k2b,k2aele,k3a,k3b,k3bLbL,k3c
       real*8 de,ddk3c,derrdk3c,MU2I,MU2I_SAV
       real*8 dECgg,derrstaECgg,derrsysECgg,dt,ECLQCDKERN
       COMMON/MFPI/MP2,M2/FFF/IKE,IPV,IREN,IRENO
       COMMON/ERRK3C/IER
       COMMON/RES/iresonances
       COMMON/RESFIT/facbw,IRESON
       common/lema/ml,IGEV
       common /rennew/irennew
       common /fermi/fm
       common /flavor/NFL
       COMMON /NNLO/ADP,NNLO
       COMMON/MFK/MU2I
       COMMON /BWRF/IBWR
       external FK,k2a,k2b,k2aele,k3a,k3b,k3bLbL,k3c,ECLQCDKERN
       IF (IKE.EQ.-1) THEN
          FACT=1.d0
          RETURN
       ENDIF
       IER_SAV=IER
c save iresonances and reset on return
c in case iresonances is changed by renormalize2c via alphaQED routines
       iresonances_sav=iresonances
       IRESON_SAV=IRESON
       MU2I_SAV=MU2I
c       fm=1.d0/0.1973269631d0   ! fermi in GeV^{-1}
       t=0.85*fm
       E0=1.d0/t
       E0=sqrt(M2)
       reno1=1.d0
       reno2=1.d0
       E=dsqrt(dabs(s))
       mm2=ml(2)**2
       me2=ml(1)**2
       EGeV=E
       if (IGEV.EQ.0) then
          mm2=mm2*1.d6
          me2=me2*1.d6
c          EGeV=E*1.d-3
          MU2I=MU2I/1.d6
       else
       endif
c       write (499,*) ' FACT: E,E0,E/E0=',E,E0,E/E0
c       if (irennew.eq.-1) then
c          reno=1.d0
c       else
cc renotab2 utilizes space-like effective charge above 3 GeV in reno2
c          CALL RENORMALIZE2c(E,reno1,reno2)
c          if (irennew.eq.0) reno2=1.d0
c          reno=reno1*reno2
c       endif
       FPO=1.D0
       reno=1.d0
c       if (IBWR.EQ.1) then
c       write (*,*) ' FACT:sqrt(M2),E,mm',sqrt(M2),sqrt(s),sqrt(mm2)
c       write (*,*) ' FACT:IKE,IPV,IER',IKE,IPV,IER
c       endif
       IF((IPV.EQ.1).AND.((IKE.EQ.1).OR.(IKE.EQ.2))) THEN
          FPO =S-M2
          IF(IKE.EQ.1) FACT=M2/S*reno
          IF(IKE.EQ.2) FACT=M2/(S-M2)*reno
       ELSE
          FPO=1.D0
          IF(IKE.EQ.0) FACT=FK(S)/ S**2*reno
          IF(IKE.EQ.3) FACT=FK(S)/ S**2*reno
          IF(IKE.EQ.4) FACT=FK(S)/ S**2*reno
          IF(IKE.EQ.1) FACT=M2/S/(S-M2)*reno
          IF(IKE.EQ.2) FACT=M2/(S-M2)/(S-M2)*reno
          IF (NNLO.EQ.1) THEN
             IF(IKE.EQ.5) FACT=k3a(s,mm2)/ S*FPO*reno
             IF(IKE.EQ.6) FACT=k3b(s,mm2,me2)/ S*FPO*reno
             IF(IKE.EQ.7) FACT=k3bLbL(s,mm2,me2)/ S*FPO*reno
          ELSE
             IF(IKE.EQ.5) FACT=k2a(s,mm2)/ S*FPO*reno
             IF(IKE.EQ.6) FACT=k2b(s,mm2,me2)/ S*FPO*reno
             IF(IKE.EQ.7) FACT=k2aele(s,me2)/ S*FPO*reno
          ENDIF
          IF(IKE.EQ.8) FACT=0.5d0*E*DEXP(-E/E0)*reno/E0**3
          IF(IKE.EQ.9) FACT=0.5d0*E**2*DEXP(-E/E0)*reno/E0**4
          IF(IKE.EQ.10) FACT=1.0d0/S**2*reno
          IF(IKE.EQ.11) FACT=k3c(s,M2,mm2,me2)/ S*FPO*reno
          IF(IKE.EQ.12) then
             de=E
             if (IBWR.EQ.1) then
                de=E*1.d-3
             endif
             IF (IER.EQ.1) THEN
                call dhadrK3cdist15(de,ddk3c,derrdk3c)
                FACT=1.0d0/S*FPO*ddk3c
             ELSE IF (IER.EQ.3) THEN
                call dhadrK3cdist15(de,ddk3c,derrdk3c)
                FACT=1.0d0/S*FPO*derrdk3c
             ELSE
                FACT=1.0d0/S*FPO*1d-16
             ENDIF
          ENDIF
          IF(IKE.EQ.13) FACT=k3a(s,me2)/ S*FPO*reno
c K3c variables
c      Mel=sqrt(mf2)
c      Mmu=sqrt(m2)
c      Ms =sqrt(s)
c      Msp=sqrt(t)
       ENDIF
       MU2I=MU2I_SAV
       IER=IER_SAV
       iresonances=iresonances_sav
       IRESON=IRESON_SAV
       RETURN
       END
C
       FUNCTION FK(S)
C      --------------
C      KERNEL FOR MUON ANOMALOUS MAGNETIC MOMENT
       IMPLICITREAL*8(A-Z)
       COMMON/MFK/MU2I
       A=S*MU2I
       IF (A.GE.4.D0) THEN
          W=DSQRT(1.D0-4.D0/A)
          X=(1.D0-W)/(1.D0+W)
          DLX=DLOG(X)
          IF (X.LT.0.6D-3) THEN
            FK=(1.D0/3.D0+(17.D0/12.D0+(11.D0/30.D0+(-0.1D0+3.D0/70.D0
     .        *X)*X)*X)*X)*X+(1.D0+X)/(1.D0-X)*X**2*DLX
          ELSE
            FK=X**2/2.D0*(2.D0-X**2)+1.D0/X**2*(1.D0+X)**2*(1.D0+X**2)*
     .        (DLOG(1.D0+X)-X+X**2/2.D0)+(1.D0+X)/(1.D0-X)*X**2*DLX
          ENDIF
       ELSE IF (A.LE.0.D0) THEN
          WRITE (*,*) ' G-2 KERNEL NOT DEFINED'
       ELSE
          W=DSQRT(4.D0/A-1.D0)
          PHI=2.D0*DATAN(W)
          FK=0.5D0-A+0.5D0*A*(A-2.D0)*DLOG(A)
     &      -PHI/W*(1.D0-A+0.5D0*A*(A-2.D0))
       ENDIF
       FK=3.D0*MU2I*S*FK
       RETURN
       END
C
       SUBROUTINE SIMPS(FCT,A,B,NIN,ANS)
C      ###############################
C      SIMPSON-Rule integration Routine
       IMPLICITREAL*8(A-Z)
       INTEGER NIN,N,J
       N=NIN
       IF (NIN.LE.1) N=3
       H=(B-A)/(2*N)
       SE=0.D0
       SO=0.D0
       DO 10 J=1,N-1
       XE=A+H*2*J
       XO=XE-H
       SE=SE+FCT(XE)
       SO=SO+FCT(XO)
10     CONTINUE
       SO=SO+FCT(XE+H)
       ANS=H/3.D0*(FCT(A)+4.D0*SO+2.D0*SE+FCT(B))
c       write (*,*) ' SIMPS:',sqrt(A),sqrt(B),ANS
       RETURN
       END
C
C**************FITFCT***************************
C
      FUNCTION FITFCT(E,A)
c Fit function y(x) = sum_i a_i f_i(x)
      IMPLICIT NONE
      REAL *8 FITFCT,E,A,MP2,M2,M,XMIN,XMAX
      DIMENSION A(3)
      COMMON/MFPI/MP2,M2/RAND/XMIN,XMAX
      M=SQRT(M2)
      IF ((M.LE.XMIN).OR.(M.GE.XMAX)) M=(XMAX+XMIN)/2.d0
      FITFCT=A(1)+A(2)*(E-M)+A(3)*(E-M)**2
      RETURN
      END
C
C*****************FCT***************************
C
      FUNCTION FCT(I,X)
c Elements f_i(x) of fit function y
      IMPLICIT NONE
      INTEGER I
      REAL *8 FCT,X,MP2,M2,M,XMIN,XMAX
      COMMON/MFPI/MP2,M2/RAND/XMIN,XMAX
      M=SQRT(M2)
      IF ((M.LE.XMIN).OR.(M.GE.XMAX)) M=(XMAX+XMIN)/2.d0
      IF (I.EQ.1) THEN
         FCT=1.d0
      ELSE
         FCT=(X-M)**(I-1)
      ENDIF
      RETURN
      END
c
       FUNCTION ROREI(S)
C      ----------------
C RHO-Resonance:Integrand
C Conversion of |F_Pi|^2 to R!
C      FAPP =(SQRT(1.D0-4.D0*MP2/S))**3/4.D0
       IMPLICITREAL*8(A-Z)
       character*4 DSET
       CHARACTER*3 LEPTONflag
       INTEGER IKE,IPV,IREN,IRENO,IRENIN,iopt,iso,IUNMIX
       COMMON/MFPI/MP2,M2/FFF/IKE,IPV,IREN,IRENO
       COMMON/MMP/MPLUS,EPPMIN,EPPMAX
       common /rhowidth/iopt,iso
       COMMON /RGMIX/IUNMIX
       external grmixcorr
       external fpi2rnew
C Reno is applied here, keep current IREN on return
C Only hadronic contribution has not yet been subtracted in |F_Pi|^2
C IREN=4 : switch off reno in function FACT
       E=SQRT(S)
       IF ((E.GT.EPPMIN).AND.(E.LT.EPPMAX)) THEN
         ROREI=(DSQRT(1.D0-4.D0*MP2/S))**3*
     &         fpi2rnew(s,iso,iopt)/4.D0*FACT(S)
c typical pipi form factor errors
         IF (IER.EQ.2) ROREI=ROREI*0.02d0
         IF (IER.EQ.3) ROREI=ROREI*0.006d0
       ELSE
         ROREI=0.D0
       ENDIF
       if ((IUNMIX.EQ.1).and.(E.LT.1000.d0)) then
          grfac=grmixcorr(E)
          ROREI=ROREI/grfac
       endif
       RETURN
       END
C
       FUNCTION OMREI(S)
C      -----------------
C OMEGA-Resonance:Integrand
       IMPLICITREAL*8(A-Z)
       INTEGER IBWR
       COMMON/MFPI/MP2,M2/BWOM/M,G,P
       COMMON /BWRF/IBWR
       IBWR=1
       E=DSQRT(S)
       IF (E.LE.0.81D3) THEN
         OMREI=BW(S,M,G,P)*FACT(S)
       ELSE
         OMREI=0.0D0
       ENDIF
       IBWR=0
       RETURN
       END
C
       FUNCTION FIREI(S)
C      -----------------
C PHI-Resonance:Integrand
       IMPLICITREAL*8(A-Z)
       INTEGER IBWR
       COMMON/MFPI/MP2,M2/BWFI/M,G,P
       COMMON /BWRF/IBWR
       IBWR=1
       E=DSQRT(S)
       IF ((E.GE.1.00D3).AND.(E.LE.1.04D3)) THEN
         FIREI=BW(S,M,G,P)*FACT(S)
       ELSE
         FIREI=0.0D0
       ENDIF
       IBWR=0
       RETURN
       END
C
       FUNCTION BWREI(S)
C      -----------------
C PHI-Resonance:Integrand
       IMPLICITREAL*8(A-Z)
       INTEGER IBWR
       COMMON/MFPI/MP2,M2/BWPAR/M,G,P
       COMMON /BWRF/IBWR
       IBWR=1
       BWREI=BW(S,M,G,P)*FACT(S)
c       write (*,*) 'BWREI:M,E:',M,Sqrt(s)
       IBWR=0
       RETURN
       END
C
      subroutine resonances(E,RES,STA,SYS,STA2,SYS2)
c calculate resonance contributions from omega, phi, Psi(i), i=1,2,3 and Yps (i) i=1,2,3,4,5,6
      IMPLICIT NONE
      logical isinf
      CHARACTER *3 LEPTONflag1
      INTEGER     LW,       LIW
      PARAMETER  (LW=320000,LIW=LW/4)
      INTEGER IE,IFAIL,KOUNT,IW(LIW)
      REAL *8 W(LW)
      INTEGER J,IKE,IPV,IREN,IRENO,IREN_SAV,IRENO_SAV,NP,NPS,NYP,
     &     IPV_SAV,IRENBW,IRENBW_SAV,IER,IER_SAV
      REAL *8 PIx,ALPx,FBW,GBW,UMG,UGM,UMG2,UGM2,E,STA2,SYS2,NULL
      REAL *8 MOM,MFI,GRO,GOM,GFI,PRO,POM,PFI
      REAL*8 PSI(6),MPS(6),GPS(6),PPS(6),YPI(6),MYP(6),GYP(6),PYP(6)
      REAL*8 STAPS(6),SYSPS(6),STAYP(6),SYSYP(6)
      REAL*8 FSTA(5),FSYS(5),RES(6),STA(6),SYS(6)
      real *8 st2,als,mtop,dalept,dahadr,daltop
      REAL *8 me,ME2,COSTHETA,FA,MPLUS,MP2,M2,M3,A,B,AL,BL,
     &     EOMMIN,EOMMAX,EFIMIN,EFIMAX,ROREI,OMREI,FIREI,EPSREL,RESULT,
     &     ABSERR,RESNUM,BWINZ,BWINT,DBWIZ,RX,MM,GG,FACT,RESULT1
      double precision pi3,ALINP,ERRAL,EINP,MTOP1
      REAL*8 EMIPS(6),EMAPS(6),EMIYP(6),EMAYP(6),EOMM,EOMP,EFIM,EFIP
      REAL *8 BWM,BWG,BWP,PSINWA(6),YPINWA(6)
      REAL *8 XX0,ST0,SY0,EPPMIN,EPPMAX,ACUT,BCUT,Dalphaweak1MSb
c,XX1,ST1,SY1,XX2,ST2,SY2
      integer irennew,irennew_sav,iopt,iso,iopt_sav,iso_sav,iomegaphidat
      integer icountps(6),icountyp(6)
      common /icountre/icountps,icountyp
      common /rennew/irennew
      common /BWPAR/BWM,BWG,BWP
      COMMON /renoBW/IRENBW
      EXTERNAL ROREI,OMREI,FIREI,BWREI,FACT,BWINZ,BWINT,DBWIZ
***************** part to be included also in main program **************
       include 'common.h'
       common /parm/st2,als,mtop
       common /resu/dalept,dahadr,daltop,Dalphaweak1MSb
       common /rhowidth/iopt,iso
       COMMON/FFF/IKE,IPV,IREN,IRENO
C Parameters for this routine: LEPTONflag1 only switches between ele and lem=ele+muo
       COMMON/BHAB/ME2,COSTHETA,LEPTONflag1
       COMMON/MFPI/MP2,M2
       COMMON/MMP/MPLUS,EPPMIN,EPPMAX
       COMMON/BWOM/MOM,GOM,POM/BWFI/MFI,GFI,PFI
       COMMON/CSTS/PIx,ALPx,FA
C cpmmon Psi and Upsilon parameters set in subroutine resonances
       COMMON/PSYPPAR/MPS,GPS,PPS,MYP,GYP,PYP,STAPS,SYSPS,STAYP,SYSYP
       COMMON/RESDOMAINS/EMIPS,EMAPS,EMIYP,EMAYP,EOMM,EOMP,EFIM,EFIP
       COMMON/RESRELERR/FSTA,FSYS
       common/var1/pi3,ALINP,ERRAL,EINP,MTOP1
       COMMON /NAGRELERR/EPSREL
       COMMON /NWA/PSI,PSINWA,YPI,YPINWA
       COMMON /OMEPHIDAT/iomegaphidat
       common /energyindex/IE
       COMMON/ERR/IER
       NULL=0.d0
       call constants()
       call constants_qcd()     ! fills commons var and RCUTS
c     sin2ell=0.23153 ! pm 0.00016 LEPEEWG Phys Rep 427 (2006) 257
c to be set in main program
c       st2=0.23153d0            ! Reference value for weak mixing parameter
       als=ALINP                ! alpha strong
       mtop=MTOP1               ! top quark mass
       me=ml(1)
       me2=me*me
       COSTHETA=0.6D0
       LEPTONflag1='ele'
C LEPTONflag=all,had,lep,ele,muo,tau,lem -> iLEP=-3,-2,-1,1,2,3,4
C Some VP subtractuions only include the electron = ele or electron plus muon = lem
C Default:
C      LEPTONflag='all'
C      iLEP  = -3  ! for sum of leptons + quarks
C Decomment in main the folowwing line for changing the default
c      LEPTONflag='all'   ! choose had,lep,top,ele,muo,tau,lem
************************************************************************
c Set parameters in above common blocks: BWOM,BWFI,PSYPPAR, RESRELERR
c rho, omega, phi, J/psi and Upsilon series
c
       call resonances_data()
c
c called from ..../for/vap/qed/resonances_dat.f
************************************************************************
c
      UMG=1.D-3                 ! convert MeV into GeV
      UGM=1.D3                  ! convert GeV into MeV
      UMG2=1.D-6
      UGM2=1.D6
      MPLUS=139.56995D0
      MP2=MPLUS**2
c
C conversion of Breit-Wigner FUNCTION BW() to R!
      FBW=(3.D0/2.D0/ALP)**2
C conversion R to Adler
      GBW=4.D0*PI*FBW
      M2=E*ABS(E)*UGM2
      do j=1,6
         res(j)=NULL
         sta(j)=NULL
         sys(j)=NULL
         PSI(j)   =NULL
         PSINWA(j)=NULL
         YPI(j)   =NULL
         YPINWA(j)=NULL
      enddo
c save environment flags
      IPV_SAV=IPV
      IREN_SAV=IREN
      IRENO_SAV=IRENO
      irennew_sav=irennew
      iopt_sav=iopt
      iso_sav=iso
      IRENBW_SAV=IRENBW
      IER_SAV=IER
      IER=1
C rho is not included as a narrow width resonance
C Here we need it to calculate the I=0 part to be subtracted in the SU(2) coupling
C need omega region only
      IFAIL=0
      EPPMIN=695.d0
      EPPMAX=880.d0
      A=EPPMIN**2
      B=EPPMAX**2
      iopt=11
      iso=0
c      write (*,*) ' ROREI Calling:',sqrt(a),sqrt(b),sqrt(m2)
      IF(((M2.GT.A).AND.(M2.LT.B)).AND.((IKE.EQ.1).OR.(IKE.EQ.2)))THEN
         IPV=1
         IFAIL=-1
         CALL D01AQF(ROREI,A,B,M2,0.D0,EPSREL,RESULT,ABSERR,
     .        W,LW,IW,LIW,IFAIL)
         IF (IFAIL.NE.0) WRITE (*,99996) 'resonances -- 1:', IFAIL
      ELSE
         IPV=0
         IFAIL=-1
         CALL D01AJF(ROREI,A,B,0.D0,EPSREL,RESULT,ABSERR,
     .        W,LW,IW,LIW,IFAIL)
         IF (IFAIL.NE.0) WRITE (*,99996) 'resonances -- 1:', IFAIL
      ENDIF
      XX0=RESULT*FA
      RX=DABS(XX0)
      ST0=FSTA(1)*RX
      SY0=FSYS(1)*RX
      RES(1)=XX0
      STA(1)=ST0
      SYS(1)=SY0
C      *******************
C      OMEGA,PHI BW-Resonances
C Normalization factor:
C FA*FBW is -3/(16*pi^2*alpha^2)*1000(scale factor) if multiplied by
C e^2=4 pi alpha we have   -3/(4*pi*alpha)*1000. The narrow width contribution
C to "Re Pi_had" is FA*FBW*4*pi*GR*PR/MR/1000
C Assume widths quoted by PDG are the physical widths ->
C apply factor reno=(alpha/alpha(s))**2 -> (IREN=2 means full vapo subtraction)
      IFAIL=0
c      IRENO=-1
      IREN=2                    ! with physical (dressed) width from PDG
      irennew=0  ! for resonance integration assuming physical widths
      IF (iomegaphidat.EQ.1) THEN
c         RES(2)=null
c         RES(3)=null
c         STA(2)=null
c         STA(3)=null
c         SYS(2)=null
c         SYS(3)=null
         STA2=null
         SYS2=null
      ELSE
      M3=E*UGM
      AL=3.D0*MPLUS
      IF (M3.LE.AL.AND.M3.GT.AL-5.D0*GOM) AL=AL+5.D0*GOM
      IF (M3.GT.AL.AND.M3.LT.AL+5.D0*GOM) AL=AL-5.D0*GOM
      BL=0.81D3
      IF (ABS(M3-BL).LT.2.5D2) BL=BL+5.0D2
      EOMMIN=AL
      EOMMAX=BL
c change end feb 2012 paremeters set in resonances_dat
      AL=EOMM*UGM
      BL=EOMP*UGM
      A =AL**2
      B =BL**2
      IF(((M2.GT.A).AND.(M2.LT.B)).AND.((IKE.EQ.1).OR.(IKE.EQ.2)))THEN
         IPV=1
         IFAIL=-1
         CALL D01AQF(OMREI,A,B,M2,0.D0,EPSREL,RESULT,ABSERR,
     .        W,LW,IW,LIW,IFAIL)
         IF (IFAIL.NE.0) WRITE (*,99996) 'resonances -- 2:', IFAIL
      ELSE
         IPV=0
         IFAIL=-1
         CALL D01AJF(OMREI,A,B,0.D0,EPSREL,RESULT,ABSERR,
     .        W,LW,IW,LIW,IFAIL)
         IF (IFAIL.NE.0) WRITE (*,99996) 'resonances -- 2:', IFAIL
      ENDIF
      RES(2)=RESULT*FA*FBW
      RX=ABS(RES(2))
      STA(2)=FSTA(2)*RX
      SYS(2)=FSYS(2)*RX
      STA2=STA2+STA(2)**2
      SYS2=SYS2+SYS(2)**2
c Phi
      EFIMIN=1.00D3
      EFIMAX=1.04D3
      AL=EFIMIN
      BL=EFIMAX
c change end feb 2012 parameters set in resonances_dat
      AL=EFIM*UGM
      BL=EFIP*UGM
      A =AL**2
      B =BL**2
      IF(((M2.GT.A).AND.(M2.LT.B)).AND.((IKE.EQ.1).OR.(IKE.EQ.2)))THEN
         IPV=1
         IFAIL=-1
         CALL D01AQF(FIREI,A,B,M2,0.D0,EPSREL,RESULT,ABSERR,
     .        W,LW,IW,LIW,IFAIL)
         IF (IFAIL.NE.0) WRITE (*,99996) 'resonances -- 3:', IFAIL
      ELSE
         IPV=0
         IFAIL=-1
         CALL D01AJF(FIREI,A,B,0.D0,EPSREL,RESULT,ABSERR,
     .        W,LW,IW,LIW,IFAIL)
         IF (IFAIL.NE.0) WRITE (*,99996) 'resonances -- 3:', IFAIL
      ENDIF
      RES(3)=RESULT*FA*FBW
      RX=ABS(RES(3))
      STA(3)=FSTA(3)*RX
      SYS(3)=FSYS(3)*RX
      STA2=STA2+STA(3)**2
      SYS2=SYS2+SYS(3)**2
      ENDIF
C     PSI-FAMILY   NARROW WIDTH
      AL=2.09D3
      IF (ABS(M3-AL).LT.0.5D3) AL=2.D3
      BL=5.D3
      IF (ABS(M3-BL).LT.1.0D3) BL=7.D3
      DO NPS=1,3
         IF (IRENBW.EQ.2) THEN
            IF (NPS.LE.2) IRENBW=1
            IF (NPS.GT.2) IRENBW=2
         ENDIF
         BWM=MPS(NPS)
         BWG=GPS(NPS)
         BWP=PPS(NPS)
         A=EMIPS(NPS)**2*UGM2
         B=EMAPS(NPS)**2*UGM2
c         write (*,*) ' reso J/Psi',NPS
c         write (*,*) E,BWM,BWG,BWP,EMIPS(NPS),EMAPS(NPS)
         IPV=0
         IF (IKE.EQ.1) THEN
            PSINWA(NPS)=BWINZ(M3,MPS(NPS),GPS(NPS),PPS(NPS),AL,BL)
     &           *FA*FBW
         ELSE IF (IKE.EQ.2) THEN
            PSINWA(NPS)=DBWIZ(M3,MPS(NPS),GPS(NPS),PPS(NPS),AL,BL)
     &           *FA*FBW
         ELSE
            MM=MPS(NPS)**2
            GG=GPS(NPS)
            PSINWA(NPS)=MM*GG*PPS(NPS)/MPS(NPS)*FACT(MM)*FA*GBW
         ENDIF
         IF (((M2.GT.A).AND.(M2.LT.B)).AND.((IKE.EQ.1).OR.(IKE.EQ.2)))
     &        THEN
            IPV=1
            IFAIL=-1
            CALL D01AQF(BWREI,A,B,M2,0.D0,EPSREL,RESULT,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            PSI(NPS)=RESULT*FA*FBW
            IF (IFAIL.NE.0) WRITE (*,99996) 'resonances -- 4:', IFAIL
            IPV=0
         ELSE
C renormalization of BW by time-like strongly varying alpha_eff requires
C numerical integration by NAG routine, here PV integral may cause problems
            IFAIL=-1
            CALL D01AJF(BWREI,A,B,0.D0,EPSREL,RESULT,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            PSI(NPS)=RESULT*FA*FBW
            IF (IFAIL.NE.0) WRITE (*,99996) 'resonances -- 4:', IFAIL
         ENDIF
         STA(4)=SQRT(STA(4)**2+(ABS(PSINWA(NPS))*STAPS(NPS))**2)
         SYS(4)=SQRT(SYS(4)**2+(ABS(PSINWA(NPS))*SYSPS(NPS))**2)
         if ((isinf(PSI(NPS))).or.(isnan(PSI(NPS))))
     &        PSI(NPS)=PSINWA(NPS)
         if ((E.GE.EMIPS(NPS)).AND.(E.LE.EMAPS(NPS))) then
            icountps(nps)=icountps(nps)+1
            write (110+NPS,124) E,PSI(NPS),PSINWA(NPS)
         endif
      ENDDO
      RES(4)=PSI(1)+PSI(2)+PSI(3)
      STA2=STA2+STA(4)**2
      SYS2=SYS2+SYS(4)**2
C     YPSILON-FAMILY   NARROW WIDTH
      AL=9.D3
      IF (ABS(M3-AL).LT.0.5D3) AL=8.D3
      BL=12.D3
      IF (ABS(M3-BL).LT.1.0D3) BL=14.D3
      DO NYP=1,6
         IF (IRENBW.EQ.2) THEN
            IF (NYP.LE.3) IRENBW=1
            IF (NYP.GT.3) IRENBW=2
         ENDIF
         BWM=MYP(NYP)
         BWG=GYP(NYP)
         BWP=PYP(NYP)
         A=EMIYP(NYP)**2*UGM2
         B=EMAYP(NYP)**2*UGM2
         IPV=0
         IF (IKE.EQ.1) THEN
            YPINWA(NYP)=BWINZ(M3,MYP(NYP),GYP(NYP),PYP(NYP),AL,BL)
     &           *FA*FBW
         ELSE IF (IKE.EQ.2) THEN
            YPINWA(NYP)=DBWIZ(M3,MYP(NYP),GYP(NYP),PYP(NYP),AL,BL)
     &           *FA*FBW
         ELSE
            MM=MYP(NYP)**2
            GG=GYP(NYP)
            YPINWA(NYP)=MM*GG*PYP(NYP)/MYP(NYP)*FACT(MM)*FA*GBW
         ENDIF
c         IF (((E.EQ.10.473D0).OR.(IE.EQ.1823)).OR.
c     &        ((E.EQ.10.687D0).OR.(IE.EQ.1893))) THEN
c            YPI(NPS)=YPINWA(NYP)
c         ELSE
         IF (((M2.GT.A).AND.(M2.LT.B)).AND.((IKE.EQ.1).OR.(IKE.EQ.2)))
     &           THEN
            IPV=1
            IFAIL=-1
            CALL D01AQF(BWREI,A,B,M2,0.D0,EPSREL,RESULT,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            YPI(NYP)=RESULT*FA*FBW
            IF (IFAIL.NE.0) WRITE (*,99996) 'resonances -- 5:', IFAIL
            IPV=0
         ELSE
            IFAIL=-1
            CALL D01AJF(BWREI,A,B,0.D0,EPSREL,RESULT,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            YPI(NYP)=RESULT*FA*FBW
            IF (IFAIL.NE.0) WRITE (*,99996) 'resonances -- 5:', IFAIL
         ENDIF
         STA(5)=SQRT(STA(5)**2+(ABS(YPINWA(NYP))*STAYP(NYP))**2)
         SYS(5)=SQRT(SYS(5)**2+(ABS(YPINWA(NYP))*SYSYP(NYP))**2)
         if ((isinf(YPI(NPS))).or.(isnan(YPI(NPS))))
     &        YPI(NPS)=YPINWA(NPS)
         if ((E.GE.EMIYP(NYP)).AND.(E.LE.EMAYP(NYP))) then
            icountyp(nyp)=icountyp(nyp)+1
            write (120+NYP,124) E,YPI(NYP),YPINWA(NYP)
         endif
      ENDDO
      RES(5)=YPI(1)+YPI(2)+YPI(3)+YPI(4)+YPI(5)+YPI(6)
      STA2=STA2+STA(5)**2
      SYS2=SYS2+SYS(5)**2
      IPV=IPV_SAV
      IREN=IREN_SAV
      IRENO=IRENO_SAV
      irennew=irennew_sav
      iopt=iopt_sav
      iso=iso_sav
      IRENBW=IRENBW_SAV
      IER=IER_SAV
      RETURN
 124  format (1X,3(2x,1pe11.4))
99996 FORMAT (1X,A,I4)
      END

      FUNCTION dfridr1(func,ipar,x,h,err)
      INTEGER NTAB,ipar,ipar1
      REAL *8 dfridr1,err,h,x,func,CON,CON2,BIG,SAFE
      PARAMETER (CON=1.4d0,CON2=CON*CON,BIG=1.d30,NTAB=10,SAFE=2.d0)
      EXTERNAL func
CU    USES func
      INTEGER i,j
      REAL *8 errt,fac,hh,a(NTAB,NTAB)
      COMMON /NPAR/ipar1
      ipar1=ipar
c      if(h.eq.0.) pause 'h must be nonzero in dfridr'
      hh=h
      a(1,1)=(func(x+hh)-func(x-hh))/(2.0d0*hh)
      err=BIG
      do 12 i=2,NTAB
        hh=hh/CON
        a(1,i)=(func(x+hh)-func(x-hh))/(2.0d0*hh)
        fac=CON2
        do 11 j=2,i
          a(j,i)=(a(j-1,i)*fac-a(j-1,i-1))/(fac-1.)
          fac=CON2*fac
          errt=max(abs(a(j,i)-a(j-1,i)),abs(a(j,i)-a(j-1,i-1)))
          if (errt.le.err) then
            err=errt
            dfridr1=a(j,i)
          endif
11      continue
        if(abs(a(i,i)-a(i-1,i-1)).ge.SAFE*err)return
12    continue
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software -1$393.

      subroutine MYPVINT(MPOL,POLP,POLM,RESPVINT,ERRT)
      implicit none
      INTEGER ipar,i,N,N1,IER,IER_SAV,IERx
      PARAMETER(N=100)
      REAL *8 dfridr1,err,h,x,RS40x,MPOL,POLP,POLM,S,Smin,Smax,RESPVINT
      REAL *8 LL,XX,dRS40,e0,FIRST,SECOND,THIRD,RESIDUAL,ERROR,ERRT,ddRS
      REAL *8 RS40test,eps
      REAL *8 xi(N),y(N),e(N)
      external RS40x,RS40test,dfridr1,ddRS
      COMMON /MYPVI/xi,y,e,IERx
      common /NX/N1
      COMMON /ERR/IER
      N1=N
      ipar=1
      S=MPOL*MPOL
      Smin=POLM
      Smax=POLP
      LL=DLOG(Smax/Smin)
      XX=DLOG((Smax-S)/(S-Smin))
      h=(Smax-Smin)/N
c      dRS40=dfridr1(RS40test,ipar,s,h,err)
      dRS40=dfridr1(RS40x,ipar,s,h,err)
      e0=err
      x=Smin
      eps=1.d-6
      IER_SAV=IER
      IER=1
      do i=1,N
         xi(i)=x
         if (abs(x-s).lt.eps) then
c            y(i)=(dfridr1(RS40test,ipar,x,h,err)-dRS40)/x*S
            y(i)=(dfridr1(RS40x,ipar,x,h,err)-dRS40)/x*S
            e(i)=sqrt(abs(err**2-e0**2))/x*S
         else
c            y(i)=((RS40test(x)-RS40test(s))/(x-s)-dRS40)/x*S
            y(i)=((RS40x(x)-RS40x(s))/(x-s)-dRS40)/x*S
         endif
         x=x+h
      enddo
      IERx=0
      CALL SIMPS(ddRS,Smin,Smax,N,RESIDUAL)
      IERx=1
      CALL SIMPS(ddRS,Smin,Smax,N,ERROR)
      IERx=0
c      FIRST=RS40test(S)*(XX-LL)
      FIRST=RS40x(s)*(XX-LL)
      SECOND=dRS40*LL*S
      THIRD=RESIDUAL
      ERROR=ERROR
      RESPVINT=FIRST+SECOND+THIRD
      ERRT=ERROR
      write (20,123) MPOL,FIRST,SECOND,THIRD
      write (21,123) MPOL,RESPVINT,XX-LL,ERROR
c      write (22,123) MPOL,RS40x(s),dRS40
      IER=IER_SAV
      return
 123  format(1x,1pe11.4,3(2x,1pe11.4))
      end

      function ddRS(S)
      implicit none
      integer N,j,IER
      parameter(N=100)
      REAL *8 ddRS,S,x,y,e
      dimension x(N),y(N),e(N)
      COMMON /MYPVI/x,y,e,IER
      call getindex(S,N,x,j)
      if (IER.EQ.0) then
         ddRS=(y(j+1)-y(j))/(x(j+1)-x(j))*(s-x(j))
      else
         ddRS=(e(j+1)-e(j))/(x(j+1)-x(j))*(s-x(j))
      endif
      return
      end

      function rs40test(s)
      implicit none
      real *8 rs40test,s
      rs40test=1.d0
      rs40test=1.d0+s
      rs40test=1.d0+s+0.1d0*s**2
      return
      end
c
      subroutine skipprint(iunit,icount,e,y,z,u)
c this is a list of encounered numerical instable point which we skip to be printed
      implicit none
      integer iunit,icount,IKE,IPV,IREN,IRENO
      real *8 e,y,z,u,fm
      COMMON/FFF/IKE,IPV,IREN,IRENO
      common /fermi/fm
      if ((e.eq.1.d0).or.(e.eq.1.04d0).or.(e.eq.1.4d0).or.
     &     (e.eq.1.85d0).or.
     &    (e.eq.3.2d0).or.(e.eq.3.6d0).or.(e.eq.5.2d0)) then
         icount=icount-1
      else
         if ((IKE.EQ.8).OR.(IKE.EQ.9)) then
            write (iunit,123) fm/e,y,z,u
         else
            write (iunit,123) e,y,z,u
         endif
      endif
 123  format(1x,1pe15.8,3(2x,1pe13.6))
      return
      end
C
      subroutine ASTOR(icount,E1,E2,R1,R2,R3,R4)
      implicit none
      integer icount
      real *8 E1,E2,R1,R2,R3,R4
      real *8 ETA(20,2),TAB(20,4)
      common /ASTO/ETA,TAB
      ETA(icount,1)=E1
      ETA(icount,2)=E2
      TAB(icount,1)=R1
      TAB(icount,2)=R2
      TAB(icount,3)=R3
      TAB(icount,4)=R4
      return
      end
c
      double precision function XABS(X)
      implicit none
      double precision X
c      XABS=ABS(X)
      XABS=X
      return
      end
c
      subroutine testRSflavorsplitting
      implicit none
      integer i,n,ier,ier_sav
      INTEGER IKE,IPV,IREN,IRENO,ICH,IKESAV
      double precision emin,emax,de,e,s,RS40,RS3G,RS33,RSGG,
     &     rs3gsmoothed,RS3GDAT,RSB3G,RSDAT,RSBGI,rs40smoothed
      external RS40,RS3G,RS33,rs3gsmoothed,RS3GDAT,RSB3G,
     &     RSDAT,RSBGI,r40gsmoothed
c,RSGG
      REAL *8 POLP,POLM,CHPTCUT,EC,ECUT
      COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
      COMMON/CHAN/ICH
      COMMON/RCUTS/CHPTCUT,EC,ECUT
      IKESAV=IKE
      IKE=-1
      ier_sav=ier
      ier=1
      n=260
      emin=0.32d0
      emax=2.50d0
      de=(emax-emin)/n
      e=emin
      WRITE(99,*) '       3.0000'
      WRITE(99,*) '       1.0000'
      WRITE(99,*) '     ',float(N)
      WRITE(99,*) ' e, rs3g, rs3gsmoothed, RS3GDAT, RSB3G'
      WRITE(80,*) '       3.0000'
      WRITE(80,*) '       1.0000'
      WRITE(80,*) '     ',float(N)
      WRITE(80,*) ' e, rs40, rs40smoothed, RSDAT, RSBGI'
      do i=1,n
         s=e*e
c         write (99,124) e,rs40(s,ier),
c     &        rs3g(s,ier),rs33(s,ier)
         write (99,124) e,rs3g(s,ier),
     &        rs3gsmoothed(s,ier),RS3GDAT(s),RSB3G(s)
         write (80,124) e,rs40(s,ier),
     &        rs40smoothed(s,ier),RSDAT(s),RSBGI(s)
         e=e+de
      enddo
      close(99)
      close(80)
      ier=ier_sav
      IKE=IKESAV
 124  format(1x,1pe15.8,4(2x,1pe13.6))
      return
      end
c
      subroutine testRSBGI(emin,emax)
      implicit none
      integer i,n,ier,iersav
      INTEGER IKE,IPV,IREN,IRENO,ICH,IKESAV
      double precision emin,emax,de,e,s,RS40,RS3G,RS33,RSGG,RSBGI,RSB3G,
     &     RSDAT,RS3GDAT,RSBN2,RSN2DAT,RSN2
      external RS40,RS3G,RS33,RSGG,RSBGI,RSB3G,RSDAT,RS3GDAT,
     &     RSBN2,RSN2DAT,RSN2
      REAL *8 POLP,POLM,CHPTCUT,EC,ECUT
      COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
      COMMON/CHAN/ICH
      COMMON/RCUTS/CHPTCUT,EC,ECUT
      IKESAV=IKE
      iersav=ier
      IKE=-1
      ier=1
      n=260
c      emin=3.0d0
c      emax=3.4d0
      de=(emax-emin)/n
      e=emin
      WRITE(90,*) '       3.0000'
      WRITE(90,*) '       1.0000'
      WRITE(90,*) '     ',float(N)
      WRITE(90,*) ' e,RSBGI(s),RSB3G(s),RSBGI(s)-RSB3G(s),RSBN2(s)'
      WRITE(97,*) '       3.0000'
      WRITE(97,*) '       1.0000'
      WRITE(97,*) '     ',float(N)
      WRITE(97,*)
     &     ' e,RSDAT(s),RS3GDAT(s),RSDAT(s)-RS3GDAT(s),RSN2DAT(s)'
      do i=1,n
         s=e*e
         write (90,124) e,RSBGI(s),RSB3G(s),RSBGI(s)-RSB3G(s),RSBN2(s)
         write (97,124) e,RSDAT(s),RS3GDAT(s),RSDAT(s)-RS3GDAT(s),
     &        RSN2DAT(s)
c         write (97,124) e,RSDAT(s),RS3GDAT(s),RSDAT(s)-RS3GDAT(s),
c     &        2.d0*RSN2(s,ier)
         e=e+de
      enddo
      close(90)
      close(97)
      IKE=IKESAV
      ier=iersav
 124  format(1x,1pe15.8,4(2x,1pe13.6))
      return
      end
c
      subroutine nagdrivenGG(s)
      implicit none
      INTEGER     LW,       LIW
      PARAMETER  (LW=320000,LIW=LW/4)
      INTEGER IFAIL,KOUNT,IW(LIW)
      REAL *8 W(LW)
      integer i,j
      INTEGER IER,IER_SAV,IKE,IPV,IREN,IRENO
      integer inde(10)
      double precision s,e,de,e1,emin,emax,smin,smax,fac,
     &     dalnagroutine,M2,NULL,ANS1
      REAL *8 CHPTCUT,EC,ECUT,ANS,A,B,RSDAT,RSBGI,RES,EPSREL,ABSERR,
     &     POLM,POLP,MPOL,rat,dalanalytic,REPO,EPSRELglobal
      REAL *8 EL(15),EH(15),TB(15),TBL(15),TBH(15),TBS(15)
      COMMON /RANGES/EL,EH,inde
      COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
      COMMON/POLE/MPOL
      common /adp/fac
      common /spole/M2
      COMMON/RCUTS/CHPTCUT,EC,ECUT
      COMMON /REGG/TBL,TBH,TB,TBS
      COMMON /NAGRELERR/EPSREL
      external RSDAT,RSBGI
c      write (*,*) ' nagdrivenGG: begin'
      M2=s
      EPSRELglobal=1.d-1
      IER_SAV=IER
      IER=1
      IPV=0
      NULL=0.d0
      DO j=1,9
         TB (j)=NULL
         TBS(j)=NULL
         TBL(j)=NULL
         TBH(j)=NULL
      ENDDO
      REPO=NULL
      IF (POLM.LT.POLP) THEN
c integrate complement of pole range
c      write (*,*) ' nag pole range'
         IPV=1
         IFAIL=-1
         CALL D01AQF(RSBGI,POLM,POLP,M2,0D0,EPSREL,REPO,
     &        ABSERR,W,LW,IW,LIW,IFAIL)
         IF (IFAIL.NE.0) WRITE (*,*) ' NAGGG -- 1:', IFAIL
         REPO=fac*REPO
      ENDIF
      DO j=1,5
         A=EL(j)**2
         B=EH(j)**2
c         IFAIL=-1
c         IPV=1
c         CALL D01AQF(RSBGI,A,B,M2,0D0,EPSREL,TB(j),
c     &        ABSERR,W,LW,IW,LIW,IFAIL)
c         TB(j)=fac*TB(j)
c      ENDDO
         IPV=0
c first collect contributions below pole region
         IF (B.LE.POLM) THEN
c pole region below intergration range
c            --|---|------|-------|--
c              SL  SH    POLM    POLP
c            write (*,*) ' nag loop 2 j:',j
            IFAIL=-1
            CALL D01AJF(RSDAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAGGG -- 2:', IFAIL
            TBL(j)=ANS1
         ELSE IF (A.GE.POLP) THEN
c pole region above intergration range
c            --|-------|----|---|--
c             POLM    POLP  SL  SH
c            write (*,*) ' nag loop 3 j:',j
            IFAIL=-1
            CALL D01AJF(RSDAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAGGG -- 3:', IFAIL
            TBH(j)=ANS1
         ELSE IF ((A.LT.POLM).AND.(B.GE.POLM)) THEN
c POLM  contained in integration range
c            --|-------|----|---|--   or  --|-------|-----|----|---
c              SL     POLM  SH  POLP        SL     POLM  POLP SH
c            write (*,*) ' nag loop 4 j:',j
            IFAIL=-1
            CALL D01AJF(RSDAT,A,POLM,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAGGG -- 4:', IFAIL
            TBL(j)=ANS1
            IF (B.GT.POLP) THEN
c in addition POLP in integration range
c            --|-------|-----|----|---
c              SL     POLM  POLP SH
c               write (*,*) ' nag loop 5 j:',j
               IFAIL=-1
               CALL D01AJF(RSDAT,POLP,B,0.D0,EPSREL,ANS1,ABSERR,
     .              W,LW,IW,LIW,IFAIL)
               IF (IFAIL.NE.0) WRITE (*,*) ' NAGGG -- 5:', IFAIL
               TBH(j)=ANS1
            ENDIF
         ELSE IF ((A.GE.POLM).AND.(B.GT.POLP)) THEN
c remaining case
c            ---|----|---|-----|---
c              POLM  SL  POLP  SH
c            write (*,*) ' nag loop 6 j:',j
            IFAIL=-1
            CALL D01AJF(RSDAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAGGG -- 6:', IFAIL
            TBH(j)=ANS1
         ELSE
         ENDIF
         TBL(j)=fac*TBL(j)
         TBH(j)=fac*TBH(j)
         TBS(J)=TBL(j)+TBH(j)
         IF ((MPOL.GT.EL(j)).and.(MPOL.LE.EH(j))) THEN
            TBS(j)=TBS(j)+REPO
         ENDIF
      ENDDO
      IER=IER_SAV
      do i=1,3
         write (84,*) TBL(i)
      enddo
      return
      end
c
      subroutine nagdriven3G(s)
      implicit none
      INTEGER     LW,       LIW
      PARAMETER  (LW=320000,LIW=LW/4)
      INTEGER IFAIL,KOUNT,IW(LIW)
      REAL *8 W(LW)
      integer j
      INTEGER IER,IER_SAV,IKE,IPV,IREN,IRENO
      integer inde(10)
      double precision s,e,de,e1,emin,emax,smin,smax,fac,
     &     dalnagroutine,M2,NULL,ANS1
      REAL *8 CHPTCUT,EC,ECUT,ANS,A,B,RS3GDAT,RSB3G,RES,EPSREL,ABSERR,
     &     POLM,POLP,MPOL,rat,dalanalytic,REPO
      REAL *8 EL(15),EH(15),TG(15),TGL(15),TGH(15),TGS(15)
      COMMON /RANGES/EL,EH,inde
      COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
      COMMON/POLE/MPOL
      common /adp/fac
      common /spole/M2
      COMMON/RCUTS/CHPTCUT,EC,ECUT
      COMMON /RE3G/TGL,TGH,TG,TGS
      COMMON /NAGRELERR/EPSREL
      external RS3GDAT,RSB3G
c      write (*,*) ' nagdriven3G: begin'
      M2=s
c      EPSREL=1.d-3
      IER_SAV=IER
      IER=1
      IPV=0
      NULL=0.d0
      DO j=1,9
         TG (j)=NULL
         TGS(j)=NULL
         TGL(j)=NULL
         TGH(j)=NULL
      ENDDO
      REPO=NULL
      IF (POLM.LT.POLP) THEN
c integrate complement of pole range
         IPV=1
         IFAIL=-1
         CALL D01AQF(RSB3G,POLM,POLP,M2,0D0,EPSREL,REPO,
     &        ABSERR,W,LW,IW,LIW,IFAIL)
         IF (IFAIL.NE.0) WRITE (*,*) ' NAG3G -- 1:', IFAIL
         REPO=fac*REPO
      ENDIF
      DO j=1,9
         A=EL(j)**2
         B=EH(j)**2
c         IFAIL=-1
c         IPV=1
c         CALL D01AQF(RSB3G,A,B,M2,0D0,EPSREL,TG(j),
c     &        ABSERR,W,LW,IW,LIW,IFAIL)
c         TG(j)=fac*TG(j)
c      ENDDO
         IPV=0
c first collect contributions below pole region
         IF (B.LE.POLM) THEN
c pole region below intergration range
c            --|---|------|-------|--
c              SL  SH    POLM    POLP
            IFAIL=-1
            CALL D01AJF(RS3GDAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAG3G -- 2:', IFAIL
            TGL(j)=ANS1
         ELSE IF (A.GE.POLP) THEN
c pole region above intergration range
c            --|-------|----|---|--
c             POLM    POLP  SL  SH
            IFAIL=-1
            CALL D01AJF(RS3GDAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAG3G -- 3:', IFAIL
            TGH(j)=ANS1
         ELSE IF ((A.LT.POLM).AND.(B.GE.POLM)) THEN
c POLM  contained in integration range
c            --|-------|----|---|--   or  --|-------|-----|----|---
c              SL     POLM  SH  POLP        SL     POLM  POLP SH
            IFAIL=-1
            CALL D01AJF(RS3GDAT,A,POLM,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAG3G -- 4:', IFAIL
            TGL(j)=ANS1
            IF (B.GT.POLP) THEN
c in addition POLP in integration range
c            --|-------|-----|----|---
c              SL     POLM  POLP SH
               IFAIL=-1
               CALL D01AJF(RS3GDAT,POLP,B,0.D0,EPSREL,ANS1,ABSERR,
     .              W,LW,IW,LIW,IFAIL)
               IF (IFAIL.NE.0) WRITE (*,*) ' NAG3G -- 5:', IFAIL
               TGH(j)=ANS1
            ENDIF
         ELSE IF ((A.GE.POLM).AND.(B.GT.POLP)) THEN
c remaining case
c            ---|----|---|-----|---
c              POLM  SL  POLP  SH
            IFAIL=-1
            CALL D01AJF(RS3GDAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAG3G -- 6:', IFAIL
            TGH(j)=ANS1
         ELSE
         ENDIF
         TGL(j)=fac*TGL(j)
         TGH(j)=fac*TGH(j)
         TGS(J)=TGL(j)+TGH(j)
         IF ((MPOL.GT.EL(j)).and.(MPOL.LE.EH(j))) THEN
            TGS(j)=TGS(j)+REPO
         ENDIF
      ENDDO
      IER=IER_SAV
      return
      end
c
      subroutine nagdriven33(s)
      implicit none
      INTEGER     LW,       LIW
      PARAMETER  (LW=320000,LIW=LW/4)
      INTEGER IFAIL,KOUNT,IW(LIW)
      REAL *8 W(LW)
      integer j
      INTEGER IER,IER_SAV,IKE,IPV,IREN,IRENO
      integer inde(10)
      double precision s,e,de,e1,emin,emax,smin,smax,fac,
     &     dalnagroutine,M2,NULL,ANS1
      REAL *8 CHPTCUT,EC,ECUT,ANS,A,B,RS33DAT,RSB33,RES,EPSREL,ABSERR,
     &     POLM,POLP,MPOL,rat,dalanalytic,REPO
      REAL *8 EL(15),EH(15),TG(15),TGL(15),TGH(15),TGS(15)
      COMMON /RANGES/EL,EH,inde
      COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
      COMMON/POLE/MPOL
      common /adp/fac
      common /spole/M2
      COMMON/RCUTS/CHPTCUT,EC,ECUT
      COMMON /RE33/TGL,TGH,TG,TGS
      COMMON /NAGRELERR/EPSREL
      external RS33DAT,RSB33
c      write (*,*) ' nagdriven33: begin'
      M2=s
c      EPSREL=1.d-3
      IER_SAV=IER
      IER=1
      IPV=0
      NULL=0.d0
      DO j=1,9
         TG (j)=NULL
         TGS(j)=NULL
         TGL(j)=NULL
         TGH(j)=NULL
      ENDDO
      REPO=NULL
      IF (POLM.LT.POLP) THEN
c integrate complement of pole range
         IPV=1
         IFAIL=-1
         CALL D01AQF(RSB33,POLM,POLP,M2,0D0,EPSREL,REPO,
     &        ABSERR,W,LW,IW,LIW,IFAIL)
         IF (IFAIL.NE.0) WRITE (*,*) ' NAG33 -- 1:', IFAIL
         REPO=fac*REPO
      ENDIF
      DO j=1,5
         A=EL(j)**2
         B=EH(j)**2
c         IFAIL=-1
c         IPV=1
c         CALL D01AQF(RSB33,A,B,M2,0D0,EPSREL,TG(j),
c     &        ABSERR,W,LW,IW,LIW,IFAIL)
c         TG(j)=fac*TG(j)
c      ENDDO
         IPV=0
c first collect contributions below pole region
         IF (B.LE.POLM) THEN
c pole region below intergration range
c            --|---|------|-------|--
c              SL  SH    POLM    POLP
            IFAIL=-1
            CALL D01AJF(RS33DAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAG33 -- 2:', IFAIL
            TGL(j)=ANS1
         ELSE IF (A.GE.POLP) THEN
c pole region above intergration range
c            --|-------|----|---|--
c             POLM    POLP  SL  SH
            IFAIL=-1
            CALL D01AJF(RS33DAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAG33 -- 3:', IFAIL
            TGH(j)=ANS1
         ELSE IF ((A.LT.POLM).AND.(B.GE.POLM)) THEN
c POLM  contained in integration range
c            --|-------|----|---|--   or  --|-------|-----|----|---
c              SL     POLM  SH  POLP        SL     POLM  POLP SH
            IFAIL=-1
            CALL D01AJF(RS33DAT,A,POLM,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAG33 -- 4:', IFAIL
            TGL(j)=ANS1
            IF (B.GT.POLP) THEN
c in addition POLP in integration range
c            --|-------|-----|----|---
c              SL     POLM  POLP SH
               IFAIL=-1
               CALL D01AJF(RS33DAT,POLP,B,0.D0,EPSREL,ANS1,ABSERR,
     .              W,LW,IW,LIW,IFAIL)
               IF (IFAIL.NE.0) WRITE (*,*) ' NAG33 -- 5:', IFAIL
               TGH(j)=ANS1
            ENDIF
         ELSE IF ((A.GE.POLM).AND.(B.GT.POLP)) THEN
c remaining case
c            ---|----|---|-----|---
c              POLM  SL  POLP  SH
            IFAIL=-1
            CALL D01AJF(RS33DAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAG33 -- 6:', IFAIL
            TGH(j)=ANS1
         ELSE
         ENDIF
         TGL(j)=fac*TGL(j)
         TGH(j)=fac*TGH(j)
         TGS(J)=TGL(j)+TGH(j)
         IF ((MPOL.GT.EL(j)).and.(MPOL.LE.EH(j))) THEN
            TGS(j)=TGS(j)+REPO
         ENDIF
      ENDDO
      IER=IER_SAV
      return
      end
c
      subroutine nagdrivenNF2(s)
      implicit none
      INTEGER     LW,       LIW
      PARAMETER  (LW=320000,LIW=LW/4)
      INTEGER IFAIL,KOUNT,IW(LIW)
      REAL *8 W(LW)
      integer j
      INTEGER IER,IER_SAV,IKE,IPV,IREN,IRENO
      integer inde(10)
      double precision s,e,de,e1,emin,emax,smin,smax,fac,
     &     dalnagroutine,M2,NULL,ANS1
      REAL *8 CHPTCUT,EC,ECUT,ANS,A,B,RSN2DAT,RSBN2,RES,EPSREL,ABSERR,
     &     POLM,POLP,MPOL,rat,dalanalytic,REPO
      REAL *8 EL(15),EH(15),TG(15),TGL(15),TGH(15),TGS(15)
      COMMON /RANGES/EL,EH,inde
      COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
      COMMON/POLE/MPOL
      common /adp/fac
      common /spole/M2
      COMMON/RCUTS/CHPTCUT,EC,ECUT
      COMMON /REN2/TGL,TGH,TG,TGS
      COMMON /NAGRELERR/EPSREL
      external RSN2DAT,RSBN2
c      write (*,*) ' nagdrivenNF2: begin'
      M2=s
c      EPSREL=1.d-3
      IER_SAV=IER
      IER=1
      IPV=0
      NULL=0.d0
      DO j=1,9
         TG (j)=NULL
         TGS(j)=NULL
         TGL(j)=NULL
         TGH(j)=NULL
      ENDDO
      REPO=NULL
      IF (POLM.LT.POLP) THEN
c integrate complement of pole range
         IPV=1
         IFAIL=-1
         CALL D01AQF(RSBN2,POLM,POLP,M2,0D0,EPSREL,REPO,
     &        ABSERR,W,LW,IW,LIW,IFAIL)
         IF (IFAIL.NE.0) WRITE (*,*) ' NAGN2 -- 1:', IFAIL
         REPO=fac*REPO
      ENDIF
      DO j=1,5
         A=EL(j)**2
         B=EH(j)**2
c         IFAIL=-1
c         IPV=1
c         CALL D01AQF(RSBN2,A,B,M2,0D0,EPSREL,TG(j),
c     &        ABSERR,W,LW,IW,LIW,IFAIL)
c         TG(j)=fac*TG(j)
c      ENDDO
         IPV=0
c first collect contributions below pole region
         IF (B.LE.POLM) THEN
c pole region below intergration range
c            --|---|------|-------|--
c              SL  SH    POLM    POLP
            IFAIL=-1
            CALL D01AJF(RSN2DAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAGN2 -- 2:', IFAIL
            TGL(j)=ANS1
         ELSE IF (A.GE.POLP) THEN
c pole region above intergration range
c            --|-------|----|---|--
c             POLM    POLP  SL  SH
            IFAIL=-1
            CALL D01AJF(RSN2DAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAGN2 -- 3:', IFAIL
            TGH(j)=ANS1
         ELSE IF ((A.LT.POLM).AND.(B.GE.POLM)) THEN
c POLM  contained in integration range
c            --|-------|----|---|--   or  --|-------|-----|----|---
c              SL     POLM  SH  POLP        SL     POLM  POLP SH
            IFAIL=-1
            CALL D01AJF(RSN2DAT,A,POLM,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAGN2 -- 4:', IFAIL
            TGL(j)=ANS1
            IF (B.GT.POLP) THEN
c in addition POLP in integration range
c            --|-------|-----|----|---
c              SL     POLM  POLP SH
               IFAIL=-1
               CALL D01AJF(RSN2DAT,POLP,B,0.D0,EPSREL,ANS1,ABSERR,
     .              W,LW,IW,LIW,IFAIL)
               IF (IFAIL.NE.0) WRITE (*,*) ' NAGN2 -- 5:', IFAIL
               TGH(j)=ANS1
            ENDIF
         ELSE IF ((A.GE.POLM).AND.(B.GT.POLP)) THEN
c remaining case
c            ---|----|---|-----|---
c              POLM  SL  POLP  SH
            IFAIL=-1
            CALL D01AJF(RSN2DAT,A,B,0.D0,EPSREL,ANS1,ABSERR,
     .           W,LW,IW,LIW,IFAIL)
            IF (IFAIL.NE.0) WRITE (*,*) ' NAGN2 -- 6:', IFAIL
            TGH(j)=ANS1
         ELSE
         ENDIF
         TGL(j)=fac*TGL(j)
         TGH(j)=fac*TGH(j)
         TGS(J)=TGL(j)+TGH(j)
         IF ((MPOL.GT.EL(j)).and.(MPOL.LE.EH(j))) THEN
            TGS(j)=TGS(j)+REPO
         ENDIF
      ENDDO
      IER=IER_SAV
      return
      end
cc
c      function FN2(s)
c      implicit none
c      integer IER
c      double precision fm,FN2,x,RSN2,FACT,E,E0,s,omega,MP2,M2
c      external RSN2
c      COMMON/MFPI/MP2,M2
c      common /fermi/fm
c      common /errorflag/ier
cc       fm=1.d0/0.1973269631d0   ! fermi in GeV^{-1}
cc       t=0.85*fm
cc       E0=1.d0/t
c      E0=sqrt(M2)
c      E=sqrt(s)
c      FACT=0.5d0*E*DEXP(-E/E0)/E0**3
c      FACT=FACT*9.d0/40.d0
cc normalize it to <3g>
c      FN2=2.d0*RSN2(s,IER)*FACT
c      return
c      end
cc
       FUNCTION RSN2DAT(S)
C      -----------------
c normalize it to <3g>= 2<33>
C Background: Integrand
c In case of PV pole in data set (time-like energies below ECUT) we need
c spline or least square fit in interval (POLM,POLP) around the pole: in
c that interval RSBGI=0 if IPV=0 and RSBGI=spline (ISPLINE=1) or fit if
c IPV=1; outside that interval RSBGI=linear interpolation of data
c new normalization 29/06/2013
C in data:
c <gg>=I_0+I_1+I_s + ...
c <3g>=1/2 I_1+3/4 I_s + ...
c <33>=1/4 I_1+9/16 I_s + ...
c <n2>=1/4 I_1
c use I_1 i.e. 4<n2>
       IMPLICIT NONE
       INTEGER IER,IKE,IPV,IREN,IRENO,Ismooth
       integer inde(10)
       REAL *8 E,S,RSN2DAT,RSN2,FACT,RSN2smoothed,
     &      CHPTCUT,EC,ECUT,POLP,POLM,MPOL,ELOW,EHIGH,N2pQCDcut
       REAL *8 EL(15),EH(15)
       EXTERNAL RSN2,FACT,RSN2smoothed
       COMMON /RANGES/EL,EH,inde
       COMMON/SMOO/Ismooth
       COMMON/RCUTS/CHPTCUT,EC,ECUT
       common /N2CUT/N2pQCDcut
       COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
       COMMON/POLE/MPOL
c this is required for trapezoidal rule intergration of data to avoid double counting with pQCD
c parametrized regions
       E=SQRT(S)
       ELOW =EL(2)
       EHIGH=EL(6)
       RSN2DAT=0.d0
c       write (14,*) ' RSN2DAT in :',e,n2pqcdcut,ier
       IF (E.GT.N2pQCDcut) THEN
c          write (*,*) ' RSN2DAT:',e,n2pqcdcut
          RSN2DAT=0.d0
       ELSE
          IF (((S.GT.POLM).AND.(S.LT.POLP)).AND.(IPV.EQ.0)) THEN
             RSN2DAT=0.D0
          ELSE
c             IF ((E.GT.ELOW).and.(E.LT.EHIGH)) THEN
c                RSN2DAT=2.d0*RSN2(s,IER)*FACT(s)
c             ELSE
c                if (icomplement.eq.1) then
c                   RSN2DAT=0.D0
c                else
             if (Ismooth.eq.1) then
                RSN2DAT=4.d0*RSN2smoothed(s,IER)*FACT(s)
             else
                RSN2DAT=4.d0*RSN2(s,IER)*FACT(s)
             endif
c                endif
c             ENDIF
          ENDIF
       ENDIF
c       write (14,*) ' RSN2DAT out ',RSN2DAT,ier
       RETURN
       END
C
       FUNCTION RSBN2(S)
C      -----------------
c normalize it to <3g>= 2<33>
C Background: Integrand
c 3g version of RSBGI i.e. in range 0.6 - 2.0 RSN2 data
c nonzero in range 0.61 to 2.0
c to be used in pole interval [POLM,POLP] and when IPV=1 where RSN2DAT is set zero
c new normalization 29/06/2013
C in data:
c <gg>=I_0+I_1+I_s + ...
c <3g>=1/2 I_1+3/4 I_s + ...
c <33>=1/4 I_1+9/16 I_s + ...
c <n2>=1/4 I_1
c use I_1 i.e. 4<n2>
       IMPLICIT NONE
       INTEGER IER,IKE,IPV,IREN,IRENO,Ismooth
       integer inde(10)
       REAL *8 RSN2,FACT,RSBN2,RSN2smoothed
       REAL *8 S,E,CHPTCUT,EC,ECUT,POLP,POLM,MPOL,ELOW,EHIGH,N2pQCDcut
       REAL *8 EL(15),EH(15)
       COMMON /RANGES/EL,EH,inde
       COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
       COMMON/RCUTS/CHPTCUT,EC,ECUT
       COMMON/POLE/MPOL
       COMMON/SMOO/Ismooth
       common /N2CUT/N2pQCDcut
       EXTERNAL RSN2,FACT,RSN2smoothed
       E=DSQRT(S)
       ELOW =EL(2)
       EHIGH=EL(6)
       RSBN2=0.d0
       IF (E.GT.N2pQCDcut) THEN
          RSBN2=0.d0
          RETURN
       ENDIF
       IF ((S.GT.POLM).AND.(S.LT.POLP)) THEN
          IF (IPV.EQ.0) THEN
             RSBN2=0.D0
          ELSE IF (IPV.EQ.1) THEN
c             IF ((MPOL.GT.ELOW).and.(MPOL.LT.EHIGH)) THEN
c extrapolate to full pole region [POLM,POLP]
             RSBN2=4.d0*RSN2smoothed(s,IER)*FACT(s)
c             RSBN2=2.d0*RSN2(s,IER)*FACT(s)
c             ELSE
c                RSBN2=RSN2smoothed(s,IER)*FACT(s)
c             ENDIF
          ENDIF
       ELSE
          if (Ismooth.eq.1) then
             RSBN2=4.d0*RSN2smoothed(s,IER)*FACT(s)
          else
             RSBN2=4.d0*RSN2(s,IER)*FACT(s)
          endif
c          ICH=8
c          IF ((E.GT.ELOW).and.(E.LT.EHIGH)) THEN
c          ELSE
c             RSBN2=2.d0*RSN2(s,IER)*FACT(s)
c          ENDIF
       ENDIF
       RETURN
       END

      function ECIfun(t,a,L,loss)
c Euclidean correlator for R=1 modulo 1/(12 pi^2)
c t^3 int_a^L domega omega^2 exp(-omega*t)
      implicit none
      double precision ECIfun,t,a,L,term1,term2,loss
      term1=(a**2*t**2+2.d0*a*t+2.d0)*dexp(-a*t)
      term2=(L**2*t**2+2.d0*L*t+2.d0)*dexp(-L*t)
      ECIfun=term1-term2
      loss=ECIfun/(term1+term2)
      return
      end

      function ECJfun(t,a,L,loss)
c Euclidean correlator for R=1 modulo 1/(12 pi^2)
c t^4 int_a^L domega omega^3 exp(-omega*t)
      implicit none
      double precision ECJfun,t,a,L,term1,term2,loss
      term1=(a**3*t**3+3.d0*a**2*t**2+6.d0*a*t+6.d0)*dexp(-a*t)
      term2=(t**3*L**3+3.d0*t**2*L**2+6.d0*t*L+6.d0)*dexp(-L*t)
      ECJfun=term1-term2
      loss=ECJfun/(term1+term2)
      return
      end

      logical function isinf(x)
      implicit none
      double precision pinf
      double precision ninf
      parameter(pinf=z'7FF0000000000000',ninf=z'7FF0000000000000')
      double precision x
      if ((x.eq.pinf).or.(x.eq.ninf)) then
         isinf=.true.
      else
         isinf=.false.
      endif
      return
      end

      include 'printscan.f'
C
       FUNCTION RS33DAT(S)
C      -----------------
C Background: Integrand
c In case of PV pole in data set (time-like energies below ECUT) we need
c spline or least square fit in interval (POLM,POLP) around the pole: in
c that interval RSBGI=0 if IPV=0 and RSBGI=spline (ISPLINE=1) or fit if
c IPV=1; outside that interval RSBGI=linear interpolation of data
       IMPLICIT NONE
       integer itestflavorsplitting,icomplement
       INTEGER IER,IKE,IPV,IREN,IRENO,Ismooth
       integer inde(10)
       REAL *8 E,S,RS33DAT,RS33,FACT,RS40,RS33smoothed,
     &      CHPTCUT,EC,ECUT,POLP,POLM,MPOL,ELOW,EHIGH
       REAL *8 EL(15),EH(15)
       EXTERNAL RS33,RSDAT,RS40,RS33smoothed
       COMMON /RANGES/EL,EH,inde
       COMMON/SMOO/Ismooth
       COMMON/RCUTS/CHPTCUT,EC,ECUT
       COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
       COMMON/POLE/MPOL
       common /testflavor/itestflavorsplitting,icomplement
c this is required for trapezoidal rule intergration of data to avoid double counting with pQCD
c parametrized regions
       E=SQRT(S)
       ELOW =EL(2)
       EHIGH=EL(6)
       RS33DAT=0.d0
       IF (((E.GT.EC).and.(E.LT.9.5d0)).or.(E.GT.ECUT)) THEN
          RS33DAT=0.d0
       ELSE
          IF (((S.GT.POLM).AND.(S.LT.POLP)).AND.(IPV.EQ.0)) THEN
             RS33DAT=0.D0
          ELSE
c             IF ((E.GT.ELOW).and.(E.LT.EHIGH)) THEN
c                RS33DAT=RS33(s,IER)*FACT(s)
c             ELSE
c                if (icomplement.eq.1) then
c                   RS33DAT=0.D0
c                else
             if (Ismooth.eq.1) then
                RS33DAT=RS33smoothed(s,IER)*FACT(s)
             else
                RS33DAT=RS33(s,IER)*FACT(s)
             endif
c                endif
c             ENDIF
          ENDIF
       ENDIF
       RETURN
       END
C
       FUNCTION RS3GDAT(S)
C      -----------------
C Background: Integrand
c In case of PV pole in data set (time-like energies below ECUT) we need
c spline or least square fit in interval (POLM,POLP) around the pole: in
c that interval RSBGI=0 if IPV=0 and RSBGI=spline (ISPLINE=1) or fit if
c IPV=1; outside that interval RSBGI=linear interpolation of data
       IMPLICIT NONE
       integer itestflavorsplitting,icomplement
       INTEGER IER,IKE,IPV,IREN,IRENO,Ismooth
       integer inde(10)
       REAL *8 E,S,RS3GDAT,RS3G,FACT,RS40,RS3Gsmoothed,
     &      CHPTCUT,EC,ECUT,POLP,POLM,MPOL,ELOW,EHIGH
       REAL *8 EL(15),EH(15)
       EXTERNAL RS3G,RSDAT,RS40,RS3Gsmoothed
       COMMON /RANGES/EL,EH,inde
       COMMON/SMOO/Ismooth
       COMMON/RCUTS/CHPTCUT,EC,ECUT
       COMMON/ERR/IER/POL/POLP,POLM/FFF/IKE,IPV,IREN,IRENO
       COMMON/POLE/MPOL
       common /testflavor/itestflavorsplitting,icomplement
c this is required for trapezoidal rule intergration of data to avoid double counting with pQCD
c parametrized regions
       E=SQRT(S)
       ELOW =EL(2)
       EHIGH=EL(6)
       RS3GDAT=0.d0
       IF (((E.GT.EC).and.(E.LT.9.5d0)).or.(E.GT.ECUT)) THEN
          RS3GDAT=0.d0
       ELSE
          IF (((S.GT.POLM).AND.(S.LT.POLP)).AND.(IPV.EQ.0)) THEN
             RS3GDAT=0.D0
          ELSE
c             IF ((E.GT.ELOW).and.(E.LT.EHIGH)) THEN
c                RS3GDAT=RS3G(s,IER)*FACT(s)
c             ELSE
c                if (icomplement.eq.1) then
c                   RS3GDAT=0.D0
c                else
             if (Ismooth.eq.1) then
                RS3GDAT=RS3Gsmoothed(s,IER)*FACT(s)
             else
                RS3GDAT=RS3G(s,IER)*FACT(s)
             endif
c                endif
c             ENDIF
          ENDIF
       ENDIF
       RETURN
       END
*
